@@ -18,9 +18,5 @@ Z* -------------------------------------------------------------------
 
 /* OBSOLETE -- USE THE FEEDBACK FACILITY */
 
-unsigned int DebugState=0;
-
-/* 
-DebugSelector | DebugParser | DebugMolecule | DebugMap
- */
 
+typedef int this_source_file_is_not_empty;
