@@ -18,12 +18,4 @@ Z* -------------------------------------------------------------------
 
 /* OBSOLETE -- USE THE FEEDBACK FACILITY */
 
-#define DebugSelector (1)
-#define DebugParser   (1<<1)
-#define DebugMolecule (1<<2)
-#define DebugPython   (1<<3)
-#define DebugMap      (1<<4)
-
-extern unsigned int DebugState;
-
 #endif
