@@ -33,6 +33,7 @@ typedef struct {
   unsigned int base_size;
 } CField;
 
+
 #define Ffloat3(f,a,b,c) (*((float*)((f)->data + \
                                      (a)*(f)->stride[0] + \
                                      (b)*(f)->stride[1] + \
@@ -89,8 +90,11 @@ CField *FieldNew(PyMOLGlobals *G,int *dim,int n_dim,unsigned int base_size,int t
 void FieldZero(CField *I);
 void FieldFree(CField *I);
 float FieldInterpolatef(CField *I,int a,int b,int c,float x,float y,float z);
+void FieldInterpolate3f(CField *I,int *locus,float *fract, float *result);
 
 PyObject *FieldAsPyList(CField *I);
 CField *FieldNewFromPyList(PyMOLGlobals *G,PyObject *list);
+CField *FieldNewCopy(PyMOLGlobals *G,CField *src);
+int FieldSmooth3f(CField *I);
 
 #endif
