@@ -41,71 +41,70 @@ float MapGetDiv(MapType *I)
 {
   return I->Div;
 }
+
 void MapFree(MapType *I)
 {
-  if(I)
-	 {
-      PyMOLGlobals *G=I->G;
-		CacheFreeP(G,I->Head,I->group_id,I->block_base + cCache_map_head_offset,false);
-		CacheFreeP(G,I->Link,I->group_id,I->block_base + cCache_map_link_offset,false);
-		CacheFreeP(G,I->EHead,I->group_id,I->block_base + cCache_map_ehead_offset,false);
-		CacheFreeP(G,I->EMask,I->group_id,I->block_base + cCache_map_emask_offset,false);
-		VLACacheFreeP(G,I->EList,I->group_id,I->block_base + cCache_map_elist_offset,false);
-	 }
+  if(I) {
+    CacheFreeP(I->G,I->Head,I->group_id,I->block_base + cCache_map_head_offset,false);
+    CacheFreeP(I->G,I->Link,I->group_id,I->block_base + cCache_map_link_offset,false);
+    CacheFreeP(I->G,I->EHead,I->group_id,I->block_base + cCache_map_ehead_offset,false);
+    CacheFreeP(I->G,I->EMask,I->group_id,I->block_base + cCache_map_emask_offset,false);
+    VLACacheFreeP(I->G,I->EList,I->group_id,I->block_base + cCache_map_elist_offset,false);
+  }
   OOFreeP(I);
 }
 
 void MapCacheInit(MapCache *M,MapType *I,int group_id,int block_base)
 {
   PyMOLGlobals *G=I->G;
-  int a,*p;
   M->G = G;
   M->block_base = I->block_base;
-  M->Cache = CacheAlloc(G,int,I->NVert,group_id,block_base + cCache_map_cache_offset);
+  M->Cache = CacheCalloc(G,int,I->NVert,group_id,block_base + cCache_map_cache_offset);
   M->CacheLink = CacheAlloc(G,int,I->NVert,group_id,block_base + cCache_map_cache_link_offset);
   M->CacheStart = -1;
-  p=M->Cache;
+  /*  p=M->Cache;
   for(a=0;a<I->NVert;a++)
-	 *(p++) = 0;
+  *(p++) = 0;*/
 }
 
 void MapCacheReset(MapCache *M)
 {
-	int		i		= M->CacheStart;
+	register int    i		= M->CacheStart;
 	register int* 	cachep	= M->Cache;
 	register int* 	clinkp	= M->CacheLink;
-	register int    i1 = 0, i2 = 0, i3 = 0, i4 = 0;
+	register int    i1 = 0, i2 = 0, i3 = 0, i4 = 0, ii;
 	while(i >= 0)  /* believe it or not, unrolling gives us almost 10%!!! */
 	{
+		ii = clinkp[i];
 		i1 = i;
-		i = clinkp[i];
-		if(i >= 0) {
+		i = ii;
+		if(ii >= 0) {
+			ii = clinkp[ii];
 			i2 = i;
-			i = clinkp[i];
+			i = ii;
 		}
-		if(i >= 0) {
+		cachep[i1] = 0; /* this doesn't look safe, but it is. i1-i4 are always valid indices*/
+		if(ii >= 0) {
+			ii = clinkp[ii];
 			i3 = i;
-			i = clinkp[i];
+			i = ii;
 		}	
-		if(i >= 0) {
+		cachep[i2] = 0; /* this doesn't look safe, but it is. i1-i4 are always valid indices*/
+		if(ii >= 0) {
+			ii = clinkp[ii];
 			i4 = i;
-			i = clinkp[i];
-		}
-      
-      /* this doesn't look safe, but it is. i1-i4 are always valid indices*/	
-		cachep[i1] = 0;
-		cachep[i2] = 0;
-		cachep[i3] = 0;
-		cachep[i4] = 0;
+			i = ii;
+		}      
+		cachep[i3] = 0; /* this doesn't look safe, but it is. i1-i4 are always valid indices*/
+		cachep[i4] = 0; /* this doesn't look safe, but it is. i1-i4 are always valid indices*/
 	}
 	M->CacheStart = -1;
 }
 
 void MapCacheFree(MapCache *M,int group_id,int block_base)
 {
-  PyMOLGlobals *G = M->G;
-  CacheFreeP(G,M->Cache,group_id,block_base + cCache_map_cache_offset,false);
-  CacheFreeP(G,M->CacheLink,group_id,block_base + cCache_map_cache_link_offset,false);
+  CacheFreeP(M->G,M->Cache,group_id,block_base + cCache_map_cache_offset,false);
+  CacheFreeP(M->G,M->CacheLink,group_id,block_base + cCache_map_cache_link_offset,false);
 }
 
 #define MapSafety 0.01F
@@ -236,196 +235,193 @@ int MapInsideXY(MapType *I,float *v,int *a,int *b,int *c) /* special version for
 	return(true);  
 }
 
-void MapSetupExpressXY(MapType *I,int n_vert) /* setup a list of XY neighbors for each square */
-{
-   PyMOLGlobals *G=I->G;
-	int n, a,b,c,flag;
-	register int d,e,i;
-	unsigned int mapSize;
-	int st, dim2;
-	int n_alloc = n_vert * 15; /* emprical est. */
+#define ELIST_GROW_FACTOR 3
 
-	PRINTFD(G,FB_Map)
+void MapSetupExpressXY(MapType *I,int n_vert, int negative_start) /* setup a list of XY neighbors for each square */
+{
+  PyMOLGlobals *G=I->G;
+  int n, a,b,c,flag;
+  register int d,e,i;
+  unsigned int mapSize;
+  int st, dim2;
+  int n_alloc = n_vert * 15; /* emprical est. */
+  
+  PRINTFD(G,FB_Map)
 	" MapSetupExpressXY-Debug: entered.\n"
 	ENDFD;
-	
-	mapSize		= I->Dim[0]*I->Dim[1]*I->Dim[2];
-	I->EHead	= CacheCalloc(G,int,mapSize,I->group_id,I->block_base + cCache_map_ehead_offset);
-	ErrChkPtr(G,I->EHead);
-	I->EList	= VLACacheMalloc(G,n_alloc,sizeof(int),5,0,
-                             I->group_id,I->block_base + cCache_map_elist_offset); 
-	I->EMask    = CacheCalloc(G,int,I->Dim[0]*I->Dim[1],
-                             I->group_id,I->block_base + cCache_map_emask_offset);
-	
-	n		= 1;
-	dim2	= I->Dim[2];
-	
-	for(a = I->iMin[0]; a <= I->iMax[0]; a++)
-	{
-		for(b = I->iMin[1]; b <= I->iMax[1]; b++)
-		{
-			for(c = I->iMin[2]; c <= I->iMax[2]; c++) /* a better alternative exists... */
-			{
-				int	*iPtr1	= (I->Head + ((a-1) * I->D1D2) + ((b-1)*dim2) + c);
-				
-				st		= n;
-				flag	= false;
-				
-				for(d = a-1; d <= a+1; d++)
-				{
-					/*int	*iPtr2	= (I->Head + (d * I->D1D2) + ((b-1)*dim2) + c);*/
-					int	*iPtr2	= iPtr1;
-					
-					for(e = b-1; e <= b+1; e++)
-					{
-						/*i	= *MapFirst(I,d,e,c);*/
-						i	= *iPtr2;
-						if(i >= 0)
-						{
-							flag	= true;
-							while(i >= 0)
-							{
-								VLACacheCheck(G,I->EList,int,n,I->group_id,I->block_base + cCache_map_elist_offset);
-								I->EList[n]	= i;
-								n++;
-								i	= MapNext(I,i);
-							}
-						}
-						
-						iPtr2	+= dim2;
-					}
-					
-					iPtr1 += I->D1D2;
-				}
-				
-				if(flag) 
-				{
-					*(I->EMask + I->Dim[1]*a + b) = true;
-					*(MapEStart(I,a,b,c))=st;
-					VLACacheCheck(G,I->EList,int,n,I->group_id,I->block_base + cCache_map_elist_offset);
-					I->EList[n]=-1;
-					n++;
-				}
-			}
-		}
-	}
-		
-	I->NEElem=n;
-	VLACacheSize(G,I->EList,int,I->NEElem,I->group_id,I->block_base + cCache_map_elist_offset);
-	PRINTFD(G,FB_Map)
-		" MapSetupExpressXY-Debug: leaving...\n"
-	ENDFD;
+  
+  mapSize = I->Dim[0]*I->Dim[1]*I->Dim[2];
+  I->EHead = CacheCalloc(G,int,mapSize,I->group_id,I->block_base + cCache_map_ehead_offset);
+  ErrChkPtr(G,I->EHead);
+  I->EList = VLACacheMalloc(G,n_alloc,sizeof(int),ELIST_GROW_FACTOR,0,
+                            I->group_id,I->block_base + cCache_map_elist_offset); 
+  I->EMask = CacheCalloc(G,int,I->Dim[0]*I->Dim[1],
+                         I->group_id,I->block_base + cCache_map_emask_offset);
+  
+  n		= 1;
+  dim2	= I->Dim[2];
+  
+  for(a = I->iMin[0]; a <= I->iMax[0]; a++) {
+    for(b = I->iMin[1]; b <= I->iMax[1]; b++) {
+      for(c = I->iMin[2]; c <= I->iMax[2]; c++) { /* a better alternative exists... */
+        int	*iPtr1	= (I->Head + ((a-1) * I->D1D2) + ((b-1)*dim2) + c);
+        
+        st		= n;
+        flag	= false;
+        
+        for(d = a-1; d <= a+1; d++) {
+          /*int	*iPtr2	= (I->Head + (d * I->D1D2) + ((b-1)*dim2) + c);*/
+          int	*iPtr2	= iPtr1;
+          
+          for(e = b-1; e <= b+1; e++) {
+            /*i	= *MapFirst(I,d,e,c);*/
+            i	= *iPtr2;
+            if(i >= 0) {
+              flag	= true;
+              while(i >= 0) {
+                
+                VLACacheCheck(G,I->EList,int,n,I->group_id,I->block_base + cCache_map_elist_offset);
+                I->EList[n]	= i;
+                n++;
+                i	= MapNext(I,i);
+              }
+            }
+            
+            iPtr2	+= dim2;
+          }
+          
+          iPtr1 += I->D1D2;
+        }
+        
+        if(flag) {
+          *(I->EMask + I->Dim[1]*a + b) = true;
+          *(MapEStart(I,a,b,c))= negative_start ? -st : st;
+          VLACacheCheck(G,I->EList,int,n,I->group_id,I->block_base + cCache_map_elist_offset);
+          I->EList[n]=-1;
+          n++;
+        }
+      }
+    }
+  }
+  
+  PRINTFB(G,FB_Map,FB_Blather)
+    " MapSetupExpressXY: %d rows in express table\n",n
+    ENDFB(G);
+  
+  I->NEElem=n;
+  VLACacheSize(G,I->EList,int,I->NEElem,I->group_id,I->block_base + cCache_map_elist_offset);
+  PRINTFD(G,FB_Map)
+    " MapSetupExpressXY-Debug: leaving...\n"
+    ENDFD;
 }
 
-
-
-void MapSetupExpressXYVert(MapType *I,float *vert,int n_vert) /* setup a list of XY neighbors for each square */
+void MapSetupExpressXYVert(MapType *I,float *vert,int n_vert,int negative_start) /* setup a list of XY neighbors for each square */
 {
-   PyMOLGlobals *G=I->G;
-	int		h, n, a,b,c;
-	int		j,k,dim2;
-	int		d,e;
-	float	*v;
-	int		*eBase, *hBase;
-	int      n_alloc = n_vert * 15; /* emprical est. */
+  PyMOLGlobals *G=I->G;
+  int		h, n, a,b,c;
+  int		j,k,dim2;
+  float	*v;
+  int		*eBase, *hBase;
+  int      n_alloc = n_vert * 15; /* emprical est. */
 	
-	PRINTFD(G,FB_Map)
-		" MapSetupExpressXY-Debug: entered.\n"
+  PRINTFD(G,FB_Map)
+    " MapSetupExpressXYVert-Debug: entered n_vert = %d negative_start = %d\n",n_vert, negative_start
 	ENDFD;
 	
-	/*mapSize 	= I->Dim[0]*I->Dim[1]*I->Dim[2];*/
-	I->EHead	= CacheCalloc(G,int, I->Dim[0]*I->Dim[1]*I->Dim[2],
+  /*mapSize 	= I->Dim[0]*I->Dim[1]*I->Dim[2];*/
+  I->EHead	= CacheCalloc(G,int, I->Dim[0]*I->Dim[1]*I->Dim[2],
                           I->group_id,I->block_base + cCache_map_ehead_offset);
-	I->EMask    = CacheCalloc(G,int,I->Dim[0]*I->Dim[1],
-                             I->group_id,I->block_base + cCache_map_emask_offset);
-	ErrChkPtr(G,I->EHead);
-	I->EList	= VLACacheMalloc(G,n_alloc,sizeof(int),5,0,
+  I->EMask    = CacheCalloc(G,int,I->Dim[0]*I->Dim[1],
+                            I->group_id,I->block_base + cCache_map_emask_offset);
+  ErrChkPtr(G,I->EHead);
+  I->EList	= VLACacheMalloc(G,n_alloc,sizeof(int),ELIST_GROW_FACTOR,0,
                              I->group_id,I->block_base + cCache_map_elist_offset); /* autozero */
 	
-	n		= 1;
-	v		= vert;
-	dim2	= I->Dim[2];
+  n		= 1;
+  v		= vert;
+  dim2	= I->Dim[2];
 		
-	for(h = 0; h < n_vert; h++) 
-	{ 
-		MapLocus(I,v,&j,&k,&c);
+  for(h = 0; h < n_vert; h++) {
+    MapLocus(I,v,&j,&k,&c);
+      
+    eBase	= I->EHead + ((j-1) * I->D1D2) + ((k-1)*dim2) + c;
+    hBase	= I->Head + (((j-1)-1) * I->D1D2);
+      
+    for(a = j-1; a <= j+1; a++) {
+      int *ePtr1	= eBase;
+        
+      for(b = k-1; b <= k+1; b++) {
+        if( *ePtr1 == 0 ) { /* if we haven't yet expanded this voxel... */
+          int	st		= n;
+          int	flag	= false;
+          int *hPtr1 = hBase + dim2 * (b-1) + (c-1);
 
-		eBase	= I->EHead + ((j-1) * I->D1D2) + ((k-1)*dim2) + c;
-		hBase	= I->Head + (((j-1)-1) * I->D1D2) + c;
-		
-		for(a = j-1; a <= j+1; a++)
-		{
-			int		*ePtr1	= eBase;
-			
-			for(b = k-1; b <= k+1; b++)
-			{
-				if( *ePtr1 == 0 )
-				{
-					int *hPtr1	= hBase + ((b-1) * dim2);
-					int	st		= n;
-					int	flag	= false;
-					
-					for(d = a-1; d <= a+1; d++)
-					{
-						int *hPtr2	= hPtr1;
-						
-						for(e = b-1; e <= b+1; e++)
-						{
-							int	i	= *hPtr2;
-							/*i	= *MapFirst(I,d,e,c);*/
-							
-							if(i > -1)
-							{
-								flag = true;
-								while(i > -1) 
-								{
-									VLACacheCheck(G,I->EList,int,n,I->group_id,I->block_base + cCache_map_elist_offset);
-									I->EList[n]	= i;
-									n++;
-									i = MapNext(I,i);
-								}
-							}
-							
-							hPtr2	+= dim2;
-						}
+          register int d,e,f;
 
-						hPtr1	+= I->D1D2;
-					}					
+          for(d = a-1; d <= a+1; d++) {
+            int *hPtr2 = hPtr1;
+            for(e = b-1; e <= b+1; e++) {
+              int *hPtr3 = hPtr2;
+              for(f = c-1; f <= c+1; f++) {
+                /*                register int i	= *MapFirst(I,d,e,f);*/
+                register int i	= *hPtr3;
+
+                if(i > -1) {
+                  flag = true;
+                  while(i > -1) {
+                    VLACacheCheck(G,I->EList,int,n,I->group_id,I->block_base + cCache_map_elist_offset);
+                    I->EList[n]	= i;
+                    n++;
+                    i = MapNext(I,i);
+                  }
+                }
+                hPtr3 ++;
+              }
+              hPtr2 += dim2;
+            }
+            hPtr1 += I->D1D2;
+          }
 					
-					if(flag) 
-					{
-						*(I->EMask + I->Dim[1]*a + b) = true;
-						*(MapEStart(I,a,b,c))	= st;
-						VLACacheCheck(G,I->EList,int,n,I->group_id,I->block_base + cCache_map_elist_offset);
-						I->EList[n] = -1;
-						n++;
-					}
-				}
+          if(flag) {
+            *(I->EMask + I->Dim[1]*a + b) = true;
+            *(MapEStart(I,a,b,c))=  negative_start ? -st : st;
+            VLACacheCheck(G,I->EList,int,n,I->group_id,I->block_base + cCache_map_elist_offset);
+            I->EList[n] = -1;
+            n++;
+          }
+        }
 				
-				ePtr1	+= dim2;
-			}
+        ePtr1	+= dim2;
+      }
 			
-			eBase	+= I->D1D2;
-			hBase	+= I->D1D2;
-		}
+      eBase	+= I->D1D2;
+      hBase	+= I->D1D2;
+    }
 		
-		v += 3;
-	}
+    v += 3;
+  }
+
+  PRINTFB(G,FB_Map,FB_Blather)
+    " MapSetupExpressXYVert: %d rows in express table\n",n
+    ENDFB(G);
 
 	I->NEElem = n;
+    VLACacheSize(G,I->EList,int,I->NEElem,I->group_id,I->block_base + cCache_map_elist_offset);
+
 	PRINTFD(G,FB_Map)
-		" MapSetupExpressXY-Debug: leaving...\n"
+		" MapSetupExpressXYVert-Debug: leaving...\n"
 	ENDFD;
 }
 
-void MapSetupExpressPerp(MapType *I, float *vert, float front)
+void MapSetupExpressPerp(MapType *I, float *vert, float front,int nVertHint,int negative_start)
 {
   PyMOLGlobals *G=I->G;
   int n=0;
-  int a,b,c,d,e,f,i;
+  int a,b,c,i;
 
   unsigned int mapSize;
-  int st,flag;
+  int st;
+  int      n_alloc = nVertHint * 15; /* emprical est. */
 
   register int iMin0 = I->iMin[0];
   register int iMin1 = I->iMin[1];
@@ -443,13 +439,13 @@ void MapSetupExpressPerp(MapType *I, float *vert, float front)
     ENDFD;
 
   mapSize = I->Dim[0]*I->Dim[1]*I->Dim[2];
-  I->EHead=CacheAlloc(G,int,mapSize,
+  I->EHead=CacheCalloc(G,int,mapSize,
                  I->group_id,I->block_base + cCache_map_ehead_offset);
   ErrChkPtr(G,I->EHead);
-  I->EList=VLACacheMalloc(G,1000,sizeof(int),5,0,
-                     I->group_id,I->block_base + cCache_map_elist_offset);
-
-    I->EMask    = CacheCalloc(G,int,I->Dim[0]*I->Dim[1],
+  I->EList=VLACacheMalloc(G,n_alloc,sizeof(int),ELIST_GROW_FACTOR,0,
+                          I->group_id,I->block_base + cCache_map_elist_offset);
+  
+  I->EMask = CacheCalloc(G,int,I->Dim[0]*I->Dim[1],
                             I->group_id,I->block_base + cCache_map_emask_offset);
   
   emask = I->EMask;
@@ -458,79 +454,95 @@ void MapSetupExpressPerp(MapType *I, float *vert, float front)
   premult = -front*iDiv;
 
   n=1;
-  for(a=(I->iMin[0]-1);a<=(I->iMax[0]+1);a++)
-	 for(b=(I->iMin[1]-1);b<=(I->iMax[1]+1);b++)
-		for(c=(I->iMin[2]-1);c<=(I->iMax[2]+1);c++)
-		  {
-          /* compute a "shadow" mask for all vertices */
+  for(a=(iMin0-1);a<=(iMax0+1);a++)
+    for(b=(iMin1-1);b<=(iMax1+1);b++)
+      for(c=(I->iMin[2]-1);c<=(I->iMax[2]+1);c++) {
 
-          i=*MapFirst(I,a,b,c);
-          while(i>=0) {
-            v0 = vert + 3*i;
-            perp_factor = premult/v0[2];
-            base0 = v0[0] * perp_factor;
-            base1 = v0[1] * perp_factor;
-            
-            d	= (int)(base0 - min0) + MapBorder;
-            e	= (int)(base1 - min1) + MapBorder;
-            
-            if(d < iMin0) {
-              d = iMin0; 
-            } else if(d > iMax0) { 
-              d = iMax0; 
-            }
-            
-            if(e < iMin1) { 
-              e = iMin1;
-            } else if(e > iMax1) {
-              e = iMax1;
-            }
-            i = link[i];
-            ptr2 = (ptr1 = emask + dim1*(d-1) + (e-1));
-            *(ptr2++) = true;
-            *(ptr2++) = true;
-            *(ptr2++) = true;
-            ptr2 = (ptr1 += dim1);
-            *(ptr2++) = true;
-            *(ptr2++) = true;
-            *(ptr2++) = true;
-            ptr2 = (ptr1 += dim1);
-            *(ptr2++) = true;
-            *(ptr2++) = true;
-            *(ptr2++) = true;
+        register int d,e,f;
+        
+        /* compute a "shadow" mask for all vertices */
+        
+        i=*MapFirst(I,a,b,c);
+        while(i>=0) {
+          v0 = vert + 3*i;
+          perp_factor = premult/v0[2];
+          base0 = v0[0] * perp_factor;
+          base1 = v0[1] * perp_factor;
+          
+          d	= (int)(base0 - min0);
+          e	= (int)(base1 - min1);
+          
+          d += MapBorder;
+          e += MapBorder;
+          
+          if(d < iMin0) {
+            d = iMin0; 
+          } else if(d > iMax0) { 
+            d = iMax0; 
           }
-
-            
+          
+          if(e < iMin1) { 
+            e = iMin1;
+          } else if(e > iMax1) {
+            e = iMax1;
+          }
+          i = link[i];
+          ptr2 = (ptr1 = emask + dim1*(d-1) + (e-1));
+          *(ptr2++) = true;
+          *(ptr2++) = true;
+          *(ptr2++) = true;
+          ptr2 = (ptr1 += dim1);
+          *(ptr2++) = true;
+          *(ptr2++) = true;
+          *(ptr2++) = true;
+          ptr2 = (ptr1 += dim1);
+          *(ptr2++) = true;
+          *(ptr2++) = true;
+          *(ptr2++) = true;
+        }
+        
+        {
+          const int am1=a-1, ap1=a+1, bm1=b-1, bp1=b+1, cm1=c-1, cp1=c+1;
+          const int dim2 = I->Dim[2];
+          register int flag=false;
+          register int *hPtr1	= I->Head + ((am1) * I->D1D2) + ((bm1)*dim2) + cm1;
           st=n;
-          flag=false;
-			 for(d=a-1;d<=a+1;d++)
-				for(e=b-1;e<=b+1;e++)
-				  for(f=c-1;f<=c+1;f++)
-					 {
-						i=*MapFirst(I,d,e,f);
-						if(i>=0) {
-						  flag=true;
-						  while(i>=0) {
-							 VLACacheCheck(G,I->EList,int,n,I->group_id,
-                                    I->block_base + cCache_map_elist_offset);
-							 I->EList[n]=i;
-                      i=link[i];
-							 n++;
-						  }
-						}
-					 }
-
-			 if(flag) {
-				*(MapEStart(I,a,b,c))=st;
-				VLACacheCheck(G,I->EList,int,n,I->group_id,
+          for(d=am1;d<=ap1;d++) {
+            register int *hPtr2 = hPtr1;
+            for(e=bm1;e<=bp1;e++) {
+              register int *hPtr3 = hPtr2;
+              for(f=cm1;f<=cp1;f++) {
+                i = *(hPtr3++);
+                /*                i=*MapFirst(I,d,e,f);*/
+                if(i>=0) {
+                  flag=true;
+                  while(i>=0) {
+                    VLACacheCheck(G,I->EList,int,n,I->group_id,
+                                  I->block_base + cCache_map_elist_offset);
+                    I->EList[n]=i;
+                    i=link[i];
+                    n++;
+                  }
+                }
+              }
+              hPtr2 += dim2;
+            }
+            hPtr1 += I->D1D2;
+          }
+          if(flag) {
+            *(MapEStart(I,a,b,c)) = negative_start ? -st : st;
+            VLACacheCheck(G,I->EList,int,n,I->group_id,
                           I->block_base + cCache_map_elist_offset);
-				I->EList[n]=-1;
-				n++;
-			 } else {
-				*(MapEStart(I,a,b,c))=0;
-			 }
-		  }
-
+            I->EList[n]=-1;
+            n++;
+          }
+        }
+      }
+  PRINTFB(G,FB_Map,FB_Blather)
+    " MapSetupExpressPerp: %d rows in express table\n",n
+    ENDFB(G);
+  I->NEElem=n;
+  VLACacheSize(G,I->EList,int,I->NEElem,I->group_id,I->block_base + cCache_map_elist_offset);
 
   PRINTFD(G,FB_Map)
     " MapSetupExpress-Debug: leaving...n=%d\n",n
@@ -548,18 +560,19 @@ void MapSetupExpress(MapType *I) /* setup a list of neighbors for each square */
   register int st,flag;
   register int *i_ptr3,*i_ptr4,*i_ptr5;
   register int *e_list;
+#ifdef _MemoryCache_ON
   register int block_offset = I->block_base + cCache_map_elist_offset;
   register int group_id = I->group_id;
+#endif
   int mx0=I->iMax[0],mx1=I->iMax[1],a,am1,ap2,*i_ptr1,b,bm1,bp2,*i_ptr2;
   unsigned int mapSize;
 
-
   PRINTFD(G,FB_Map)
     " MapSetupExpress-Debug: entered.\n"
     ENDFD;
 
   mapSize = I->Dim[0]*I->Dim[1]*I->Dim[2];
-  I->EHead=CacheAlloc(G,int,mapSize,group_id,I->block_base + cCache_map_ehead_offset);
+  I->EHead=CacheCalloc(G,int,mapSize,group_id,I->block_base + cCache_map_ehead_offset);
   ErrChkPtr(G,I->EHead);
   e_list=VLACacheMalloc(G,1000,sizeof(int),5,0,group_id,block_offset);
 
@@ -608,7 +621,8 @@ void MapSetupExpress(MapType *I) /* setup a list of neighbors for each square */
     }
   }
   I->EList = e_list;
-
+  I->NEElem=n;
+  VLACacheSize(G,I->EList,int,I->NEElem,group_id,block_offset);
   PRINTFD(G,FB_Map)
     " MapSetupExpress-Debug: leaving...n=%d\n",n
     ENDFD;
@@ -675,13 +689,20 @@ int MapExclLocus(MapType *I,float *v,int *a,int *b,int *c)
 float MapGetSeparation(PyMOLGlobals *G,float range,float *mx,float *mn,float *diagonal)
 {
   float maxSize;
-  float size,subDiv;
-
+  float size,maxSubDiv,divSize, subDiv[3];
+  float maxCubed, subDivCubed;
+  int a;
   maxSize = SettingGet(G,cSetting_hash_max);
 
+  maxCubed = maxSize * maxSize * maxSize;
+
   /* find longest axis */
 
   subtract3f(mx,mn,diagonal);
+  diagonal[0] = (float)fabs(diagonal[0]);
+  diagonal[1] = (float)fabs(diagonal[1]);
+  diagonal[2] = (float)fabs(diagonal[2]);
+  
   size=diagonal[0];
   if(diagonal[1]>size) size=diagonal[1];
   if(diagonal[2]>size) size=diagonal[2];
@@ -692,21 +713,43 @@ float MapGetSeparation(PyMOLGlobals *G,float range,float *mx,float *mn,float *di
     diagonal[2]=1.0;
     size = 1.0;
   }
+
   /* compute maximum number of subdivisions */
-  subDiv = (float)(size/(range+MapSafety)); 
-  if(subDiv>maxSize ) subDiv = maxSize; /* keep it reasonable - we're talking N^3 here... */
-  if(subDiv<1.0) subDiv = 1.0;
+  maxSubDiv = (float)(size/(range+MapSafety));
+  if(maxSubDiv<1.0F)
+    maxSubDiv = 1.0F;
+
+  /* find resulting divSize */
+  divSize = size/maxSubDiv;
+  if(divSize<MapSafety)
+    divSize = MapSafety;
+  for(a=0;a<3;a++) {
+    subDiv[a] = (float)((int)((diagonal[a] / divSize)+0.5F));
+    subDiv[a] = (subDiv[a] < 1.0F) ? 1.0F : subDiv[a];
+  }
+  subDivCubed = subDiv[0]*subDiv[1]*subDiv[2];
+
+  if(subDivCubed>maxCubed) {
+    divSize = (float)(divSize / pow(maxCubed/subDivCubed,0.33333F));
+  } else if(subDivCubed<maxCubed) {
+    divSize = (float)(divSize * pow(subDivCubed/maxCubed,0.33333F));    
+  }
+
+  if(divSize<(range+MapSafety))
+    divSize = range+MapSafety;
 
   if(Feedback(G,FB_Map,FB_Debugging)) {
     PRINTF
-      " MapGetSeparation: range %8.3f maxSize %8.3f subDiv %8.3f size %8.3f\n",range,maxSize,subDiv,size
+      " MapGetSeparation: range %8.3f divSize %8.3f size %8.3f\n",range,divSize,size
       ENDF(G);
-    dump3f(mx,"mx");
+    /*    dump3f(mx,"mx");
     dump3f(mn,"mn");
     dump3f(diagonal,"diagonal");
+    printf("%8.3f\n",divSize);
+  printf("divSize %8.3f\n",divSize);
+    */
   }
-
-  return(size/subDiv);
+  return(divSize);
 }
 
 MapType *MapNew(PyMOLGlobals *G,float range,float *vert,int nVert,float *extent)
@@ -756,70 +799,62 @@ static MapType *_MapNew(PyMOLGlobals *G,float range,float *vert,int nVert,float
   for(a=0;a<nVert;a++)
 	 I->Link[a] = -1;
 
-  if(extent)
-	 {
-		I->Min[0]=extent[0];
-		I->Max[0]=extent[1];
-		I->Min[1]=extent[2];
-		I->Max[1]=extent[3];
-		I->Min[2]=extent[4];
-		I->Max[2]=extent[5];
-	 }
-  else
-	 {
-		I->Min[0]=0.0F;
-		I->Max[0]=0.0F;
-		I->Min[1]=0.0F;
-		I->Max[1]=0.0F;
-		I->Min[2]=0.0F;
-		I->Max[2]=0.0F;
-		if(flag) {
-		  firstFlag=true;
-		  v=vert;
-		  for(a=0;a<nVert;a++)
-			 {
-				if(flag[a]) {
-				  if(firstFlag) {
-					 for(c=0;c<3;c++)
-						{
-						  I->Min[c]=v[c];
-						  I->Max[c]=v[c];
-						}
-					 firstFlag=false;
-				  } else {
-					 for(c=0;c<3;c++)
-						{
-						  if(I->Min[c]>v[c]) I->Min[c]=v[c];
-						  if(I->Max[c]<v[c]) I->Max[c]=v[c];
-						}
-				  }
-				}
-				v+=3;
-			 }
-		} else {
-        if(nVert) {
-          v=vert;
-          for(c=0;c<3;c++)
-            {
-              I->Min[c] = v[c];
-              I->Max[c] = v[c];
+  if(extent) {
+    I->Min[0]=extent[0];
+    I->Max[0]=extent[1];
+    I->Min[1]=extent[2];
+    I->Max[1]=extent[3];
+    I->Min[2]=extent[4];
+    I->Max[2]=extent[5];
+  }  else	 {
+    I->Min[0]=0.0F;
+    I->Max[0]=0.0F;
+    I->Min[1]=0.0F;
+    I->Max[1]=0.0F;
+    I->Min[2]=0.0F;
+    I->Max[2]=0.0F;
+    if(flag) {
+      firstFlag=true;
+      v=vert;
+      for(a=0;a<nVert;a++) {
+        if(flag[a]) {
+          if(firstFlag) {
+            for(c=0;c<3;c++) {
+              I->Min[c]=v[c];
+              I->Max[c]=v[c];
             }
-          v+=3;
-          for(a=1;a<nVert;a++)
-            {
-              for(c=0;c<3;c++)
-                {
-                  if(I->Min[c]>v[c])
-                    I->Min[c]=v[c];
-                  if(I->Max[c]<v[c])
-                    I->Max[c]=v[c];
-                }
-              v+=3;
+            firstFlag=false;
+          } else {
+            for(c=0;c<3;c++) {
+              
+              if(I->Min[c]>v[c]) I->Min[c]=v[c];
+              if(I->Max[c]<v[c]) I->Max[c]=v[c];
             }
+          }
         }
-		}
-	 }
-
+        v+=3;
+      }
+    } else {
+      if(nVert) {
+        v=vert;
+        for(c=0;c<3;c++) {
+            I->Min[c] = v[c];
+            I->Max[c] = v[c];
+          }
+        v+=3;
+        for(a=1;a<nVert;a++)  {
+          for(c=0;c<3;c++)  {
+            if(I->Min[c]>v[c])
+              I->Min[c]=v[c];
+            if(I->Max[c]<v[c])
+              I->Max[c]=v[c];
+          }
+          v+=3;
+        }
+      }
+    }
+  }
+  
   /* sanity check */
   for(a=0;a<3;a++) {
     if(I->Min[a]>I->Max[a]) {
@@ -828,7 +863,7 @@ static MapType *_MapNew(PyMOLGlobals *G,float range,float *vert,int nVert,float
       I->Min[a]=tmp_f;
     }
   }
-     
+  
   if(Feedback(G,FB_Map,FB_Debugging)) {
     printf(" MapSetup: %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f\n",
            I->Min[0],I->Min[1],I->Min[2],
@@ -903,27 +938,26 @@ static MapType *_MapNew(PyMOLGlobals *G,float range,float *vert,int nVert,float
   /* create 3-D hash of the vertices*/
   if(flag) {
 	 v=vert;
-	 for(a=0;a<nVert;a++)
-		{
-		  if(flag[a]) 
-			 if(MapExclLocus(I,v,&h,&k,&l)) {
-				list = MapFirst(I,h,k,l);
-				I->Link[a] = *list; 
-				*list = a; /*add to top of list*/
-			 }
-		  v+=3;
-		}
+	 for(a=0;a<nVert;a++)	{
+       if(flag[a]) 
+         if(MapExclLocus(I,v,&h,&k,&l)) {
+           list = MapFirst(I,h,k,l);
+           I->Link[a] = *list; 
+           *list = a; /*add to top of list*/
+         }
+       v+=3;
+     }
   } else {
-	 v=vert;
-	 for(a=0;a<nVert;a++)
-		{
-		  if(MapExclLocus(I,v,&h,&k,&l)) {
-			 list = MapFirst(I,h,k,l);
-			 I->Link[a] = *list; 
-			 *list = a; /*add to top of list*/
-		  }
-		  v+=3;
-		}
+    v=vert;
+    for(a=0;a<nVert;a++) {
+      if(MapExclLocus(I,v,&h,&k,&l)) {
+        list = MapFirst(I,h,k,l);
+        /*        printf("LINK %d %d %d %d %5.2f %5.2f %5.2f\n",a,h,k,l,v[0],v[1],v[2]);*/
+        I->Link[a] = *list; 
+        *list = a; /*add to top of list*/
+      }
+      v+=3;
+    }
   }
 
   PRINTFD(G,FB_Map)
