@@ -54,10 +54,50 @@ int MatrixInvTransformExtentsR44d3f(double *matrix,
     if(!c) {
       copy3d(out_tst,out_max);
       copy3d(out_tst,out_min);
+    } else {
+      for(a=0;a<3;a++) {
+        if(out_min[a]>out_tst[a]) out_min[a] = out_tst[a];
+        if(out_max[a]<out_tst[a]) out_max[a] = out_tst[a];
+      }
     }
-    for(a=0;a<3;a++) {
-      if(out_min[a]>out_tst[a]) out_min[a] = out_tst[a];
-      if(out_max[a]<out_tst[a]) out_max[a] = out_tst[a];
+  }
+  copy3d3f(out_min,new_min);
+  copy3d3f(out_max,new_max);
+  return 1;
+}
+int MatrixTransformExtentsR44d3f(double *matrix, 
+                              float *old_min, float *old_max,
+                              float *new_min, float *new_max)
+{
+  /* just brute-forcing this for now... */
+  int a;
+  int c;
+
+    
+  double inp_min[3],inp_max[3];
+  double out_min[3],out_max[3];
+  double inp_tst[3],out_tst[3];
+
+  if(!matrix) 
+    return 0;
+
+  copy3f3d(old_min,inp_min);
+  copy3f3d(old_max,inp_max);
+  
+  for(c=0;c<8;c++) {
+    inp_tst[0] = c&0x1 ? inp_min[0] : inp_max[0];
+    inp_tst[1] = c&0x2 ? inp_min[1] : inp_max[1];
+    inp_tst[2] = c&0x4 ? inp_min[2] : inp_max[2];
+
+    transform44d3d(matrix,inp_tst,out_tst);
+    if(!c) {
+      copy3d(out_tst,out_max);
+      copy3d(out_tst,out_min);
+    } else {
+      for(a=0;a<3;a++) {
+        if(out_min[a]>out_tst[a]) out_min[a] = out_tst[a];
+        if(out_max[a]<out_tst[a]) out_max[a] = out_tst[a];
+      }
     }
   }
   copy3d3f(out_min,new_min);
@@ -362,10 +402,10 @@ int *MatrixFilter(float cutoff,int window,int n_pass,int nv,float *v1,float *v2)
 /*========================================================================*/
 void MatrixTransformTTTfN3f( unsigned int n, float *q, float *m, float *p )
 {
-  const register float m0 = m[0],  m4 = m[4],  m8 = m[8],  m12 = m[12];
-  const register float m1 = m[1],  m5 = m[5],  m9 = m[9],  m13 = m[13];
-  const register float m2 = m[2],  m6 = m[6],  m10 = m[10],  m14 = m[14];
-  const register float m3 = m[3],  m7 = m[7],  m11 = m[11];
+  register const float m0 = m[0],  m4 = m[4],  m8 = m[8],  m12 = m[12];
+  register const float m1 = m[1],  m5 = m[5],  m9 = m[9],  m13 = m[13];
+  register const float m2 = m[2],  m6 = m[6],  m10 = m[10],  m14 = m[14];
+  register const float m3 = m[3],  m7 = m[7],  m11 = m[11];
   register float p0,p1,p2;
   while(n--) {
 	p0=*(p++) + m12;
@@ -379,10 +419,10 @@ void MatrixTransformTTTfN3f( unsigned int n, float *q, float *m, float *p )
 /*========================================================================*/
 void MatrixTransformR44fN3f( unsigned int n, float *q, float *m, float *p )
 {
-  const register float m0 = m[0],  m4 = m[4],  m8 = m[8];
-  const register float m1 = m[1],  m5 = m[5],  m9 = m[9];
-  const register float m2 = m[2],  m6 = m[6],  m10 = m[10];
-  const register float m3 = m[3],  m7 = m[7],  m11 = m[11];
+  register const float m0 = m[0],  m4 = m[4],  m8 = m[8];
+  register const float m1 = m[1],  m5 = m[5],  m9 = m[9];
+  register const float m2 = m[2],  m6 = m[6],  m10 = m[10];
+  register const float m3 = m[3],  m7 = m[7],  m11 = m[11];
   register float p0,p1,p2;
   while(n--) {
     p0=*(p++);
@@ -481,18 +521,18 @@ void MatrixMultiplyC44f( const float *b, float *m )
 void MatrixTransformC44f3f(float *m, float *q,float *p)
 {
   register float q0  = *q    , q1  = *(q+1), q2  = *(q+2);
-  *(p++) = m[ 0]*q0+m[ 4]*q1+m[ 8]*q2+m[12];
-  *(p++) = m[ 1]*q0+m[ 5]*q1+m[ 9]*q2+m[13];
-  *(p++) = m[ 2]*q0+m[ 6]*q1+m[10]*q2+m[14];
+  p[0] = m[ 0]*q0+m[ 4]*q1+m[ 8]*q2+m[12];
+  p[1] = m[ 1]*q0+m[ 5]*q1+m[ 9]*q2+m[13];
+  p[2] = m[ 2]*q0+m[ 6]*q1+m[10]*q2+m[14];
 }
 /*========================================================================*/
 void MatrixTransformC44f4f(float *m, float *q,float *p)
 {
   register float q0  = *q    , q1  = *(q+1), q2  = *(q+2);
-  *(p++) = m[ 0]*q0+m[ 4]*q1+m[ 8]*q2+m[12];
-  *(p++) = m[ 1]*q0+m[ 5]*q1+m[ 9]*q2+m[13];
-  *(p++) = m[ 2]*q0+m[ 6]*q1+m[10]*q2+m[14];
-  *(p++) = m[ 3]*q0+m[ 7]*q1+m[11]*q2+m[15];
+  p[0] = m[ 0]*q0+m[ 4]*q1+m[ 8]*q2+m[12];
+  p[1] = m[ 1]*q0+m[ 5]*q1+m[ 9]*q2+m[13];
+  p[2] = m[ 2]*q0+m[ 6]*q1+m[10]*q2+m[14];
+  p[3] = m[ 3]*q0+m[ 7]*q1+m[11]*q2+m[15];
 }
 /*========================================================================*/
 void MatrixInvTransformC44fAs33f3f(float *m, float *q,float *p)
@@ -500,17 +540,17 @@ void MatrixInvTransformC44fAs33f3f(float *m, float *q,float *p)
   /* multiplying a column major rotation matrix as row-major will
    * give the inverse rotation */
   register float q0  = *q    , q1  = *(q+1), q2  = *(q+2);
-  *(p++) = m[ 0]*q0+m[ 1]*q1+m[ 2]*q2;
-  *(p++) = m[ 4]*q0+m[ 5]*q1+m[ 6]*q2;
-  *(p++) = m[ 8]*q0+m[ 9]*q1+m[10]*q2;
+  p[0] = m[ 0]*q0+m[ 1]*q1+m[ 2]*q2;
+  p[1] = m[ 4]*q0+m[ 5]*q1+m[ 6]*q2;
+  p[2] = m[ 8]*q0+m[ 9]*q1+m[10]*q2;
 }
 /*========================================================================*/
 void MatrixTransformC44fAs33f3f(float *m, float *q, float *p) 
 {
   register float q0  = *q    , q1  = *(q+1), q2  = *(q+2);
-  *(p++) = m[ 0]*q0+m[ 4]*q1+m[ 8]*q2;
-  *(p++) = m[ 1]*q0+m[ 5]*q1+m[ 9]*q2;
-  *(p++) = m[ 2]*q0+m[ 6]*q1+m[10]*q2;
+  p[0] = m[ 0]*q0+m[ 4]*q1+m[ 8]*q2;
+  p[1] = m[ 1]*q0+m[ 5]*q1+m[ 9]*q2;
+  p[2] = m[ 2]*q0+m[ 6]*q1+m[10]*q2;
 }
 /*========================================================================*/
 float MatrixGetRMS(PyMOLGlobals *G,int n,float *v1,float *v2,float *wt)
@@ -596,20 +636,19 @@ float MatrixFitRMSTTTf(PyMOLGlobals *G,int n,float *v1,float *v2,float *wt,float
   vv2=v2;
 
   if(wt) {
-	for(c=0;c<n;c++)
-	  {
-		for(a=0;a<3;a++) {
-		  t1[a] += wt[c]*vv1[a];
-		  t2[a] += wt[c]*vv2[a];
-		}
-		if (wt[c]!=0.0F) {
-		  sumwt = sumwt + wt[c];
-		} else {
-		  sumwt = sumwt + 1.0F; /* WHAT IS THIS? */
-		}
-		vv1+=3;
-		vv2+=3;
-	  }
+	for(c=0;c<n;c++) {
+      for(a=0;a<3;a++) {
+        t1[a] += wt[c]*vv1[a];
+        t2[a] += wt[c]*vv2[a];
+      }
+      if (wt[c]!=0.0F) {
+        sumwt = sumwt + wt[c];
+      } else {
+        sumwt = sumwt + 1.0F; /* WHAT IS THIS? */
+      }
+      vv1+=3;
+      vv2+=3;
+    }
   } else {
 	for(c=0;c<n;c++)
 	  {
@@ -630,25 +669,24 @@ float MatrixFitRMSTTTf(PyMOLGlobals *G,int n,float *v1,float *v2,float *wt,float
   /* Calculate correlation matrix */
   vv1=v1;
   vv2=v2;
-  for(c=0;c<n;c++)
-	{
-	  if(wt) {
-		for(a=0;a<3;a++) {
-		  x[a] = wt[c]*(vv1[a] - t1[a]);
-		  xx[a] = wt[c]*(vv2[a] - t2[a]);
-		}
-	  } else {
-		for(a=0;a<3;a++) {
-		  x[a] = vv1[a] - t1[a];
-		  xx[a] = vv2[a] - t2[a];
-		}
-	  }
-	  for(a=0;a<3;a++)
-		for(b=0;b<3;b++)
-		  aa[a][b] = aa[a][b] + xx[a]*x[b];
-	  vv1+=3;
-	  vv2+=3;
-	}
+  for(c=0;c<n;c++) {
+    if(wt) {
+      for(a=0;a<3;a++) {
+        x[a] = wt[c]*(vv1[a] - t1[a]);
+        xx[a] = wt[c]*(vv2[a] - t2[a]);
+      }
+    } else {
+      for(a=0;a<3;a++) {
+        x[a] = vv1[a] - t1[a];
+        xx[a] = vv2[a] - t2[a];
+      }
+    }
+    for(a=0;a<3;a++)
+      for(b=0;b<3;b++)
+        aa[a][b] = aa[a][b] + xx[a]*x[b];
+    vv1+=3;
+    vv2+=3;
+  }
   if(n>1) {
     /* Primary iteration scheme to determine rotation matrix for molecule 2 */
     iters = 0;
@@ -675,13 +713,12 @@ float MatrixFitRMSTTTf(PyMOLGlobals *G,int n,float *v1,float *v2,float *wt,float
       sig = aa[iz][iy] - aa[iy][iz];
       gam = aa[iy][iy] + aa[iz][iz];
 
-      if(iters>=maxiter) 
-        {
-          PRINTFB(G,FB_Matrix,FB_Details)
-            " Matrix: Warning: no convergence (%1.8f<%1.8f after %d iterations).\n",(float)tol,(float)gam,iters
-            ENDFB(G);
-          break;
-        }
+      if(iters>=maxiter) {
+        PRINTFB(G,FB_Matrix,FB_Details)
+          " Matrix: Warning: no convergence (%1.8f<%1.8f after %d iterations).\n",(float)tol,(float)gam,iters
+          ENDFB(G);
+        break;
+      }
 
       /* Determine size of off-diagonal element.  If off-diagonals exceed the
          diagonal elements * tolerance, perform Jacobi rotation. */
@@ -735,43 +772,26 @@ float MatrixFitRMSTTTf(PyMOLGlobals *G,int n,float *v1,float *v2,float *wt,float
 
   err=err/sumwt;
   err=sqrt1d(err);
-  /*
-  ttt[0]=(float)m[0][0];
-  ttt[1]=(float)m[0][1];
-  ttt[2]=(float)m[0][2];
-  ttt[3]=(float)-t1[0];
-  ttt[4]=(float)m[1][0];
-  ttt[5]=(float)m[1][1];
-  ttt[6]=(float)m[1][2];
-  ttt[7]=(float)-t1[1];
-  ttt[8]=(float)m[2][0];
-  ttt[9]=(float)m[2][1];
-  ttt[10]=(float)m[2][2];
-  ttt[11]=(float)-t1[2];
-  ttt[12]=(float)t2[0];
-  ttt[13]=(float)t2[1];
-  ttt[14]=(float)t2[2];
-  ttt[15]=1.0F; 
-  */
 
   /* NOTE: TTT's are now row-major (to be more like homogenous matrices) */
 
-  ttt[ 0]=(float)m[0][0];
-  ttt[ 1]=(float)m[1][0];
-  ttt[ 2]=(float)m[2][0];
-  ttt[ 3]=(float)t2[0];
-  ttt[ 4]=(float)m[0][1];
-  ttt[ 5]=(float)m[1][1];
-  ttt[ 6]=(float)m[2][1];
-  ttt[ 7]=(float)t2[1];
-  ttt[ 8]=(float)m[0][2];
-  ttt[ 9]=(float)m[1][2];
-  ttt[10]=(float)m[2][2];
-  ttt[11]=(float)t2[2];
-  ttt[12]=(float)-t1[0];
-  ttt[13]=(float)-t1[1];
-  ttt[14]=(float)-t1[2];
-
+  if(ttt) {
+    ttt[ 0]=(float)m[0][0];
+    ttt[ 1]=(float)m[1][0];
+    ttt[ 2]=(float)m[2][0];
+    ttt[ 3]=(float)t2[0];
+    ttt[ 4]=(float)m[0][1];
+    ttt[ 5]=(float)m[1][1];
+    ttt[ 6]=(float)m[2][1];
+    ttt[ 7]=(float)t2[1];
+    ttt[ 8]=(float)m[0][2];
+    ttt[ 9]=(float)m[1][2];
+    ttt[10]=(float)m[2][2];
+    ttt[11]=(float)t2[2];
+    ttt[12]=(float)-t1[0];
+    ttt[13]=(float)-t1[1];
+    ttt[14]=(float)-t1[2];
+  }
 /* for compatibility with normal 4x4 matrices */
 
   if(fabs(err)<R_SMALL4)
@@ -1605,7 +1625,7 @@ integer *ierr;
     /* Local variables */
      doublereal norm;
      integer i__, j, k, l=0, m=0;
-     doublereal p, q=0.0, r__=0.0, s, t, w, x, y;
+     doublereal p=0.0, q=0.0, r__=0.0, s, t, w, x, y;
      integer na, en, ll, mm;
      doublereal zz;
      logical notlas;
@@ -1959,11 +1979,11 @@ integer *ierr;
     /* Local variables */
      doublereal norm;
      integer i__, j, k, l=0, m=0;
-     doublereal p, q, r__=0.0, s=0.0, t, w, x, y;
+     doublereal p=0.0, q=0.0, r__=0.0, s=0.0, t, w, x, y;
      integer na, ii, en, jj;
      doublereal ra, sa;
      integer ll, mm, nn;
-     doublereal vi, vr, zz;
+     doublereal vi, vr, zz=0.0;
      logical notlas;
      integer mp2, itn, its, enm2;
      doublereal tst1, tst2;
