@@ -18,7 +18,7 @@ Z* -------------------------------------------------------------------
 
 #include"PyMOLGlobals.h"
 
-int MyPNGWrite(PyMOLGlobals *G,char *file_name,unsigned char *p,unsigned int width,unsigned int height);
+int MyPNGWrite(PyMOLGlobals *G,char *file_name,unsigned char *p,unsigned int width,unsigned int height,float dpi);
 int MyPNGRead(char *file_name,unsigned char **p_ptr,unsigned int *width_ptr,unsigned int *height_ptr);
 
 #endif
