@@ -39,6 +39,7 @@ typedef struct _CPop CPop;
 typedef struct _CScene CScene;
 typedef struct _CSeq CSeq;
 typedef struct _CSetting CSetting;
+typedef struct _CSettingUnique CSettingUnique;
 typedef struct _CText CText;
 typedef struct _CWizard CWizard;
 typedef struct _CAtomInfo CAtomInfo;
@@ -49,8 +50,18 @@ typedef struct _CExecutive CExecutive;
 typedef struct _CSeeker CSeeker;
 typedef struct _CSelector CSelector;
 typedef struct _CTexture CTexture;
+typedef struct _CType CType;
 typedef struct _CMain CMain;
-typedef struct _CGO CGO;
+typedef struct _CPlugIOManager CPlugIOManager;
+
+#ifndef _PYMOL_NOPY
+typedef struct _CP_inst CP_inst;
+#endif
+
+#ifndef OVLexicon_DEFINED
+typedef struct _OVLexicon OVLexicon;
+#define OVLexicon_DEFINED
+#endif
 
 #ifndef CPyMOLOptions_DEFINED
 typedef struct _CPyMOLOptions CPyMOLOptions;
@@ -67,6 +78,11 @@ typedef struct _CPyMOL CPyMOL;
 #define CPyMOL_DEFINED
 #endif
 
+#ifndef CGO_DEFINED
+typedef struct _CGO                CGO;
+#define CGO_DEFINED
+#endif
+
 typedef struct _PyMOLGlobals PyMOLGlobals;
 struct _PyMOLGlobals {
 
@@ -91,6 +107,7 @@ struct _PyMOLGlobals {
   CGO       *DebugCGO; /* for debugging rendering */
   CSeq      *Seq;
   CSetting  *Setting;
+  CSettingUnique *SettingUnique;
   CText     *Text;
   CWizard   *Wizard;
   CAtomInfo *AtomInfo;
@@ -101,10 +118,18 @@ struct _PyMOLGlobals {
   CSeeker   *Seeker;
   CSelector *Selector;
   CTexture  *Texture;
+  CType     *Type;
   OVContext *Context;
   CMain     *Main; /* host/platform-specific "main" code */
   CPyMOLOptions *Option; 
   CPyMOL    *PyMOL; /* the instance */
+  OVLexicon *Lexicon; /* lexicon for data (e.g. label) strings */
+  CPlugIOManager *PlugIOManager;
+
+#ifndef _PYMOL_NOPY
+  CP_inst *P_inst;
+#endif
+
   /* global variables */
 
   int HaveGUI; /* do we have an OpenGL graphics window or are we
@@ -132,18 +157,13 @@ struct _PyMOLGlobals {
 
 };
 
-/* the following transitional global will disappear once we've
-   completely removed global state from PyMOL's code */
+/* if we're running PyMOL as a global singleton (old way / backward
+   compatible) then this global variable will contain a pointer to
+   PyMOL global state variables */
 
 #ifndef _PYMOL_NOPY
-extern PyMOLGlobals *TempPyMOLGlobals;
+extern PyMOLGlobals *SingletonPyMOLGlobals;
 #endif
 
-/* not a global, but CRay widely used and Ray.h definitely isn't a
- * lightweight include... */
-
-typedef struct _CRay               CRay;
-
-
 #endif
 
