@@ -24,6 +24,7 @@ Z* -------------------------------------------------------------------
 #include"Crystal.h"
 #include"Vector.h"
 #include"Feedback.h"
+#include"P.h"
 
 #define Trace_OFF
 
@@ -218,7 +219,7 @@ static int ProcessTetrahedron(int *edge,int nv,int v0,int v1,int v2,int v3,
   return nv;
 }
 /*===========================================================================*/
-int	TetsurfInit(PyMOLGlobals *G)
+static CTetsurf *TetsurfNew(PyMOLGlobals *G)
 {
 
    /* there are six tetrahedrons in each cube, and there are 
@@ -314,14 +315,12 @@ int	TetsurfInit(PyMOLGlobals *G)
 #define cM_101_111 0x20000
 #define cM_110_111 0x40000
 
-   register CTetsurf *I;
-	int	ok=true;
-	int	c;
+   register CTetsurf *I = Calloc(CTetsurf,1);
+   int	c;
    int   nv=1;
    int   last_nv;
    int v000,v100,v010,v110,v001,v101,v011,v111;
 
-   I = (G->Tetsurf = Calloc(CTetsurf,1));
    I->G=G;
    I->Tri = NULL;
    I->PtLink = NULL;
@@ -408,18 +407,127 @@ int	TetsurfInit(PyMOLGlobals *G)
         }
         }
    */
-	return(ok);
+   return I;
+}
+int	TetsurfInit(PyMOLGlobals *G)
+{
+  G->Tetsurf = TetsurfNew(G);
+  return 1;
+}
+/*===========================================================================*/
+static void  _TetsurfFree(CTetsurf *I)
+{
+  FreeP(I);
 }
 
-/*===========================================================================*/
 void  TetsurfFree(PyMOLGlobals *G)
 {
-  FreeP(G->Tetsurf);
+  _TetsurfFree(G->Tetsurf);
+  G->Tetsurf = NULL;
 }
-
 /*===========================================================================*/
-void TetsurfGetRange(Isofield *field,CCrystal *cryst,float *mn,float *mx,int *range)
+void TetsurfGetRange(PyMOLGlobals *G,
+                     Isofield *field,CCrystal *cryst,
+                     float *mn,float *mx,int *range)
 {
+  float rmn[3],rmx[3];
+  float imn[3],imx[3];
+  float mix[24],imix[24];
+  int a,b;
+  PRINTFD(G,FB_Isosurface)
+    " IsosurfGetRange: entered mn: %4.2f %4.2f %4.2f mx: %4.2f %4.2f %4.2f\n",
+    mn[0],mn[1],mn[2],mx[0],mx[1],mx[2]
+    ENDFD;
+
+  for(a=0;a<3;a++) {
+    rmn[a] = F4(field->points,0,0,0,a);
+    rmx[a] = F4(field->points,
+                field->dimensions[0]-1,
+                field->dimensions[1]-1,
+                field->dimensions[2]-1,a);
+  }
+
+  /* get min/max extents of map in fractional space */
+
+  transform33f3f(cryst->RealToFrac,rmn,imn);
+  transform33f3f(cryst->RealToFrac,rmx,imx);
+
+  mix[ 0]=mn[0];
+  mix[ 1]=mn[1];
+  mix[ 2]=mn[2];
+
+  mix[ 3]=mx[0];
+  mix[ 4]=mn[1];
+  mix[ 5]=mn[2];
+
+  mix[ 6]=mn[0];
+  mix[ 7]=mx[1];
+  mix[ 8]=mn[2];
+
+  mix[ 9]=mn[0];
+  mix[10]=mn[1];
+  mix[11]=mx[2];
+
+  mix[12]=mx[0];
+  mix[13]=mx[1];
+  mix[14]=mn[2];
+
+  mix[15]=mx[0];
+  mix[16]=mn[1];
+  mix[17]=mx[2];
+
+  mix[18]=mn[0];
+  mix[19]=mx[1];
+  mix[20]=mx[2];
+
+  mix[21]=mx[0];
+  mix[22]=mx[1];
+  mix[23]=mx[2];
+
+  /* compute min/max of query in fractional space */
+
+  for(b=0;b<8;b++) {
+    transform33f3f(cryst->RealToFrac,mix+3*b,imix+3*b);
+  }
+
+  for(a=0;a<3;a++) {
+    if(imx[a]!=imn[a]) { /* protect against div by zero */
+      int b;
+      int mini=0, maxi=0, tst_min, tst_max;
+      float cur;
+      for(b=0;b<8;b++) {
+        cur = ((field->dimensions[a]-1)*(imix[a+3*b]-imn[a])/(imx[a]-imn[a]));
+        tst_min = (int)floor(cur);
+        tst_max = ((int)ceil(cur))+1;
+        
+        if(!b) {
+          mini=tst_min;
+          maxi=tst_max;
+        } else {
+          if(mini>tst_min)
+            mini=tst_min;
+          if(maxi<=tst_max)
+            maxi=tst_max;
+        }
+      }
+
+      range[a] = mini;
+
+      range[a+3] = maxi;
+    } else {
+      range[a] = 0;
+      range[a+3] = 1;
+    }
+    if(range[a]<0) range[a]=0;
+    if(range[a]>field->dimensions[a]) range[a]=field->dimensions[a];
+    if(range[a+3]<0) range[a+3]=0;
+    if(range[a+3]>field->dimensions[a]) range[a+3]=field->dimensions[a];
+  }
+  PRINTFD(G,FB_Isosurface)
+    " IsosurfGetRange: returning range: %d %d %d %d %d %d\n",
+    range[0],range[1],range[2],range[3],range[4],range[5]
+    ENDFD;
+#if 0
   float fmn[3],fmx[3];
   float rmn[3],rmx[3];
   float imn[3],imx[3];
@@ -428,7 +536,9 @@ void TetsurfGetRange(Isofield *field,CCrystal *cryst,float *mn,float *mx,int *ra
   transform33f3f(cryst->RealToFrac,mx,fmx);
   for(a=0;a<3;a++) {
     rmn[a] = F4(field->points,0,0,0,a);
-    rmx[a] = F4(field->points,field->dimensions[0]-1,field->dimensions[1]-1,
+    rmx[a] = F4(field->points,
+                field->dimensions[0]-1,
+                field->dimensions[1]-1,
                 field->dimensions[2]-1,a);
   }
   
@@ -436,66 +546,76 @@ void TetsurfGetRange(Isofield *field,CCrystal *cryst,float *mn,float *mx,int *ra
   transform33f3f(cryst->RealToFrac,rmx,imx);
 
   for(a=0;a<3;a++) {
-    range[a] = (int)((field->dimensions[a]*(fmn[a]-imn[a])/(imx[a]-imn[a])));
+    range[a] = floor(((field->dimensions[a]-1)*(fmn[a]-imn[a])/(imx[a]-imn[a])));
+    range[a+3] = (int)(((field->dimensions[a]-1)*(fmx[a]-imn[a])/(imx[a]-imn[a])))+1;
     if(range[a]<0) range[a]=0;
-    range[a+3] = (int)((field->dimensions[a]*(fmx[a]-imn[a])/(imx[a]-imn[a]))+0.999F);
-    if(range[a]>field->dimensions[a])
-      range[a]=field->dimensions[a];
-    if(range[a+3]>field->dimensions[a])
-      range[a+3]=field->dimensions[a];
+    if(range[a]>field->dimensions[a]) range[a]=field->dimensions[a];
+    if(range[a+3]<0) range[a+3]=0;
+    if(range[a+3]>field->dimensions[a])  range[a+3]=field->dimensions[a];
   }
+#endif
+
 }
   /*===========================================================================*/
 int	TetsurfVolume(PyMOLGlobals *G,Isofield *field,float level,int **num,float **vert,
                     int *range,int mode,MapType *voxelmap,float *a_vert,
                     float carvebuffer, int side)
 {
-  register CTetsurf *I=G->Tetsurf;
+  
+  register CTetsurf *I;
+  if(PIsGlutThread()) {
+    I = G->Tetsurf;
+  } else {
+    I = TetsurfNew(G);
+  }
+
+  {
 	int	ok=true;
 	int	Steps[3];
 	int	c,i,j,k;
-   int range_store[6];
-   int n_strip = 0;
-   int n_vert = 0;
-
-   if(mode==3) 
-     IsofieldComputeGradients(G,field);
-
-   I->TotPrim=0;
-   if(range) {
-     for(c=0;c<3;c++)
-       {
-         I->AbsDim[c]=field->dimensions[c];
-         I->CurDim[c]=TetsurfSubSize+1;
-         Steps[c]=1+((range[3+c]-range[c])-1)/TetsurfSubSize;
-       }     
-   } else {
-     range=range_store;
-     for(c=0;c<3;c++)
-       {
-         range[c]=0;
-         range[3+c]=field->dimensions[c];
-         I->AbsDim[c]=field->dimensions[c];
-         I->CurDim[c]=TetsurfSubSize+1;
-         Steps[c]=1+(I->AbsDim[c]-1)/TetsurfSubSize;
-       }
-   }
-   
-   /*   for(c=0;c<3;c++) {
-        printf("range %d %d %d\n",c,range[c],range[c+3]);
-        printf("steps %d\n",Steps[c]);
+    int range_store[6];
+    int n_strip = 0;
+    int n_vert = 0;
+    int tot_prim = 0;
+    
+    if(mode==3) 
+      IsofieldComputeGradients(G,field);
+    
+    I->TotPrim=0;
+    if(range) {
+      for(c=0;c<3;c++)
+        {
+          I->AbsDim[c]=field->dimensions[c];
+          I->CurDim[c]=TetsurfSubSize+1;
+          Steps[c]=1+((range[3+c]-range[c])-1)/TetsurfSubSize;
+        }     
+    } else {
+      range=range_store;
+      for(c=0;c<3;c++)
+        {
+          range[c]=0;
+          range[3+c]=field->dimensions[c];
+          I->AbsDim[c]=field->dimensions[c];
+          I->CurDim[c]=TetsurfSubSize+1;
+          Steps[c]=1+(I->AbsDim[c]-1)/TetsurfSubSize;
         }
-   */
-     
+    }
+    
+    /*   for(c=0;c<3;c++) {
+         printf("range %d %d %d\n",c,range[c],range[c+3]);
+         printf("steps %d\n",Steps[c]);
+         }
+    */
+    
 	I->Coord=field->points;
 	I->Grad=field->gradients;
 	I->Data=field->data;
 	I->Level=level;
 	if(ok) ok=TetsurfAlloc(I);
-
+    
 	if(ok)
-		{
-
+      {
+        
 		for(i=0;i<Steps[0];i++)
 		for(j=0;j<Steps[1];j++)
 		for(k=0;k<Steps[2];k++)
@@ -546,7 +666,13 @@ int	TetsurfVolume(PyMOLGlobals *G,Isofield *field,float level,int **num,float **
    VLASize(*vert,float,n_vert*3);
    VLASize(*num,int,n_strip);
 
-	return(I->TotPrim);
+   tot_prim = I->TotPrim;
+   
+   if(!PIsGlutThread()) {
+     _TetsurfFree(I);
+   }
+   return(tot_prim);
+  }
 }
 /*===========================================================================*/
 static int	TetsurfAlloc(CTetsurf *II)
@@ -614,7 +740,10 @@ static void	TetsurfPurge(CTetsurf *II)
     }
 }
 /*===========================================================================*/
-__inline__ static void	TetsurfInterpolate2(float *pt,float *v0,float l0,float *v1,float l1,float level)
+#ifdef _PYMOL_INLINE
+__inline__
+#endif
+static void	TetsurfInterpolate2(float *pt,float *v0,float l0,float *v1,float l1,float level)
 {
   float	ratio;
   ratio=(level-l0)/(l1-l0);
@@ -624,7 +753,10 @@ __inline__ static void	TetsurfInterpolate2(float *pt,float *v0,float l0,float *v
 }
 
 /*===========================================================================*/
-__inline__ static void	TetsurfInterpolate4(float *pt,float *v0,float l0,float *v1,float l1
+#ifdef _PYMOL_INLINE
+__inline__
+#endif
+static void	TetsurfInterpolate4(float *pt,float *v0,float l0,float *v1,float l1
                           ,float l2,float l3,float level)
 
 {
@@ -647,7 +779,10 @@ __inline__ static void	TetsurfInterpolate4(float *pt,float *v0,float l0,float *v
 }
 
 /*===========================================================================*/
-__inline__  static void	TetsurfInterpolate8(float *pt,float *v0,float l0,float *v1,float l1,
+#ifdef _PYMOL_INLINE
+__inline__
+#endif
+static void	TetsurfInterpolate8(float *pt,float *v0,float l0,float *v1,float l1,
                           float l2,float l3,float l4,
                           float l5,float l6,float l7,float level)
 {
