@@ -20,12 +20,14 @@ Z* -------------------------------------------------------------------
 
 void UtilZeroMem(void *ptr,unsigned int howMuch);
 void UtilCopyMem(void *dst,void *src,unsigned int howMuch);
-void *UtilArrayMalloc(unsigned int *dim,int ndim,unsigned int atom_size);
+void *UtilArrayCalloc(unsigned int *dim,int ndim,unsigned int atom_size);
 char *UtilConcat(char *where,char *what);
 void UtilNConcat(char *dst,char *str,int n);
 void UtilConcatVLA(char **vla,int *cc,char *str);
+void UtilNPadVLA(char **vla,int *cc,char *str,int len);
 void UtilFillVLA(char **vla,int *cc,char what,int len);
 void UtilNCopy(char *dst,char *src,int n); /* up to N-1 chars */
+void UtilNCopyToLower(char *dst,char *src,int n); /* up to N-1 chars */
 void UtilCleanStr(char *s);
 int UtilCountStringVLA(char *vla);
 
@@ -35,6 +37,9 @@ void UtilFree(PyMOLGlobals *G);
 
 typedef int UtilOrderFn(void *array,int l,int r);
 void UtilSortIndex(int n,void *array,int *x,UtilOrderFn* fOrdered);
+
+void UtilSemiSortFloatIndex(int n,float *array,int *x,int forward);
+
 void UtilApplySortedIndices(int n,int *x, int rec_size, void *src, void *dst);
 
 void UtilSortInPlace(PyMOLGlobals *G,void *array,int nItem,unsigned int itemSize,UtilOrderFn *fOrdered);
