@@ -70,6 +70,14 @@ float get_random0to1f()
   return(rand()/(_1+RAND_MAX));
 }
 
+int pymol_roundf(float f)
+{
+  if(f>0.0F)
+    return (int)(f+0.49999F);
+  else
+    return (int)(f-0.49999F);
+}
+
 double slow_sqrt1f(float f) { /* no good as a macro because f is used twice */
   if(f>_0)
 	 return(sqrt(f));
@@ -105,30 +113,43 @@ void dump4f( float *v, char *prefix ) /* for debugging */
 
 void dump33f( float *m, char *prefix ) /* for debugging */
 {
-  printf("%s:0 %8.3f %8.3f %8.3f\n",prefix,m[0],m[1],m[2]);
-  printf("%s:1 %8.3f %8.3f %8.3f\n",prefix,m[3],m[4],m[5]);
-  printf("%s:2 %8.3f %8.3f %8.3f\n",prefix,m[6],m[7],m[8]);
+  if(m) {
+    printf("%s:0 %8.3f %8.3f %8.3f\n",prefix,m[0],m[1],m[2]);
+    printf("%s:1 %8.3f %8.3f %8.3f\n",prefix,m[3],m[4],m[5]);
+    printf("%s:2 %8.3f %8.3f %8.3f\n",prefix,m[6],m[7],m[8]);
+  } else {
+    printf("%s: (null matrix pointer)\n",prefix);
+  }
 }
 
 void dump44f( float *m, char *prefix ) /* for debugging */
 {
-  if(prefix) {
+  if(m) {
+    if(prefix) {
+      printf("%s:0 %8.3f %8.3f %8.3f %8.3f\n",prefix,m[0],m[1],m[2],m[3]);
+      printf("%s:1 %8.3f %8.3f %8.3f %8.3f\n",prefix,m[4],m[5],m[6],m[7]);
+      printf("%s:2 %8.3f %8.3f %8.3f %8.3f\n",prefix,m[8],m[9],m[10],m[11]);
+      printf("%s:3 %8.3f %8.3f %8.3f %8.3f\n",prefix,m[12],m[13],m[14],m[15]);
+    } else {
+    }
+  } else {
+    printf("%s: (null matrix pointer)\n",prefix);
+  }
+
+}
+
+void dump44d( double *m, char *prefix ) /* for debugging */
+{
+  if(m) {
     printf("%s:0 %8.3f %8.3f %8.3f %8.3f\n",prefix,m[0],m[1],m[2],m[3]);
     printf("%s:1 %8.3f %8.3f %8.3f %8.3f\n",prefix,m[4],m[5],m[6],m[7]);
     printf("%s:2 %8.3f %8.3f %8.3f %8.3f\n",prefix,m[8],m[9],m[10],m[11]);
     printf("%s:3 %8.3f %8.3f %8.3f %8.3f\n",prefix,m[12],m[13],m[14],m[15]);
   } else {
+    printf("%s: (null matrix pointer)\n",prefix);
   }
 }
 
-void dump44d( double *m, char *prefix ) /* for debugging */
-{
-  printf("%s:0 %8.3f %8.3f %8.3f %8.3f\n",prefix,m[0],m[1],m[2],m[3]);
-  printf("%s:1 %8.3f %8.3f %8.3f %8.3f\n",prefix,m[4],m[5],m[6],m[7]);
-  printf("%s:2 %8.3f %8.3f %8.3f %8.3f\n",prefix,m[8],m[9],m[10],m[11]);
-  printf("%s:3 %8.3f %8.3f %8.3f %8.3f\n",prefix,m[12],m[13],m[14],m[15]);
-}
-
 void dump33d( double *m, char *prefix ) /* for debugging */
 {
   printf("%s:0 %8.3f %8.3f %8.3f\n",prefix,m[0],m[1],m[2]);
@@ -267,6 +288,13 @@ void zero3f(float *v1)
 	v1[2]=_0;
 }
 
+void ones3f(float *v1)
+{
+  v1[0]=_1;
+  v1[1]=_1;
+  v1[2]=_1;
+}
+
 float dot_product3f ( float *v1, float *v2 )
 {
   return( v1[0]*v2[0] + v1[1]*v2[1] + v1[2]*v2[2]);
@@ -282,6 +310,13 @@ void invert3f (float *v)
   v[2]=-v[2];
 }
 
+void invert3f3f (float *v1,float *v2)
+{
+  v2[0]=-v1[0];
+  v2[1]=-v1[1];
+  v2[2]=-v1[2];
+}
+
 void scale3f ( float *v1,float v0,float *v2)
 {
   v2[0]=v1[0]*v0;
@@ -304,6 +339,7 @@ void copy3d ( double *v1,double *v2)
 }
 
 
+
 void add3f ( float *v1, float *v2, float *v3 )
 {
   v3[0]=v1[0]+v2[0];
@@ -413,6 +449,13 @@ void identity44f ( float *m1 )
   for(a=0;a<16;a++) m1[a]=_0;
   for(a=0;a<16;a=a+5) m1[a]=_1;
 }
+void identity44d ( double *m1 )
+{
+  int a;
+  for(a=0;a<16;a++) m1[a]=_d0;
+  for(a=0;a<16;a=a+5) m1[a]=_d1;
+}
+
 
 void copy44f ( float *src, float *dst )
 {
@@ -478,6 +521,70 @@ void copy44d33f ( double *src, float *dst )
   src++;
 }
 
+void copy44f33f ( float *src, float *dst )
+{
+  *(dst++)=*(src++);
+  *(dst++)=*(src++);
+  *(dst++)=*(src++);
+  src++;
+
+  *(dst++)=*(src++);
+  *(dst++)=*(src++);
+  *(dst++)=*(src++);
+  src++;
+
+  *(dst++)=*(src++);
+  *(dst++)=*(src++);
+  *(dst++)=*(src++);
+  src++;
+}
+
+void copy44f44d ( float *src, double *dst)
+{
+  *(dst++)=(double)*(src++);
+  *(dst++)=(double)*(src++);
+  *(dst++)=(double)*(src++);
+  *(dst++)=(double)*(src++);
+
+  *(dst++)=(double)*(src++);
+  *(dst++)=(double)*(src++);
+  *(dst++)=(double)*(src++);
+  *(dst++)=(double)*(src++);
+
+  *(dst++)=(double)*(src++);
+  *(dst++)=(double)*(src++);
+  *(dst++)=(double)*(src++);
+  *(dst++)=(double)*(src++);
+
+  *(dst++)=(double)*(src++);
+  *(dst++)=(double)*(src++);
+  *(dst++)=(double)*(src++);
+  *(dst++)=(double)*(src++);
+}
+
+void copy44d44f ( double *src, float *dst)
+{
+  *(dst++)=(float)*(src++);
+  *(dst++)=(float)*(src++);
+  *(dst++)=(float)*(src++);
+  *(dst++)=(float)*(src++);
+
+  *(dst++)=(float)*(src++);
+  *(dst++)=(float)*(src++);
+  *(dst++)=(float)*(src++);
+  *(dst++)=(float)*(src++);
+
+  *(dst++)=(float)*(src++);
+  *(dst++)=(float)*(src++);
+  *(dst++)=(float)*(src++);
+  *(dst++)=(float)*(src++);
+
+  *(dst++)=(float)*(src++);
+  *(dst++)=(float)*(src++);
+  *(dst++)=(float)*(src++);
+  *(dst++)=(float)*(src++);
+}
+
 
 void copy33f44d ( float *src, double *dst )
 {
@@ -503,6 +610,30 @@ void copy33f44d ( float *src, double *dst )
   *(dst++)=_1;
 }
 
+void copy33f44f ( float *src, float *dst )
+{
+  const float _0 = 0.0;
+  *(dst++)=*(src++);
+  *(dst++)=*(src++);
+  *(dst++)=*(src++);
+  *(dst++)=_0;
+
+  *(dst++)=*(src++);
+  *(dst++)=*(src++);
+  *(dst++)=*(src++);
+  *(dst++)=_0;
+
+  *(dst++)=*(src++);
+  *(dst++)=*(src++);
+  *(dst++)=*(src++);
+  *(dst++)=_0;
+
+  *(dst++)=_0;
+  *(dst++)=_0;
+  *(dst++)=_0;
+  *(dst++)=_1;
+}
+
 void transform33f3f (float *m1, float *m2, float *m3) 
 {
   float m2r0=m2[0];
@@ -526,6 +657,29 @@ void transpose33f33f ( float  *m1, float  *m2)
   m2[8] = m1[8];
 }
 
+void transpose44f44f ( float  *m1, float  *m2)
+{
+  m2[0 ] = m1[0 ];
+  m2[1 ] = m1[4 ];
+  m2[2 ] = m1[8 ];
+  m2[3 ] = m1[12];
+
+  m2[4 ] = m1[1 ];
+  m2[5 ] = m1[5 ];
+  m2[6 ] = m1[9 ];
+  m2[7 ] = m1[13];
+
+  m2[8 ] = m1[2 ];
+  m2[9 ] = m1[6 ];
+  m2[10] = m1[10];
+  m2[11] = m1[14];
+
+  m2[12] = m1[3 ];
+  m2[13] = m1[7 ];
+  m2[14] = m1[11];
+  m2[15] = m1[15];
+}
+
 void transform33Tf3f (float *m1, float *m2, float *m3) 
 {
   float m2r0=m2[0];
@@ -555,6 +709,15 @@ void transform44d3f (double *m1, float *m2, float *m3)
   m3[1] = (float) (m1[ 4] * m2r0 + m1[ 5] * m2r1 + m1[ 6] * m2r2 + m1[ 7]);
   m3[2] = (float) (m1[ 8] * m2r0 + m1[ 9] * m2r1 + m1[10] * m2r2 + m1[11]);
 }
+void transform44d3d (double *m1, double *m2, double *m3)
+{
+  register double m2r0 = m2[0];
+  register double m2r1 = m2[1];
+  register double m2r2 = m2[2];
+  m3[0] = (float) (m1[ 0] * m2r0 + m1[ 1] * m2r1 + m1[ 2] * m2r2 + m1[ 3]);
+  m3[1] = (float) (m1[ 4] * m2r0 + m1[ 5] * m2r1 + m1[ 6] * m2r2 + m1[ 7]);
+  m3[2] = (float) (m1[ 8] * m2r0 + m1[ 9] * m2r1 + m1[10] * m2r2 + m1[11]);
+}
 
 void transform44d3fas33d3f (double *m1, float *m2, float *m3)
 {
@@ -566,6 +729,36 @@ void transform44d3fas33d3f (double *m1, float *m2, float *m3)
   m3[2] = (float) (m1[ 8] * m2r0 + m1[ 9] * m2r1 + m1[10] * m2r2);
 }
 
+void transform44f3fas33f3f (float *m1, float *m2, float *m3)
+{
+  register float m2r0 = m2[0];
+  register float m2r1 = m2[1];
+  register float m2r2 = m2[2];
+  m3[0] =  (m1[ 0] * m2r0 + m1[ 1] * m2r1 + m1[ 2] * m2r2);
+  m3[1] =  (m1[ 4] * m2r0 + m1[ 5] * m2r1 + m1[ 6] * m2r2);
+  m3[2] =  (m1[ 8] * m2r0 + m1[ 9] * m2r1 + m1[10] * m2r2);
+}
+
+void inverse_transformC44f3f (float *m1, float *m2, float *m3)
+{
+  register float m2r0 = m2[0] - m1[12];
+  register float m2r1 = m2[1] - m1[13];
+  register float m2r2 = m2[2] - m1[14];
+  m3[0] = (float) (m1[ 0] * m2r0 + m1[ 1] * m2r1 + m1[ 2] * m2r2);
+  m3[1] = (float) (m1[ 4] * m2r0 + m1[ 5] * m2r1 + m1[ 6] * m2r2);
+  m3[2] = (float) (m1[ 8] * m2r0 + m1[ 9] * m2r1 + m1[10] * m2r2);
+}
+
+void inverse_transform44f3f (float *m1, float *m2, float *m3)
+{
+  register float m2r0 = m2[0] - m1[3];
+  register float m2r1 = m2[1] - m1[7];
+  register float m2r2 = m2[2] - m1[11];
+  m3[0] = (float) (m1[ 0] * m2r0 + m1[ 4] * m2r1 + m1[ 8] * m2r2);
+  m3[1] = (float) (m1[ 1] * m2r0 + m1[ 5] * m2r1 + m1[ 9] * m2r2);
+  m3[2] = (float) (m1[ 2] * m2r0 + m1[ 6] * m2r1 + m1[10] * m2r2);
+}
+
 void inverse_transform44d3f (double *m1, float *m2, float *m3)
 {
   register double m2r0 = m2[0] - m1[3];
@@ -598,7 +791,6 @@ void transform44f4f (float *m1, float *m2, float *m3)
 }
 
 
-
 void initializeTTT44f ( float *m )
 {
   int a;
@@ -616,36 +808,35 @@ void combineTTT44f44f( float *m1, float *m2, float *m3)
    TTTs are designed for easily creating movies of rotating 
    bodies! */
 {
+  float m1_homo[16];
+  float m2_homo[16];
+  float *src,*dst;
+  float pre[3],post[3];
 
-  /*  dump44f(m1,"m1");
-      dump44f(m2,"m2");*/
-  /* pre-translation */
+  /* convert the existing TTT into a homogenous transformation matrix */
+  
+  convertTTTfR44f(m1, m1_homo);
+  convertTTTfR44f(m2, m2_homo);
+  
+  /* combine the matrices */
 
-  m3[12] = m1[12] + m2[12];
-  m3[13] = m1[13] + m2[13];
-  m3[14] = m1[14] + m2[14];
+  left_multiply44f44f(m1_homo, m2_homo);
 
-  /* unused... */
+  /* now use the origin from the most recent TTT */
 
-  m3[15] = m1[15] * m2[15];
+  src = m1+12;
+  invert3f3f(src, pre);
 
-  /* transformation */
+  transform44f3fas33f3f(m2_homo, pre, post);
 
-  m3[ 0] = m1[ 0] * m2[ 0] + m1[ 1]  * m2[ 4] + m1[ 2] * m2[ 8];
-  m3[ 4] = m1[ 4] * m2[ 0] + m1[ 5]  * m2[ 4] + m1[ 6] * m2[ 8];
-  m3[ 8] = m1[ 8] * m2[ 0] + m1[ 9]  * m2[ 4] + m1[10] * m2[ 8];
-  m3[ 1] = m1[ 0] * m2[ 1] + m1[ 1]  * m2[ 5] + m1[ 2] * m2[ 9];
-  m3[ 5] = m1[ 4] * m2[ 1] + m1[ 5]  * m2[ 5] + m1[ 6] * m2[ 9];
-  m3[ 9] = m1[ 8] * m2[ 1] + m1[ 9]  * m2[ 5] + m1[10] * m2[ 9];
-  m3[ 2] = m1[ 0] * m2[ 2] + m1[ 1]  * m2[ 6] + m1[ 2] * m2[10];
-  m3[ 6] = m1[ 4] * m2[ 2] + m1[ 5]  * m2[ 6] + m1[ 6] * m2[10];
-  m3[10] = m1[ 8] * m2[ 2] + m1[ 9]  * m2[ 6] + m1[10] * m2[10];
+  m2_homo[ 3] += post[0];
+  m2_homo[ 7] += post[1];
+  m2_homo[11] += post[2];
 
-  /* post-translation */
+  dst = m2_homo+12;
 
-  m3[ 3] = m1[ 3] + m2[ 3];
-  m3[ 7] = m1[ 7] + m2[ 7];
-  m3[11] = m1[11] + m2[11];
+  copy3f(src,dst);
+  copy44f(m2_homo,m3);
 
 }
 
@@ -686,6 +877,39 @@ void convertTTTfR44d( float *ttt, double *homo)
 
 }
 
+void convertTTTfR44f( float *ttt, float *homo)
+{
+  /* takes the PyMOL-specific TTT matrix and 
+     makes a homogenous 4x4 txf matrix homo of it */
+
+  register float ttt_3  = ttt[3];
+  register float ttt_7  = ttt[7];
+  register float ttt_11 = ttt[11];
+  register float ttt_12 = ttt[12];
+  register float ttt_13 = ttt[13];
+  register float ttt_14 = ttt[14];
+
+  homo[ 0] = ttt[ 0];
+  homo[ 1] = ttt[ 1];
+  homo[ 2] = ttt[ 2];
+  homo[ 4] = ttt[ 4];
+  homo[ 5] = ttt[ 5];
+  homo[ 6] = ttt[ 6];
+  homo[ 8] = ttt[ 8];
+  homo[ 9] = ttt[ 9];
+  homo[10] = ttt[10];
+
+  homo[ 3] = (homo[ 0] * ttt_12) + (homo[ 1] * ttt_13) + (homo[ 2] * ttt_14) + ttt_3;
+  homo[ 7] = (homo[ 4] * ttt_12) + (homo[ 5] * ttt_13) + (homo[ 6] * ttt_14) + ttt_7;
+  homo[11] = (homo[ 8] * ttt_12) + (homo[ 9] * ttt_13) + (homo[10] * ttt_14) + ttt_11;
+
+  homo[12] = 0.0;
+  homo[13] = 0.0;
+  homo[14] = 0.0;
+  homo[15] = 1.0;
+
+}
+
 void convert44d44f(double *dbl, float *flt)
 {
   flt[ 0] = (float)dbl[ 0];
@@ -863,6 +1087,139 @@ void right_multiply44d44d( double *left, double *right)
 
 }
 
+
+
+void multiply44f44f44f( float *left, float *right, float *product)
+{
+  register float rA = right[ 0];
+  register float rB = right[ 4];
+  register float rC = right[ 8];
+  register float rD = right[12];
+  
+  product[ 0] = left[ 0] * rA + left[ 1] * rB + left[ 2] * rC + left[ 3] * rD;
+  product[ 4] = left[ 4] * rA + left[ 5] * rB + left[ 6] * rC + left[ 7] * rD;
+  product[ 8] = left[ 8] * rA + left[ 9] * rB + left[10] * rC + left[11] * rD;
+  product[12] = left[12] * rA + left[13] * rB + left[14] * rC + left[15] * rD;
+
+  rA = right[ 1];
+  rB = right[ 5];
+  rC = right[ 9];
+  rD = right[13];
+  
+  product[ 1] = left[ 0] * rA + left[ 1] * rB + left[ 2] * rC + left[ 3] * rD;
+  product[ 5] = left[ 4] * rA + left[ 5] * rB + left[ 6] * rC + left[ 7] * rD;
+  product[ 9] = left[ 8] * rA + left[ 9] * rB + left[10] * rC + left[11] * rD;
+  product[13] = left[12] * rA + left[13] * rB + left[14] * rC + left[15] * rD;
+
+  rA = right[ 2];
+  rB = right[ 6];
+  rC = right[10];
+  rD = right[14];
+  
+  product[ 2] = left[ 0] * rA + left[ 1] * rB + left[ 2] * rC + left[ 3] * rD;
+  product[ 6] = left[ 4] * rA + left[ 5] * rB + left[ 6] * rC + left[ 7] * rD;
+  product[10] = left[ 8] * rA + left[ 9] * rB + left[10] * rC + left[11] * rD;
+  product[14] = left[12] * rA + left[13] * rB + left[14] * rC + left[15] * rD;
+
+  rA = right[ 3];
+  rB = right[ 7];
+  rC = right[11];
+  rD = right[15];
+  
+  product[ 3] = left[ 0] * rA + left[ 1] * rB + left[ 2] * rC + left[ 3] * rD;
+  product[ 7] = left[ 4] * rA + left[ 5] * rB + left[ 6] * rC + left[ 7] * rD;
+  product[11] = left[ 8] * rA + left[ 9] * rB + left[10] * rC + left[11] * rD;
+  product[15] = left[12] * rA + left[13] * rB + left[14] * rC + left[15] * rD;
+}
+
+void left_multiply44f44f( float *left, float *right)
+{
+  register float rA = right[ 0];
+  register float rB = right[ 4];
+  register float rC = right[ 8];
+  register float rD = right[12];
+  
+  right[ 0] = left[ 0] * rA + left[ 1] * rB + left[ 2] * rC + left[ 3] * rD;
+  right[ 4] = left[ 4] * rA + left[ 5] * rB + left[ 6] * rC + left[ 7] * rD;
+  right[ 8] = left[ 8] * rA + left[ 9] * rB + left[10] * rC + left[11] * rD;
+  right[12] = left[12] * rA + left[13] * rB + left[14] * rC + left[15] * rD;
+
+  rA = right[ 1];
+  rB = right[ 5];
+  rC = right[ 9];
+  rD = right[13];
+  
+  right[ 1] = left[ 0] * rA + left[ 1] * rB + left[ 2] * rC + left[ 3] * rD;
+  right[ 5] = left[ 4] * rA + left[ 5] * rB + left[ 6] * rC + left[ 7] * rD;
+  right[ 9] = left[ 8] * rA + left[ 9] * rB + left[10] * rC + left[11] * rD;
+  right[13] = left[12] * rA + left[13] * rB + left[14] * rC + left[15] * rD;
+
+  rA = right[ 2];
+  rB = right[ 6];
+  rC = right[10];
+  rD = right[14];
+  
+  right[ 2] = left[ 0] * rA + left[ 1] * rB + left[ 2] * rC + left[ 3] * rD;
+  right[ 6] = left[ 4] * rA + left[ 5] * rB + left[ 6] * rC + left[ 7] * rD;
+  right[10] = left[ 8] * rA + left[ 9] * rB + left[10] * rC + left[11] * rD;
+  right[14] = left[12] * rA + left[13] * rB + left[14] * rC + left[15] * rD;
+
+  rA = right[ 3];
+  rB = right[ 7];
+  rC = right[11];
+  rD = right[15];
+  
+  right[ 3] = left[ 0] * rA + left[ 1] * rB + left[ 2] * rC + left[ 3] * rD;
+  right[ 7] = left[ 4] * rA + left[ 5] * rB + left[ 6] * rC + left[ 7] * rD;
+  right[11] = left[ 8] * rA + left[ 9] * rB + left[10] * rC + left[11] * rD;
+  right[15] = left[12] * rA + left[13] * rB + left[14] * rC + left[15] * rD;
+}
+
+void right_multiply44f44f( float *left, float *right)
+{
+  register float cA = left[ 0];
+  register float cB = left[ 1];
+  register float cC = left[ 2];
+  register float cD = left[ 3];
+  
+  left[ 0] = cA * right[ 0] + cB * right[ 4] + cC * right[ 8] + cD * right[12];
+  left[ 1] = cA * right[ 1] + cB * right[ 5] + cC * right[ 9] + cD * right[13];
+  left[ 2] = cA * right[ 2] + cB * right[ 6] + cC * right[10] + cD * right[14];
+  left[ 3] = cA * right[ 3] + cB * right[ 7] + cC * right[11] + cD * right[15];
+
+  cA = left[ 4];
+  cB = left[ 5];
+  cC = left[ 6];
+  cD = left[ 7];
+  
+  left[ 4] = cA * right[ 0] + cB * right[ 4] + cC * right[ 8] + cD * right[12];
+  left[ 5] = cA * right[ 1] + cB * right[ 5] + cC * right[ 9] + cD * right[13];
+  left[ 6] = cA * right[ 2] + cB * right[ 6] + cC * right[10] + cD * right[14];
+  left[ 7] = cA * right[ 3] + cB * right[ 7] + cC * right[11] + cD * right[15];
+
+  cA = left[ 8];
+  cB = left[ 9];
+  cC = left[10];
+  cD = left[11];
+  
+  left[ 8] = cA * right[ 0] + cB * right[ 4] + cC * right[ 8] + cD * right[12];
+  left[ 9] = cA * right[ 1] + cB * right[ 5] + cC * right[ 9] + cD * right[13];
+  left[10] = cA * right[ 2] + cB * right[ 6] + cC * right[10] + cD * right[14];
+  left[11] = cA * right[ 3] + cB * right[ 7] + cC * right[11] + cD * right[15];
+
+  cA = left[12];
+  cB = left[13];
+  cC = left[14];
+  cD = left[15];
+  
+  left[12] = cA * right[ 0] + cB * right[ 4] + cC * right[ 8] + cD * right[12];
+  left[13] = cA * right[ 1] + cB * right[ 5] + cC * right[ 9] + cD * right[13];
+  left[14] = cA * right[ 2] + cB * right[ 6] + cC * right[10] + cD * right[14];
+  left[15] = cA * right[ 3] + cB * right[ 7] + cC * right[11] + cD * right[15];
+
+}
+
+
 void invert_special44d44d(double *orig, double *inv)
 {
   /* inverse of the rotation matrix */
@@ -890,6 +1247,211 @@ void invert_special44d44d(double *orig, double *inv)
 
 }
 
+void invert_special44f44f(float *orig, float *inv)
+{
+  /* inverse of the rotation matrix */
+
+  inv[ 0] = orig[ 0];
+  inv[ 1] = orig[ 4];
+  inv[ 2] = orig[ 8];
+  inv[ 4] = orig[ 1];
+  inv[ 5] = orig[ 5];
+  inv[ 6] = orig[ 9];
+  inv[ 8] = orig[ 2];
+  inv[ 9] = orig[ 6];
+  inv[10] = orig[10];
+
+  /* invert the translation portion */
+
+  inv[ 3] = -(orig[ 3] * orig[ 0] + orig[ 7] * orig[ 4] + orig[11] * orig[ 8]);
+  inv[ 7] = -(orig[ 3] * orig[ 1] + orig[ 7] * orig[ 5] + orig[11] * orig[ 9]);
+  inv[11] = -(orig[ 3] * orig[ 2] + orig[ 7] * orig[ 6] + orig[11] * orig[10]);
+
+  inv[12] = 0.0F;
+  inv[13] = 0.0F;
+  inv[14] = 0.0F;
+  inv[15] = 1.0F;
+
+}
+
+static void normalize3dp( double *v1, double *v2, double *v3 )
+{
+  double vlen = sqrt1d((v1[0]*v1[0]) + 
+                       (v2[0]*v2[0]) + 
+                       (v3[0]*v3[0]));
+  if(vlen>R_SMALL)
+	 {
+		v1[0]/=vlen;
+		v2[0]/=vlen;
+		v3[0]/=vlen;
+	 }
+  else
+	 {
+		v1[0]=_0;
+		v2[1]=_0;
+		v3[2]=_0;
+	 }
+} 
+
+static void normalize3df( float *v1, float *v2, float *v3 )
+{
+  float vlen = (float)sqrt1f((v1[0]*v1[0]) + 
+                             (v2[0]*v2[0]) + 
+                             (v3[0]*v3[0]));
+  if(vlen>R_SMALL)
+    {
+      v1[0]/=vlen;
+      v2[0]/=vlen;
+      v3[0]/=vlen;
+    }
+  else
+    {
+      v1[0]=_0;
+      v2[1]=_0;
+      v3[2]=_0;
+    }
+} 
+
+void scale3d ( double *v1,double v0,double *v2)
+{
+  v2[0]=v1[0]*v0;
+  v2[1]=v1[1]*v0;
+  v2[2]=v1[2]*v0;
+}
+
+void add3d ( double *v1, double *v2, double *v3 )
+{
+  v3[0]=v1[0]+v2[0];
+  v3[1]=v1[1]+v2[1];
+  v3[2]=v1[2]+v2[2];
+}
+
+void cross_product3d ( double *v1, double *v2, double *cross )
+{
+  cross[0] = (v1[1]*v2[2]) - (v1[2]*v2[1]);
+  cross[1] = (v1[2]*v2[0]) - (v1[0]*v2[2]);
+  cross[2] = (v1[0]*v2[1]) - (v1[1]*v2[0]);
+}
+
+void remove_component3d ( double *v1, double *unit, double *result)
+{
+  double dot;
+
+  dot = v1[0]*unit[0] + v1[1]*unit[1] + v1[2]*unit[2];
+  result[0]=v1[0]-unit[0]*dot;
+  result[1]=v1[1]-unit[1]*dot;
+  result[2]=v1[2]-unit[2]*dot;  
+}
+
+void reorient44d(double *matrix)
+{
+  double tmp[16];
+  int a;
+
+  /* restore orthogonality and recondition */
+
+  for(a=0;a<3;a++) {
+    normalize3d(matrix);
+    normalize3d(matrix+4);
+    normalize3d(matrix+8);
+    cross_product3d(matrix+4, matrix+8, tmp);
+    cross_product3d(matrix+8, matrix, tmp+4);
+    cross_product3d(matrix, matrix+4, tmp+8);
+    normalize3d(tmp);
+    normalize3d(tmp+4);
+    normalize3d(tmp+8);
+    scale3d(tmp,2.0,tmp);
+    scale3d(tmp+4,2.0,tmp+4);
+    scale3d(tmp+8,2.0,tmp+8);
+    add3d(matrix,tmp,tmp);
+    add3d(matrix+4,tmp+4,tmp+4);
+    add3d(matrix+8,tmp+8,tmp+8);
+    copy3d(tmp,matrix);
+    copy3d(tmp+4,matrix+4);
+    copy3d(tmp+8,matrix+8);
+  }
+   
+  normalize3d(matrix);
+  normalize3d(matrix+4);
+  normalize3d(matrix+8);
+  
+  copy3d(matrix,tmp);
+  remove_component3d(matrix+4,tmp,tmp+4);
+  cross_product3d(tmp, tmp+4, tmp+8);    
+  normalize3d(tmp+4);
+  normalize3d(tmp+8);
+  
+  recondition44d(tmp);
+
+  copy3d(tmp,matrix);
+  copy3d(tmp+4,matrix+4);
+  copy3d(tmp+8,matrix+8);
+  
+}
+
+void recondition33d(double *matrix)
+{
+  normalize3d(matrix);
+  normalize3d(matrix+3);
+  normalize3d(matrix+6);
+  normalize3dp(matrix + 0, matrix + 3, matrix + 6);
+  normalize3dp(matrix + 1, matrix + 4, matrix + 7);
+  normalize3dp(matrix + 2, matrix + 5, matrix + 8);
+  normalize3d(matrix);
+  normalize3d(matrix+3);
+  normalize3d(matrix+6);
+  normalize3dp(matrix + 0, matrix + 3, matrix + 6 );
+  normalize3dp(matrix + 1, matrix + 4, matrix + 7 );
+  normalize3dp(matrix + 2, matrix + 5, matrix + 8);
+  normalize3d(matrix);
+  normalize3d(matrix+3);
+  normalize3d(matrix+6);
+}
+
+void recondition44d(double *matrix)
+{
+  normalize3d(matrix);
+  normalize3d(matrix+4);
+  normalize3d(matrix+8);
+  normalize3dp(matrix + 0, matrix + 4, matrix + 8 );
+  normalize3dp(matrix + 1, matrix + 5, matrix + 9 );
+  normalize3dp(matrix + 2, matrix + 6, matrix + 10);
+  normalize3d(matrix);
+  normalize3d(matrix+4);
+  normalize3d(matrix+8);
+  normalize3dp(matrix + 0, matrix + 4, matrix + 8 );
+  normalize3dp(matrix + 1, matrix + 5, matrix + 9 );
+  normalize3dp(matrix + 2, matrix + 6, matrix + 10);
+  normalize3d(matrix);
+  normalize3d(matrix+4);
+  normalize3d(matrix+8);
+}
+
+void invert_rotation_only44d44d(double *orig, double *inv)
+{
+  /* inverse of the rotation matrix */
+
+  inv[ 0] = orig[ 0];
+  inv[ 1] = orig[ 4];
+  inv[ 2] = orig[ 8];
+  inv[ 4] = orig[ 1];
+  inv[ 5] = orig[ 5];
+  inv[ 6] = orig[ 9];
+  inv[ 8] = orig[ 2];
+  inv[ 9] = orig[ 6];
+  inv[10] = orig[10];
+
+  inv[ 3] = 0.0;
+  inv[ 7] = 0.0;
+  inv[11] = 0.0;
+
+  inv[12] = 0.0;
+  inv[13] = 0.0;
+  inv[14] = 0.0;
+  inv[15] = 1.0;
+
+}
+
 
 void transformTTT44f3f (float *m1, float *m2, float *m3)
 {
@@ -977,6 +1539,7 @@ void get_rotation_about3f3fTTTf(float angle, float *dir, float *origin, float *t
   ttt[ 3] = origin[0];
   ttt[ 7] = origin[1];
   ttt[11] = origin[2];
+  ttt[15] = 1.0F;
 }
 
 void rotation_to_matrix33f(float *axis, float angle, Matrix33f mat)
@@ -1155,6 +1718,7 @@ void normalize3d( double *v1 )
 } 
 
 
+
 double length3d ( double *v1 )
 {
   return(sqrt1d((v1[0]*v1[0]) + 
@@ -1207,6 +1771,22 @@ int slow_within3fsq(float *v1,float *v2,float dist,float dist2)
   return((dx*dx + dy*dy + dz*dz)<=(dist2));
 }
 
+int slow_within3fret(float *v1,float *v2,float cutoff, float cutoff2, float *diff, float *dist)
+{
+  register float dx,dy,dz,dist2;
+  dx = (float)fabs( (diff[0] = v1[0]-v2[0]) );
+  dy = (float)fabs( (diff[1] = v1[1]-v2[1]) );
+  if(dx>cutoff) return 0;
+  dz = (float)fabs( (diff[2] = v1[2]-v2[2]) );
+  dx = dx * dx;
+  if(dy>cutoff) return 0;
+  dy = dy * dy;
+  if(dz>cutoff) return 0;
+  if((dist2 = ((dx + dy) + dz*dz))>cutoff2) 
+    return 0;
+  *dist = (float)sqrt1f(dist2);
+  return 1;
+}
 
 void slow_remove_component3f ( float *v1, float *unit, float *result)
 {
@@ -1267,17 +1847,19 @@ double distance_halfline2point3f(float *base,float *normal,float *point,float *a
 
 void matrix_transform33f3f(Matrix33f m1,float *v1,float *v2)
 {
-#if 0
-	int b;
-	for(b=0;b<3;b++)
-		v2[b] = m1[b][0]*v1[0] + m1[b][1]*v1[1] + m1[b][2]*v1[2];
-#else
-	v2[0] = m1[0][0]*v1[0] + m1[0][1]*v1[1] + m1[0][2]*v1[2];
-	v2[1] = m1[1][0]*v1[0] + m1[1][1]*v1[1] + m1[1][2]*v1[2];
-	v2[2] = m1[2][0]*v1[0] + m1[2][1]*v1[1] + m1[2][2]*v1[2];
-#endif
+  v2[0] = m1[0][0]*v1[0] + m1[0][1]*v1[1] + m1[0][2]*v1[2];
+  v2[1] = m1[1][0]*v1[0] + m1[1][1]*v1[1] + m1[1][2]*v1[2];
+  v2[2] = m1[2][0]*v1[0] + m1[2][1]*v1[1] + m1[2][2]*v1[2];
 }
 
+void matrix_inverse_transform33f3f(Matrix33f m1,float *v1,float *v2)
+{
+	v2[0] = m1[0][0]*v1[0] + m1[1][0]*v1[1] + m1[2][0]*v1[2];
+	v2[1] = m1[0][1]*v1[0] + m1[1][1]*v1[1] + m1[2][1]*v1[2];
+	v2[2] = m1[0][2]*v1[0] + m1[1][2]*v1[1] + m1[2][2]*v1[2];
+}
+
+
 #if 0
 double matdiffsq ( float *v1, oMatrix5f m, float *v2 )
 {
@@ -1403,153 +1985,9 @@ void rotation_to_matrix(Matrix53f rot,float *axis, float angle)
   rotation_matrix3f(angle,axis[0],axis[1],axis[2],&rot[0][0]);
 }
 
-void matrix_interpolate(Matrix53f imat,Matrix53f mat,float *pivot,
-								float *axis,float angle,float tAngle,
-								int linear,int tLinear,float fxn)
-{
-  int a;
-  float pos[3],rotaxis[3],adj[3],adjdir[3],opp[3],oppdir[3];
-  float p0[3],p1[3],center[3];
-  float hyplen,adjlen,opplen;
-  float tAlpha;
-
-			 /*           ______--------______
-           *        /____________          \
-           *     /   \   opp     |adj         \
-           *   |      \          |        trans   | 
-           * (CM)---------------------------------->(CM)
-           *    \       \        |               /
-           *       \      \hyp   |             /
-			  *          \p0   \    |adjdir   p1/
-			  *             \    \  |        /
-			  *                \  \ |     /     
-			  *                   \\|  /
-           *            <--------O pivot
-			  *                      F-raxis
-			  */
 
 
-  for(a=0;a<3;a++)
-	 {
-		imat[a][0]=_0;
-		imat[a][1]=_0;
-		imat[a][2]=_0;
-		imat[a][a]=_1;
-	 }
-
-  if(!linear) {
-	 rotation_to_matrix(imat,axis,fxn*angle);
-  }
-  else {
-	 tLinear = true;
-  }
-  if(!tLinear)
-	 {
-		subtract3f(mat[3],pivot,p0);
-		subtract3f(mat[4],pivot,p1);
-
-		/*		printf("length match? %8.3f %8.3f\n",length3f(p0),length3f(p1));*/
-		hyplen = (float)length3f(p0);
-		
-		average3f(mat[3],mat[4],center);
-		
-		cross_product3f(p1,p0,rotaxis);
-		normalize3f(rotaxis);
-		subtract3f(center,pivot,adjdir);
-		normalize3f(adjdir);
-		cross_product3f(rotaxis,adjdir,oppdir);
-		normalize3f(oppdir);
-		
-		tAlpha = (float)(fabs(0.5-fxn)*tAngle);
-		
-		opplen = (float)fabs(hyplen * sin(tAlpha));
-		adjlen = (float)fabs(hyplen * cos(tAlpha));
-		
-		scale3f(oppdir,opplen,opp);
-		scale3f(adjdir,adjlen,adj);
-		
-		add3f(pivot,adj,pos);
-		
-		if(fxn<=0.5) {
-		  add3f(pos,opp,pos);
-		} else {
-		  subtract3f(pos,opp,pos);
-      }
-	 }
-  else
-	 {
-		for(a=0;a<3;a++)
-		  pos[a] = (mat[3][a]*(_1-fxn))+(fxn*mat[4][a]);
-	 }
-
-  for(a=0;a<3;a++)
-	 {
-		imat[3][a] = mat[3][a];
-		imat[4][a] = pos[a];
-	 }
-
-}
-
-void matrix_to_rotation(Matrix53f rot,float *axis, float *angle)
-{
-  float perp[3],tmp[3],rperp[3],dirck[3];
-  Matrix33d rot3d;
-  Matrix53f rotck;
-  int a,b;
-
-#ifdef MATCHK
-  printf("starting matrix\n");
-  for(a=0;a<5;a++)
-	 printf("%8.3f %8.3f %8.3f\n",rot[a][0],rot[a][1],rot[a][2]);
-#endif
-
-  for(a=0;a<3;a++)
-	 for(b=0;b<3;b++)
-		rot3d[a][b] = rot[a][b];
-
-  find_axis(rot3d,axis);
-
-  /* find a perpendicular vector */
-
-  perp[0]=axis[1]*axis[0]-axis[2]*axis[2];
-  perp[1]=axis[2]*axis[1]-axis[0]*axis[0];
-  perp[2]=axis[0]*axis[2]-axis[1]*axis[1];
-
-  if(length3f(perp)<R_SMALL)
-	 {
-		tmp[0]=axis[0];
-		tmp[1]=-2*axis[1];
-		tmp[2]=axis[2];
-		cross_product3f(axis,tmp,perp);
-	 }
-
-  normalize3f(perp);
-
-  transform33d3f(rot3d,perp,rperp);
-
-  *angle = get_angle3f(perp,rperp);
-
-  cross_product3f(perp,rperp,dirck);
-  if(((dirck[0]*axis[0])+(dirck[1]*axis[1])+(dirck[2]*axis[2]))<_0)
-	 *angle = -*angle;
-	 
-
-  /*  printf("angle %8.3f \n",*angle);*/
-
-  rotation_to_matrix(rotck,axis,*angle);
-
-
-#ifdef MATCHK  
-  printf("reconstructed matrix: \n");
-  for(a=0;a<3;a++)
-	 printf("%8.3f %8.3f %8.3f\n",rotck[a][0],rotck[a][1],rotck[a][2]);
-  printf("\n");
-#endif
-
-}
-
-
-void find_axis( Matrix33d a, float *axis)
+static void find_axis( Matrix33d a, float *axis)
 {
   doublereal at[3][3],v[3][3],vt[3][3],fv1[3][3];
   integer iv1[3];
@@ -1563,6 +2001,8 @@ void find_axis( Matrix33d a, float *axis)
   n = 3;
   matz = 1;
 
+  recondition33d(&a[0][0]); /* IMPORTANT! */
+
   for(x=0;x<3;x++)
 	 {
 		for(y=0;y<3;y++)
@@ -1586,24 +2026,38 @@ void find_axis( Matrix33d a, float *axis)
   axis[2]=0.0F;
 
   {
-    double max_real = 0.0F;
+    doublereal max_real = 0.0F, test_real;
+    doublereal min_imag = 1.0F, test_imag;
+    float test_inp[3],test_out[3];
 
     for(x=0;x<3;x++) { /* looking for an eigvalue of (1,0) */
       /*      printf("wr %8.3f wi %8.3f\n",wr[x],wi[x]);
       printf("%8.3f %8.3f %8.3f\n",
       v[0][x],v[1][x],v[2][x]);*/
-
-      if(fabs(wr[x])>=max_real) {
+      test_real = fabs(wr[x]);
+      test_imag = fabs(wi[x]);
+      
+      if((test_real>=max_real)&&(test_imag<=min_imag)) {
         for(y=0;y<3;y++)
-          axis[y] = (float)v[y][x];
-        max_real = wr[x];
+          test_inp[y] = (float)v[y][x];
+        transform33d3f(a,test_inp,test_out); /* confirm that axis is invariant to rotation */
+        test_out[0] -= test_inp[0];
+        test_out[1] -= test_inp[1];
+        test_out[2] -= test_inp[2];
+        if((test_out[0]*test_out[0]+
+            test_out[1]*test_out[1]+
+            test_out[2]*test_out[2])<0.1) {
+          for(y=0;y<3;y++)
+            axis[y] = test_inp[y];
+          max_real = test_real;
+          min_imag = test_imag;
+        }
       } else {
         /*for(y=0;y<3;y++)
           v[y][x]=_0;*/
       }
     }
   }
-
   /*
     printf("eigenvectors\n%8.3f %8.3f %8.3f\n",v[0][0],v[0][1],v[0][2]);
     printf("%8.3f %8.3f %8.3f\n",v[1][0],v[1][1],v[1][2]);
@@ -1612,14 +2066,75 @@ void find_axis( Matrix33d a, float *axis)
     
     printf("eigenvalues\n%8.3f %8.3f %8.3f\n",wr[0],wr[1],wr[2]);
     printf("%8.3f %8.3f %8.3f\n",wi[0],wi[1],wi[2]);
-    
-    matrix_multiply33d33d(a,v,p);
+  */
+
+    /*    matrix_multiply33d33d(a,v,p);
     
     printf("invariance\n");
     printf("%8.3f %8.3f %8.3f\n",p[0][0],p[0][1],p[0][2]);
     printf("%8.3f %8.3f %8.3f\n",p[1][0],p[1][1],p[1][2]);
     printf("%8.3f %8.3f %8.3f\n",p[2][0],p[2][1],p[2][2]);
-  */
+    */
+
+}
+
+void matrix_to_rotation(Matrix53f rot,float *axis, float *angle)
+{
+  float perp[3],tmp[3],rperp[3],dirck[3];
+  Matrix33d rot3d;
+  Matrix53f rotck;
+  int a,b;
+
+
+#ifdef MATCHK
+  printf("starting matrix\n");
+  for(a=0;a<3;a++)
+	 printf("%8.3f %8.3f %8.3f\n",rot[a][0],rot[a][1],rot[a][2]);
+#endif
+
+  for(a=0;a<3;a++)
+	 for(b=0;b<3;b++)
+		rot3d[a][b] = (double)rot[a][b];
+
+  find_axis(rot3d,axis);
+
+
+  /* find a perpendicular vector */
+
+  perp[0]=axis[1]*axis[0]-axis[2]*axis[2];
+  perp[1]=axis[2]*axis[1]-axis[0]*axis[0];
+  perp[2]=axis[0]*axis[2]-axis[1]*axis[1];
+
+  if(length3f(perp)<R_SMALL) {
+		tmp[0]=axis[0];
+		tmp[1]=-2*axis[1];
+		tmp[2]=axis[2];
+		cross_product3f(axis,tmp,perp);
+  }
+
+  normalize3f(perp);
+
+  transform33d3f(rot3d,perp,rperp);
+
+  *angle = get_angle3f(perp,rperp);
+
+  cross_product3f(perp,rperp,dirck);
+  if(((dirck[0]*axis[0])+(dirck[1]*axis[1])+(dirck[2]*axis[2]))<_0)
+	 *angle = -*angle;
+	 
+  /*  printf("angle %8.3f \n",*angle);*/
+
+  rotation_to_matrix(rotck,axis,*angle);
+
+
+#ifdef MATCHK  
+  printf("reconstructed matrix: \n");
+  for(a=0;a<3;a++)
+	 printf("%8.3f %8.3f %8.3f\n",rotck[a][0],rotck[a][1],rotck[a][2]);
+  printf("\n");
+#endif
+
 }
 
 
+
