@@ -19,8 +19,14 @@ Z* -------------------------------------------------------------------
 #include"os_predef.h"
 /* NOTE THIS VERSION USES RADIANS BY DEFAULT! */
 
-/* NOTE: All matrices are assumed to be row-major (NOTE that means
- * C-like, not OpenGL-like) in this module */
+/* NOTE: Matrices are assumed to be row-major (C-like not
+ * OpenGL-like) unless explictly labeled as per the following
+ * conventions:
+ * 
+ * row-major:    33f, 33d, 44f, 44d, R33f, R33d, R44f, R44d
+ * column-major: C33f, C33d, C44f, C44d
+ */
+
 
 #define cPI            3.14159265358979323846  /* pi */
 
@@ -63,6 +69,10 @@ void get_system2f3f(float *x,float *y,float *z); /* make system in direction of
 double dot_product3d ( double *v1, double *v2 );
 float slow_project3f ( float *v1, float *v2, float *proj );
 void slow_remove_component3f ( float *v1, float *unit, float *result);
+void remove_component3d ( double *v1, double *unit, double *result);
+void cross_product3d ( double *v1, double *v2, double *cross );
+void scale3d ( double *v1, double v0, double *v2);
+void add3d ( double *v1, double *v0, double *v2);
 
 double distance_line2point3f(float *base,float *normal,float *point,float *alongNormalSq);
 double distance_halfline2point3f(float *base,float *normal,float *point,float *alongNormalSq);
@@ -71,9 +81,13 @@ double slow_diffsq3f ( float *v1, float *v2 );
 double slow_diff3f ( float *v1, float *v2 );
 int slow_within3f(float *v1,float *v2,float dist);
 int slow_within3fsq(float *v1,float *v2,float dist,float dist2);
+int slow_within3fret(float *v1,float *v2,float cutoff, float cutoff2, float *diff, float *dist);
 
 int equal3f(float *v1,float *v2);
 
+
+int pymol_roundf(float f);
+
 float get_angle3f( float *v1, float *v2 );
 float get_dihedral3f( float *v0, float *v1, float *v2, float *v3 );
 double length3d ( double *v1 );
@@ -96,11 +110,16 @@ void copy44d ( double *src, double *dst );
 void identity33f ( float *m1 );
 void identity33d( double *m);
 void identity44f ( float *m1 );
+void identity44d ( double *m1 );
 
 void copy44f44f ( float *src, float *dst );
+void copy44d44f ( double *src, float *dst );
+void copy44f44d ( float *src, double *dst );
 
 void copy44d33f ( double *src, float *dst );
+void copy44f33f ( float *src, float *dst );
 void copy33f44d ( float *src, double *dst );
+void copy33f44f ( float *src, float *dst );
 void copy3d3f ( double *v1,float *v2);
 void copy3f3d ( float *v1,double *v2);
 
@@ -108,6 +127,7 @@ void copy3f3d ( float *v1,double *v2);
    the last two matrices can be the same matrix! */
 
 void transpose33f33f ( float  *m1, float  *m2);
+void transpose44f44f ( float  *m1, float  *m2);
 void transform33f3f ( float  *m1, float  *m2,  float  *m3 );
 void transform33Tf3f ( float  *m1, float  *m2,  float  *m3 ); /* uses transpose */
 
@@ -115,8 +135,12 @@ void transform44f3f ( float  *m1, float  *m2,  float  *m3 );
 void transform44f4f ( float  *m1, float  *m2,  float  *m3 );
 
 void transform44d3f ( double  *m1, float  *m2,  float  *m3 );
+void transform44d3d (double *m1, double *m2, double *m3);
+void inverse_transformC44f3f (float *m1, float *m2, float *m3);
+void inverse_transform44f3f (float *m1, float *m2, float *m3);
 void inverse_transform44d3f (double *m1, float *m2, float *m3);
 void inverse_transform44d3d (double *m1, double *m2, double *m3);
+void transform44f3fas33f3f (float *m1, float *m2, float *m3);
 void transform44d3fas33d3f (double *m1, float *m2, float *m3);
 
 void multiply33f33f ( float  *m1, float  *m2,  float  *m3 );
@@ -125,6 +149,8 @@ void multiply33d33d ( double *m1, double *m2, double  *m3 );
 /* as matrix types */
 
 void matrix_transform33f3f ( Matrix33f m1,float *v1,float *v2);
+void matrix_inverse_transform33f3f ( Matrix33f m1,float *v1,float *v2);
+
 void rotation_to_matrix33f (float *axis, float angle, Matrix33f mat);
 void matrix_multiply33f33f ( Matrix33f m1,Matrix33f m2,Matrix33f m3);
 void matrix_multiply33d33d ( Matrix33d m1,Matrix33d m2,Matrix33d m3);
@@ -143,11 +169,23 @@ void multiply44d44d44d( double *left, double *right, double *product);
 void left_multiply44d44d( double *left, double *right);
 void right_multiply44d44d( double *left, double *right);
 
+void multiply44f44f44f( float *left, float *right, float *product);
+void left_multiply44f44f( float *left, float *right);
+void right_multiply44f44f( float *left, float *right);
+
+void reorient44d(double *matrix);
+
+void recondition33d(double *matrix);
+void recondition44d(double *matrix);
 /* invert a 4x4 homogenous that contains just rotation & tranlation
   (e.g. no scaling & fourth row is 0,0,0,1) */
 void invert_special44d44d( double *original, double *inv);
+void invert_special44f44f( float *original, float *inv);
+
+void invert_rotation_only44d44d( double *original, double *inv);
 
 void convertTTTfR44d( float *ttt, double *homo);
+void convertTTTfR44f( float *ttt, float *homo);
 void convertR44dTTTf( double *homo, float *ttt);
 void convert44d44f(double *dbl, float *flt);
 void convert44f44d(float *flt, double *dbl);
@@ -168,7 +206,6 @@ typedef float *oMatrix3f[3];
 typedef float *oMatrix3d[3];
 
 double matdiffsq ( float *v1, oMatrix5f m, float *v2 );
-void find_axis( Matrix33d a, float *axis);
 
 /*void matcopy ( oMatrix5f to, oMatrix5f from );
   void mattran ( oMatrix5f nm, oMatrix5f om, int axis, float dist );
@@ -178,9 +215,6 @@ void find_axis( Matrix33d a, float *axis);
 void matrix_to_rotation(Matrix53f rot,float *axis, float *angle);
 void rotation_to_matrix(Matrix53f rot,float *axis, float angle);
 
-void matrix_interpolate(Matrix53f imat,Matrix53f mat,float *pivot,
-								float *axis,float angle,float tAngle,
-								int linear,int tLinear,float fxn);
 
 void transform3d3f ( oMatrix3d m1,float *v1,float *v2);
 void transform33d3f ( Matrix33d m1,float *v1,float *v2);
@@ -194,6 +228,7 @@ void transform5f3f ( oMatrix5f m, float *v1, float *v2 );
 
 float dot_product3f ( float *v1, float *v2 );
 void  invert3f ( float *v );
+void  invert3f3f (float *v1, float *v2);
 void  scale3f ( float *v1, float v0, float *v2);
 void  copy3f( float *src , float *dst);
 void  copy3d( double *src , double *dst);
@@ -205,6 +240,7 @@ double length3f ( float *v1 );
 void  cross_product3f ( float *v1, float *v2, float *cross );
 void  average3f ( float *v1, float *v2, float *avg );
 void  zero3f ( float *v1 )
+void  ones3f ( float *v1);
 void  set3f ( float *v1,float x,float y,float z );
 void  swap1f (float *f, float *g);
 
@@ -212,8 +248,10 @@ void  swap1f (float *f, float *g);
 
 #define set3f(v1,x,y,z) { (v1)[0]=(x);(v1)[1]=(y);(v1)[2]=(z); }
 #define zero3f(v1) { (v1)[0]=0.0;(v1)[1]=0.0;(v1)[2]=0.0; }
+#define ones3f(v1) { (v1)[0]=1.0F;(v1)[1]=1.0F;(v1)[2]=1.0F; }
 #define dot_product3f(v1,v2) ((v1)[0]*(v2)[0] + (v1)[1]*(v2)[1] + (v1)[2]*(v2)[2])
 #define invert3f(v) {(v)[0]=-(v)[0]; (v)[1]=-(v)[1]; (v)[2]=-(v)[2];}
+#define invert3f3f(v1,v2) {(v2)[0]=-(v1)[0]; (v2)[1]=-(v1)[1]; (v2)[2]=-(v1)[2];}
 #define scale3f(v1,v0,v2) {(v2)[0]=(v1)[0]*(v0); (v2)[1]=(v1)[1]*(v0); (v2)[2]=(v1)[2]*(v0);}
 #define copy3f(v1,v2) {(v2)[0]=(v1)[0]; (v2)[1]=(v1)[1]; (v2)[2]=(v1)[2];}
 #define copy3d(v1,v2) {(v2)[0]=(v1)[0]; (v2)[1]=(v1)[1]; (v2)[2]=(v1)[2];}
@@ -254,12 +292,13 @@ static const double R_SMALLd_inline = 0.000000001;
 #define diffsq3f inline_diffsq3f
 #define within3f inline_within3f
 #define within3fsq inline_within3fsq
+#define within3fret inline_within3fret
 #define remove_component3f inline_remove_component3f
 #define project3f inline_project3f
 
 __inline__ static double inline_sqrt1f(float f) { /* no good as a macro because f is used twice */
   if(f>_0f_inline)
-	 return(sqrt(f));
+    return(sqrt(f));
   else
 	 return(_0d_inline);
 }
@@ -343,6 +382,22 @@ __inline__ static int inline_within3fsq(float *v1,float *v2,float dist,float dis
   return((dx + dz)<=(dist2));
 }
 
+__inline__ static int inline_within3fret(float *v1,float *v2,float cutoff, float cutoff2, float *diff, float *dist)
+{
+  register float dx,dy,dz,dist2;
+  dx = (float)fabs( (diff[0] = v1[0]-v2[0]) );
+  dy = (float)fabs( (diff[1] = v1[1]-v2[1]) );
+  if(dx>cutoff) return 0;
+  dz = (float)fabs( (diff[2] = v1[2]-v2[2]) );
+  dx = dx * dx;
+  if(dy>cutoff) return 0;
+  dy = dy * dy;
+  if(dz>cutoff) return 0;
+  if((dist2 = ((dx + dy) + dz*dz))>cutoff2) 
+    return 0;
+  *dist = (float)sqrt1f(dist2);
+  return 1;
+}
 
 __inline__ static void inline_remove_component3f ( float *v1, float *unit, float *result)
 {
@@ -375,6 +430,7 @@ __inline__ static float inline_project3f ( float *v1, float *v2, float *proj )
 #define diffsq3f slow_diffsq3f
 #define within3f slow_within3f
 #define within3fsq slow_within3fsq
+#define within3fret slow_within3fret
 #define project3f slow_project3f
 #define remove_component3f slow_remove_component3f
 
