@@ -18,7 +18,7 @@ Z* -------------------------------------------------------------------
 
 #include "PyMOLGlobals.h"
 
-#define WordLength 64
+#define WordLength 256
 
 typedef char WordType[WordLength];
 
@@ -42,6 +42,8 @@ typedef struct {
   int lists;
   int ignore_case;
   int allow_hyphen;
+  int allow_plus;
+  int space_lists;
   char wildcard;
 } CWordMatchOptions;
 
@@ -49,17 +51,21 @@ typedef struct _CWordMatcher CWordMatcher;
 
 void WordMatchOptionsConfigInteger(CWordMatchOptions *I);
 void WordMatchOptionsConfigAlpha(CWordMatchOptions *I, char wildcard, int ignore_case);
+void WordMatchOptionsConfigAlphaList(CWordMatchOptions *I, char wildcard, int ignore_case);
 void WordMatchOptionsConfigMixed(CWordMatchOptions *I, char wildcard, int ignore_case);
+void WordMatchOptionsConfigNameList(CWordMatchOptions *I, char wildcard, int ignore_case);
 
 CWordMatcher *WordMatcherNew(PyMOLGlobals *G, char *st, CWordMatchOptions *option, int force);
 int WordMatcherMatchAlpha(CWordMatcher *I, char *text);
 int WordMatcherMatchMixed(CWordMatcher *I, char *text,int value);
 int WordMatcherMatchInteger(CWordMatcher *I,int value);
 void WordMatcherFree(CWordMatcher *I);
+int WordMatchNoWild(PyMOLGlobals *G,char *p,char *q,int ignCase) ;
 
 CWordList *WordListNew(PyMOLGlobals *G,char *st);
 void WordListFreeP(CWordList *I);
 void WordListDump(CWordList *I,char *prefix);
+int WordListIterate(PyMOLGlobals *G,CWordList *I,char **ptr, int *hidden);
 int WordListMatch(PyMOLGlobals *G,CWordList *I,char *name, int ignore_case);
 
 int WordInit(PyMOLGlobals *G);
@@ -75,9 +81,51 @@ int WordMatchCommaExact(PyMOLGlobals *G,char *p,char *q,int ignCase);
 
 /* (<0) exact match, (>0) inexact match, =0 no match */
 
-int WordCompare(PyMOLGlobals *G,char *p,char *q,int ignCase);
 int WordIndex(PyMOLGlobals *G,WordType *list,char *word,int minMatch,int ignCase);
 int WordKey(PyMOLGlobals *G,WordKeyValue *list,char *word,int minMatch,int ignCase,int *exact);
 
+#ifdef _PYMOL_INLINE
+__inline__ static int WordCompare(PyMOLGlobals *G,char *p,char *q,int ignCase) 
+/* all things equal, shorter is smaller */
+{
+  int result=0;
+  register char cp,cq,tlp,tlq;
+  if(ignCase) {
+    while((cp=*p)&&(cq=*q))	{
+      p++;
+      q++;
+      if(cp!=cq) {
+        (tlp=tolower(cp));
+	(tlq=tolower(cq));
+	if(tlp<tlq) 
+          return -1;
+        else if(tlp>tlq) {
+          return 1;
+        }
+      }
+    }
+  } else {
+    while((cp=*p)&&(cq=*q))	{
+      p++;
+      q++;
+      if(cp!=cq) {
+        if(cp<cq) {
+          return -1;
+        } else if(cp>cq) {
+          return 1;
+        }
+      }
+    }
+  }
+  if((!result)&&(!*p)&&(*q))
+    return -1;
+  else if((!result)&&(*p)&&(!*q))
+    return 1;
+  return 0;
+}
+#else
+int WordCompare(PyMOLGlobals *G,char *p,char *q,int ignCase);
+
+#endif
 
 #endif
