@@ -95,14 +95,17 @@ void PyMOLDrawPixels(GLsizei width,
 
 }
 
-void PyMOLCheckOpenGLErr(char *pos)
+int PyMOLCheckOpenGLErr(char *pos)
 {
+  int flag = 0;
   GLenum  glerr = glGetError( );
   while( glerr != GL_NO_ERROR ) 
     {
-      printf("OpenGL-Error: Where? %s: %s\n",pos,gluErrorString(glerr));
+      printf("OpenGL-Error: Where? %s: %s\n",pos,(char*)gluErrorString(glerr));
       glerr = glGetError( );
+	  flag=1;
     }
+   return flag;
 }
 
 #ifndef _PYMOL_NO_GLUT
@@ -182,6 +185,7 @@ void     p_glutMotionFunc(void (*func)(int x, int y)) {}
 void     p_glutPassiveMotionFunc(void (*func)(int x, int y)) {}
 void     p_glutSpecialFunc(void (*func)(int key, int x, int y)) {}
 void     p_glutIdleFunc(void (*func)(void)) { idleFunc = func; }
+void     p_glutPositionWindow(int x,int y) {};
 
 #endif
 
@@ -275,7 +279,7 @@ void     p_glutBitmapCharacter(void *font, int character)
   plutBitmapCharacter(character);
 #endif
 }
-void     p_glutSwapBuffers(void){PBlock();PyRun_SimpleString("pymol._swap_buffers()");PUnblock();}
+void     p_glutSwapBuffers(void){PBlockLegacy();PyRun_SimpleString("if hasattr(pymol,'_swap_buffers'): pymol._swap_buffers()");PUnblockLegacy();}
 
 
 void     p_glutPopWindow(void){}
@@ -305,6 +309,7 @@ void     p_glutMotionFunc(void (*func)(int x, int y)) { motionFunc=func;}
 void     p_glutPassiveMotionFunc(void (*func)(int x, int y)) { passiveMotionFunc=func;}
 void     p_glutSpecialFunc(void (*func)(int key, int x, int y)) { specialFunc=func;}
 void     p_glutIdleFunc(void (*func)(void)) { idleFunc = func; }
+void     p_glutPositionWindow(int x,int y) {};
 
 #endif
 
