@@ -16,10 +16,24 @@ Z* -------------------------------------------------------------------
 #ifndef _H_os_predef
 #define _H_os_predef
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+/* Macros used by Fortify source in GCC 4.1.x are incompatible with
+   PyMOL's Feedback system... */
+
+#ifdef _FORTIFY_SOURCE
+#undef _FORTIFY_SOURCE
+#endif
+
+/* BEGIN PROPRIETARY CODE SEGMENT (see disclaimer in "os_proprietary.h") */ 
+
 #ifdef WIN32
 #define __inline__ __inline
 #endif
 
+#ifndef _PYMOL_MINGW
 #ifdef _WIN32
 #ifndef _AFXDLL
 	#ifndef _UNICODE
@@ -100,6 +114,51 @@ Z* -------------------------------------------------------------------
 #endif
 
 #endif
+#endif
+
+/* commercial product */
+
+#ifdef PYMOL_COMM
+#ifndef _PYMOL_IP_SPLASH
+#define _PYMOL_IP_SPLASH
+#endif
+#ifndef _PYMOL_IP_EXTRAS
+#define _PYMOL_IP_EXTRAS
+#endif
+#endif
+
+/* collaboration product (placarded) */
+
+#ifdef PYMOL_COLL
+#ifndef _PYMOL_IP_SPLASH
+#define _PYMOL_IP_SPLASH
+#endif
+#ifndef _PYMOL_IP_EXTRAS
+#define _PYMOL_IP_EXTRAS
+#endif
+#endif
+
+/* educational product (placarded) */
+
+#ifdef PYMOL_EDU
+#ifndef _PYMOL_IP_SPLASH
+#define _PYMOL_IP_SPLASH
+#endif
+#ifndef _PYMOL_IP_EXTRAS
+#define _PYMOL_IP_EXTRAS
+#endif
+#endif
+
+/* evaluation product (placarded) */
+
+#ifdef PYMOL_EVAL
+#ifndef _PYMOL_IP_SPLASH
+#define _PYMOL_IP_SPLASH
+#endif
+#endif
+
+
+/* END PROPRIETARY CODE SEGMENT */
 
 #endif
 
