@@ -34,14 +34,14 @@ typedef struct {
   int vert;
   float v1[3],v2[3],v3[3];
   float n0[3],n1[3],n2[3],n3[3];
-  float c1[3],c2[3],c3[3];
+  float c1[3],c2[3],c3[3],ic[3],tr[3];
   float r1,l1; 
   float trans;
   int char_id;
   char type,cap1,cap2,cull;
-  char wobble;
+  char wobble,ramped;
   /* float wobble_param[3] eliminated to save space */
-} CPrimitive;
+} CPrimitive; /* currently 172 bytes -> appoximately 6.5 million primitives per gigabyte */
 
 typedef struct {
   PyMOLGlobals *G;
@@ -52,6 +52,7 @@ typedef struct {
   int NVertex;
   int NNormal;
   float LightNormal[3]; /* for lights - this is the direction of the light rays */
+  float SpecNormal[3]; /* for computing specular reflections */
   float Color[3]; /* for lights */
   Matrix33f Matrix;
 } CBasis;
@@ -74,14 +75,16 @@ typedef struct {
 typedef struct {
   CBasis *Basis;
   RayInfo *rr;
-  int except;
+  int except1,except2; /* primitives to avoid */
   int *vert2prim;
   int shadow;
   float front;
   float back;
   float excl_trans;
   int trans_shadows;
+  int nearest_shadow;
   int check_interior;
+  int label_shadow_mode;
   CPrimitive *prim;
   MapCache cache;
   float fudge0,fudge1;
@@ -96,7 +99,7 @@ void BasisInit(PyMOLGlobals *G,CBasis *I,int group_id);
 void BasisFinish(CBasis *I,int group_id);
 void BasisMakeMap(CBasis *I,int *vert2prim,CPrimitive *prim,float *volume,
                   int group_id,int block_base,
-                  int perspective,float front);
+                  int perspective,float front,float size_hint);
 
 void BasisSetupMatrix(CBasis *I);
 void BasisGetTriangleNormal(CBasis *I,RayInfo *r,int i,float *fc,int perspective);
