@@ -49,6 +49,10 @@ struct _CGO {
   PyMOLGlobals *G;
   float *op;
   int c;
+  int z_flag;
+  float z_min, z_max;
+  float z_vector[3];
+  int *i_start,i_size;
 };
 
 /* instructions and data segment sizes */
@@ -87,25 +91,30 @@ struct _CGO {
 #define CGO_CUSTOM_CYLINDER_SZ   15
 #define CGO_DOTWIDTH             0x10
 #define CGO_DOTWIDTH_SZ          1
-
+#define CGO_ALPHA_TRIANGLE       0x11
+#define CGO_ALPHA_TRIANGLE_SZ    35
 #define CGO_FONT                 0x13
 #define CGO_FONT_SZ              3   /*  size, face, style */
 #define CGO_FONT_SCALE           0x14 
 #define CGO_FONT_SCALE_SZ        2 
 #define CGO_FONT_VERTEX          0x15 
-#define CGO_FONT_VERTEX_SZ       3   /*  principle axes (zeros -> use camera x y or z */
+#define CGO_FONT_VERTEX_SZ       3   /*  principle axes (zeros -> use camera x y z */
 #define CGO_FONT_AXES            0x16 
-#define CGO_FONT_AXES_SZ         9   /*  principle axes (zeros -> use camera x y or z */
+#define CGO_FONT_AXES_SZ         9   /*  principle axes (zeros -> use camera x y z */
 #define CGO_CHAR                 0x17
 #define CGO_CHAR_SZ              1
 #define CGO_INDENT               0x18
 #define CGO_INDENT_SZ            2
 #define CGO_ALPHA                0x19 
 #define CGO_ALPHA_SZ             1
+#define CGO_RESET_NORMAL         0x1E
+#define CGO_RESET_NORMAL_SZ      1
 #define CGO_PICK_COLOR           0x1F
 #define CGO_PICK_COLOR_SZ        2
 #define CGO_MASK                 0x1F
 
+#define CGO_LIGHTING             0x0B50
+
 int CGORendererInit(PyMOLGlobals *G);
 void CGORendererFree(PyMOLGlobals *G);
 CGO *CGONew(PyMOLGlobals *G);
@@ -129,6 +138,8 @@ int CGOFromFloatArray(CGO *I,float *src,int len);
 void CGOBegin(CGO *I,int mode);
 void CGOEnd(CGO *I);
 
+void CGOSphere(CGO *I,float *v1, float r);
+void CGOSausage(CGO *I,float *v1,float *v2,float r,float *c1,float *c2);
 void CGOVertex(CGO *I,float v1,float v2,float v3);
 void CGOVertexv(CGO *I,float *v);
 void CGOAlpha(CGO *I,float alpha);
@@ -136,6 +147,7 @@ void CGOColor(CGO *I,float v1,float v2,float v3);
 void CGOColorv(CGO *I,float *v);
 void CGONormal(CGO *I,float v1,float v2,float v3);
 void CGONormalv(CGO *I,float *v);
+void CGOResetNormal(CGO *I,int mode);
 void CGOLinewidth(CGO *I,float v);
 void CGODotwidth(CGO *I,float v);
 void CGOChar(CGO *I,char c);
@@ -146,6 +158,7 @@ void CGOChar(CGO *I,char c);
 void CGOIndent(CGO *I,char c,float dir);
 void CGOWrite(CGO *I,char *str);
 void CGOWriteLeft(CGO *I,char *str);
+void CGOWriteIndent(CGO *I,char *str,float indent);
 
 /*void CGOFontScale(CGO *I,float v);
   void CGOFont(CGO *I,float size,int face,int style);*/
@@ -158,11 +171,19 @@ void CGOStop(CGO *I);
 void CGOCylinderv(CGO *I,float *p1,float *p2,float r,float *c1,float *c2);
 void CGOCustomCylinderv(CGO *I,float *p1,float *p2,float r,float *c1,float *c2,
                         float cap1,float cap2);
-
+void CGOAlphaTriangle(CGO *I,
+                      float *v1, float *v2, float *v3,
+                      float *n1, float *n2, float *n3,
+                      float *c1, float *c2, float *c3,
+                      float a1, float a2, float a3,
+                      int reverse);
+void CGOSetZVector(CGO *I, float z0, float z1, float z2);
 struct GadgetSet;
 CGO *CGOProcessShape(CGO *I,struct GadgetSet *gs,CGO *result);
-void CGORenderGLPickable(CGO *I,Pickable **pick,void *ptr,CSetting *set1,CSetting *set2);
-void CGORenderGL(CGO *I,float *color,CSetting *set1,CSetting *set2);
+void CGORenderGLPicking(CGO *I,Picking **pick,
+                         PickContext *context,CSetting *set1,CSetting *set2);
+void CGORenderGL(CGO *I,float *color,CSetting *set1,CSetting *set2,RenderInfo *info);
+void CGORenderGLAlpha(CGO *I,RenderInfo *info);
 void CGORenderRay(CGO *I,CRay *ray,float *color,CSetting *set1,CSetting *set2);
 void CGOReset(CGO *I);
 
