@@ -23,14 +23,17 @@ typedef unsigned char CharColor[4];
 
 
 typedef struct {
-  int text_id;
-  short int ch;
-  short int height;
-  CharColor color;
-} CharInfo; /* currently 5 short ints */
+  int text_id; /* 32 bits (+ 2)*/
+  unsigned int ch; /* 32 bits (+ 2)*/
+  short int size; /* 16 bits (+ 1) */
+  CharColor color; /* 32 bits (+ 2)*/
+  CharColor outline_color; /* 32 bits (+ 2)*/
+  short int flat; /* 16 bits (+ 1) */
+} CharInfo; /* 10 short ints 
+             * if you change this, update hash functions in Character.c */
 
 typedef struct {
-  unsigned short int data[6];
+  unsigned short int data[10];
 } CharData;
 
 typedef union {
@@ -48,6 +51,8 @@ typedef struct {
   CPixmap Pixmap;
   int Width;
   int Height;
+  float Advance;
+  float XOrig,YOrig;
   int Next,Prev,HashNext,HashPrev;
   CharFngrprnt Fngrprnt;
   float extent[2]; /* texture extent */
@@ -71,17 +76,27 @@ void CharacterFree(PyMOLGlobals *G);
 int CharacterGetNew(PyMOLGlobals *G);
 int CharacterGetWidth(PyMOLGlobals *G,int id);
 int CharacterGetHeight(PyMOLGlobals *G,int id);
+int CharacterGetGeometry(PyMOLGlobals *G,int id,
+                         int *width, int *height, 
+                         float *xorig, float *yorig, float *advance);
+float CharacterGetAdvance(PyMOLGlobals *G,int sampling, int id);
 
 int CharacterNewFromBitmap(PyMOLGlobals *G,int width, int height,
                            unsigned char *bitmap,
-                           CharFngrprnt *fprnt);
+                           float x_orig, float y_orig, float advance,
+                           CharFngrprnt *fprnt,int sampling);
+
+int CharacterNewFromBytemap(PyMOLGlobals *G, int width, int height,
+                            int pitch, unsigned char *bytemap, 
+                            float x_orig, float y_orig, float advance,
+                            CharFngrprnt *fprnt);
 
 int CharacterFind(PyMOLGlobals *G,CharFngrprnt *fprnt);
 
 float CharacterInterpolate(PyMOLGlobals *G,int id,float *v);
 void CharacterSetRetention(PyMOLGlobals *G,int retail_all);
 unsigned char *CharacterGetPixmapBuffer(PyMOLGlobals *G,int id);
-void CharacterRenderOpenGL(PyMOLGlobals *G,int id, float x_orig, float y_orig, float advance);
+void CharacterRenderOpenGL(PyMOLGlobals *G,RenderInfo *info,int id);
 
 #endif
 
