@@ -41,7 +41,7 @@ static int AutoColor[] = {
 9    , /* salmon */
 29   , /* hydrogen */
 11   , /* slate */
-30   , /* brightorange */
+13   , /* orange */
 10   , /* lime */
 5262 , /* deepteal */
 12   , /* hotpink */
@@ -78,8 +78,60 @@ static int AutoColor[] = {
 
 static int nAutoColor = 40;
 
-#define cColorExtCutoff (-10)
+void ColorGetBkrdContColor(PyMOLGlobals *G,float *rgb, int invert_flag) 
+{
+  float *bkrd=SettingGetfv(G,cSetting_bg_rgb);
+  
+  if(!invert_flag) {
+    if((bkrd[0]+bkrd[1]+bkrd[2])>0.5F) {
+      rgb[0] = 1.0F;
+      rgb[1] = 1.0F;
+      rgb[2] = 1.0F;
+    } else {
+      rgb[0] = 0.0F;
+      rgb[1] = 0.0F;
+      rgb[2] = 0.0F;
+    }
+  }
 
+  { 
+    int a;
+    for(a=0;a<3;a++) 
+      if(fabs(bkrd[a]-rgb[a])<0.5F) {
+        rgb[a] = 1.0F - rgb[a];
+        if(fabs(bkrd[a]-rgb[a])<0.5F) {
+          if(bkrd[a]>0.5F)
+            rgb[a] = 0.0F;
+      else
+        rgb[a] = 1.0F;
+        }
+      }
+  }
+}
+
+unsigned int ColorGet32BitWord(PyMOLGlobals *G,float *rgba)
+{
+  CColor *I=G->Color;
+  unsigned int rc, gc, bc, ac;
+  unsigned int result;
+
+  rc = (int)(255*rgba[0]+0.49999F);
+  gc = (int)(255*rgba[1]+0.49999F);
+  bc = (int)(255*rgba[2]+0.49999F);
+  ac = (int)(255*rgba[3]+0.49999F);
+
+  if(rc>255) rc=255;
+  if(bc>255) bc=255;
+  if(gc>255) gc=255;
+  if(ac>255) ac=255;
+
+  if(I->BigEndian) {
+    result = (rc<<24)|(gc<<16)|(bc<<8)|ac;
+  } else {
+    result = (ac<<24)|(bc<<16)|(gc<<8)|rc;
+  }
+  return result;
+}
 
 int ColorGetNext(PyMOLGlobals *G) 
 {
@@ -130,7 +182,7 @@ ObjectGadgetRamp* ColorGetRamp(PyMOLGlobals *G,int index)
   return result;
 }
 
-int ColorGetRamped(PyMOLGlobals *G,int index,float *vertex,float *color)
+int ColorGetRamped(PyMOLGlobals *G,int index,float *vertex,float *color,int state)
 {
   register CColor *I=G->Color;
   int ok=false;
@@ -144,7 +196,7 @@ int ColorGetRamped(PyMOLGlobals *G,int index,float *vertex,float *color)
       if(I->Ext[index].Ptr) 
         ok = ObjectGadgetRampInterVertex(
                                          (ObjectGadgetRamp*)I->Ext[index].Ptr,
-                                         vertex,color);
+                                         vertex,color,state);
     }
   
   }
@@ -259,13 +311,14 @@ PyObject *ColorAsPyList(PyMOLGlobals *G)
   color=I->Color;
   for(a=0;a<I->NColor;a++) {
     if(color->Custom||color->ClampedFlag) {
-      list = PyList_New(6);
+      list = PyList_New(7);
       PyList_SetItem(list,0,PyString_FromString(color->Name));
       PyList_SetItem(list,1,PyInt_FromLong(a));
       PyList_SetItem(list,2,PConvFloatArrayToPyList(color->Color,3));
       PyList_SetItem(list,3,PyInt_FromLong((int)color->Custom));
       PyList_SetItem(list,4,PyInt_FromLong((int)color->ClampedFlag));
       PyList_SetItem(list,5,PConvFloatArrayToPyList(color->Clamped,3));
+      PyList_SetItem(list,6,PyInt_FromLong((int)color->Fixed));
       PyList_SetItem(result,c,list);
       c++;
     }
@@ -275,7 +328,36 @@ PyObject *ColorAsPyList(PyMOLGlobals *G)
 #endif
 }
 
-int ColorExtFromPyList(PyMOLGlobals *G,PyObject *list)
+int ColorConvertOldSessionIndex(PyMOLGlobals *G,int index)
+{
+  register CColor *I=G->Color;
+  if(index>cColorExtCutoff) {
+    if(I->HaveOldSessionColors) {
+      ColorRec *col=I->Color + (I->NColor-1);
+      int a;
+      for(a=I->NColor-1;a>=0;a--) {
+        if(index==col->old_session_index) {
+          index = a;
+          break;
+        }
+        col--;
+      }
+    }
+  } else if(I->HaveOldSessionExtColors) {
+    ExtRec *ext=I->Ext + (I->NExt-1);
+    int a;
+    for(a=I->NExt-1;a>=0;a--) {
+      if(index==ext->old_session_index) {
+        index = cColorExtCutoff-a;
+        break;
+      }
+      ext--;
+    }
+  }
+  return index; /* failsafe */
+}
+
+int ColorExtFromPyList(PyMOLGlobals *G,PyObject *list,int partial_restore)
 {
 #ifdef _PYMOL_NOPY
   return 0;
@@ -287,6 +369,18 @@ int ColorExtFromPyList(PyMOLGlobals *G,PyObject *list)
   register CColor *I=G->Color;
   PyObject *rec;
   ExtRec *ext;
+
+  if(partial_restore) {
+    ext=I->Ext;
+    for(a=0;a<I->NExt;a++) {
+      ext->old_session_index = 0;
+      ext++;
+    }
+    I->HaveOldSessionExtColors = true;
+  } else {
+    I->HaveOldSessionExtColors = false;
+  }
+
   if(ok) ok=(list!=NULL);
   if(ok) ok=PyList_Check(list);
 
@@ -296,17 +390,23 @@ int ColorExtFromPyList(PyMOLGlobals *G,PyObject *list)
 
   if(ok) {
     n_ext=PyList_Size(list);
-    VLACheck(I->Ext,ExtRec,n_ext); 
-    ext=I->Ext;
+    if(partial_restore) {
+      VLACheck(I->Ext,ExtRec,n_ext + I->NExt); 
+      ext=I->Ext+I->NExt;
+    } else {
+      VLACheck(I->Ext,ExtRec,n_ext); 
+      ext=I->Ext;
+    }
     for(a=0;a<n_ext;a++) {
       rec=PyList_GetItem(list,a);
       if(ok) ok=(rec!=NULL);
       if(ok) ok=PyList_Check(rec);
       if(ok) ok=PConvPyStrToStr(PyList_GetItem(rec,0),ext->Name,sizeof(ColorName));
       if(ok) ok=PConvPyIntToInt(PyList_GetItem(rec,1),&ext->Type);
+      ext->old_session_index = cColorExtCutoff-a;
       ext++;
     }
-    if(ok) I->NExt=n_ext;
+    if(ok) I->NExt=(ext-I->Ext);
 
   }
   return(ok);
@@ -314,7 +414,7 @@ int ColorExtFromPyList(PyMOLGlobals *G,PyObject *list)
 }
 
 /*========================================================================*/
-int ColorFromPyList(PyMOLGlobals *G,PyObject *list)
+int ColorFromPyList(PyMOLGlobals *G,PyObject *list,int partial_restore)
 {
 #ifdef _PYMOL_NOPY
   return 0;
@@ -322,12 +422,22 @@ int ColorFromPyList(PyMOLGlobals *G,PyObject *list)
 
   int n_custom=0;
   int a;
-  int index=0;
+  int index=0,old_session_index=0;
   int ok=true;
-  int ll;
+  int ll=0;
   register CColor *I=G->Color;
   PyObject *rec;
-  ColorRec *color;
+  ColorRec *color = NULL;
+
+  if(partial_restore) {
+    color = I->Color;
+    for(a=0;a<I->NColor;a++) {
+      color->old_session_index=0;
+      color++;
+    }
+  }
+  I->HaveOldSessionColors=false;
+
   if(ok) ok=(list!=NULL);
   if(ok) ok=PyList_Check(list);
   if(ok) {
@@ -341,11 +451,19 @@ int ColorFromPyList(PyMOLGlobals *G,PyObject *list)
          Always check ll when adding new PyList_GetItem's */
       if(ok) ok=PConvPyIntToInt(PyList_GetItem(rec,1),&index);
       if(ok) {
+        old_session_index = index;
+        if(partial_restore) {
+          if(I->NColor>index) { /* conflicts with an existing color...*/
+            I->HaveOldSessionColors = true;
+            index = I->NColor;
+          }
+        }
         if(index>=I->NColor) {
           VLACheck(I->Color,ColorRec,index); /* auto-zeros */
           I->NColor=index+1;
         }
         color=I->Color+index;
+        color->old_session_index = old_session_index; 
         if(ok) ok=PConvPyStrToStr(PyList_GetItem(rec,0),color->Name,sizeof(ColorName));
         if(ok) ok=PConvPyListToFloatArrayInPlace(PyList_GetItem(rec,2),color->Color,3);
         if(PyList_Size(rec)>=6) {
@@ -356,6 +474,11 @@ int ColorFromPyList(PyMOLGlobals *G,PyObject *list)
           if(ok) {color->Custom=true;}
         }
       }
+      if(ok&&(ll>6)) {
+        if(ok) ok=PConvPyIntToChar(PyList_GetItem(rec,6),&color->Fixed);
+      } else if(ok&&color) {
+        color->Fixed =false;
+      }
       if(!ok) break;
     }
   }
@@ -364,7 +487,7 @@ int ColorFromPyList(PyMOLGlobals *G,PyObject *list)
 }
 
 /*========================================================================*/
-void ColorDef(PyMOLGlobals *G,char *name,float *v)
+void ColorDef(PyMOLGlobals *G,char *name,float *v,int mode,int quiet)
 {
   register CColor *I=G->Color;
   int color=-1;
@@ -387,15 +510,30 @@ void ColorDef(PyMOLGlobals *G,char *name,float *v)
     I->NColor++;
   }
   strcpy(I->Color[color].Name,name);
+
   I->Color[color].Color[0]=v[0];
   I->Color[color].Color[1]=v[1];
   I->Color[color].Color[2]=v[2];
+  
+  switch(mode) {
+  case 1:
+    I->Color[color].Fixed=true;
+    break;
+  default:
+    I->Color[color].Fixed=false;
+    break;
+  }
+
   I->Color[color].Custom=true;
   ColorUpdateClamp(G,color);
 
-  PRINTFB(G,FB_Executive,FB_Actions)
-    " Color: \"%s\" defined as [ %3.3f, %3.3f, %3.3f ].\n",name,v[0],v[1],v[2] 
-    ENDFB(G);
+  if(!quiet) {
+    PRINTFB(G,FB_Executive,FB_Actions)
+      " Color: \"%s\" defined as [ %3.3f, %3.3f, %3.3f ].\n",name,v[0],v[1],v[2] 
+      ENDFB(G);
+
+  }
+
   PRINTFD(G,FB_Color) 
     " Color: and assigned number %d.\n",color
     ENDFD;
@@ -433,10 +571,23 @@ int ColorGetIndex(PyMOLGlobals *G,char *name)
         return(ColorGetNext(G));
       else if(i==cColorCurAuto)
         return(ColorGetCurrent(G));
+      else if(i==cColorAtomic)
+        return cColorAtomic;
+      else if(i==cColorObject)
+        return cColorObject;
       else if(i==-1)
         return -1;
     }
   }
+  if((name[0]=='0')&&(name[1]=='x')) { /* explicit hex RGB 0x000000 */
+    int tmp_color;
+    if(sscanf(name+2,"%x",(unsigned int*)&tmp_color)==1) {
+      tmp_color = ( cColor_TRGB_Bits |
+                    (tmp_color     &0x00FFFFFF) | 
+                    ((tmp_color>>2)&0x3F000000));
+      return tmp_color;
+    }
+  }
   if(WordMatch(G,name,"default",true))
     return(-1);
   if(WordMatch(G,name,"auto",true))
@@ -445,7 +596,8 @@ int ColorGetIndex(PyMOLGlobals *G,char *name)
     return(ColorGetCurrent(G));
   if(WordMatch(G,name,"atomic",true))
     return(cColorAtomic);
-
+  if(WordMatch(G,name,"object",true))
+    return(cColorObject);
   if(!I->Lex) { /* populate the dictionary */
 
     OVreturn_word result;
@@ -500,8 +652,23 @@ char *ColorGetName(PyMOLGlobals *G,int index)
   register CColor *I=G->Color;
   if((index>=0)&&(index<I->NColor))
     return(I->Color[index].Name);
-  else
-    return(NULL);
+  else if((index&cColor_TRGB_Mask)==cColor_TRGB_Bits) {
+    index = (((index&0xFFFFFF) | 
+              ((index<<2)&0xFC000000) | /* convert 6 bits of trans into 8 */
+              ((index>>4)&0x03000000)));
+    if(index&0xFF000000) /* if transparent */
+      sprintf(I->RGBName,"%08x",index);
+    else /* else */
+      sprintf(I->RGBName,"%06x",index);
+    return I->RGBName;
+  } else if(index<=cColorExtCutoff) {
+    int a = cColorExtCutoff - index;
+    if(a<I->NExt) {
+      return I->Ext[a].Name;
+    } else 
+      return NULL;
+  }
+  return(NULL);
 }
 /*========================================================================*/
 int ColorGetStatus(PyMOLGlobals *G,int index)
@@ -681,7 +848,8 @@ void ColorReset(PyMOLGlobals *G)
     { 0.0, 1.0, 1.0 }, /* cyan - 999 */
   };
 
-  float spectrumO[29][3] = { /* a rainbow with perceptive color balancing and extra blue/red at the ends */
+  float spectrumO[29][3] = { 
+    /* a rainbow with perceptive color balancing and extra blue/red at the ends */
     { 1.0, 0.0,  1.0 }, /* violet */
     { 0.8F,0.0,  1.0 }, 
 
@@ -918,9 +1086,9 @@ void ColorReset(PyMOLGlobals *G)
   n_color++;
 
   strcpy(color[n_color].Name,"sulfur");
-  color[n_color].Color[0]=1.0F;
-  color[n_color].Color[1]=0.5F;
-  color[n_color].Color[2]=0.0F;
+  color[n_color].Color[0]=0.9F; /* needs to be far enough from "yellow" */
+  color[n_color].Color[1]=0.775F; /* to be resolved, while still slightly on */
+  color[n_color].Color[2]=0.25F; /* the yellow side of yelloworange */
   n_color++;
 
   strcpy(color[n_color].Name,"tv_red");
@@ -1967,8 +2135,19 @@ void ColorReset(PyMOLGlobals *G)
 
   n_color++;
 
-  /* if any more colors need to be added, add them here at the end so that existing files won't have their colors changed */
+  strcpy(color[n_color].Name,"lonepair");
+  color[n_color].Color[0]=0.5F;
+  color[n_color].Color[1]=0.5F;
+  color[n_color].Color[2]=0.5F;
 
+  n_color++;
+
+  strcpy(color[n_color].Name,"pseudoatom");
+  color[n_color].Color[0]=0.9F;
+  color[n_color].Color[1]=0.9F;
+  color[n_color].Color[2]=0.9F;
+
+  n_color++;
   for(a=0;a<n_color;a++) { 
     /* mark all current colors non-custom so that they don't get saved in session files */
     color[a].Custom=false;
@@ -1976,7 +2155,6 @@ void ColorReset(PyMOLGlobals *G)
 
   I->NColor = n_color;
   I->NExt = 0;
-
 }
 
 int ColorTableLoad(PyMOLGlobals *G,char *fname,int quiet)
@@ -2150,6 +2328,129 @@ int ColorTableLoad(PyMOLGlobals *G,char *fname,int quiet)
   }
   return(ok);
 }
+#if 0
+static void unclamp_color(unsigned int *table, float *in, float *out, int big_endian)
+{
+  /* simple iterative approach to finding closest input for a desired
+   output given the current color mapping table */
+  
+  float cur_in[3], cur_out[3], diff[3];
+  copy3f(in, cur_in);
+  for(a=0;a<10;a++) {
+    clamp_color(table,cur_in,cur_out,big_endian);
+    diff3f(cur_out,in,diff);
+    scale3f(0.5,diff,diff);
+    subtract3f(cur_in,diff,cur_in);
+  }
+  copy3f(cur_in,out);
+}
+#endif
+
+static void clamp_color(unsigned int *table, float *in, float *out, int big_endian)
+{
+  unsigned int r,g,b,rr,gr,br;
+  unsigned int ra,ga,ba;
+  unsigned int rc[2][2][2],gc[2][2][2],bc[2][2][2];
+  unsigned int *entry;
+  int x,y,z;  
+  float fr,fg,fb,frm1x, fgm1,fbm1,rct,gct,bct;
+  const float _1 = 1.0F, _2 = 2.0F, _0 = 0.0F, _05 = 0.5F,
+    _04999 = 0.4999F;
+  const float inv255 = 1.0F/255.0F;
+
+  r = ((int)(255*in[0]+_05))&0xFF;
+  g = ((int)(255*in[1]+_05))&0xFF;
+  b = ((int)(255*in[2]+_05))&0xFF;
+
+  rr = r&0x3;
+  gr = g&0x3;
+  br = b&0x3;
+  
+  r = (r>>2);
+  g = (g>>2);
+  b = (b>>2);
+  
+  /* now for a crude little trilinear */
+  
+  for(x=0;x<2;x++) {
+    ra = r + x;
+    if(ra>63) ra=63;
+    for(y=0;y<2;y++) {
+      ga = g + y;
+      if(ga>63) ga=63;
+      for(z=0;z<2;z++) {
+        ba = b + z;
+        if(ba>63) ba=63;
+        
+        entry = table + (ra<<12) + (ga<<6) + ba;
+        
+        if(big_endian) {
+          rc[x][y][z] = 0xFF&((*entry)>>24);
+          gc[x][y][z] = 0xFF&((*entry)>>16);
+          bc[x][y][z] = 0xFF&((*entry)>>8);
+        } else {
+          rc[x][y][z] = 0xFF&((*entry)    );
+          gc[x][y][z] = 0xFF&((*entry)>> 8);
+          bc[x][y][z] = 0xFF&((*entry)>>16);
+        }
+      }
+    }
+  }
+  
+  frm1x = rr/4.0F;
+  fgm1 = gr/4.0F;
+  fbm1 = br/4.0F;
+  
+  fr = 1.0F - frm1x;
+  fg = 1.0F - fgm1;
+  fb = 1.0F - fbm1;
+  
+  rct = _04999 + 
+    (fr   * fg   * fb   * rc[0][0][0]) + 
+    (frm1x * fg   * fb   * rc[1][0][0]) + 
+    (fr   * fgm1 * fb   * rc[0][1][0]) + 
+    (fr   * fg   * fbm1 * rc[0][0][1]) + 
+    (frm1x * fgm1 * fb   * rc[1][1][0]) + 
+    (fr   * fgm1 * fbm1 * rc[0][1][1]) + 
+    (frm1x * fg   * fbm1 * rc[1][0][1]) + 
+    (frm1x * fgm1 * fbm1 * rc[1][1][1]);
+  
+  gct = _04999 + 
+    (fr   * fg   * fb   * gc[0][0][0]) + 
+    (frm1x * fg   * fb   * gc[1][0][0]) + 
+    (fr   * fgm1 * fb   * gc[0][1][0]) + 
+    (fr   * fg   * fbm1 * gc[0][0][1]) + 
+    (frm1x * fgm1 * fb   * gc[1][1][0]) + 
+    (fr   * fgm1 * fbm1 * gc[0][1][1]) + 
+    (frm1x * fg   * fbm1 * gc[1][0][1]) + 
+    (frm1x * fgm1 * fbm1 * gc[1][1][1]);
+  
+  bct = _04999 + 
+    (fr   * fg   * fb   * bc[0][0][0]) + 
+    (frm1x * fg   * fb   * bc[1][0][0]) + 
+    (fr   * fgm1 * fb   * bc[0][1][0]) + 
+    (fr   * fg   * fbm1 * bc[0][0][1]) + 
+    (frm1x * fgm1 * fb   * bc[1][1][0]) + 
+    (fr   * fgm1 * fbm1 * bc[0][1][1]) + 
+    (frm1x * fg   * fbm1 * bc[1][0][1]) + 
+    (frm1x * fgm1 * fbm1 * bc[1][1][1]);
+  
+  if(r>=63) rct+=rr;
+  if(g>=63) gct+=gr;
+  if(b>=63) bct+=br;
+  
+  if(rct<=_2) rct=_0; /* make sure black is black */
+  if(gct<=_2) gct=_0;
+  if(bct<=_2) bct=_0;
+        
+  out[0] = rct*inv255;
+  out[1] = gct*inv255;
+  out[2] = bct*inv255;
+  if(out[0]>_1) out[0]=_1;
+  if(out[1]>_1) out[1]=_1;
+  if(out[2]>_1) out[2]=_1;
+
+}
 
 /*========================================================================*/
 void ColorUpdateClamp(PyMOLGlobals *G,int index)
@@ -2157,13 +2458,7 @@ void ColorUpdateClamp(PyMOLGlobals *G,int index)
   int i;
   int once=false;
   register CColor *I=G->Color; 
-  unsigned int *entry;
   float *color,*new_color;
-  unsigned int r,g,b,rr,gr,br;
-  unsigned int ra,ga,ba;
-  unsigned int rc[2][2][2],gc[2][2][2],bc[2][2][2];
-  float fr,fg,fb,frm1,fgm1,fbm1,rct,gct,bct;
-  int x,y,z;
 
   i = index;
   if(index>=0) {
@@ -2175,101 +2470,11 @@ void ColorUpdateClamp(PyMOLGlobals *G,int index)
     if(index<I->NColor) {
       if(!I->ColorTable) {
         I->Color[index].ClampedFlag = false;
-      } else {
+      } else if(!I->Color[index].Fixed) {
         color = I->Color[index].Color;
-        r = ((int)(255*color[0]+0.5F))&0xFF;
-        g = ((int)(255*color[1]+0.5F))&0xFF;
-        b = ((int)(255*color[2]+0.5F))&0xFF;
-
-        rr = r&0x3;
-        gr = g&0x3;
-        br = b&0x3;
-
-        r = (r>>2);
-        g = (g>>2);
-        b = (b>>2);
-
-        /* now for a crude little trilinear */
-
-        for(x=0;x<2;x++) {
-          ra = r + x;
-          if(ra>63) ra=63;
-          for(y=0;y<2;y++) {
-            ga = g + y;
-            if(ga>63) ga=63;
-            for(z=0;z<2;z++) {
-              ba = b + z;
-              if(ba>63) ba=63;
-              
-              entry = I->ColorTable + (ra<<12) + (ga<<6) + ba;
-              
-              if(I->BigEndian) {
-                rc[x][y][z] = 0xFF&((*entry)>>24);
-                gc[x][y][z] = 0xFF&((*entry)>>16);
-                bc[x][y][z] = 0xFF&((*entry)>>8);
-              } else {
-                rc[x][y][z] = 0xFF&((*entry)    );
-                gc[x][y][z] = 0xFF&((*entry)>> 8);
-                bc[x][y][z] = 0xFF&((*entry)>>16);
-              }
-            }
-          }
-        }
-
-        frm1 = rr/4.0F;
-        fgm1 = gr/4.0F;
-        fbm1 = br/4.0F;
-        
-        fr = 1.0F - frm1;
-        fg = 1.0F - fgm1;
-        fb = 1.0F - fbm1;
-
-        rct = 0.4999F + 
-          (fr   * fg   * fb   * rc[0][0][0]) + 
-          (frm1 * fg   * fb   * rc[1][0][0]) + 
-          (fr   * fgm1 * fb   * rc[0][1][0]) + 
-          (fr   * fg   * fbm1 * rc[0][0][1]) + 
-          (frm1 * fgm1 * fb   * rc[1][1][0]) + 
-          (fr   * fgm1 * fbm1 * rc[0][1][1]) + 
-          (frm1 * fg   * fbm1 * rc[1][0][1]) + 
-          (frm1 * fgm1 * fbm1 * rc[1][1][1]);
-
-        gct = 0.4999F + 
-          (fr   * fg   * fb   * gc[0][0][0]) + 
-          (frm1 * fg   * fb   * gc[1][0][0]) + 
-          (fr   * fgm1 * fb   * gc[0][1][0]) + 
-          (fr   * fg   * fbm1 * gc[0][0][1]) + 
-          (frm1 * fgm1 * fb   * gc[1][1][0]) + 
-          (fr   * fgm1 * fbm1 * gc[0][1][1]) + 
-          (frm1 * fg   * fbm1 * gc[1][0][1]) + 
-          (frm1 * fgm1 * fbm1 * gc[1][1][1]);
-
-        bct = 0.4999F + 
-          (fr   * fg   * fb   * bc[0][0][0]) + 
-          (frm1 * fg   * fb   * bc[1][0][0]) + 
-          (fr   * fgm1 * fb   * bc[0][1][0]) + 
-          (fr   * fg   * fbm1 * bc[0][0][1]) + 
-          (frm1 * fgm1 * fb   * bc[1][1][0]) + 
-          (fr   * fgm1 * fbm1 * bc[0][1][1]) + 
-          (frm1 * fg   * fbm1 * bc[1][0][1]) + 
-          (frm1 * fgm1 * fbm1 * bc[1][1][1]);
-
-        if(r>=63) rct+=rr;
-        if(g>=63) gct+=gr;
-        if(b>=63) bct+=br;
-
-        if(rct<=2.0F) rct=0.0F; /* make sure black is black */
-        if(gct<=2.0F) gct=0.0F;
-        if(bct<=2.0F) bct=0.0F;
-
         new_color = I->Color[index].Clamped;
-        new_color[0] = rct/255.0F;
-        if(new_color[0]>1.0F) new_color[0]=1.0F;
-        new_color[1] = gct/255.0F;
-        if(new_color[1]>1.0F) new_color[1]=1.0F; 
-        new_color[2] = bct/255.0F;
-        if(new_color[2]>1.0F) new_color[2]=1.0F;
-
+        clamp_color(I->ColorTable, color, new_color, I->BigEndian);
+        
         PRINTFD(G,FB_Color)
           "%5.3f %5.3f %5.3f -> %5.3f %5.3f %5.3f\n",
                color[0],color[1],color[2],
@@ -2309,6 +2514,19 @@ int ColorInit(PyMOLGlobals *G)
 }
 
 /*========================================================================*/
+float *ColorGetSpecial(PyMOLGlobals *G,int index)
+{
+  if(index>=0) 
+    return ColorGet(G,index);
+  else {
+    register CColor *I=G->Color;
+    I->RGBColor[0] = (float)index;
+    I->RGBColor[1] = -1.0F;
+    I->RGBColor[2] = -1.0F;
+    return I->RGBColor;
+  }
+}
+
 float *ColorGet(PyMOLGlobals *G,int index)
 {
   register CColor *I=G->Color;
@@ -2319,7 +2537,48 @@ float *ColorGet(PyMOLGlobals *G,int index)
     else
       ptr = I->Color[index].Color;
     return(ptr);
-  } else /* invalid color id, then simply return white */
+  } else if((index&cColor_TRGB_Mask)==cColor_TRGB_Bits) { /* a 24-bit RGB color */
+    I->RGBColor[0] = ((index&0x00FF0000) >> 16) / 255.0F;
+    I->RGBColor[1] = ((index&0x0000FF00) >> 8 ) / 255.0F;
+    I->RGBColor[2] = ((index&0x000000FF)      ) / 255.0F;
+    if(I->ColorTable)
+      clamp_color(I->ColorTable, I->RGBColor, I->RGBColor, I->BigEndian);
+    return I->RGBColor;
+  } else {
+            /* invalid color id, then simply return white */
 	 return(I->Color[0].Color);
+  }
+}
+
+int ColorGetEncoded(PyMOLGlobals *G,int index,float *color)
+{
+  register CColor *I=G->Color;
+  float *ptr;
+  if((index>=0)&&(index<I->NColor)) {
+    if(I->Color[index].ClampedFlag&&SettingGetGlobal_b(G,cSetting_clamp_colors))
+       ptr = I->Color[index].Clamped;
+     else
+       ptr = I->Color[index].Color;
+     copy3f(ptr,color);
+   } else if((index&cColor_TRGB_Mask)==cColor_TRGB_Bits) { /* a 24-bit RGB color */
+     float rgb_color[3];
+     rgb_color[0] = ((index&0x00FF0000) >> 16) / 255.0F;
+     rgb_color[1] = ((index&0x0000FF00) >> 8 ) / 255.0F;
+     rgb_color[2] = ((index&0x000000FF)      ) / 255.0F;
+     if(I->ColorTable)
+       clamp_color(I->ColorTable, rgb_color, rgb_color, I->BigEndian);
+     copy3f(rgb_color,color);
+   } else if(index<=cColorExtCutoff) {
+     color[0]=(float)index;
+     color[1]=0.0F;
+     color[2]=0.0F;
+   } else {
+     color[0] = 1.0F;
+     color[1] = 1.0F;
+     color[2] = 1.0F;
+     /* invalid color id, then simply return white */
+    return 0;
+  }
+  return 1;
 }
 
