@@ -35,55 +35,94 @@ Z* -------------------------------------------------------------------
 #include"PyMOL.h"
 
 struct _CMovie {
-  ImageType *Image;
+  ImageType **Image;
   int *Sequence;
   MovieCmdType *Cmd;
   int NImage,NFrame;
-  unsigned Width,Height;
   int MatrixFlag;
   SceneViewType Matrix;
   int Playing;
   int Locked;
   int CacheSave;
+  int OverlaySave;
   CViewElem *ViewElem;
   int RecursionFlag;
 };
 
 void MovieCopyPrepare(PyMOLGlobals *G,int *width,int *height,int *length)
 {
-  /* assumed locked api, blocked threads, and master thread on entry */
+	/* assumed locked api, blocked threads, and master thread on entry */
+	
+	/* writes the current movie sequence to a set of PNG files 
+	* this routine can take a LOT of time -- 
+	* TODO: develop an interrupt mechanism */
+	
+	int start,stop;
+	register CMovie *I=G->Movie;
+	int nFrame;
+	
 
-  /* writes the current movie sequence to a set of PNG files 
-  * this routine can take a LOT of time -- 
-  * TODO: develop an interrupt mechanism */
-
-  int start,stop;
-  register CMovie *I=G->Movie;
-  int nFrame;
-
-  I->CacheSave = (int)SettingGet(G,cSetting_cache_frames); 
-  if(!I->CacheSave)
-    MovieClearImages(G);
-  SettingSet(G,cSetting_cache_frames,1.0);
-  nFrame = I->NFrame;
-  if(!nFrame) {
-	 nFrame=SceneGetNFrame(G);
-  }
-  start=0;
-  stop=nFrame;
-  if((start!=0)||(stop!=(nFrame+1)))
-    SceneSetFrame(G,0,0);
-  MoviePlay(G,cMoviePlay);
-  VLACheck(I->Image,ImageType,nFrame);
-  SceneGetWidthHeight(G,width,height);
-  *length = nFrame;
+	I->CacheSave = (int)SettingGet(G,cSetting_cache_frames); 
+	I->OverlaySave = (int)SettingGet(G,cSetting_overlay);
+	if(!I->CacheSave)
+		MovieClearImages(G);
+	SettingSet(G,cSetting_cache_frames,1.0);
+	SettingSet(G,cSetting_overlay,5);
+	nFrame = I->NFrame;
+	if(!nFrame) {
+		nFrame=SceneGetNFrame(G,NULL);
+	}
+	start=0;
+	stop=nFrame;
+	if((start!=0)||(stop!=(nFrame+1)))
+		SceneSetFrame(G,0,0);
+	MoviePlay(G,cMoviePlay);
+	VLACheck(I->Image,ImageType*,nFrame);
+	SceneGetWidthHeight(G,width,height);
+	{ 
+      int uniform_height = -1;
+      int uniform_width = -1;
+		int uniform_flag = false;
+		int scene_match = true;
+		int a;
+		ImageType *image;
+		/* make sure all the movie frames match the screen size or are pre-rendered and are already the same size */
+		for(a=0;a<nFrame;a++) {
+			image = I->Image[a];
+			if(image) {
+				if((image->height!=*height)||
+				   (image->width!=*width)) {
+					scene_match = false;
+					if(uniform_height<0) {
+						uniform_height = image->height;
+						uniform_width = image->width;
+					} else {
+						if((image->height!=uniform_height)||
+						   (image->width!=uniform_width))
+							uniform_flag = false;
+					}
+				}
+			} else 
+			   uniform_flag = false; /* missing at least one image, so not uniform */
+		}
+		if(!scene_match) {
+			if(uniform_flag) {
+				*height = uniform_height;
+				*width = uniform_width;
+			} else {
+			    MovieClearImages(G);
+			}
+		}
+		
+	}
+	*length = nFrame;
 }
 
 void MovieFlushCommands(PyMOLGlobals *G)
 {
   register CMovie *I=G->Movie;
   I->RecursionFlag=true;
-  PFlush();
+  PFlush(G);
   I->RecursionFlag=false;
 }
 
@@ -92,32 +131,32 @@ int MovieCopyFrame(PyMOLGlobals *G,int frame,int width,int height,int rowbytes,v
   register CMovie *I=G->Movie;
   int result=false;
   int nFrame;
-  
   nFrame = I->NFrame;
   if(!nFrame) {
-    nFrame=SceneGetNFrame(G);
+    nFrame=SceneGetNFrame(G,NULL);
   }
   
-  if((width==(signed int)I->Width)&&
-     (height==(signed int)I->Height)&&
-     (frame<nFrame)&&(ptr)) {
+  if((frame<nFrame)&&(ptr)) {
     int a = frame;
     int i;
     SceneSetFrame(G,0,a);
     MovieDoFrameCommand(G,a);
     MovieFlushCommands(G);
     i=MovieFrameToImage(G,a);
-    VLACheck(I->Image,ImageType,i);
+    VLACheck(I->Image,ImageType*,i);
     if(!I->Image[i]) {
-      SceneMakeMovieImage(G);
+      SceneUpdate(G);
+	  SceneMakeMovieImage(G,false);
     }
     if(!I->Image[i]) {
       PRINTFB(G,FB_Movie,FB_Errors) 
         "MoviePNG-Error: Missing rendered image.\n"
         ENDFB(G);
     } else {
-      {
-        unsigned char *srcImage = (unsigned char*)I->Image[i];
+      if((I->Image[i]->height == height) &&
+	     (I->Image[i]->width == width))
+		 {
+        unsigned char *srcImage = (unsigned char*)I->Image[i]->data;
         int i,j;
         for (i=0; i< height; i++)
           {
@@ -133,28 +172,58 @@ int MovieCopyFrame(PyMOLGlobals *G,int frame,int width,int height,int rowbytes,v
               }
           }
         result = true;
-      }
+      } else { 
+	     /* mismatched dimensions, so furnish a white image */
+	     memset(ptr, 0xFF, 4*height*width);
+	  }
       ExecutiveDrawNow(G);
       if(G->HaveGUI) PyMOL_SwapBuffers(G->PyMOL);
     }
     if(!I->CacheSave) {
-      if(I->Image[i])
-        mfree(I->Image[i]);
-      I->Image[i]=NULL;
+      if(I->Image[i]) {
+        FreeP(I->Image[i]->data);
+        FreeP(I->Image[i]);
+      }
     }
   }
   return result;
 }
 
+int MoviePurgeFrame(PyMOLGlobals *G,int frame)
+{
+  register CMovie *I=G->Movie;
+  int result=false;
+  int nFrame;
+  int i;
+  nFrame = I->NFrame;
+  if(!nFrame) {
+    nFrame=SceneGetNFrame(G,NULL);
+  }
+  if(!I->CacheSave) {
+  if(frame<nFrame) {
+    int a = frame;
+    i=MovieFrameToImage(G,a);
+    VLACheck(I->Image,ImageType*,i);
+    if(I->Image[i]) {
+		FreeP(I->Image[i]->data);
+		FreeP(I->Image[i]);
+		I->Image[i]=NULL;
+		result=true;
+		}
+	}
+	}
+	return result;
+}
+
 void MovieCopyFinish(PyMOLGlobals *G) 
 {
   register CMovie *I=G->Movie;
-  SceneDirty(G); /* important */
+  SceneInvalidate(G); /* important */
   SettingSet(G,cSetting_cache_frames,(float)I->CacheSave);
+  SettingSet(G,cSetting_overlay,(float)I->OverlaySave);
   MoviePlay(G,cMovieStop);
   if(!I->CacheSave) {
     MovieClearImages(G); 
-    SceneSuppressMovieFrame(G);
   }
 }
 
@@ -257,7 +326,7 @@ int MovieFromPyList(PyMOLGlobals *G,PyObject *list,int *warning)
     I->ViewElem = NULL;
     tmp = PyList_GetItem(list,6);
     if(tmp && !(tmp == Py_None))
-      ok = ViewElemVLAFromPyList(tmp,&I->ViewElem,I->NFrame);
+      ok = ViewElemVLAFromPyList(G,tmp,&I->ViewElem,I->NFrame);
   }
   if(!ok) {
     MovieReset(G);
@@ -309,7 +378,7 @@ PyObject *MovieAsPyList(PyMOLGlobals *G)
     PyList_SetItem(result,5,PConvAutoNone(NULL));
   }
   if(I->ViewElem) {
-    PyList_SetItem(result,6,ViewElemVLAAsPyList(I->ViewElem,I->NFrame));
+    PyList_SetItem(result,6,ViewElemVLAAsPyList(G,I->ViewElem,I->NFrame));
   } else {
     PyList_SetItem(result,6,PConvAutoNone(NULL));
   }
@@ -332,7 +401,9 @@ int MoviePlaying(PyMOLGlobals *G)
   register CMovie *I=G->Movie;
   if(I->Locked)
     return false;
-  return(I->Playing);
+  return(I->Playing||I->RecursionFlag); 
+  /* returns true if movie is playing OR if we're in the process of evaluating
+     movie commands */
 }
 /*========================================================================*/
 void MoviePlay(PyMOLGlobals *G,int cmd)
@@ -349,7 +420,7 @@ void MoviePlay(PyMOLGlobals *G,int cmd)
     if(!(int)SettingGet(G,cSetting_movie_loop)) { 
       /* if not looping, and at end of movie, then automatically rewind
        and force execution of the first movie command */
-      if((SettingGetGlobal_i(G,cSetting_frame))==(SceneGetNFrame(G))) {
+      if((SettingGetGlobal_i(G,cSetting_frame))==(SceneGetNFrame(G,NULL))) {
         SceneSetFrame(G,7,0);
       }
     }
@@ -357,7 +428,7 @@ void MoviePlay(PyMOLGlobals *G,int cmd)
 	 break;
   }
   OrthoDirty(G);
-  SceneRestartTimers(G);
+  SceneRestartFrameTimer(G);
 }
 /*========================================================================*/
 int  MovieMatrix(PyMOLGlobals *G,int action)
@@ -376,7 +447,7 @@ int  MovieMatrix(PyMOLGlobals *G,int action)
 	 break;
   case cMovieMatrixRecall:
 	 if(I->MatrixFlag) 
-		SceneSetView(G,I->Matrix,true,0);
+		SceneSetView(G,I->Matrix,true,0,0);
     else
       result = 0;
 	 break;
@@ -389,12 +460,10 @@ int  MovieMatrix(PyMOLGlobals *G,int action)
 /*========================================================================*/
 void MovieSetSize(PyMOLGlobals *G,unsigned int width,unsigned int height)
 {  
-  register CMovie *I=G->Movie;
-  I->Width=width;
-  I->Height=height;
+  return;
 }
 /*========================================================================*/
-int MoviePNG(PyMOLGlobals *G,char *prefix,int save,int start,int stop)
+int MoviePNG(PyMOLGlobals *G,char *prefix,int save,int start,int stop,int missing_only)
 {
   /* assumed locked api, blocked threads, and master thread on entry */
 
@@ -408,7 +477,8 @@ int MoviePNG(PyMOLGlobals *G,char *prefix,int save,int start,int stop)
   char fname[255];
   char buffer[255];
   int nFrame;
-
+  double accumTiming = 0.0;
+  int file_missing = true;
   save = (int)SettingGet(G,cSetting_cache_frames); 
   if(!save)
     MovieClearImages(G);
@@ -416,7 +486,7 @@ int MoviePNG(PyMOLGlobals *G,char *prefix,int save,int start,int stop)
   OrthoBusyPrime(G);
   nFrame = I->NFrame;
   if(!nFrame) {
-	 nFrame=SceneGetNFrame(G);
+	 nFrame=SceneGetNFrame(G,NULL);
   }
   if(start<0) start=0;
   if(start>nFrame) start=nFrame;
@@ -428,55 +498,82 @@ int MoviePNG(PyMOLGlobals *G,char *prefix,int save,int start,int stop)
     
   SceneSetFrame(G,0,0);
   MoviePlay(G,cMoviePlay);
-  VLACheck(I->Image,ImageType,nFrame);
+  VLACheck(I->Image,ImageType*,nFrame);
 
   OrthoBusySlow(G,0,nFrame);
-  for(a=0;a<nFrame;a++)
-	 {
-      PRINTFB(G,FB_Movie,FB_Debugging)
-        " MoviePNG-DEBUG: Cycle %d...\n",a
-        ENDFB(G);
-		sprintf(fname,"%s%04d.png",prefix,a+1);
-		SceneSetFrame(G,0,a);
-		MovieDoFrameCommand(G,a);
-      MovieFlushCommands(G);
-		i=MovieFrameToImage(G,a);
-      VLACheck(I->Image,ImageType,i);
-      printf("here %d\n",a);
-      if((a>=start)&&(a<=stop)) { /* only render frames in the specified interval */
-        if(!I->Image[i]) {
-          SceneMakeMovieImage(G);
-        }
-        if(!I->Image[i]) {
-          PRINTFB(G,FB_Movie,FB_Errors) 
-            "MoviePNG-Error: Missing rendered image.\n"
-            ENDFB(G);
-        } else {
-          MyPNGWrite(G,fname,I->Image[i],I->Width,I->Height);		
-          ExecutiveDrawNow(G);
-          OrthoBusySlow(G,a,nFrame);
-          if(G->HaveGUI) PyMOL_SwapBuffers(G->PyMOL);
-          PRINTFB(G,FB_Movie,FB_Debugging)
-            " MoviePNG-DEBUG: i = %d, I->Image[i] = %p\n",i,I->Image[i]
-            ENDFB(G);
-          if(Feedback(G,FB_Movie,FB_Actions)) {
-            printf(" MoviePNG: wrote %s\n",fname);
-          }
+  for(a=0;a<nFrame;a++) {
+    double timing = UtilGetSeconds(G); /* start timing the process */
+    PRINTFB(G,FB_Movie,FB_Debugging)
+      " MoviePNG-DEBUG: Cycle %d...\n",a
+      ENDFB(G);
+    sprintf(fname,"%s%04d.png",prefix,a+1);
+    if(missing_only) {
+      FILE *tmp = fopen(fname,"rb");
+      if(tmp) {
+        fclose(tmp);
+        file_missing=false;
+      } else {
+        file_missing=true;
+      }
+    }
+    SceneSetFrame(G,0,a);
+    MovieDoFrameCommand(G,a);
+    MovieFlushCommands(G);
+    i=MovieFrameToImage(G,a);
+    VLACheck(I->Image,ImageType*,i);
+    if((a>=start)&&(a<=stop)&&(file_missing)) { /* only render frames in the specified interval */
+      if(!I->Image[i]) {
+        SceneUpdate(G);
+        SceneMakeMovieImage(G,false);
+      }
+      if(!I->Image[i]) {
+        PRINTFB(G,FB_Movie,FB_Errors) 
+          "MoviePNG-Error: Missing rendered image.\n"
+          ENDFB(G);
+      } else {
+        MyPNGWrite(G,fname,I->Image[i]->data,I->Image[i]->width,I->Image[i]->height,SettingGetGlobal_f(G,cSetting_image_dots_per_inch));		
+        ExecutiveDrawNow(G);
+        OrthoBusySlow(G,a,nFrame);
+        if(G->HaveGUI) PyMOL_SwapBuffers(G->PyMOL);
+        PRINTFB(G,FB_Movie,FB_Debugging)
+          " MoviePNG-DEBUG: i = %d, I->Image[i] = %p\n",i,I->Image[i]->data
+          ENDFB(G);
+        if(Feedback(G,FB_Movie,FB_Actions)) {
+          printf(" Movie: wrote %s\n",fname);
         }
       }
-      if(I->Image[i])
-        mfree(I->Image[i]);
-		I->Image[i]=NULL;
-	 }
-  SceneDirty(G); /* important */
+    }
+    if(I->Image[i]) {
+      FreeP(I->Image[i]->data);
+      FreeP(I->Image[i]);
+    }
+    timing = UtilGetSeconds(G)-timing;
+    accumTiming += timing; 
+    { 
+      double est1 =       (nFrame-a)*timing;
+      double est2 =       ((nFrame-a)/(float)(a+1))*accumTiming;
+      
+      PRINTFB(G,FB_Movie,FB_Details)
+        " Movie: frame %4d of %4d, %4.2f sec. (%d:%02d:%02d - %d:%02d:%02d to go).\n", 
+        a+1,nFrame,
+        timing,
+        (int)(est1/3600),
+        ((int)(est1/60)) % 60,
+        ((int)est1) % 60,
+        (int)(est2/3600),
+        ((int)(est2/60)) % 60,
+        ((int)est2) % 60
+        ENDFB(G);
+    }
+  }
+  SceneInvalidate(G); /* important */
   PRINTFB(G,FB_Movie,FB_Debugging)
     " MoviePNG-DEBUG: done.\n"
     ENDFB(G);
   SettingSet(G,cSetting_cache_frames,(float)save);
   MoviePlay(G,cMovieStop);
   MovieClearImages(G);
-  SceneSuppressMovieFrame(G);
-  return(true);
+   return(true);
 }
 /*========================================================================*/
 void MovieAppendSequence(PyMOLGlobals *G,char *str,int start_from)
@@ -545,7 +642,7 @@ void MovieAppendSequence(PyMOLGlobals *G,char *str,int start_from)
     I->NFrame=start_from;
   }
            
-  VLACheck(I->Image,ImageType,I->NFrame);
+  VLACheck(I->Image,ImageType*,I->NFrame);
   PRINTFB(G,FB_Movie,FB_Debugging)
     " MovieSequence: leaving... I->NFrame%d\n",I->NFrame
     ENDFB(G);
@@ -577,7 +674,7 @@ int MovieFrameToIndex(PyMOLGlobals *G,int frame)
   } else return(frame);
 }
 /*========================================================================*/
-void MovieSetImage(PyMOLGlobals *G,int index,ImageType image)
+void MovieSetImage(PyMOLGlobals *G,int index,ImageType *image)
 {
   register CMovie *I=G->Movie;
 
@@ -585,7 +682,7 @@ void MovieSetImage(PyMOLGlobals *G,int index,ImageType image)
     " MovieSetImage: setting movie image %d\n",index+1
     ENDFB(G);
 
-  VLACheck(I->Image,ImageType,index);
+  VLACheck(I->Image,ImageType*,index);
   if(I->Image[index]) FreeP(I->Image[index]);
   I->Image[index]=image;
   if(I->NImage<(index+1))
@@ -598,16 +695,26 @@ void MovieDoFrameCommand(PyMOLGlobals *G,int frame)
   if(frame==0)
 	 MovieMatrix(G,cMovieMatrixRecall);
   if(!I->Locked) {
-    if((frame>=0)&&(frame<I->NFrame))
-      {
-        if(I->Cmd[frame][0]) {
-          if(!I->RecursionFlag) {
-            PParse(I->Cmd[frame]);
+    if((frame>=0)&&(frame<I->NFrame)) {
+      if(I->Cmd[frame][0]) {
+        if(!I->RecursionFlag) {
+          PParse(G,I->Cmd[frame]);
+        }
+      }
+      if(I->ViewElem) { 
+        if(I->ViewElem[frame].scene_flag) {
+          char *st = OVLexicon_FetchCString(G->Lexicon,I->ViewElem[frame].scene_name);
+          if(strcmp(st,SettingGetGlobal_s(G,cSetting_scene_current_name))) {
+#ifndef _PYMOL_NOPY            
+            PBlock(G);
+            PXDecRef(PyObject_CallMethod(G->P_inst->cmd,"scene","sssi",st,"recall","", 0));
+            PUnblock(G);
+#endif
           }
         }
-        if(I->ViewElem) 
-          SceneFromViewElem(G,I->ViewElem+frame);
+        SceneFromViewElem(G,I->ViewElem+frame);
       }
+    }
   }
 }
 /*========================================================================*/
@@ -632,7 +739,11 @@ void MovieSetCommand(PyMOLGlobals *G,int frame,char *command)
 }
 
 /*========================================================================*/
-int MovieView(PyMOLGlobals *G,int action,int first,int last,float power,float bias)
+int MovieView(PyMOLGlobals *G,int action,int first,
+              int last,float power,float bias,
+              int simple, float linear,int wrap,
+              int hand,int window,int cycles,
+              char *scene_name, float scene_cut, int quiet)
 {
   register CMovie *I=G->Movie;
   int frame;
@@ -646,10 +757,13 @@ int MovieView(PyMOLGlobals *G,int action,int first,int last,float power,float bi
       for(frame=first;frame<=last;frame++) {
         if((frame>=0)&&(frame<I->NFrame)) {
           VLACheck(I->ViewElem,CViewElem,frame);
-          PRINTFB(G,FB_Movie,FB_Details)
-            " MovieView: Setting frame %d.\n",frame+1
-            ENDFB(G);
+          if(!quiet) {
+            PRINTFB(G,FB_Movie,FB_Details)
+              " MovieView: Setting frame %d.\n",frame+1
+              ENDFB(G);
+          }
           SceneToViewElem(G,I->ViewElem+frame);          
+          
           I->ViewElem[frame].specification_level = 2;
         }
       }
@@ -664,6 +778,7 @@ int MovieView(PyMOLGlobals *G,int action,int first,int last,float power,float bi
       for(frame=first;frame<=last;frame++) {
         if((frame>=0)&&(frame<I->NFrame)) {
           VLACheck(I->ViewElem,CViewElem,frame);
+          ViewElemArrayPurge(G,I->ViewElem+frame,1);
           UtilZeroMem((void*)(I->ViewElem+frame),sizeof(CViewElem));
         }
       }
@@ -673,49 +788,118 @@ int MovieView(PyMOLGlobals *G,int action,int first,int last,float power,float bi
   case 3:
     {
       CViewElem *first_view=NULL,*last_view=NULL;
+      int zero_flag = -1;
       if(first<0)
         first = 0;
-      if(last<0)
-        last = SceneGetNFrame(G)-1;
+
+      if(first>I->NFrame) {
+        first = I->NFrame-1;
+      }
+
+      /* note that we're leaving a blank frame at the end... */
+
+      if(last<0) {
+        last = SceneGetNFrame(G,NULL);
+        if(last && (!wrap))
+          last--;
+      }
+      if(last>=I->NFrame) {
+        last = I->NFrame;
+        if(last && (!wrap))
+          last--;
+      }
 
       VLACheck(I->ViewElem,CViewElem,last);
-      if(action==2) {
-        PRINTFB(G,FB_Movie,FB_Details)
-          " MovieView: interpolating unspecified frames %d to %d.\n",first+1,last+1
-          ENDFB(G);
-      } else {
-        PRINTFB(G,FB_Movie,FB_Details)
-          " MovieView: reinterpolating all frames %d to %d.\n",first+1,last+1
-          ENDFB(G);
+      
+      if(wrap && (last == I->NFrame)) {
+        /* if we're interpolating beyond the
+           last frame, then wrap by copying
+           first to last */
+        ViewElemCopy(G,I->ViewElem, I->ViewElem+last);
+        zero_flag = last;
       }
-      for(frame=first;frame<=last;frame++) {
-        if((frame>=0)&&(frame<I->NFrame)) {
-          if(!first_view) {
-            if(I->ViewElem[frame].specification_level==2) { /* specified */
-              first_view = I->ViewElem + frame;
-            }
+
+      if(!quiet) {
+        if(action==2) {
+          if(last == I->NFrame) {
+            PRINTFB(G,FB_Movie,FB_Details)
+              " MovieView: interpolating unspecified frames %d to %d (wrapping)\n",first+1,last
+              ENDFB(G);
           } else {
-            CViewElem *view;
-            int interpolate_flag = false;
-            if(I->ViewElem[frame].specification_level==2) { /* specified */
-              last_view = I->ViewElem + frame;
-              if(action==2) {/* interpolate */
-                for(view=first_view+1;view<last_view;view++) {
-                  if(!view->specification_level)
-                    interpolate_flag = true;
-                }
-              } else {
-                interpolate_flag=true;
-              }
-              if(interpolate_flag) {
-                ViewElemInterpolate(first_view,last_view,power,bias);
-              }
-              first_view = last_view;
-              last_view = NULL;
-            }
+            PRINTFB(G,FB_Movie,FB_Details)
+              " MovieView: interpolating unspecified frames %d to %d.\n",first+1,last+1
+              ENDFB(G);
+          }
+          
+        } else {
+          if(last == I->NFrame) {
+            PRINTFB(G,FB_Movie,FB_Details)
+              " MovieView: reinterpolating all frames %d to %d (wrapping).\n",first+1,last
+              ENDFB(G);
+          } else {
+            PRINTFB(G,FB_Movie,FB_Details)
+              " MovieView: reinterpolating all frames %d to %d.\n",first+1,last+1
+              ENDFB(G);
           }
         }
       }
+      for(frame=first;frame<=last;frame++) {
+        if(!first_view) {
+          if(I->ViewElem[frame].specification_level==2) { /* specified */
+            first_view = I->ViewElem + frame;
+          }
+        } else {
+          CViewElem *view;
+          int interpolate_flag = false;
+          if(I->ViewElem[frame].specification_level==2) { /* specified */
+            last_view = I->ViewElem + frame;
+            if(action==2) {/* interpolate */
+              for(view=first_view+1;view<last_view;view++) {
+                if(!view->specification_level)
+                  interpolate_flag = true;
+              }
+            } else {
+              interpolate_flag=true;
+            }
+            if(interpolate_flag) {
+              ViewElemInterpolate(G,first_view,last_view,
+                                  power,bias,simple,linear,hand,scene_cut);
+            }
+            first_view = last_view;
+            last_view = NULL;
+          }
+        }
+      }
+      if(zero_flag>=0) { /* erase temporary view */
+        UtilZeroMem((void*)(I->ViewElem + last), sizeof(CViewElem));
+      }
+    }
+    break;
+  case 4: /* smooth */
+   {
+      if(first<0)
+        first = 0;
+
+      if(last<0) {
+        last = SceneGetNFrame(G,NULL)-1;
+      }
+      if(last>=I->NFrame) {
+        last = I->NFrame-1;
+      }
+      if(first<=last) {
+        int a;
+        VLACheck(I->ViewElem,CViewElem,last);
+        for(a=0;a<cycles;a++) {
+          ViewElemSmooth(I->ViewElem+first, I->ViewElem + last, window,wrap);
+        }
+      }
+   }
+   break;
+  case 5: /* reset */
+    if(I->ViewElem) {
+      int size = VLAGetSize(I->ViewElem);
+      VLAFreeP(I->ViewElem);
+      I->ViewElem = VLACalloc(CViewElem, size);
     }
     break;
   }
@@ -743,7 +927,7 @@ void MovieAppendCommand(PyMOLGlobals *G,int frame,char *command)
   }
 }
 /*========================================================================*/
-ImageType MovieGetImage(PyMOLGlobals *G,int index)
+ImageType *MovieGetImage(PyMOLGlobals *G,int index)
 {
   register CMovie *I=G->Movie;
   if((index>=0)&&(index<I->NImage))
@@ -783,12 +967,14 @@ void MovieClearImages(PyMOLGlobals *G)
   for(a=0;a<I->NImage;a++)
 	 {
 		if(I->Image[a]) {
+		  FreeP(I->Image[a]->data);
 		  FreeP(I->Image[a]);
 		  I->Image[a]=NULL;
 		}
 	 }
   I->NImage=0;
-  SceneDirty(G);
+  SceneInvalidate(G);
+  SceneSuppressMovieFrame(G);
 }
 /*========================================================================*/
 void MovieReset(PyMOLGlobals *G) {
