@@ -22,7 +22,6 @@ Z* -------------------------------------------------------------------
 #include"Scene.h"
 #include"View.h"
 
-typedef unsigned char *ImageType;
 typedef char MovieCmdType[OrthoLineLength];
 
 int MovieFromPyList(PyMOLGlobals *G,PyObject *list,int *warning);
@@ -33,7 +32,7 @@ void MovieFree(PyMOLGlobals *G);
 void MovieReset(PyMOLGlobals *G);
 void MovieDump(PyMOLGlobals *G);
 void MovieAppendSequence(PyMOLGlobals *G,char *seq,int start_from);
-int MoviePNG(PyMOLGlobals *G,char *prefix,int save,int start,int stop);
+int MoviePNG(PyMOLGlobals *G,char *prefix,int save,int start,int stop,int missing_only);
 void MovieSetCommand(PyMOLGlobals *G,int frame,char *command);
 void MovieAppendCommand(PyMOLGlobals *G,int frame,char *command);
 
@@ -41,6 +40,7 @@ void MovieDoFrameCommand(PyMOLGlobals *G,int frame);
 
 void MovieCopyPrepare(PyMOLGlobals *G,int *width,int *height,int *length);
 int MovieCopyFrame(PyMOLGlobals *G,int frame,int width,int height,int rowbytes,void *ptr);
+int MoviePurgeFrame(PyMOLGlobals *G,int frame);
 void MovieCopyFinish(PyMOLGlobals *G);
 
 #define cMovieStop 0
@@ -52,8 +52,8 @@ int MoviePlaying(PyMOLGlobals *G);
 void MovieSetSize(PyMOLGlobals *G,unsigned int width,unsigned int height);
 
 void MovieClearImages(PyMOLGlobals *G);
-ImageType MovieGetImage(PyMOLGlobals *G,int image);
-void MovieSetImage(PyMOLGlobals *G,int index,ImageType image);
+ImageType *MovieGetImage(PyMOLGlobals *G,int image);
+void MovieSetImage(PyMOLGlobals *G,int index,ImageType *image);
 
 int MovieGetLength(PyMOLGlobals *G);
 int MovieFrameToImage(PyMOLGlobals *G,int frame);
@@ -61,7 +61,11 @@ int MovieFrameToIndex(PyMOLGlobals *G,int frame);
 int MovieLocked(PyMOLGlobals *G);
 void MovieSetLock(PyMOLGlobals *G,int);
 int MovieDefined(PyMOLGlobals *G);
-int MovieView(PyMOLGlobals *G,int action,int first,int last,float power,float bias);
+int MovieView(PyMOLGlobals *G,int action,int first,
+              int last,float power,float bias,
+              int simple, float linear,int wrap,
+              int hand,int window,int cycles,
+              char *scene_name, float scene_cut, int quiet);
 void MovieFlushCommands(PyMOLGlobals *G);
 
 #define cMovieMatrixClear  0
