@@ -60,7 +60,6 @@ Z* -------------------------------------------------------------------
 #define ButModeMargin 20
 #define ControlMargin 0
 
-
 struct _COrtho {
   Block *Blocks;
   Block *GrabbedBy,*ClickedIn;
@@ -81,7 +80,7 @@ struct _COrtho {
   int SavedPC,SavedCC;
   float TextColor[3],OverlayColor[3],WizardBackColor[3],WizardTextColor[3];
   int DirtyFlag;
-  double BusyLast;
+  double BusyLast, BusyLastUpdate;
   int BusyStatus[4];
   char BusyMessage[255];
   char *WizardPromptVLA;
@@ -93,9 +92,14 @@ struct _COrtho {
   CQueue *feedback;
   int Pushed;
   CDeferred *deferred;
+  int RenderMode;
+  GLint ViewPort[4];
+  int WrapXFlag;
+  GLenum ActiveGLBuffer;
+  double DrawTime, LastDraw;
+
 };
 
-static void OrthoBusyDraw(PyMOLGlobals *G,int force);
 
 void OrthoParseCurrentLine(PyMOLGlobals *G);
 static void OrthoDrawWizardPrompt(PyMOLGlobals *G);
@@ -116,6 +120,42 @@ void OrthoKeyAlt(PyMOLGlobals *G,unsigned char k);
 #define cWizardLeftMargin 15
 #define cWizardBorder 7
 
+static int get_wrap_x(int x, int *last_x, int width)
+{
+  int width_2 = width/2;
+  int width_3 = width/3;
+  if(!last_x) {
+    if(x>width_2)
+      x-=width_2;
+  } else {
+    if((x-(*last_x))>width_3)
+      x-=width_2;
+    else if(((*last_x)-x)>width_3)
+      x+=width_2;
+  }
+  return x;
+}
+
+void OrthoDrawBuffer(PyMOLGlobals *G,GLenum mode)
+{
+  register COrtho *I=G->Ortho;
+  if((mode!=I->ActiveGLBuffer) && G->HaveGUI && G->ValidContext) {  
+    glDrawBuffer(mode);
+    I->ActiveGLBuffer = mode;
+  }
+}
+
+int OrthoGetDirty(PyMOLGlobals *G)
+{
+  register COrtho *I=G->Ortho;
+  return I->DirtyFlag;
+}
+int OrthoGetRenderMode(PyMOLGlobals *G)
+{
+  register COrtho *I=G->Ortho;
+  return I->RenderMode;
+}
+
 void OrthoSetLoopRect(PyMOLGlobals *G,int flag, BlockRect *rect)
 {
   register COrtho *I=G->Ortho;
@@ -124,6 +164,12 @@ void OrthoSetLoopRect(PyMOLGlobals *G,int flag, BlockRect *rect)
   OrthoDirty(G);
 }
 
+int OrthoDeferredWaiting(PyMOLGlobals *G)
+{
+  register COrtho *I=G->Ortho;
+  return (I->deferred!=NULL);
+}
+
 void OrthoExecDeferred(PyMOLGlobals *G)
 {
   register COrtho *I=G->Ortho;
@@ -149,10 +195,21 @@ void OrthoDefer(PyMOLGlobals *G,CDeferred *D)
 
 int OrthoGetWidth(PyMOLGlobals *G)
 {
-  register COrtho *I=G->Ortho;
-  return(I->Width);
+  if(G) {
+    register COrtho *I=G->Ortho;
+    return(I->Width);
+  }
+  return 0;
 }
 
+int OrthoGetHeight(PyMOLGlobals *G)
+{
+  if(G) {
+    register COrtho *I=G->Ortho;
+    return(I->Height);
+  }
+  return 0;
+}
 /*========================================================================*/
 void OrthoFakeDrag(PyMOLGlobals *G) /* for timing-based events, such as pop-ups */
 {
@@ -251,13 +308,16 @@ void  OrthoRemoveSplash(PyMOLGlobals *G)
 /*========================================================================*/
 int  OrthoCommandOut(PyMOLGlobals *G,char *buffer)
 {
-  register COrtho *I=G->Ortho;
-  if(I->cmds) {
-    int result;
-    result = QueueStrOut(I->cmds,buffer);
-    return(result);
+  if(G&&buffer) {
+    register COrtho *I=G->Ortho;
+    if(I && I->cmds) {
+      int result;
+      result = QueueStrOut(I->cmds,buffer);
+      return(result);
+    } else
+      return 0;
   } else
-	return(0);
+    return 0;
 }
 /*========================================================================*/
 int  OrthoCommandWaiting(PyMOLGlobals *G)
@@ -316,46 +376,57 @@ void OrthoBusyMessage(PyMOLGlobals *G,char *message)
 /*========================================================================*/
 void OrthoBusySlow(PyMOLGlobals *G,int progress,int total)
 {
-  register COrtho *I=G->Ortho;
-  PRINTFD(G,FB_Ortho)
-    " OrthoBusySlow-DEBUG: progress %d total %d\n",progress,total
-    ENDFD;
-  I->BusyStatus[0]=progress;
-  I->BusyStatus[1]=total;
-  if(PyMOL_GetBusy(G->PyMOL,false)) { /* harmless race condition */
+	register COrtho *I=G->Ortho;
+    double   time_yet = (-I->BusyLastUpdate) + UtilGetSeconds(G);
+
+	PRINTFD(G,FB_Ortho)
+		" OrthoBusySlow-DEBUG: progress %d total %d\n",progress,total
+		ENDFD;
+	I->BusyStatus[0]=progress;
+	I->BusyStatus[1]=total;
+	if(SettingGetGlobal_b(G,cSetting_show_progress)&&(time_yet>0.15F)) {
+		if(PyMOL_GetBusy(G->PyMOL,false)) { /* harmless race condition */
 #ifndef _PYMOL_NOPY
-    int blocked = PAutoBlock();
-    PLockStatus();
+			int blocked = PAutoBlock(G);
+			if(PLockStatusAttempt(G)) {
 #endif
-    PyMOL_SetProgress(G->PyMOL,PYMOL_PROGRESS_SLOW,progress,total);
+			PyMOL_SetProgress(G->PyMOL,PYMOL_PROGRESS_SLOW,progress,total);
+            I->BusyLastUpdate = UtilGetSeconds(G);
+
 #ifndef _PYMOL_NOPY
-    PUnlockStatus();
-    PAutoUnblock(blocked);
+			PUnlockStatus(G);
+            }
+			PAutoUnblock(G,blocked);
 #endif
-  }
-  OrthoBusyDraw(G,false);
+		}
+	}
+	OrthoBusyDraw(G,false);
 }
 /*========================================================================*/
 void OrthoBusyFast(PyMOLGlobals *G,int progress,int total)
 {
-  register COrtho *I=G->Ortho;
-  PRINTFD(G,FB_Ortho)
-    " OrthoBusyFast-DEBUG: progress %d total %d\n",progress,total
-    ENDFD;
-  I->BusyStatus[2]=progress;
-  I->BusyStatus[3]=total;
-  if(PyMOL_GetBusy(G->PyMOL,false)) { /* harmless race condition */
+	register COrtho *I=G->Ortho;
+    double   time_yet = (-I->BusyLastUpdate) + UtilGetSeconds(G);
+	PRINTFD(G,FB_Ortho)
+		" OrthoBusyFast-DEBUG: progress %d total %d\n",progress,total
+		ENDFD;
+	I->BusyStatus[2]=progress;
+	I->BusyStatus[3]=total;
+	if(SettingGetGlobal_b(G,cSetting_show_progress)&&(time_yet>0.15F)) {
+		if(PyMOL_GetBusy(G->PyMOL,false)) { /* harmless race condition */
 #ifndef _PYMOL_NOPY
-    int blocked = PAutoBlock();
-    PLockStatus();
+			int blocked = PAutoBlock(G);
+			if(PLockStatusAttempt(G)) {
 #endif
-    PyMOL_SetProgress(G->PyMOL,PYMOL_PROGRESS_FAST,progress,total);
+			PyMOL_SetProgress(G->PyMOL,PYMOL_PROGRESS_FAST,progress,total);
+            I->BusyLastUpdate = UtilGetSeconds(G);
 #ifndef _PYMOL_NOPY
-    PUnlockStatus();
-    PAutoUnblock(blocked);
+			PUnlockStatus(G);
+            }
+			PAutoUnblock(G,blocked);
 #endif
-  }
-
+		}
+	}
   OrthoBusyDraw(G,false);
 }
 /*========================================================================*/
@@ -367,9 +438,17 @@ void OrthoBusyPrime(PyMOLGlobals *G)
 	 I->BusyStatus[a]=0;
   I->BusyMessage[0]=0;
   I->BusyLast = UtilGetSeconds(G);
+  I->BusyLastUpdate = UtilGetSeconds(G);
 }
+
+/* BEGIN PROPRIETARY CODE SEGMENT (see disclaimer in "os_proprietary.h") */ 
+#ifdef _MACPYMOL_XCODE
+void MacPyMOLSetProgress(float value);
+#endif
+/* END PROPRIETARY CODE SEGMENT */
+
 /*========================================================================*/
-static void OrthoBusyDraw(PyMOLGlobals *G,int force)
+void OrthoBusyDraw(PyMOLGlobals *G,int force)
 {
   register COrtho *I=G->Ortho;
   double now;
@@ -384,85 +463,122 @@ static void OrthoBusyDraw(PyMOLGlobals *G,int force)
     
     I->BusyLast=now;
     if(PIsGlutThread()) {
+
+#ifdef _MACPYMOL_XCODE
+/* BEGIN PROPRIETARY CODE SEGMENT (see disclaimer in "os_proprietary.h") */ 
+      float busyValue;
+      if(I->BusyStatus[1]) {
+        busyValue=(I->BusyStatus[0]*1.0F/I->BusyStatus[1]);
+      }
+      if(I->BusyStatus[3]) {
+        busyValue=(I->BusyStatus[2]*1.0F/I->BusyStatus[3]);
+      }
+      MacPyMOLSetProgress(busyValue);
+/* END PROPRIETARY CODE SEGMENT */
+#else
       if(G->HaveGUI && G->ValidContext) {      
         char *c;
         int x,y;
         float black[3] = {0,0,0};
         float white[3] = {1,1,1};
-
+		int draw_both = SceneMustDrawBoth(G);
         OrthoPushMatrix(G);
+
+        {
+          int pass = 0;
+          glClear(GL_DEPTH_BUFFER_BIT);
+          while(1) {
+            if(draw_both) {
+              if(!pass) 
+                OrthoDrawBuffer(G,GL_FRONT_LEFT); 
+              else
+                OrthoDrawBuffer(G,GL_FRONT_RIGHT);
+            } else {
+              OrthoDrawBuffer(G,GL_FRONT); /* draw into the front buffer */
+            }
         
-        glDrawBuffer(GL_FRONT);
-        glClear(GL_DEPTH_BUFFER_BIT);
         
-        glColor3fv(black);
-        glBegin(GL_POLYGON);
-        glVertex2i(0,I->Height);
-        glVertex2i(cBusyWidth,I->Height);
-        glVertex2i(cBusyWidth,I->Height-cBusyHeight);
-        glVertex2i(0,I->Height-cBusyHeight);
-        glVertex2i(0,I->Height); /* needed on old buggy Mesa */
-        glEnd();
+            glColor3fv(black);
+            glBegin(GL_POLYGON);
+            glVertex2i(0,I->Height);
+            glVertex2i(cBusyWidth,I->Height);
+            glVertex2i(cBusyWidth,I->Height-cBusyHeight);
+            glVertex2i(0,I->Height-cBusyHeight);
+            glVertex2i(0,I->Height); /* needed on old buggy Mesa */
+            glEnd();
+            
+            glColor3fv(white);	 
+            
+            y=I->Height-cBusyMargin;
+            c=I->BusyMessage;
+            if(*c) {
+              TextSetColor(G,white);
+              TextSetPos2i(G,cBusyMargin,y-(cBusySpacing/2));
+              TextDrawStr(G,c);
+              y-=cBusySpacing;
+            }
+            
+            if(I->BusyStatus[1]) {
+              glBegin(GL_LINE_LOOP);
+              glVertex2i(cBusyMargin,y);
+              glVertex2i(cBusyWidth-cBusyMargin,y);
+              glVertex2i(cBusyWidth-cBusyMargin,y-cBusyBar);
+              glVertex2i(cBusyMargin,y-cBusyBar);
+              glVertex2i(cBusyMargin,y); /* needed on old buggy Mesa */
+              glEnd();
+              glColor3fv(white);	 
+              glBegin(GL_POLYGON);
+              glVertex2i(cBusyMargin,y);
+              x=(I->BusyStatus[0]*(cBusyWidth-2*cBusyMargin)/I->BusyStatus[1])+cBusyMargin;
+              glVertex2i(x,y);
+              glVertex2i(x,y-cBusyBar);
+              glVertex2i(cBusyMargin,y-cBusyBar);
+              glVertex2i(cBusyMargin,y); /* needed on old buggy Mesa */
+              glEnd();
+              y-=cBusySpacing;
+            }
+            
+            if(I->BusyStatus[3]) {
+              glColor3fv(white);	 
+              glBegin(GL_LINE_LOOP);
+              glVertex2i(cBusyMargin,y);
+              glVertex2i(cBusyWidth-cBusyMargin,y);
+              glVertex2i(cBusyWidth-cBusyMargin,y-cBusyBar);
+              glVertex2i(cBusyMargin,y-cBusyBar);
+              glVertex2i(cBusyMargin,y); /* needed on old buggy Mesa */
+              glEnd();
+              x=(I->BusyStatus[2]*(cBusyWidth-2*cBusyMargin)/I->BusyStatus[3])+cBusyMargin;
+              glColor3fv(white);	 
+              glBegin(GL_POLYGON);
+              glVertex2i(cBusyMargin,y);
+              glVertex2i(x,y);
+              glVertex2i(x,y-cBusyBar);
+              glVertex2i(cBusyMargin,y-cBusyBar);
+              glVertex2i(cBusyMargin,y); /* needed on old buggy Mesa */
+              glEnd();
+              y-=cBusySpacing;
+            }
+            
+            if(!draw_both)
+              break;
+            if(pass>1)
+              break;
+            pass++;
+          }
         
-        glColor3fv(white);	 
-        
-        y=I->Height-cBusyMargin;
-        c=I->BusyMessage;
-        if(*c) {
-          TextSetColor(G,white);
-          TextSetPos2i(G,cBusyMargin,y-(cBusySpacing/2));
-          TextDrawStr(G,c);
-          y-=cBusySpacing;
+          glFlush();
+          glFinish();
+		  
+          if(draw_both)
+            OrthoDrawBuffer(G,GL_BACK_LEFT);
+          else
+            OrthoDrawBuffer(G,GL_BACK);      
         }
-        
-        if(I->BusyStatus[1]) {
-          glBegin(GL_LINE_LOOP);
-          glVertex2i(cBusyMargin,y);
-          glVertex2i(cBusyWidth-cBusyMargin,y);
-          glVertex2i(cBusyWidth-cBusyMargin,y-cBusyBar);
-          glVertex2i(cBusyMargin,y-cBusyBar);
-          glVertex2i(cBusyMargin,y); /* needed on old buggy Mesa */
-          glEnd();
-          glColor3fv(white);	 
-          glBegin(GL_POLYGON);
-          glVertex2i(cBusyMargin,y);
-          x=(I->BusyStatus[0]*(cBusyWidth-2*cBusyMargin)/I->BusyStatus[1])+cBusyMargin;
-          glVertex2i(x,y);
-          glVertex2i(x,y-cBusyBar);
-          glVertex2i(cBusyMargin,y-cBusyBar);
-          glVertex2i(cBusyMargin,y); /* needed on old buggy Mesa */
-          glEnd();
-          y-=cBusySpacing;
-        }
-        
-        if(I->BusyStatus[3]) {
-          glColor3fv(white);	 
-          glBegin(GL_LINE_LOOP);
-          glVertex2i(cBusyMargin,y);
-          glVertex2i(cBusyWidth-cBusyMargin,y);
-          glVertex2i(cBusyWidth-cBusyMargin,y-cBusyBar);
-          glVertex2i(cBusyMargin,y-cBusyBar);
-          glVertex2i(cBusyMargin,y); /* needed on old buggy Mesa */
-          glEnd();
-          x=(I->BusyStatus[2]*(cBusyWidth-2*cBusyMargin)/I->BusyStatus[3])+cBusyMargin;
-          glColor3fv(white);	 
-          glBegin(GL_POLYGON);
-          glVertex2i(cBusyMargin,y);
-          glVertex2i(x,y);
-          glVertex2i(x,y-cBusyBar);
-          glVertex2i(cBusyMargin,y-cBusyBar);
-          glVertex2i(cBusyMargin,y); /* needed on old buggy Mesa */
-          glEnd();
-          y-=cBusySpacing;
-        }
-        
-        glFlush();
-        glFinish();
-        glDrawBuffer(GL_BACK);
-        
         OrthoPopMatrix(G);
-        OrthoDirty(G);/* switched from SceneDirty */
+        OrthoDirty(G);
       }
+#endif               
+
     }
   }
 
@@ -510,9 +626,9 @@ void OrthoKeyControl(PyMOLGlobals *G,unsigned char k) {
   /* safer...*/
 
   sprintf(buffer,"cmd._ctrl(chr(%d))",k);
-  PLog(buffer,cPLog_pym);
-  PParse(buffer);
-  PFlush();
+  PLog(G,buffer,cPLog_pym);
+  PParse(G,buffer);
+  PFlush(G);
 
 }
 /*========================================================================*/
@@ -521,15 +637,16 @@ void OrthoKeyAlt(PyMOLGlobals *G,unsigned char k) {
 
   /* safer...*/
 
-  sprintf(buffer,"cmd._alt(chr(%d))",k);
-  PLog(buffer,cPLog_pym);
-  PParse(buffer);
-  PFlush();
-
-  /*  PBlockAndUnlockAPI();
-      sprintf(buffer,"cmd._ctrl('%c')",k+64);
-      PRunString(buffer);
-      PLockAPIAndUnblock(); */
+  if(k=='@') { 
+    /* option G produces '@' on some non-US keyboards, so simply
+       ignore the modifier */
+    OrthoKey(G,k,0,0,0);
+  } else {
+    sprintf(buffer,"cmd._alt(chr(%d))",k);
+    PLog(G,buffer,cPLog_pym);
+    PParse(G,buffer);
+    PFlush(G);
+  }
 
 }
 /*========================================================================*/
@@ -663,7 +780,7 @@ void OrthoKey(PyMOLGlobals *G,unsigned char k,int x,int y,int mod)
         curLine=I->CurLine&OrthoSaveLines;
         if(I->PromptChar) {
           strcpy(buffer,I->Line[curLine]);
-          if(PComplete(buffer+I->PromptChar,
+          if(PComplete(G,buffer+I->PromptChar,
                     sizeof(OrthoLineType)-I->PromptChar)); /* just print, don't complete */
         }
       }
@@ -676,7 +793,7 @@ void OrthoKey(PyMOLGlobals *G,unsigned char k,int x,int y,int mod)
         if(I->PromptChar) {
           strcpy(buffer,I->Line[curLine]);
           
-          if(PComplete(buffer+I->PromptChar,
+          if(PComplete(G,buffer+I->PromptChar,
                        sizeof(OrthoLineType)-I->PromptChar))
             {
               OrthoRestorePrompt(G);
@@ -713,9 +830,11 @@ void OrthoKey(PyMOLGlobals *G,unsigned char k,int x,int y,int mod)
       }
       break;
     case 22: /* CTRL V -- paste */
-      PBlockAndUnlockAPI();
-      PRunString("cmd.paste()");
-      PLockAPIAndUnblock();
+#ifndef _PYMOL_NOPY
+      PBlockAndUnlockAPI(G);
+      PRunStringInstance(G,"cmd.paste()");
+      PLockAPIAndUnblock(G);
+#endif
       break;
 	 default:
       OrthoKeyControl(G,(unsigned char)(k+64));
@@ -740,10 +859,10 @@ void OrthoParseCurrentLine(PyMOLGlobals *G)
       I->History[I->HistoryLine][0]=0;
       I->HistoryView=I->HistoryLine;
       if(WordMatch(G,buffer,"quit",true)==0) /* don't log quit */
-        PLog(buffer,cPLog_pml);
+        PLog(G,buffer,cPLog_pml);
       OrthoNewLine(G,NULL,true);
       OrthoDirty(G); /* this will force a redraw, if necessary */
-      PParse(buffer);
+      PParse(G,buffer);
       OrthoRestorePrompt(G);
     }
 #endif
@@ -906,7 +1025,22 @@ void OrthoDetach(PyMOLGlobals *G,Block *block)
   ListDetach(I->Blocks,block,next,Block);
 }
 /*========================================================================*/
-void OrthoDoDraw(PyMOLGlobals *G)
+/* BEGIN PROPRIETARY CODE SEGMENT (see disclaimer in "os_proprietary.h") */ 
+#ifdef PYMOL_EVAL
+#include "OrthoEvalMessage.h"
+#endif
+#ifdef JYMOL_EVAL
+#include "OrthoJyMolEvalMessage.h"
+#endif
+#ifdef PYMOL_EDU
+#include "OrthoEduMessage.h"
+#endif
+#ifdef PYMOL_COLL
+#include "OrthoCollMessage.h"
+#endif
+/* END PROPRIETARY CODE SEGMENT */
+
+void OrthoDoDraw(PyMOLGlobals *G,int render_mode)
 {
   register COrtho *I=G->Ortho;
 
@@ -925,7 +1059,7 @@ void OrthoDoDraw(PyMOLGlobals *G)
   int skip_prompt = 0;
   int render = false;
 
-  
+  I->RenderMode = render_mode;
   if(SettingGetGlobal_b(G,cSetting_seq_view)) {
     SeqUpdate(G); 
     I->HaveSeqViewer = true;
@@ -955,7 +1089,7 @@ void OrthoDoDraw(PyMOLGlobals *G)
 
     if(Feedback(G,FB_OpenGL,FB_Debugging))
       PyMOLCheckOpenGLErr("OrthoDoDraw checkpoint 0");
-
+    
     if(SettingGetGlobal_b(G,cSetting_internal_gui)) {
       switch(SettingGetGlobal_i(G,cSetting_internal_gui_mode)) {
       case 0:
@@ -980,45 +1114,59 @@ void OrthoDoDraw(PyMOLGlobals *G)
 
     if(text) overlay=0;
     
-    glClearColor(v[0],v[1],v[2],1.0);
+    {
+      float alpha = (SettingGetGlobal_b(G,cSetting_opaque_background) ? 1.0F : 0.0F);
+      glClearColor(v[0],v[1],v[2],alpha);
+    }
 
     if(overlay||(!text)) 
       if(!SceneRenderCached(G))
         render=true;
     
-    if((SceneGetStereo(G)==1)||double_pump) {
-      glDrawBuffer(GL_BACK_LEFT);
-      glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
-      glDrawBuffer(GL_BACK_RIGHT);
-      glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
-      glClearColor(0.0,0.0,0.0,1.0);
-      times = 2;
-      double_pump = true;
+    if(render_mode<2) {
+      if(SceneMustDrawBoth(G)) {
+        OrthoDrawBuffer(G,GL_BACK_LEFT);
+        glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
+        OrthoDrawBuffer(G,GL_BACK_RIGHT);
+        glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
+        times = 2;
+        double_pump = true;
+      } else {
+        OrthoDrawBuffer(G,GL_BACK);
+        glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
+        times = 1;
+        double_pump=false;
+      }    
     } else {
-      glDrawBuffer(GL_BACK);
-      glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
-      glClearColor(0.0,0.0,0.0,1.0);
       times = 1;
       double_pump=false;
-    }    
+    }
 
-    if(render)
-      SceneRender(G,NULL,0,0,NULL);
+    I->DrawTime = -I->LastDraw;
+    I->LastDraw = UtilGetSeconds(G);
+    I->DrawTime += I->LastDraw;
+    ButModeSetRate(G,(float)I->DrawTime); 
+    
+    if(render&&(render_mode<2))
+      SceneRender(G,NULL,0,0,NULL,0,0);
+
+    glClearColor(0.0,0.0,0.0,1.0);
     
     while(times--) {
+
       switch(times) {
       case 1:
-        glDrawBuffer(GL_BACK_RIGHT);
+        OrthoDrawBuffer(G,GL_BACK_LEFT);
+
         break;
       case 0:
         if(double_pump) {
-          glDrawBuffer(GL_BACK_LEFT);
+          OrthoDrawBuffer(G,GL_BACK_RIGHT);
         } else
-          glDrawBuffer(GL_BACK);
+          OrthoDrawBuffer(G,GL_BACK);
         break;
       }
 
-
       OrthoPushMatrix(G);
       
       x = I->X;
@@ -1076,9 +1224,11 @@ void OrthoDoDraw(PyMOLGlobals *G)
         lcount = 0;
         x = cOrthoLeftMargin;
         y = cOrthoBottomMargin;
-        
+
 #ifdef _PYMOL_SHARP3D
-        y+=(7*cOrthoLineHeight)/10;
+        if(SceneGetStereo(G)&&SettingGetGlobal_b(G,cSetting_overlay)) {
+          y+=(7*cOrthoLineHeight)/10;
+        }
 #endif
         if((int)SettingGet(G,cSetting_text)||I->SplashFlag)
           showLines=I->ShowLines;
@@ -1124,12 +1274,29 @@ void OrthoDoDraw(PyMOLGlobals *G)
       }
       
       OrthoDrawWizardPrompt(G);
+ 
+/* BEGIN PROPRIETARY CODE SEGMENT (see disclaimer in "os_proprietary.h") */ 
+#ifdef PYMOL_EVAL
+      OrthoDrawEvalMessage(G);
+#endif
+#ifdef JYMOL_EVAL
+      OrthoDrawEvalMessage(G);
+#endif
+#ifdef PYMOL_EDU
+      OrthoDrawEduMessage(G);
+#endif
+#ifdef PYMOL_COLL
+      OrthoDrawCollMessage(G);
+#endif
+/* END PROPRIETARY CODE SEGMENT */
+
+
       OrthoPopMatrix(G);
 
       if(Feedback(G,FB_OpenGL,FB_Debugging))
         PyMOLCheckOpenGLErr("OrthoDoDraw final checkpoint");
 
-    }
+    } /* while */
   }
 
   I->DirtyFlag =false;
@@ -1139,6 +1306,7 @@ void OrthoDoDraw(PyMOLGlobals *G)
 
 }
 /*========================================================================*/
+
 static void OrthoDrawWizardPrompt(PyMOLGlobals *G)
 {
   /* assumes PMGUI */
@@ -1275,6 +1443,13 @@ void OrthoReshape(PyMOLGlobals *G,int width, int height,int force)
     " OrthoReshape-Debug: %d %d\n",width,height
     ENDFD;
 
+  if((width>0)&&(SettingGetGlobal_i(G,cSetting_stereo_mode)==4)) {
+    width = width / 2;
+    I->WrapXFlag = true;
+  } else {
+    I->WrapXFlag = false;
+  }
+
   if((width!=I->Width)||(height!=I->Height)||force) {
   if(width<0) width=I->Width;
   if(height<0) height=I->Height;
@@ -1434,6 +1609,7 @@ Block *OrthoFindBlock(PyMOLGlobals *G,int x,int y)
 
   return(BlockRecursiveFind(I->Blocks,x,y));
 }
+
 /*========================================================================*/
 int OrthoButton(PyMOLGlobals *G,int button,int state,int x,int y,int mod)
 {
@@ -1442,6 +1618,22 @@ int OrthoButton(PyMOLGlobals *G,int button,int state,int x,int y,int mod)
   Block *block=NULL;
   int handled = 0; 
 
+
+  switch(button) {
+  case 3:
+  case 4:
+    block = SceneGetBlock(G);
+    break;
+  }
+  
+  if(I->WrapXFlag) {
+    if(state==P_GLUT_DOWN) {
+      x = get_wrap_x(x,NULL,G->Option->winX);
+    } else {
+      x = get_wrap_x(x,&I->LastX,G->Option->winX);
+    }
+  }
+
   OrthoRemoveSplash(G);
   I->X=x;
   I->Y=y;
@@ -1449,44 +1641,35 @@ int OrthoButton(PyMOLGlobals *G,int button,int state,int x,int y,int mod)
   I->LastY = y;
   I->LastModifiers = mod;
 
-  if(state==P_GLUT_DOWN)
-	 {
-		I->ActiveButton = button;
-		if(I->GrabbedBy)
-		  {
-			 if(I->GrabbedBy->inside)
-				block = BlockRecursiveFind(I->GrabbedBy->inside,x,y);
-			 else
-				block = I->GrabbedBy;
-		  }
-		else
-		  block = OrthoFindBlock(G,x,y);
-		if(block)
-		  {
-			 I->ClickedIn = block;
-			 if(block->fClick)
-				{
-				  handled = block->fClick(block,button,x,y,mod);
-				}
-		  }
-	 }
-  else if(state==P_GLUT_UP)
-	 {
-      if(I->GrabbedBy)
-        {
-			 block=I->GrabbedBy;
-			 if(block->fRelease)
-            handled = block->fRelease(block,button,x,y,mod);
-			 I->ClickedIn = NULL;
-        }
-		if(I->ClickedIn)
-		  {
-			 block=I->ClickedIn;
-			 if(block->fRelease)
-            handled = block->fRelease(block,button,x,y,mod);
-			 I->ClickedIn = NULL;
-		  }
-	 }
+  if(state==P_GLUT_DOWN) {
+    I->ActiveButton = button;
+    if(I->GrabbedBy) {
+      if(I->GrabbedBy->inside)
+        block = BlockRecursiveFind(I->GrabbedBy->inside,x,y);
+      else
+        block = I->GrabbedBy;
+    } else if(!block)
+      block = OrthoFindBlock(G,x,y);
+    if(block) {
+      I->ClickedIn = block;
+      if(block->fClick) {
+        handled = block->fClick(block,button,x,y,mod);
+      }
+    }
+  } else if(state==P_GLUT_UP) {
+    if(I->GrabbedBy) {
+      block=I->GrabbedBy;
+      if(block->fRelease)
+        handled = block->fRelease(block,button,x,y,mod);
+      I->ClickedIn = NULL;
+    }
+    if(I->ClickedIn) {
+      block=I->ClickedIn;
+      if(block->fRelease)
+        handled = block->fRelease(block,button,x,y,mod);
+      I->ClickedIn = NULL;
+    }
+  }
 #if 0
   if(block&&!handled) {
     if(SceneGetBlock(G)==block) {
@@ -1513,6 +1696,10 @@ int OrthoDrag(PyMOLGlobals *G,int x, int y,int mod)
   Block *block=NULL;
   int handled = 0;
 
+ if(I->WrapXFlag) {
+   x = get_wrap_x(x,&I->LastX,G->Option->winX);
+ }
+
   I->LastX = x;
   I->LastY = y;
   I->LastModifiers = mod;
@@ -1533,46 +1720,50 @@ int OrthoDrag(PyMOLGlobals *G,int x, int y,int mod)
 	 }
   return(handled);
 }
+
 /*========================================================================*/
 void OrthoSplash(PyMOLGlobals *G) 
 {
-  OrthoNewLine(G,NULL,true);
+/* BEGIN PROPRIETARY CODE SEGMENT (see disclaimer in "os_proprietary.h") */ 
+#ifdef _PYMOL_IP_SPLASH
+#include"OrthoIPSplash.h"
+#else
   if(G->Option->incentive_product) {
-    PRINTF " This PyMOL(TM) Incentive Product is Copyright (C) 2005 DeLano Scientific LLC.\n \n" ENDF(G);
-    PRINTF " A current PyMOL Maintenance and/or Support Subscription may be required for\n" ENDF(G);
-    PRINTF " legal usage of this Build beyond a finite honor-system evaluation period.\n" ENDF(G);
-    PRINTF " Please visit http://www.pymol.org/funding.html for more information.\n \n"ENDF(G);
-    PRINTF " This PyMOL Executable Build is based on Open-Source PyMOL version " ENDF(G);
-    PRINTF _PyMOL_VERSION ENDF(G);
-    PRINTF ".\n " ENDF(G);
-  } else {
-    PRINTF " PyMOL(TM) Molecular Graphics System, Version " ENDF(G);
+    PRINTF " PyMOL(TM) Incentive Product - Copyright (C) 2007 DeLano Scientific LLC.\n \n" ENDF(G);
+    PRINTF " This Executable Build integrates and extends Open-Source PyMOL " ENDF(G);
     PRINTF _PyMOL_VERSION ENDF(G);
     PRINTF ".\n" ENDF(G);
-    PRINTF " Copyright (C) 2005 by DeLano Scientific LLC.\n All Rights Reserved.\n \n" ENDF(G);
-    
-    PRINTF "    Created by Warren L. DeLano, Ph.D. \n \n" ENDF(G);
-    
-    PRINTF "    Other Major Authors and Contributors:\n\n" ENDF(G);
-    PRINTF "       Ralf W. Grosse-Kunstleve, Ph.D.\n \n" ENDF(G);
-    
-    PRINTF "    PyMOL is user-supported open-source software.  Although some versions\n" ENDF(G);
-    PRINTF "    are freely available, PyMOL is not in the public domain.\n \n" ENDF(G);
-    
-    PRINTF "    If PyMOL is helpful in your work or study, then please volunteer \n" ENDF(G);
-    PRINTF "    support for our ongoing efforts to create open and affordable scientific\n" ENDF(G);
-    PRINTF "    software by purchasing a PyMOL Maintenance and/or Support subscription.\n\n" ENDF(G);
-    
-    PRINTF "    More information can be found at \"http://www.pymol.org\".\n \n" ENDF(G);
-    
-    PRINTF "    Enter \"help\" for a list of commands.\n" ENDF(G);
-    PRINTF 
-      "    Enter \"help <command-name>\" for information on a specific command.\n\n"
-      ENDF(G);
-
-    PRINTF " Hit ESC anytime to toggle between text and graphics.\n\n" ENDF(G);
-  }
-
+  } else 
+/* END PROPRIETARY CODE SEGMENT */
+    {
+      /* Splash message for unrestricted access open-source versions... */
+      PRINTF " PyMOL(TM) Molecular Graphics System, Version " ENDF(G);
+      PRINTF _PyMOL_VERSION ENDF(G);
+      PRINTF ".\n" ENDF(G);
+      PRINTF " Copyright (C) 2007 by DeLano Scientific LLC.\n All Rights Reserved.\n \n" ENDF(G);
+      
+      PRINTF "    Created by Warren L. DeLano, Ph.D. \n \n" ENDF(G);
+      
+      PRINTF "    Other Major Authors and Contributors:\n\n" ENDF(G);
+      PRINTF "       Ralf W. Grosse-Kunstleve, Ph.D.\n \n" ENDF(G);
+      
+      PRINTF "    PyMOL is user-supported open-source software.  Although some versions\n" ENDF(G);
+      PRINTF "    are freely available, PyMOL is not in the public domain.\n \n" ENDF(G);
+      
+      PRINTF "    If PyMOL is helpful in your work or study, then please volunteer \n" ENDF(G);
+      PRINTF "    support for our ongoing efforts to create open and affordable scientific\n" ENDF(G);
+      PRINTF "    software by purchasing a PyMOL Maintenance and/or Support subscription.\n\n" ENDF(G);
+      
+      PRINTF "    More information can be found at \"http://www.pymol.org\".\n \n" ENDF(G);
+      
+      PRINTF "    Enter \"help\" for a list of commands.\n" ENDF(G);
+      PRINTF 
+        "    Enter \"help <command-name>\" for information on a specific command.\n\n"
+        ENDF(G);
+      
+      PRINTF " Hit ESC anytime to toggle between text and graphics.\n\n" ENDF(G);
+    }
+#endif
 }
 /*========================================================================*/
 int OrthoInit(PyMOLGlobals *G,int showSplash)
@@ -1584,10 +1775,12 @@ int OrthoInit(PyMOLGlobals *G,int showSplash)
 
   ListInit(I->Blocks);
 
-  I->Pushed = false;
+  I->Pushed = 0;
   I->cmds = QueueNew(G,0xFFFF); /* 65K for commands */
   I->feedback = QueueNew(G,0x3FFFF); /* ~256K for output */
   I->deferred = NULL;
+  I->RenderMode = 0;
+  I->WrapXFlag = false;
 
   I->WizardBackColor[0]=0.2F;
   I->WizardBackColor[1]=0.2F;
@@ -1619,12 +1812,14 @@ int OrthoInit(PyMOLGlobals *G,int showSplash)
   I->ShowLines = 1;
   I->Saved[0]=0;
   I->DirtyFlag = true;
-  
+  I->ActiveGLBuffer = GL_NONE;
+  I->LastDraw = UtilGetSeconds(G);
+  I->DrawTime = 0.0;
   if(showSplash) {
 	 OrthoSplash(G);
     I->SplashFlag=true;
   }
-  OrthoFeedbackIn(G," ");
+  /*  OrthoFeedbackIn(G," ");*/
   I->CurLine++;
   strcpy(I->Prompt,"PyMOL>");
   strcpy(I->Line[I->CurLine],I->Prompt);
@@ -1655,21 +1850,38 @@ void OrthoFree(PyMOLGlobals *G)
   I->cmds=NULL;
   QueueFree(I->feedback);
   I->feedback=NULL;
+  if(I->deferred) {
+    DeferredFree(I->deferred);
+    I->deferred = NULL;
+  }
   FreeP(G->Ortho);
 }
 /*========================================================================*/
 void OrthoPushMatrix(PyMOLGlobals *G)
 {
   register COrtho *I=G->Ortho;
-  GLint ViewPort[4];
 
   if(G->HaveGUI && G->ValidContext) {
 
-    glGetIntegerv(GL_VIEWPORT,ViewPort);
+    if(!I->Pushed) {
+      glGetIntegerv(GL_VIEWPORT,I->ViewPort);
+    }
+    switch(I->RenderMode) {
+    case 1:
+      glViewport(I->ViewPort[0],I->ViewPort[1],I->ViewPort[2],I->ViewPort[3]);
+      break;
+    case 2:
+      glViewport(I->ViewPort[0]+I->ViewPort[2],I->ViewPort[1],
+                 I->ViewPort[2],I->ViewPort[3]);
+      break;
+    default:
+      glViewport(I->ViewPort[0],I->ViewPort[1],I->ViewPort[2],I->ViewPort[3]);
+    }
+
     glMatrixMode(GL_PROJECTION);
     glPushMatrix();
     glLoadIdentity();
-    glOrtho(0,ViewPort[2],0,ViewPort[3],-100,100);
+    glOrtho(0,I->ViewPort[2],0,I->ViewPort[3],-100,100);
     glMatrixMode(GL_MODELVIEW);
     glPushMatrix();
     glLoadIdentity();
@@ -1685,10 +1897,12 @@ void OrthoPushMatrix(PyMOLGlobals *G)
     glDisable(GL_LINE_SMOOTH);
     glDisable(GL_DITHER);
     glDisable(GL_BLEND);
+	glShadeModel(GL_SMOOTH);
     if(G->Option->multisample)    
       glDisable(0x809D); /* GL_MULTISAMPLE_ARB */
-   }
-  I->Pushed=true;
+    
+    I->Pushed++;
+  }
   /*  glDisable(GL_ALPHA_TEST);
   glDisable(GL_CULL_FACE);
   glDisable(GL_POINT_SMOOTH);*/
@@ -1700,12 +1914,16 @@ void OrthoPopMatrix(PyMOLGlobals *G)
   register COrtho *I=G->Ortho;
   if(G->HaveGUI && G->ValidContext) {
 
-    glPopMatrix();
-    glMatrixMode(GL_PROJECTION);
-    glPopMatrix();
-    glMatrixMode(GL_MODELVIEW);
+    if(I->Pushed>=0) {
+      glViewport(I->ViewPort[0],I->ViewPort[1],I->ViewPort[2],I->ViewPort[3]);
+      glPopMatrix();
+      glMatrixMode(GL_PROJECTION);
+      glPopMatrix();
+      glMatrixMode(GL_MODELVIEW);
+      
+      I->Pushed--;
+    }
   }
-  I->Pushed=false;
 }
 
 int OrthoGetPushed(PyMOLGlobals *G)
