@@ -24,6 +24,7 @@ Z* -------------------------------------------------------------------
 #define cOrthoBottomSceneMargin 16
 #define cOrthoLineHeight 12
 
+#include"os_gl.h"
 #include"Block.h"
 #include"Feedback.h"
 #include"Deferred.h"
@@ -45,7 +46,8 @@ void OrthoFreeBlock(PyMOLGlobals *G,Block *block);
 
 void OrthoReshape(PyMOLGlobals *G,int width,int height,int force);
 int OrthoGetWidth(PyMOLGlobals *G);
-void OrthoDoDraw(PyMOLGlobals *G);
+int OrthoGetHeight(PyMOLGlobals *G);
+void OrthoDoDraw(PyMOLGlobals *G,int render_mode);
 
 void OrthoPushMatrix(PyMOLGlobals *G);
 void OrthoPopMatrix(PyMOLGlobals *G);
@@ -65,8 +67,10 @@ void OrthoUngrab(PyMOLGlobals *G);
 void OrthoSetLoopRect(PyMOLGlobals *G,int flag, BlockRect *rect);
 
 void OrthoRestorePrompt(PyMOLGlobals *G);
+void OrthoBusyDraw(PyMOLGlobals *G,int force);
 
 void OrthoDirty(PyMOLGlobals *G);
+int OrthoGetDirty(PyMOLGlobals *G);
 void OrthoWorking(PyMOLGlobals *G);
 void OrthoClear(PyMOLGlobals *G);
 void OrthoFakeDrag(PyMOLGlobals *G);
@@ -92,7 +96,13 @@ int OrthoTextVisible(PyMOLGlobals *G);
 void OrthoReshapeWizard(PyMOLGlobals *G,int height);
 void OrthoDefer(PyMOLGlobals *G,CDeferred *D);
 void OrthoExecDeferred(PyMOLGlobals *G);
+int OrthoDeferredWaiting(PyMOLGlobals *G);
+
 void OrthoSetLoop(PyMOLGlobals *G,int flag, int l, int r, int t, int b);
+int OrthoGetRenderMode(PyMOLGlobals *G);
+void OrthoDrawBuffer(PyMOLGlobals *G,GLenum mode);
+int OrthoGetWrapClickSide(PyMOLGlobals *G);
+
 #define OrthoLineLength 1024
 typedef char OrthoLineType[OrthoLineLength];
 
