@@ -58,6 +58,37 @@ int PConvPyListToExtent(PyObject *obj,float *mn,float *mx) /* [[min_x,min_y,min_
   return(ok);
 }
 
+int PConvPyListToStrVLAList(PyObject *obj,char **vla, int *n_str)
+{
+  int ok=false;
+  PyObject *t;
+  int n_st = 0, n_ch = 0, nn_ch, l, i;
+  if(!*vla)
+    *vla = VLAlloc(char,10);
+  if((!obj)||(!*vla)) {
+    ok=false;
+  } else if(PyList_Check(obj)) {
+    n_st = PyList_Size(obj);
+    ok = true;
+    for(i=0;i<n_st;i++) {
+      t = PyList_GetItem(obj,i);
+      if(PyString_Check(t)) {
+        l = PyString_Size(t);
+        nn_ch = n_ch+l+1;
+        VLACheck(*vla, char, nn_ch);
+        UtilNCopy((*vla)+n_ch,PyString_AsString(t),l+1);
+        n_ch = nn_ch;
+      } else {
+        VLACheck(*vla, char, n_ch+1);
+        (*vla)[n_ch] = 0;
+        n_ch++;
+      }
+    }
+  }
+  *n_str = n_st;
+  return(ok);
+}
+
 int PConvAttrToIntArrayInPlace(PyObject *obj,char *attr,int *f,int ll)
 {
   int ok=true;
@@ -133,6 +164,30 @@ int PConvCObjectToPtr(PyObject *obj,void **ptr) {
   return(ok);
 }
 
+int PConvPyStrToLexRef(PyObject *obj,OVLexicon *lex,int *lex_ref)
+{
+  int ok=true;
+  if(!obj) {
+    ok=false;
+  } else if (!PyString_Check(obj)) {
+    ok=false;
+  } else {
+    char *ptr = PyString_AsString(obj);
+    if(!ptr) {
+      ok=false;
+    } else {
+      OVreturn_word result = OVLexicon_GetFromCString(lex,ptr);
+      if(OVreturn_IS_OK(result)) {
+        *lex_ref = result.word;
+      } else {
+        ok=false;
+      }
+    }
+  }
+  return ok;
+}
+
+
 int PConvPyStrToStrPtr(PyObject *obj,char **ptr)
 {
   int ok=true;
@@ -167,9 +222,12 @@ int PConvPyIntToInt(PyObject *obj,int *ptr)
   if(!obj) {
     ok=false;
   } else if (!PyInt_Check(obj)) {
-    ok=false;
-  }
-  else {
+	  if(!PyLong_Check(obj)) {
+          ok=false;	
+	  } else {
+		  *ptr = (int)PyLong_AsLongLong(obj);
+	  }
+  } else {
     *ptr = PyInt_AsLong(obj);
   }
   return(ok);
@@ -194,7 +252,11 @@ int PConvPyIntToChar(PyObject *obj,char *ptr)
   if(!obj) {
     ok=false;
   } else if (!PyInt_Check(obj)) {
-    ok=false;
+    if(!PyLong_Check(obj)) {
+          ok=false;	
+	  } else {
+		  *ptr = (char)PyLong_AsLongLong(obj);
+	  }
   } else {
     *ptr = (char)PyInt_AsLong(obj);
   }
@@ -214,7 +276,9 @@ int PConvPyObjectToFloat(PyObject *object,float *value)
     (*value) = (float)PyFloat_AsDouble(object);
   } else if(PyInt_Check(object)) {
     (*value) = (float)PyInt_AsLong(object);
-  } else {
+  } else if(PyLong_Check(object)) {
+    (*value) = (float)PyLong_AsLongLong(object);
+  }else {
     tmp = PyNumber_Float(object);
     if(tmp) {
       (*value) = (float)PyFloat_AsDouble(tmp);
@@ -233,6 +297,8 @@ int PConvPyObjectToInt(PyObject *object,int *value)
     result=false;
   else   if(PyInt_Check(object)) {
     (*value) = (int)PyInt_AsLong(object);
+  } else if(PyLong_Check(object)) {
+    (*value) = (int)PyLong_AsLongLong(object);
   } else {
     tmp = PyNumber_Int(object);
     if(tmp) {
@@ -244,6 +310,27 @@ int PConvPyObjectToInt(PyObject *object,int *value)
   return(result);
 }
 
+int PConvPyObjectToChar(PyObject *object,char *value)
+{
+  int result = true;
+  PyObject *tmp;
+  if(!object)
+    result=false;
+  else   if(PyInt_Check(object)) {
+    (*value) = (char)PyInt_AsLong(object);
+  } else if(PyLong_Check(object)) {
+    (*value) = (char)PyLong_AsLongLong(object);
+  } else {
+    tmp = PyNumber_Int(object);
+    if(tmp) {
+      (*value) = (char)PyInt_AsLong(tmp);
+      Py_DECREF(tmp);
+    } else 
+      result=false;
+  }
+  return(result);
+}
+
 int PConvPyObjectToStrMaxLen(PyObject *object,char *value,int ln)
 {
   char *st;
@@ -697,6 +784,30 @@ int PConvPyListToSIntArrayInPlaceAutoZero(PyObject *obj,short int *ii,int ll)
   return(ok);
 }
 
+int PConvPyListToSCharArrayInPlaceAutoZero(PyObject *obj,signed char *ii,int ll)
+{
+  int ok = true;
+  int a,l;
+  if(!obj) 
+    ok=false;
+  else if(!PyList_Check(obj)) 
+    ok=false;
+  else {
+    l=PyList_Size(obj);
+    if(!l)
+      ok=-1;
+    else
+      ok=l;
+    for(a=0;(a<l)&&(a<ll);a++) 
+      *(ii++) = (signed char)PyInt_AsLong(PyList_GetItem(obj,a)); 
+    while(l<ll) {
+      *(ii++)=0;
+      l++;
+    }
+  }
+  return(ok);
+}
+
 int PConvPyListToFloatArrayInPlaceAutoZero(PyObject *obj,float *ii,int ll)
 {
   int ok = true;
@@ -799,6 +910,16 @@ PyObject *PConvSIntArrayToPyList(short int *f,int l)
   return(result);
 }
 
+PyObject *PConvSCharArrayToPyList(signed char *f,int l)
+{
+  int a;
+  PyObject *result = Py_None;
+  result=PyList_New(l);
+  for(a=0;a<l;a++) 
+    PyList_SetItem(result,a,PyInt_FromLong(*(f++)));
+  return(result);
+}
+
 PyObject *PConv3DIntArrayTo3DPyList(int ***array,int *dim)
 {
   int a,b,c;
@@ -847,7 +968,7 @@ PyObject *PConvStringVLAToPyList(char *vla)
     PyList_SetItem(result,a,PyString_FromString(p));
     while(*(p++));
   }
-  return(result);
+  return(PConvAutoNone(result));
 }
 
 int PConvPyListToStringVLA(PyObject *obj,char **vla_ptr)
@@ -882,6 +1003,69 @@ int PConvPyListToStringVLA(PyObject *obj,char **vla_ptr)
   return(vla&&1);
 }
 
+int PConvPyListToLabPosVLA(PyObject *obj,LabPosType **vla_ptr)
+{
+  int a,l;
+  int ok=true;
+  LabPosType *vla = NULL,*q;
+  PyObject *i;
+  if(obj) 
+    if(PyList_Check(obj)) {
+      l=PyList_Size(obj);
+      vla=VLACalloc(LabPosType,l);
+      q=vla;
+      for(a=0;a<l;a++) {
+        i = PyList_GetItem(obj,a);
+        if (PyList_Check(i) && (PyList_Size(i)==7)) {
+          ok = ok && PConvPyIntToInt(PyList_GetItem(i,0),&q->mode) && 
+            PConvPyFloatToFloat(PyList_GetItem(i,1),q->pos) && 
+            PConvPyFloatToFloat(PyList_GetItem(i,2),q->pos+1) && 
+            PConvPyFloatToFloat(PyList_GetItem(i,3),q->pos+2) && 
+            PConvPyFloatToFloat(PyList_GetItem(i,4),q->offset) && 
+            PConvPyFloatToFloat(PyList_GetItem(i,5),q->offset+1) &&
+            PConvPyFloatToFloat(PyList_GetItem(i,6),q->offset+2);
+        } else {
+          VLAFreeP(vla); /* just in case... */
+          vla = NULL;
+          break;
+        }
+        q++;
+      }
+    }
+  if(!ok&&(!vla)) {
+    VLAFreeP(vla);
+  }
+  (*vla_ptr)=vla;
+  return(ok);
+}
+
+PyObject *PConvLabPosVLAToPyList(LabPosType *vla,int l)
+{ /* TO DO error handling */
+  int a;
+  LabPosType *p = vla;
+  PyObject *result = Py_None;
+  if(p) {
+    PyObject *item;
+    result=PyList_New(l); 
+    for(a=0;a<l;a++) {
+      item = PyList_New(7);
+      if(item) {
+        PyList_SetItem(item,0,PyInt_FromLong(p->mode));
+        PyList_SetItem(item,1,PyFloat_FromDouble((double)p->pos[0]));
+        PyList_SetItem(item,2,PyFloat_FromDouble((double)p->pos[1]));
+        PyList_SetItem(item,3,PyFloat_FromDouble((double)p->pos[2]));
+        PyList_SetItem(item,4,PyFloat_FromDouble((double)p->offset[0]));
+        PyList_SetItem(item,5,PyFloat_FromDouble((double)p->offset[1]));
+        PyList_SetItem(item,6,PyFloat_FromDouble((double)p->offset[2]));
+        PyList_SetItem(result,a,item);
+      }
+      
+      p++;
+    }
+  }
+  return(PConvAutoNone(result));
+}
+
 void PConv44PyListTo44f(PyObject *src,float *dest) /* note lost of precision */
 {
   PyObject *row;
