@@ -26,13 +26,12 @@ Z* -------------------------------------------------------------------
 #include"P.h"
 /*========================================================================*/
 
-void RepRenderBox(struct Rep *this,CRay *ray,Pickable **pick);
 void RepInvalidate(struct Rep *I,struct CoordSet *cs,int level);
-struct Rep *RepUpdate(struct Rep *I,struct CoordSet *cs,int rep);
-struct Rep *RepRebuild(struct Rep *I,struct CoordSet *cs,int rep);
+struct Rep *RepUpdate(struct Rep *I,struct CoordSet *cs,int state,int rep);
+struct Rep *RepRebuild(struct Rep *I,struct CoordSet *cs,int state, int rep);
 
 /*========================================================================*/
-struct Rep *RepRebuild(struct Rep *I,struct CoordSet *cs,int rep)
+struct Rep *RepRebuild(struct Rep *I,struct CoordSet *cs,int state,int rep)
 {
   Rep *tmp = NULL;
 
@@ -41,7 +40,7 @@ struct Rep *RepRebuild(struct Rep *I,struct CoordSet *cs,int rep)
     ENDFD;
 
   if(I->fNew) {
-    tmp = I->fNew(cs);
+    tmp = I->fNew(cs,state);
     if(tmp) {
       tmp->fNew = I->fNew;
       I->fFree(I);
@@ -54,7 +53,7 @@ struct Rep *RepRebuild(struct Rep *I,struct CoordSet *cs,int rep)
   return(tmp);
 }
 /*========================================================================*/
-struct Rep *RepUpdate(struct Rep *I,struct CoordSet *cs,int rep)
+struct Rep *RepUpdate(struct Rep *I,struct CoordSet *cs,int state,int rep)
 {
 
   PRINTFD(I->G,FB_Rep)
@@ -74,16 +73,16 @@ struct Rep *RepUpdate(struct Rep *I,struct CoordSet *cs,int rep)
       if(I->fRecolor) {
         I->fRecolor(I,cs);
       } else {
-        I=I->fRebuild(I,cs,rep);
+        I=I->fRebuild(I,cs,state,rep);
       }
     } else if(I->MaxInvalid<=cRepInvVisib) {
       if(I->fSameVis) {
         if(!I->fSameVis(I,cs))
-          I=I->fRebuild(I,cs,rep);
+          I=I->fRebuild(I,cs,state,rep);
       } else 
-        I=I->fRebuild(I,cs,rep);
+        I=I->fRebuild(I,cs,state,rep);
     } else if(I->MaxInvalid>=cRepInvCoord) {
-      I=I->fRebuild(I,cs,rep);   
+      I=I->fRebuild(I,cs,state,rep);   
       if(!cs->Active[rep])
         {
           I->fFree(I);
@@ -95,7 +94,7 @@ struct Rep *RepUpdate(struct Rep *I,struct CoordSet *cs,int rep)
               I=tmp;
       */
     } else {
-      I=I->fRebuild(I,cs,rep);    
+      I=I->fRebuild(I,cs,state,rep);    
     }
     if(I)
       I->MaxInvalid=0;
@@ -105,7 +104,38 @@ struct Rep *RepUpdate(struct Rep *I,struct CoordSet *cs,int rep)
 /*========================================================================*/
 void RepInvalidate(struct Rep *I,struct CoordSet *cs,int level)
 {
-  if(level>I->MaxInvalid) I->MaxInvalid=level;
+  if(level > I->MaxInvalid) I->MaxInvalid=level;
+}
+/*========================================================================*/
+static void RepRenderBox(struct Rep *this,RenderInfo *info)
+{
+  register PyMOLGlobals *G = this->G;
+  if(G->HaveGUI && G->ValidContext) {
+    glBegin(GL_LINE_LOOP);
+    glVertex3f(-0.5F,-0.5F,-0.5F);
+    glVertex3f(-0.5F,-0.5F, 0.5F);
+    glVertex3f(-0.5F, 0.5F, 0.5F);
+    glVertex3f(-0.5F, 0.5F,-0.5F);
+    
+    glVertex3f( 0.5F, 0.5F,-0.5F);
+    glVertex3f( 0.5F, 0.5F, 0.5F);
+    glVertex3f( 0.5F,-0.5F, 0.5F);
+    glVertex3f( 0.5F,-0.5F,-0.5F);
+    glEnd();
+    
+    glBegin(GL_LINES);
+    glVertex3i(0,0,0);
+    glVertex3i(1,0,0);
+    
+    glVertex3i(0,0,0);
+    glVertex3i(0,2,0);
+    
+    glVertex3i(0,0,0);
+    glVertex3i(0,0,3);
+    
+    glEnd();
+  }
+
 }
 /*========================================================================*/
 void RepInit(PyMOLGlobals *G,Rep *I)
@@ -137,44 +167,13 @@ void RepPurge(Rep *I)
         }
       } else {
         char buffer[255]; /* pass this off to the main thread */
-        sprintf(buffer,"_cmd.gl_delete_lists(%d,%d)\n",I->displayList,1);
-        PParse(buffer);
+        sprintf(buffer,"_cmd.gl_delete_lists(cmd._COb,%d,%d)\n",I->displayList,1);
+        PParse(G,buffer);
       }
     }
   }
   FreeP(I->P);
 }
-/*========================================================================*/
-void RepRenderBox(struct Rep *this,CRay *ray,Pickable **pick)
-{
-  register PyMOLGlobals *G = this->G;
-  if(G->HaveGUI && G->ValidContext) {
-    glBegin(GL_LINE_LOOP);
-    glVertex3f(-0.5F,-0.5F,-0.5F);
-    glVertex3f(-0.5F,-0.5F, 0.5F);
-    glVertex3f(-0.5F, 0.5F, 0.5F);
-    glVertex3f(-0.5F, 0.5F,-0.5F);
-    
-    glVertex3f( 0.5F, 0.5F,-0.5F);
-    glVertex3f( 0.5F, 0.5F, 0.5F);
-    glVertex3f( 0.5F,-0.5F, 0.5F);
-    glVertex3f( 0.5F,-0.5F,-0.5F);
-    glEnd();
-    
-    glBegin(GL_LINES);
-    glVertex3i(0,0,0);
-    glVertex3i(1,0,0);
-    
-    glVertex3i(0,0,0);
-    glVertex3i(0,2,0);
-    
-    glVertex3i(0,0,0);
-    glVertex3i(0,0,3);
-    
-    glEnd();
-  }
-
-}
 
 
 
