@@ -31,6 +31,7 @@ Z* -------------------------------------------------------------------
 
 #define cCartoon_skip_helix -2
 #define cRepAll       -1
+#define cRepNone      -2
 
 /* WARNING: don't change these -- you'll break sessions!
    (you can add to them however, I think) */
@@ -54,7 +55,7 @@ Z* -------------------------------------------------------------------
 #define cRepSlice      16
 
 #define cRepAngle      17
-#define cRepDihedral    18
+#define cRepDihedral   18
 
 #define cRepCnt        19
 
@@ -63,14 +64,14 @@ Z* -------------------------------------------------------------------
  * These used to be used just for graphics, but are now
  * used by the molecular editor as well */
 
+/* precompted extents (can change if matrix changes) */
+#define cRepInvExtents 5
 /* invalidate external atom colors */
 #define cRepInvExtColor  10
 /* invalidate atom colors */
 #define cRepInvColor  15
 /* invalidate label text */
 #define cRepInvText   16
-/* invalidate picked atoms */
-#define cRepInvPick   19
 /* invalidate visible atoms */
 #define cRepInvVisib  20
 #define cRepInvVisib2 21
@@ -80,32 +81,40 @@ Z* -------------------------------------------------------------------
 #define cRepInvCoord  30
 /* invalidate graphic representation */
 #define cRepInvRep    35
+/* invalidate picked atoms */
+#define cRepInvPick   36
 /* invalidate bond structure */
 #define cRepInvBonds  40
 /* invalidate atomic structure */
 #define cRepInvAtoms  50
 /* invalidate everything about a structure */
 #define cRepInvAll    100
+/* invalidate and furthermore, purge existing representations */
+#define cRepInvPurge  110
 
 struct CoordSet;
 struct Object;
 
+
+
 typedef struct Rep {
   PyMOLGlobals *G;
-  void            (*fRender)(struct Rep *I,CRay *ray,Pickable **pick);  
-  struct Rep     *(*fUpdate)(struct Rep *I,struct CoordSet *cs,int rep);
+  void            (*fRender)(struct Rep *I,RenderInfo *info);
+  struct Rep     *(*fUpdate)(struct Rep *I,struct CoordSet *cs,int state,int rep);
   void        (*fInvalidate)(struct Rep *I,struct CoordSet *cs,int level);
   void              (*fFree)(struct Rep* I);
   int MaxInvalid,Active;
   struct CObject *obj;
   struct CoordSet *cs;
   Pickable *P;
+  PickContext context;
   /* private */
   void        (*fRecolor)(struct Rep *I,struct CoordSet *cs);
   int         (*fSameVis)(struct Rep *I,struct CoordSet *cs);
-  struct Rep *(*fRebuild)(struct Rep *I,struct CoordSet *cs,int rep);
-  struct Rep *(*fNew)(struct CoordSet *cs);
+  struct Rep *(*fRebuild)(struct Rep *I,struct CoordSet *cs,int state,int rep);
+  struct Rep *(*fNew)(struct CoordSet *cs,int state);
   int displayList;
+  int displayListInvalid;
 } Rep;
 
 void RepInit(PyMOLGlobals *G,Rep *I);
