@@ -26,9 +26,12 @@ typedef struct {
   int offset; 
   int atom_at; /* starting offset in list */
   int inverse;
+  int unaligned;
   int spacer;
   int state;
   int color;
+  int tag;
+  int is_abbr, hint_no_space;
 } CSeqCol;
   
 typedef struct {
@@ -36,41 +39,39 @@ typedef struct {
   int label_flag,column_label_flag;
   int color;
   char *txt;
-  CSeqCol *col;
-  int nCol;
+  CSeqCol *col,*fill;
+  int nCol,cCol;
+  int nFill;
   int *char2col;
   int *atom_lists;
-  char name[ObjNameMax]; /* associated object */
+  ObjectNameType name; /* associated object */
   struct ObjectMolecule *obj; /* this pointer only valid during update */
+  struct AtomInfoType *last_ai;
   int accum,current,title_width; /* temporary stores for aligning */
-
 } CSeqRow;
 
 typedef struct {
-  CSeqRow* (*fClick)   (CSeqRow* rowVLA,int button,int row,int col,int mod,int x,int y);
-  CSeqRow* (*fDrag)    (CSeqRow* rowVLA,int row,int col,int mod);
-  CSeqRow* (*fRelease) (CSeqRow* rowVLA,int button,int row,int col,int mod);
-  void (*fRefresh) (CSeqRow* rowVLA);
+  CSeqRow* (*fClick)   (PyMOLGlobals *G,CSeqRow* rowVLA,int button,int row,int col,int mod,int x,int y);
+  CSeqRow* (*fDrag)    (PyMOLGlobals *G,CSeqRow* rowVLA,int row,int col,int mod);
+  CSeqRow* (*fRelease) (PyMOLGlobals *G,CSeqRow* rowVLA,int button,int row,int col,int mod);
+  void (*fRefresh) (PyMOLGlobals *G,CSeqRow* rowVLA);
   int box_active,box_row;
   int box_start_col,box_stop_col;
 } CSeqHandler;
 
-void SeqInit(void);
-void SeqFree(void);
-Block *SeqGetBlock(void);
+int SeqInit(PyMOLGlobals *G);
+void SeqFree(PyMOLGlobals *G);
+Block *SeqGetBlock(PyMOLGlobals *G);
 
-void SeqInit(void);
-void SeqFree(void);
-Block *SeqGetBlock(void);
-int SeqIdling(void);
-void SeqInterrupt(void);
-int SeqGetHeight(void);
-void SeqSetHandler(CSeqHandler *handler);
-void SeqSetRowVLA(CSeqRow *row,int nRow);
+int SeqIdling(PyMOLGlobals *G);
+void SeqInterrupt(PyMOLGlobals *G);
+int SeqGetHeight(PyMOLGlobals *G);
+void SeqSetHandler(PyMOLGlobals *G,CSeqHandler *handler);
+void SeqSetRowVLA(PyMOLGlobals *G,CSeqRow *row,int nRow);
 CSeqRow *SeqGetRowVLA(void);
-void SeqDirty(void); /* sequence dirty -- need to update selections */
-void SeqChanged(void); /* sequence changed -- need to rebuild */
-void SeqUpdate(void);
+void SeqDirty(PyMOLGlobals *G); /* sequence dirty -- need to update selections */
+void SeqChanged(PyMOLGlobals *G); /* sequence changed -- need to rebuild */
+void SeqUpdate(PyMOLGlobals *G);
 
 #endif
 
