@@ -18,9 +18,26 @@ Z* -------------------------------------------------------------------
 
 #include"os_python.h"
 #include"PyMOLGlobals.h"
+#include"OVOneToOne.h"
 
 typedef char SettingName[255];
 
+/* for atomic settings */
+
+typedef struct {
+  int setting_id;
+  int type; /* must be cSetting_boolean, cSetting_int, cSetting_float, or cSetting_color */
+  int value;
+  int next; /* for per-atom setting lists & memory management */
+} SettingUniqueEntry;
+
+struct _CSettingUnique {
+  OVOneToOne *id2offset;
+  OVOneToOne *old2new;
+  SettingUniqueEntry *entry;
+  int n_alloc, next_free;
+};
+
 typedef struct {
   int defined;
   int changed;
@@ -44,14 +61,36 @@ struct _CSetting {
 #define cSetting_color       5
 #define cSetting_string      6
 
+/* Atomic Settings */
+
+void SettingUniqueDetachChain(PyMOLGlobals *G,int index);
 /* New API 
  * NOTE: get commands are not range-checked, so be careful
  * in contrast, set commands expand the current list 
  */
 
+void SettingUniqueSet_b(PyMOLGlobals *G,int unique_id,int setting_id,int value);
+void SettingUniqueSet_i(PyMOLGlobals *G,int unique_id,int setting_id,int value);
+void SettingUniqueSet_f(PyMOLGlobals *G,int unique_id,int setting_id,float value);
+void SettingUniqueSet_color(PyMOLGlobals *G,int unique_id,int setting_id,int value);
+void SettingUniqueSetTypedValue(PyMOLGlobals *G,int unique_id,int setting_id,int setting_type, void *value);
+
+int SettingUniqueCheck(PyMOLGlobals *G,int unique_id,int setting_id);
+int SettingUniqueGet_b(PyMOLGlobals *G,int unique_id,int setting_id,int *value);
+int SettingUniqueGet_i(PyMOLGlobals *G,int unique_id,int setting_id,int *value);
+int SettingUniqueGet_f(PyMOLGlobals *G,int unique_id,int setting_id,float *value);
+int SettingUniqueGet_color(PyMOLGlobals *G,int unique_id,int setting_id,int *value);
+
+void SettingUniqueResetAll(PyMOLGlobals *G);
+PyObject *SettingUniqueAsPyList(PyMOLGlobals *G);
+int SettingUniqueFromPyList(PyMOLGlobals *G,PyObject *list,int partial_restore);
+int SettingUniqueConvertOldSessionID(PyMOLGlobals *G,int old_unique_id);
+
+int SettingUniqueCopyAll(PyMOLGlobals *G, int src_unique_id, int dst_unique_id);
 void SettingInitGlobal(PyMOLGlobals *G,int alloc,int reset_gui);
 void SettingFreeGlobal(PyMOLGlobals *G);
 
+
 CSetting *SettingNew(PyMOLGlobals *G);
 void SettingFreeP(CSetting *I);
 void SettingInit(PyMOLGlobals *G,CSetting *I);
@@ -77,13 +116,15 @@ int   SettingGetGlobal_b(PyMOLGlobals *G,int index); /* always succeed */
 int   SettingGetGlobal_i(PyMOLGlobals *G,int index); /* always succeed */
 float SettingGetGlobal_f(PyMOLGlobals *G,int index); /* always succeed */
 char *SettingGetGlobal_s(PyMOLGlobals *G,int index); /* always succeeds */
+int   SettingGetGlobal_color(PyMOLGlobals *G,int index); /* always succeed */
+
 void  SettingGetGlobal_3f(PyMOLGlobals *G,int index,float *value); /* always succeeds */
 float *SettingGetGlobal_3fv(PyMOLGlobals *G,int index); /* always succeed */
 
 int   SettingSetGlobal_b(PyMOLGlobals *G,int index,int value);
 int   SettingSetGlobal_i(PyMOLGlobals *G,int index,int value);
 int   SettingSetGlobal_f(PyMOLGlobals *G,int index,float value);
-
+int   SettingSetGlobal_3f(PyMOLGlobals *G,int index, float value1,float value2,float value3);
 int   SettingSetSmart_i(PyMOLGlobals *G,CSetting *set1,CSetting *set2,int index, int value);
 /* more to come */
 
@@ -108,6 +149,7 @@ float *SettingGet_3fv(PyMOLGlobals *G,CSetting *set1,CSetting *set2,int index);
 int   SettingGet_color(PyMOLGlobals *G,CSetting *set1,CSetting *set2,int index);
 
 int SettingSetFromString(PyMOLGlobals *G,CSetting *I,int index,char *st);
+int SettingStringToTypedValue(PyMOLGlobals *G,int index,char *st, int *type, int *value);
 
 #ifndef _PYMOL_NOPY
 int SettingSetFromTuple(PyMOLGlobals *G,CSetting *I,int index,PyObject *tuple);
@@ -132,7 +174,7 @@ int SettingGetName(PyMOLGlobals *G,int index,SettingName name);
 PyObject *SettingAsPyList(CSetting *I);
 int SettingFromPyList(CSetting *I,PyObject *list);
 int SettingSetGlobalsFromPyList(PyMOLGlobals *G,PyObject *list);
-PyObject *SettingGetGlobalsPyList(PyMOLGlobals *G);
+PyObject *SettingGetGlobalsAsPyList(PyMOLGlobals *G);
 CSetting *SettingNewFromPyList(PyMOLGlobals *G,PyObject *list);
 
 /* WARNING: do not delete or change indices
@@ -476,7 +518,7 @@ CSetting *SettingNewFromPyList(PyMOLGlobals *G,PyObject *list);
 #define cSetting_transparency_picking_mode  324
 #define cSetting_virtual_trackball          325
 #define cSetting_pdb_reformat_names_mode    326
-#define cSetting_ray_pixel_scale_to_window  327
+#define cSetting_ray_pixel_scale            327
 #define cSetting_label_font_id              328
 #define cSetting_pdb_conect_all             329
 #define cSetting_button_mode_name           330
@@ -536,7 +578,7 @@ CSetting *SettingNewFromPyList(PyMOLGlobals *G,PyObject *list);
 #define cSetting_surface_optimize_subsets   384
 #define cSetting_multiplex                  385
 #define cSetting_texture_fonts              386
-#define cSetting_pqr_no_chain_id            387
+#define cSetting_pqr_workarounds            387
 #define cSetting_animation                  388
 #define cSetting_animation_duration         389
 #define cSetting_scene_animation            390
@@ -568,8 +610,150 @@ CSetting *SettingNewFromPyList(PyMOLGlobals *G,PyObject *list);
 #define cSetting_editor_auto_dihedral       416
 #define cSetting_presentation_auto_start    417
 #define cSetting_validate_object_names      418
+#define cSetting_unused_boolean_def_true    419
+#define cSetting_auto_show_spheres          420
+#define cSetting_sphere_mode                421
+#define cSetting_sphere_point_max_size      422
+#define cSetting_sphere_point_size          423
+#define cSetting_pdb_honor_model_number     424
+#define cSetting_rank_assisted_sorts        425
+#define cSetting_ribbon_nucleic_acid_mode   426
+#define cSetting_cartoon_ring_mode          427
+#define cSetting_cartoon_ring_width         428
+#define cSetting_cartoon_ring_color         429
+#define cSetting_cartoon_ring_finder        430
+#define cSetting_cartoon_tube_cap           431
+#define cSetting_cartoon_loop_cap           432
+#define cSetting_nvidia_bugs                433
+#define cSetting_image_dots_per_inch        434
+#define cSetting_opaque_background          435
+#define cSetting_draw_frames                436
+#define cSetting_show_alpha_checker         437
+#define cSetting_matrix_mode                438
+#define cSetting_editor_auto_origin         439
+#define cSetting_session_file               440
+#define cSetting_cgo_transparency           441
+#define cSetting_legacy_mouse_zoom          442
+#define cSetting_auto_number_selections     443
+#define cSetting_sculpt_vdw_vis_mode        444
+#define cSetting_sculpt_vdw_vis_min         445
+#define cSetting_sculpt_vdw_vis_mid         446
+#define cSetting_sculpt_vdw_vis_max         447
+#define cSetting_cartoon_ladder_mode        448
+#define cSetting_cartoon_ladder_radius      449
+#define cSetting_cartoon_ladder_color       450
+#define cSetting_cartoon_nucleic_acid_color 451
+#define cSetting_cartoon_ring_transparency  452
+#define cSetting_label_size                 453
+#define cSetting_spec_direct                454
+#define cSetting_light_count                455
+#define cSetting_light2                     456
+#define cSetting_light3                     457
+#define cSetting_hide_underscore_names      458
+#define cSetting_selection_round_points     459
+#define cSetting_distance_exclusion         460
+#define cSetting_h_bond_exclusion           461
+#define cSetting_label_shadow_mode          462
+#define cSetting_light4                     463
+#define cSetting_light5                     464
+#define cSetting_light6                     465
+#define cSetting_light7                     466
+#define cSetting_label_outline_color        467
+#define cSetting_ray_trace_mode             468
+#define cSetting_ray_trace_gain             469
+#define cSetting_selection_visible_only     470
+#define cSetting_label_position             471
+#define cSetting_ray_trace_depth_factor     472
+#define cSetting_ray_trace_slope_factor     473
+#define cSetting_ray_trace_disco_factor     474
+#define cSetting_ray_shadow_decay_factor    475
+#define cSetting_ray_interior_mode          476
+#define cSetting_ray_legacy_lighting        477
+#define cSetting_sculpt_auto_center         478
+#define cSetting_pdb_discrete_chains        479
+#define cSetting_pdb_unbond_cations         480
+#define cSetting_sculpt_tri_scale           481
+#define cSetting_sculpt_tri_weight          482
+#define cSetting_sculpt_tri_min             483 
+#define cSetting_sculpt_tri_max             484
+#define cSetting_sculpt_tri_mode            485
+#define cSetting_pdb_echo_tags              486
+#define cSetting_connect_bonded             487
+#define cSetting_spec_direct_power          488
+#define cSetting_light8                     489
+#define cSetting_light9                     490
+#define cSetting_ray_shadow_decay_range     491
+#define cSetting_spec_count                 492
+#define cSetting_sculpt_min_scale           493
+#define cSetting_sculpt_min_weight          494
+#define cSetting_sculpt_min_min             495
+#define cSetting_sculpt_min_max             496
+#define cSetting_sculpt_max_scale           497
+#define cSetting_sculpt_max_weight          498
+#define cSetting_sculpt_max_min             499
+#define cSetting_sculpt_max_max             500
+#define cSetting_surface_circumscribe       501
+#define cSetting_sculpt_avd_weight          502
+#define cSetting_sculpt_avd_gap             503
+#define cSetting_sculpt_avd_range           504
+#define cSetting_sculpt_avd_excl            505
+#define cSetting_async_builds               506
+#define cSetting_fetch_path                 507
+#define cSetting_cartoon_ring_radius        508
+#define cSetting_ray_color_ramps            509
+#define cSetting_ray_hint_camera            510
+#define cSetting_ray_hint_shadow            511
+#define cSetting_stick_valence_scale        512
+#define cSetting_seq_view_alignment         513
+#define cSetting_seq_view_unaligned_mode    514
+#define cSetting_seq_view_unaligned_color   515
+#define cSetting_seq_view_fill_char         516
+#define cSetting_seq_view_fill_color        517
+#define cSetting_seq_view_label_color       518
+#define cSetting_surface_carve_normal_cutoff 519
+#define cSetting_trace_atoms_mode           520
+#define cSetting_session_changed            521
+#define cSetting_ray_clip_shadows           522
+#define cSetting_mouse_wheel_scale          523
+#define cSetting_nonbonded_transparency     524
+#define cSetting_ray_spec_local             525
+#define cSetting_line_color                 526
+#define cSetting_ray_label_specular         527
+#define cSetting_mesh_skip                  528
+#define cSetting_label_digits               529
+#define cSetting_label_distance_digits      530
+#define cSetting_label_angle_digits         531
+#define cSetting_label_dihedral_digits      532
+#define cSetting_surface_negative_visible   533
+#define cSetting_surface_negative_color     534
+#define cSetting_mesh_negative_visible      535
+#define cSetting_mesh_negative_color        536
+#define cSetting_group_auto_mode            537
+#define cSetting_group_full_member_names    538
+#define cSetting_gradient_max_length        539
+#define cSetting_gradient_min_length        540
+#define cSetting_gradient_min_slope         541
+#define cSetting_gradient_normal_min_dot    542
+#define cSetting_gradient_step_size         543
+#define cSetting_gradient_spacing           544
+#define cSetting_gradient_symmetry          545
+#define cSetting_ray_trace_color            546
+#define cSetting_group_arrow_prefix         547
+#define cSetting_suppress_hidden            548
+#define cSetting_session_compression        549
+#define cSetting_movie_fps                  550
+#define cSetting_ray_transparency_oblique   551
+#define cSetting_ray_trace_trans_cutoff     552
+#define cSetting_ray_trace_persist_cutoff   553
+#define cSetting_ray_transparency_oblique_power 554
+#define cSetting_ray_scatter                555
+#define cSetting_h_bond_from_proton         556
+#define cSetting_auto_copy_images           557
+#define cSetting_moe_separate_chains        558
+#define cSetting_transparency_global_sort   559
+#define cSetting_hide_long_bonds            560
 
-/* when you add a new settingg also remember:
+/* when you add a new setting also remember:
    layer1/Setting.c
    modules/pymol/setting.py
   layer5/PyMOL.c 
@@ -577,7 +761,7 @@ CSetting *SettingNewFromPyList(PyMOLGlobals *G,PyObject *list);
 
 /* cSetting_ss_INIT must always be last setting_index +1 */
 
-#define cSetting_INIT                       419
+#define cSetting_INIT                       561
 
 #endif
 
