@@ -63,7 +63,7 @@ float ShakerDoDist(float target,float *v0,float *v1,float *d0to1,float *d1to0,fl
   len = (float)length3f(d);
   dev = target-len;
   if((result=fabs(dev))>R_SMALL8) {
-    dev_2 = wt*dev/2.0F;
+    dev_2 = wt*dev*0.5F;
     if(len>R_SMALL8) { /* nonoverlapping */
       sc = dev_2/len;
       scale3f(d,sc,push);
@@ -83,39 +83,30 @@ float ShakerDoDist(float target,float *v0,float *v1,float *d0to1,float *d1to0,fl
     result = 0.0;
   return result;
 }
-#endif
 
 float ShakerDoDistLimit(float target,float *v0,float *v1,float *d0to1,float *d1to0,float wt)
 {
   float d[3],push[3];
   float len,dev,dev_2,sc;
 
+  if(wt==0.0F) return 0.0F;
   subtract3f(v0,v1,d);
   len = (float)length3f(d);
   dev = target-len;
-  if(dev<0.0F) {
-    dev_2 = wt*dev/2.0F;
-    if(len>R_SMALL8) { /* nonoverlapping */
-      sc = dev_2/len;
-      scale3f(d,sc,push);
-      add3f(push,d0to1,d0to1);
-      subtract3f(d1to0,push,d1to0);
-    } else { /* overlapping, so just push in a random direction */
-      float rd[3];
-      get_random3f(rd);
-      d0to1[0]-=rd[0]*dev_2;
-      d1to0[0]+=rd[0]*dev_2;
-      d0to1[1]-=rd[1]*dev_2;
-      d1to0[1]+=rd[1]*dev_2;
-      d0to1[2]-=rd[2]*dev_2;
-      d1to0[2]+=rd[2]*dev_2;
-    }
+  if(dev<0.0F) { /* assuming len is non-zero since it is above target */
+    dev_2 = wt*dev*0.5F;
+    sc = dev_2/len;
+    scale3f(d,sc,push);
+    add3f(push,d0to1,d0to1);
+    subtract3f(d1to0,push,d1to0);
+    return -dev;
   } else
-    dev = 0.0F;
-  return (float)fabs(dev);
+    return 0.0F;
 }
 
-void ShakerAddDistCon(CShaker *I,int atom0,int atom1,float target,int type)
+#endif
+
+void ShakerAddDistCon(CShaker *I,int atom0,int atom1,float target,int type,float wt)
 {
   ShakerDistCon *sdc;
 
@@ -125,6 +116,7 @@ void ShakerAddDistCon(CShaker *I,int atom0,int atom1,float target,int type)
   sdc->at1=atom1;
   sdc->targ = target;
   sdc->type = type;
+  sdc->weight = wt;
   I->NDistCon++;
 }
 
@@ -161,7 +153,7 @@ float ShakerDoPyra(float target,float *v0,float *v1,float *v2,float *v3,
     sc = wt*dev;
     scale3f(cp,sc,push);
     add3f(push,p0,p0);
-    scale3f(push,1.0F/3,push);
+    scale3f(push,1.0F*0.333333F,push);
     subtract3f(p1,push,p1);
     subtract3f(p2,push,p2);
     subtract3f(p3,push,p3);
@@ -246,7 +238,7 @@ float ShakerDoPlan(float *v0,float *v1,float *v2,float *v3,
   dev = (float)fabs(cur);
   if((result = (float)fabs(dev))>R_SMALL8) {
 
-    sc = -wt*dev/2.0F;
+    sc = -wt*dev*0.5F;
 
     subtract3f(v0,v3,d0);
     normalize3f(d0);
@@ -271,11 +263,21 @@ float ShakerDoPlan(float *v0,float *v1,float *v2,float *v3,
 
 #else
     
-    float d01[3],d12[3],d23[3],cp0[3],cp1[3],dp,sc,dev,dp2,pos[3],d0[3],push[3];
-
+    float d01[3],d12[3],d23[3],d03[3],cp0[3],cp1[3],dp,sc,dev,d0[3],push[3];
+  double s01,s12,s23,s03;
+    
     subtract3f(v0,v1,d01);
     subtract3f(v1,v2,d12);
     subtract3f(v2,v3,d23);
+    subtract3f(v0,v3,d03);    
+
+    s03 = lengthsq3f(d03);
+    s01 = lengthsq3f(d01);
+    s12 = lengthsq3f(d12);
+    s23 = lengthsq3f(d23);
+    
+    if( (s03<s01) || (s03<s12) || (s03<s23))
+      return 0.0F;
 
     cross_product3f(d01,d12,cp0);
     cross_product3f(d12,d23,cp1);
@@ -289,23 +291,33 @@ float ShakerDoPlan(float *v0,float *v1,float *v2,float *v3,
     
     if(dev>R_SMALL4) {
 
-      add3f(cp0,cp1,d0);
-      normalize3f(d0);
-
-      cross_product3f(cp0,d12,pos);
-      dp2 = dot_product3f(cp1,pos);
+      /*
+        add3f(cp0,cp1,d0);
+        normalize3f(d0);
+        
+        cross_product3f(cp0,d12,pos);
+        dp2 = dot_product3f(cp1,pos);
+      */
       
       if(fixed && (dp*target<0.0F)) {
         if(dp<0.0F) {
-          sc = -wt*dev/2.0F;
+          sc = -wt*dev*0.5F;
         } else {
-          sc = wt*dev/2.0F;
+          sc = wt*dev*0.5F;
         }
       } else if(dp>0) {
-        sc = -wt*dev/2.0F;
+        sc = -wt*dev*0.5F;
       } else {
-        sc = wt*dev/2.0F;
+        sc = wt*dev*0.5F;
       }
+      
+      if(fixed) {
+        sc *= 8;
+      } else {
+        sc *= 0.2F;
+      }
+
+      /* pair-wise nudges */
 
       subtract3f(v0,v3,d0);
       normalize3f(d0);
@@ -332,10 +344,9 @@ float ShakerDoPlan(float *v0,float *v1,float *v2,float *v3,
       add3f(push,p1,p1); 
       subtract3f(p3,push,p3);
 
-    }
-    
+    } else
 #endif    
-    result = 0.0;
+      result = 0.0;
   return result;
 
 }
