@@ -17,6 +17,7 @@ Z* -------------------------------------------------------------------
 #define _H_Text
 
 #include"PyMOLGlobals.h"
+#include"Base.h"
 
 /* Here are the issues:
 
@@ -67,14 +68,23 @@ Z* -------------------------------------------------------------------
 #define cTextSrcFreeType  2
 
 int TextInit(PyMOLGlobals *G);
-int TextGetFontID(PyMOLGlobals *G,int src, int code, char *name,int size_mode, int size, int style);
+int TextGetFontID(PyMOLGlobals *G,int src, int code, char *name,int size_mode, int style);
 
 void TextFree(PyMOLGlobals *G);
 
+void TextSetLabPos(PyMOLGlobals *G,float *pos, LabPosType *labpos, char *text);
+void TextSetPickColor(PyMOLGlobals *G,int index, int pass);
+
 void TextSetPos(PyMOLGlobals *G,float *pos);
 
 void TextSetColor(PyMOLGlobals *G,float *color);
 void TextSetColor3f(PyMOLGlobals *G,float red, float green, float blue);
+void TextGetOutlineColor(PyMOLGlobals *G,
+                         unsigned char *red,
+                         unsigned char *green, 
+                         unsigned char *blue,
+                         unsigned char *alpha);
+void TextSetOutlineColor(PyMOLGlobals *G,int color);
 void TextSetPosNColor(PyMOLGlobals *G,float *pos,float *color);
 float *TextGetColor(PyMOLGlobals *G);
 float *TextGetPos(PyMOLGlobals *G);
@@ -82,13 +92,17 @@ void TextGetColorUChar(PyMOLGlobals *G,unsigned char *red,
                        unsigned char *green, 
                        unsigned char *blue,
                        unsigned char *alpha);
-char *TextRenderOpenGL(PyMOLGlobals *G,int text_id,char *st);
-char *TextRenderRay(PyMOLGlobals *G,CRay *ray,int text_id,char *st);
+
+char *TextRenderOpenGL(PyMOLGlobals *G,RenderInfo *info,int text_id,char *st,float size, float *rpos);
+char *TextRenderRay(PyMOLGlobals *G,CRay *ray,int text_id,char *st,float size, float *rpos);
+
 void TextDrawStrAt(PyMOLGlobals *G,char *st, int x, int y);
 void TextDrawStr(PyMOLGlobals *G,char *st);
+void TextIndent(PyMOLGlobals *G,float x,float y);
 void TextAdvance(PyMOLGlobals *G,float advance);
 void TextSetPos2i(PyMOLGlobals *G,int x,int y);
 void TextDrawChar(PyMOLGlobals *G,char ch);
 void TextDrawSubStrFast(PyMOLGlobals *G,char *c,int x,int y,int start,int n);
+void TextDrawCharRepeat(PyMOLGlobals *G,char c,int x,int y,int start,int n);
 
 #endif
