@@ -24,7 +24,6 @@ Z* -------------------------------------------------------------------
 #include"Ortho.h"
 #include"P.h"
 #include"PConv.h"
-#include"Grap.h"
 #include"PopUp.h"
 
 #include"Wizard.h"
@@ -45,6 +44,7 @@ Z* -------------------------------------------------------------------
 #define cWizEventScene   16
 #define cWizEventState   32
 #define cWizEventFrame   64
+#define cWizEventDirty  128
 
 typedef struct {
   int type;
@@ -82,6 +82,10 @@ int WizardUpdate(PyMOLGlobals *G)
   register CWizard *I=G->Wizard;
   int result = false;
 
+  if(OrthoGetDirty(G)) {
+      WizardDoDirty(G);    
+  }
+
   {
     int frame = SettingGetGlobal_i(G,cSetting_frame);
     int state = SettingGetGlobal_i(G,cSetting_state);
@@ -110,11 +114,11 @@ void WizardPurgeStack(PyMOLGlobals *G)
   int blocked;
   int a;
   register CWizard *I=G->Wizard;
-  blocked = PAutoBlock();
+  blocked = PAutoBlock(G);
   for(a=I->Stack;a>=0;a--)
     Py_XDECREF(I->Wiz[a]);
   I->Stack = -1;
-  PAutoUnblock(blocked);
+  PAutoUnblock(G,blocked);
 #endif
 }
 int WizardDoSelect(PyMOLGlobals *G,char *name)
@@ -130,13 +134,13 @@ int WizardDoSelect(PyMOLGlobals *G,char *name)
     if(I->Stack>=0)
       if(I->Wiz[I->Stack]) {
         sprintf(buf,"cmd.get_wizard().do_select('''%s''')",name);
-        PLog(buf,cPLog_pym);
-        PBlock(); 
+        PLog(G,buf,cPLog_pym);
+        PBlock(G); 
         if(PyObject_HasAttrString(I->Wiz[I->Stack],"do_select")) {
           result = PTruthCallStr(I->Wiz[I->Stack],"do_select",name);
         if(PyErr_Occurred()) PyErr_Print();
         }
-        PUnblock();
+        PUnblock(G);
       }
   return result;
 #endif
@@ -153,7 +157,7 @@ void WizardRefresh(PyMOLGlobals *G)
   PyObject *i;
   int a;
   int blocked;
-  blocked = PAutoBlock();
+  blocked = PAutoBlock(G);
   
   /* get the current prompt */
   if(I->Stack>=0)
@@ -180,6 +184,7 @@ void WizardRefresh(PyMOLGlobals *G)
       
       if(PyObject_HasAttrString(I->Wiz[I->Stack],"get_event_mask")) {      
         i = PyObject_CallMethod(I->Wiz[I->Stack],"get_event_mask","");
+        if(PyErr_Occurred()) PyErr_Print();
         if(!PConvPyIntToInt(i,&I->EventMask))
           I->EventMask = cWizEventPick + cWizEventSelect;
         Py_XDECREF(i);
@@ -223,7 +228,7 @@ void WizardRefresh(PyMOLGlobals *G)
   } else {
     OrthoReshapeWizard(G,0);
   }
-  PAutoUnblock(blocked);
+  PAutoUnblock(G,blocked);
 #endif
 }
 /*========================================================================*/
@@ -232,7 +237,7 @@ void WizardSet(PyMOLGlobals *G,PyObject *wiz,int replace)
 #ifndef _PYMOL_NOPY
   register CWizard *I = G->Wizard;
   int blocked;
-  blocked = PAutoBlock();
+  blocked = PAutoBlock(G);
   if(I->Wiz) {
     if((!wiz)||(wiz==Py_None)||((I->Stack>=0)&&replace)) { 
       if(I->Stack>=0) {  /* pop */
@@ -258,7 +263,7 @@ void WizardSet(PyMOLGlobals *G,PyObject *wiz,int replace)
     }
   }
   WizardRefresh(G);
-  PAutoUnblock(blocked);
+  PAutoUnblock(G,blocked);
 #endif
 }
 /*========================================================================*/
@@ -289,11 +294,11 @@ int WizardDoPick(PyMOLGlobals *G,int bondFlag)
     if(I->Stack>=0) 
       if(I->Wiz[I->Stack]) {
         if(bondFlag)
-          PLog("cmd.get_wizard().do_pick(1)",cPLog_pym);
+          PLog(G,"cmd.get_wizard().do_pick(1)",cPLog_pym);
         else
-          PLog("cmd.get_wizard().do_pick(0)",cPLog_pym);
+          PLog(G,"cmd.get_wizard().do_pick(0)",cPLog_pym);
         
-        PBlock(); 
+        PBlock(G); 
         if(I->Stack>=0)
           if(I->Wiz[I->Stack]) {
             if(PyObject_HasAttrString(I->Wiz[I->Stack],"do_pick")) {
@@ -301,7 +306,7 @@ int WizardDoPick(PyMOLGlobals *G,int bondFlag)
               if(PyErr_Occurred()) PyErr_Print();
             }
           }
-        PUnblock();
+        PUnblock(G);
       }
   return result;
 #endif
@@ -319,8 +324,8 @@ int WizardDoKey(PyMOLGlobals *G,unsigned char k, int x, int y, int mod)
       if(I->Wiz[I->Stack]) {
         OrthoLineType buffer;
         sprintf(buffer,"cmd.get_wizard().do_key(%d,%d,%d,%d)",k,x,y,mod);
-        PLog(buffer,cPLog_pym);
-        PBlock(); 
+        PLog(G,buffer,cPLog_pym);
+        PBlock(G); 
         if(I->Stack>=0)
           if(I->Wiz[I->Stack]) {
             if(PyObject_HasAttrString(I->Wiz[I->Stack],"do_key")) {
@@ -328,7 +333,7 @@ int WizardDoKey(PyMOLGlobals *G,unsigned char k, int x, int y, int mod)
               if(PyErr_Occurred()) PyErr_Print();
             }
           }
-        PUnblock();
+        PUnblock(G);
       }
   return result;
 #endif
@@ -346,8 +351,8 @@ int WizardDoScene(PyMOLGlobals *G)
       if(I->Wiz[I->Stack]) {
         OrthoLineType buffer;
         sprintf(buffer,"cmd.get_wizard().do_scene()");
-        PLog(buffer,cPLog_pym);
-        PBlock(); 
+        PLog(G,buffer,cPLog_pym);
+        PBlock(G); 
         if(I->Stack>=0)
           if(I->Wiz[I->Stack]) {
             if(PyObject_HasAttrString(I->Wiz[I->Stack],"do_scene")) {
@@ -355,7 +360,34 @@ int WizardDoScene(PyMOLGlobals *G)
               if(PyErr_Occurred()) PyErr_Print();
             }
           }
-        PUnblock();
+        PUnblock(G);
+      }
+  return result;
+#endif
+}
+
+int WizardDoDirty(PyMOLGlobals *G)
+{
+#ifdef _PYMOL_NOPY
+  return 0;
+#else
+  register CWizard *I=G->Wizard;
+  int result=false;
+  if(I->EventMask & cWizEventDirty) 
+    if(I->Stack>=0) 
+      if(I->Wiz[I->Stack]) {
+        OrthoLineType buffer;
+        sprintf(buffer,"cmd.get_wizard().do_dirty()");
+        PLog(G,buffer,cPLog_pym);
+        PBlock(G); 
+        if(I->Stack>=0)
+          if(I->Wiz[I->Stack]) {
+            if(PyObject_HasAttrString(I->Wiz[I->Stack],"do_dirty")) {
+              result = PTruthCallStr0(I->Wiz[I->Stack],"do_dirty");
+              if(PyErr_Occurred()) PyErr_Print();
+            }
+          }
+        PUnblock(G);
       }
   return result;
 #endif
@@ -375,8 +407,8 @@ int WizardDoState(PyMOLGlobals *G)
         OrthoLineType buffer;
         int state = SettingGetGlobal_i(G,cSetting_state) + 1;
         sprintf(buffer,"cmd.get_wizard().do_state(%d)",state);
-        PLog(buffer,cPLog_pym);
-        PBlock(); 
+        PLog(G,buffer,cPLog_pym);
+        PBlock(G); 
         if(I->Stack>=0)
           if(I->Wiz[I->Stack]) {
             if(PyObject_HasAttrString(I->Wiz[I->Stack],"do_state")) {
@@ -384,7 +416,7 @@ int WizardDoState(PyMOLGlobals *G)
               if(PyErr_Occurred()) PyErr_Print();
             }
           }
-        PUnblock();
+        PUnblock(G);
       }
   return result;
 #endif
@@ -404,8 +436,8 @@ int WizardDoFrame(PyMOLGlobals *G)
         OrthoLineType buffer;
         int frame = SettingGetGlobal_i(G,cSetting_frame) + 1;
         sprintf(buffer,"cmd.get_wizard().do_frame(%d)",frame);
-        PLog(buffer,cPLog_pym);
-        PBlock(); 
+        PLog(G,buffer,cPLog_pym);
+        PBlock(G); 
         if(I->Stack>=0)
           if(I->Wiz[I->Stack]) {
             if(PyObject_HasAttrString(I->Wiz[I->Stack],"do_frame")) {
@@ -413,7 +445,7 @@ int WizardDoFrame(PyMOLGlobals *G)
               if(PyErr_Occurred()) PyErr_Print();
             }
           }
-        PUnblock();
+        PUnblock(G);
       }
   return result;
 #endif
@@ -432,8 +464,8 @@ int WizardDoSpecial(PyMOLGlobals *G,int k, int x, int y, int mod)
       if(I->Wiz[I->Stack]) {
         OrthoLineType buffer;
         sprintf(buffer,"cmd.get_wizard().do_special(%d,%d,%d,%d)",k,x,y,mod);
-        PLog(buffer,cPLog_pym);
-        PBlock(); 
+        PLog(G,buffer,cPLog_pym);
+        PBlock(G); 
         if(I->Stack>=0)
           if(I->Wiz[I->Stack]) {
             if(PyObject_HasAttrString(I->Wiz[I->Stack],"do_special")) {
@@ -441,7 +473,7 @@ int WizardDoSpecial(PyMOLGlobals *G,int k, int x, int y, int mod)
               if(PyErr_Occurred()) PyErr_Print();
             }
           }
-        PUnblock();
+        PUnblock(G);
       }
   return result;
 #endif
@@ -469,7 +501,7 @@ static int WizardClick(Block *block,int button,int x,int y,int mod)
       OrthoDirty(G);
       break;
     case cWizTypePopUp:
-      PBlock(); 
+      PBlock(G); 
       if(I->Stack>=0)
         if(I->Wiz[I->Stack]) {
           if(PyObject_HasAttrString(I->Wiz[I->Stack],"get_menu")) {
@@ -485,8 +517,7 @@ static int WizardClick(Block *block,int button,int x,int y,int mod)
         PopUpNew(G,x,my,x,y,false,menuList,NULL);
       }
       Py_XDECREF(menuList);
-      PUnblock();
-
+      PUnblock(G);
       break;
     }
   }
@@ -553,13 +584,14 @@ static int WizardRelease(Block *block,int button,int x,int y,int mod)
     case cWizTypeButton:
       if(I->Stack>=0)
         if(I->Wiz[I->Stack]) {
-          PLog(I->Line[a].code,cPLog_pym);
-          PParse(I->Line[a].code);
-          PFlush();
+          PLog(G,I->Line[a].code,cPLog_pym);
+          PParse(G,I->Line[a].code);
+          PFlush(G);
         }
       break;
     }
   }
+  I->Pressed = -1;
   return(1);
 }
 
@@ -652,7 +684,7 @@ static void WizardDraw(Block *block)
 
   text_color = menuColor;
 
-  if(G->HaveGUI && G->ValidContext) {
+  if(G->HaveGUI && G->ValidContext && ((block->rect.right-block->rect.left)>6)) {
 
     glColor3fv(I->Block->BackColor);
     BlockFill(I->Block);
