@@ -21,9 +21,13 @@ Z* -------------------------------------------------------------------
 #include"Word.h"
 #include"MemoryDebug.h"
 #include"Err.h"
+#include"Feedback.h"
 #include"Util.h"
 #include"Color.h"
 #include"PConv.h"
+#include"Ortho.h"
+#include"OVOneToAny.h"
+#include"OVContext.h"
 
 struct _CAtomInfo {
   int NColor,CColor,DColor,HColor,OColor,SColor;
@@ -32,23 +36,232 @@ struct _CAtomInfo {
   int CuColor, FeColor, ZnColor;
   int SeColor;
   int DefaultColor;
-  
+  int NextUniqueID;
+  OVOneToAny *ActiveIDs;
 };
 
+int AtomInfoCheckSetting(PyMOLGlobals *G, AtomInfoType *ai, int setting_id)
+{  
+  if(!ai->has_setting) {
+    return 0;
+  } else {
+    if(!SettingUniqueCheck(G,ai->unique_id,setting_id)) {
+      return 0;
+    } else {
+      return 1;
+    }
+  }
+}
+
+int AtomInfoGetSetting_b(PyMOLGlobals *G, AtomInfoType *ai, int setting_id, int current, int *effective)
+{
+  if(!ai->has_setting) {
+    *effective = current;
+    return 0;
+  } else {
+    if(!SettingUniqueGet_b(G,ai->unique_id,setting_id,effective)) {
+      *effective = current;
+      return 0;
+    } else {
+      return 1;
+    }
+  }
+}
+int AtomInfoGetSetting_i(PyMOLGlobals *G, AtomInfoType *ai, int setting_id, int current, int *effective)
+{
+  if(!ai->has_setting) {
+    *effective = current;
+    return 0;
+  } else {
+    if(!SettingUniqueGet_i(G,ai->unique_id,setting_id,effective)) {
+      *effective = current;
+      return 0;
+    } else {
+      return 1;
+    }
+  }
+}
+int AtomInfoGetSetting_f(PyMOLGlobals *G, AtomInfoType *ai, int setting_id, float current, float *effective)
+{
+  if(!ai->has_setting) {
+    *effective = current;
+    return 0;
+  } else {
+    if(!SettingUniqueGet_f(G,ai->unique_id,setting_id,effective)) {
+      *effective = current;
+      return 0;
+    } else {
+      return 1;
+    }
+  }
+}
+int AtomInfoGetSetting_color(PyMOLGlobals *G, AtomInfoType *ai, int setting_id, int current, int *effective)
+{
+  if(!ai->has_setting) {
+    *effective = current;
+    return 0;
+  } else {
+    if(!SettingUniqueGet_color(G,ai->unique_id,setting_id,effective)) {
+      *effective = current;
+      return 0;
+    } else {
+      return 1;
+    }
+  }
+}
+
+int AtomInfoCheckBondSetting(PyMOLGlobals *G, BondType *bi, int setting_id)
+{  
+  if(!bi->has_setting) {
+    return 0;
+  } else {
+    if(!SettingUniqueCheck(G,bi->unique_id,setting_id)) {
+      return 0;
+    } else {
+      return 1;
+    }
+  }
+}
+
+int AtomInfoGetBondSetting_b(PyMOLGlobals *G, BondType *bi, int setting_id, int current, int *effective)
+{
+  if(!bi->has_setting) {
+    *effective = current;
+    return 0;
+  } else {
+    if(!SettingUniqueGet_b(G,bi->unique_id,setting_id,effective)) {
+      *effective = current;
+      return 0;
+    } else {
+      return 1;
+    }
+  }
+}
+int AtomInfoGetBondSetting_i(PyMOLGlobals *G, BondType *bi, int setting_id, int current, int *effective)
+{
+  if(!bi->has_setting) {
+    *effective = current;
+    return 0;
+  } else {
+    if(!SettingUniqueGet_i(G,bi->unique_id,setting_id,effective)) {
+      *effective = current;
+      return 0;
+    } else {
+      return 1;
+    }
+  }
+}
+int AtomInfoGetBondSetting_f(PyMOLGlobals *G, BondType *bi, int setting_id, float current, float *effective)
+{
+  if(!bi->has_setting) {
+    *effective = current;
+    return 0;
+  } else {
+    if(!SettingUniqueGet_f(G,bi->unique_id,setting_id,effective)) {
+      *effective = current;
+      return 0;
+    } else {
+      return 1;
+    }
+  }
+}
+int AtomInfoGetBondSetting_color(PyMOLGlobals *G, BondType *bi, int setting_id, int current, int *effective)
+{
+  if(!bi->has_setting) {
+    *effective = current;
+    return 0;
+  } else {
+    if(!SettingUniqueGet_color(G,bi->unique_id,setting_id,effective)) {
+      *effective = current;
+      return 0;
+    } else {
+      return 1;
+    }
+  }
+}
+
+
+static int AtomInfoPrimeUniqueIDs(PyMOLGlobals *G)
+{
+  CAtomInfo *I=G->AtomInfo;
+  if(!I->ActiveIDs) {
+    OVContext *C = G->Context;
+    I->ActiveIDs = OVOneToAny_New(C->heap);
+  }
+  return (I->ActiveIDs!=NULL);
+}
+
+int AtomInfoReserveUniqueID(PyMOLGlobals *G,int unique_id)
+{
+  CAtomInfo *I=G->AtomInfo;
+  if(!I->ActiveIDs) 
+    AtomInfoPrimeUniqueIDs(G);
+  if(I->ActiveIDs) 
+    return (OVreturn_IS_OK(OVOneToAny_SetKey(I->ActiveIDs, unique_id, 1)));
+  return 0;
+}
+
+int AtomInfoIsUniqueIDActive(PyMOLGlobals *G,int unique_id)
+{
+  CAtomInfo *I=G->AtomInfo;
+  if(!I->ActiveIDs) 
+    return 0;
+  else 
+    return (OVreturn_IS_OK(OVOneToAny_GetKey(I->ActiveIDs, unique_id)));
+  return 0;
+}
+
+int AtomInfoGetNewUniqueID(PyMOLGlobals *G)
+{
+  CAtomInfo *I=G->AtomInfo;
+  int result = 0;
+  AtomInfoPrimeUniqueIDs(G);
+  if(I->ActiveIDs) {
+    while(1) {
+      result = I->NextUniqueID++;
+      if(result) { /* skip zero */
+        if(OVOneToAny_GetKey(I->ActiveIDs,result).status == OVstatus_NOT_FOUND) {
+          if(OVreturn_IS_ERROR( OVOneToAny_SetKey(I->ActiveIDs, result, 1) ))
+            result = 0;
+          break;
+        }
+      }
+    }
+  }
+  return result;
+}
+
+int AtomInfoCheckUniqueID(PyMOLGlobals *G, AtomInfoType *ai)
+{
+  if(!ai->unique_id) 
+    ai->unique_id = AtomInfoGetNewUniqueID(G);
+  return ai->unique_id;
+}
+
+int AtomInfoCheckUniqueBondID(PyMOLGlobals *G, BondType *bi)
+{
+  if(!bi->unique_id) 
+    bi->unique_id = AtomInfoGetNewUniqueID(G);
+  return bi->unique_id;
+}
+
 int AtomInfoInit(PyMOLGlobals *G)
 {
   register CAtomInfo *I=NULL;
   if( (I=(G->AtomInfo=Calloc(CAtomInfo,1)))) {
     AtomInfoPrimeColors(G);
+    I->NextUniqueID = 1;
     return 1;
   } else
     return 0;
 }
+
 void AtomInfoFree(PyMOLGlobals *G)
 {
+  CAtomInfo *I = G->AtomInfo;
+  OVOneToAny_DEL_AUTO_NULL(I->ActiveIDs);
   FreeP(G->AtomInfo);
 }
-
 /*========================================================================*/
 void AtomInfoGetPDB3LetHydroName(PyMOLGlobals *G,char *resn, char *iname, char *oname) 
 {
@@ -584,9 +797,23 @@ int AtomInfoKnownPolymerResName(PyMOLGlobals *G,char *resn)
 
 /*========================================================================*/
 
-int AtomInfoInOrder(PyMOLGlobals *G,AtomInfoType *atom,int atom1,int atom2);
-int AtomInfoInOrigOrder(PyMOLGlobals *G,AtomInfoType *atom,int atom1,int atom2);
-int AtomInfoInOrderIgnoreHet(PyMOLGlobals *G,AtomInfoType *atom,int atom1,int atom2);
+static int AtomInfoInOrder(PyMOLGlobals *G,AtomInfoType *atom,int atom1,int atom2)
+{
+  return(AtomInfoCompare(G,atom+atom1,atom+atom2)<=0);
+}
+
+static int AtomInfoInOrderIgnoreHet(PyMOLGlobals *G,AtomInfoType *atom,int atom1,int atom2)
+{
+  return(AtomInfoCompareIgnoreHet(G,atom+atom1,atom+atom2)<=0);
+}
+
+static int AtomInfoInOrigOrder(PyMOLGlobals *G,AtomInfoType *atom,int atom1,int atom2)
+{
+  if(atom[atom1].rank!=atom[atom2].rank)
+    return (atom[atom1].rank<atom[atom2].rank);
+  else
+    return(AtomInfoCompare(G,atom+atom1,atom+atom2)<=0);
+}
 
 int AtomResvFromResi(char *resi)
 {
@@ -610,7 +837,7 @@ PyObject *AtomInfoAsPyList(PyMOLGlobals *G,AtomInfoType *I)
 #else
   PyObject *result = NULL;
 
-  result = PyList_New(40);
+  result = PyList_New(41);
   PyList_SetItem(result, 0,PyInt_FromLong(I->resv));
   PyList_SetItem(result, 1,PyString_FromString(I->chain));
   PyList_SetItem(result, 2,PyString_FromString(I->alt));
@@ -619,10 +846,21 @@ PyObject *AtomInfoAsPyList(PyMOLGlobals *G,AtomInfoType *I)
   PyList_SetItem(result, 5,PyString_FromString(I->resn));
   PyList_SetItem(result, 6,PyString_FromString(I->name));
   PyList_SetItem(result, 7,PyString_FromString(I->elem));
-  PyList_SetItem(result, 8,PyString_FromString(I->textType));
-  PyList_SetItem(result, 9,PyString_FromString(I->label));
+
+  {
+    char null_st[1] = "";
+    char *st = null_st;
+
+    if(I->textType) st = OVLexicon_FetchCString(G->Lexicon,I->textType);
+    PyList_SetItem(result, 8,PyString_FromString(st));
+
+    st = null_st;
+    if(I->label) st = OVLexicon_FetchCString(G->Lexicon,I->label);
+    PyList_SetItem(result, 9,PyString_FromString(st));
+  }
+
   PyList_SetItem(result,10,PyString_FromString(I->ssType));
-  PyList_SetItem(result,11,PyInt_FromLong(I->hydrogen));  
+  PyList_SetItem(result,11,PyInt_FromLong((char)I->hydrogen));  
   PyList_SetItem(result,12,PyInt_FromLong(I->customType));
   PyList_SetItem(result,13,PyInt_FromLong(I->priority));
   PyList_SetItem(result,14,PyFloat_FromDouble(I->b));
@@ -631,10 +869,10 @@ PyObject *AtomInfoAsPyList(PyMOLGlobals *G,AtomInfoType *I)
   PyList_SetItem(result,17,PyFloat_FromDouble(I->partialCharge));
   PyList_SetItem(result,18,PyInt_FromLong(I->formalCharge));
   PyList_SetItem(result,19,PyInt_FromLong((int)I->hetatm));
-  PyList_SetItem(result,20,PConvSIntArrayToPyList(I->visRep,cRepCnt));
+  PyList_SetItem(result,20,PConvSCharArrayToPyList(I->visRep,cRepCnt));
   PyList_SetItem(result,21,PyInt_FromLong(I->color));
   PyList_SetItem(result,22,PyInt_FromLong(I->id));
-  PyList_SetItem(result,23,PyInt_FromLong(I->cartoon));
+  PyList_SetItem(result,23,PyInt_FromLong((char)I->cartoon));
   PyList_SetItem(result,24,PyInt_FromLong(I->flags));
   PyList_SetItem(result,25,PyInt_FromLong((int)I->bonded));
   PyList_SetItem(result,26,PyInt_FromLong((int)I->chemFlag));
@@ -643,18 +881,19 @@ PyObject *AtomInfoAsPyList(PyMOLGlobals *G,AtomInfoType *I)
   PyList_SetItem(result,29,PyInt_FromLong((int)I->masked));
   PyList_SetItem(result,30,PyInt_FromLong((int)I->protekted));
   PyList_SetItem(result,31,PyInt_FromLong((int)I->protons));
-  PyList_SetItem(result,32,PyInt_FromLong(I->sculpt_id));
-  PyList_SetItem(result,33,PyInt_FromLong(I->stereo));
+  PyList_SetItem(result,32,PyInt_FromLong(I->unique_id));
+  PyList_SetItem(result,33,PyInt_FromLong((char)I->stereo));
   PyList_SetItem(result,34,PyInt_FromLong(I->discrete_state));
-  PyList_SetItem(result,35,PyFloat_FromDouble(I->bohr_radius));
+  PyList_SetItem(result,35,PyFloat_FromDouble(I->elec_radius));
   PyList_SetItem(result,36,PyInt_FromLong(I->rank));
   PyList_SetItem(result,37,PyInt_FromLong((int)I->hb_donor));
   PyList_SetItem(result,38,PyInt_FromLong((int)I->hb_acceptor));
   PyList_SetItem(result,39,PyInt_FromLong((int)I->atomic_color));
+  PyList_SetItem(result,40,PyInt_FromLong((int)I->has_setting));
   return(PConvAutoNone(result));
 #endif
 }
-
+ 
 int AtomInfoFromPyList(PyMOLGlobals *G,AtomInfoType *I,PyObject *list)
 {
 #ifdef _PYMOL_NOPY
@@ -663,61 +902,168 @@ int AtomInfoFromPyList(PyMOLGlobals *G,AtomInfoType *I,PyObject *list)
   int ok=true;
   int hetatm;
   int ll = 0;
-  if(ok) ok = PyList_Check(list);
-  if(ok) ll = PyList_Size(list);
-  if(ok) ok = PConvPyIntToInt(PyList_GetItem(list, 0),&I->resv);
-  if(ok) ok = PConvPyStrToStr(PyList_GetItem(list, 1),I->chain,sizeof(Chain));
-  if(ok) ok = PConvPyStrToStr(PyList_GetItem(list, 2),I->alt,sizeof(Chain));
-  if(ok) ok = PConvPyStrToStr(PyList_GetItem(list, 3),I->resi,sizeof(ResIdent));
-  if(ok) ok = PConvPyStrToStr(PyList_GetItem(list, 4),I->segi,sizeof(SegIdent));
-  if(ok) ok = PConvPyStrToStr(PyList_GetItem(list, 5),I->resn,sizeof(ResName));
-  if(ok) ok = PConvPyStrToStr(PyList_GetItem(list, 6),I->name,sizeof(AtomName));
-  if(ok) ok = PConvPyStrToStr(PyList_GetItem(list, 7),I->elem,sizeof(AtomName));
-  if(ok) ok = PConvPyStrToStr(PyList_GetItem(list, 8),I->textType,sizeof(TextType));
-  if(ok) ok = PConvPyStrToStr(PyList_GetItem(list, 9),I->label,sizeof(LabelType));
-  if(ok) ok = PConvPyStrToStr(PyList_GetItem(list,10),I->ssType,sizeof(SSType));
-  if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,11),&I->hydrogen);
-  if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,12),&I->customType);
-  if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,13),&I->priority);
-  if(ok) ok = PConvPyFloatToFloat(PyList_GetItem(list,14),&I->b);
-  if(ok) ok = PConvPyFloatToFloat(PyList_GetItem(list,15),&I->q);
-  if(ok) ok = PConvPyFloatToFloat(PyList_GetItem(list,16),&I->vdw);
-  if(ok) ok = PConvPyFloatToFloat(PyList_GetItem(list,17),&I->partialCharge);
-  if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,18),&I->formalCharge);
-  if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,19),&hetatm);
+  if(ok) ok = PyList_Check(list);  
+  if(ok) ll = PyList_Size(list);  
+  if(ok) ok = PConvPyIntToInt(PyList_GetItem(list, 0),&I->resv);   
+  if(ok) ok = PConvPyStrToStr(PyList_GetItem(list, 1),I->chain,sizeof(Chain));  
+  if(ok) ok = PConvPyStrToStr(PyList_GetItem(list, 2),I->alt,sizeof(Chain));  
+  if(ok) ok = PConvPyStrToStr(PyList_GetItem(list, 3),I->resi,sizeof(ResIdent));  
+  if(ok) ok = PConvPyStrToStr(PyList_GetItem(list, 4),I->segi,sizeof(SegIdent));  
+  if(ok) ok = PConvPyStrToStr(PyList_GetItem(list, 5),I->resn,sizeof(ResName));  
+  if(ok) ok = PConvPyStrToStr(PyList_GetItem(list, 6),I->name,sizeof(AtomName));  
+  if(ok) ok = PConvPyStrToStr(PyList_GetItem(list, 7),I->elem,sizeof(AtomName));  
+  if(ok) {
+    OrthoLineType temp;
+    PConvPyStrToStr(PyList_GetItem(list, 8),temp,sizeof(OrthoLineType));
+    I->textType = 0;
+    if(temp[0]) {
+      OVreturn_word result = OVLexicon_GetFromCString(G->Lexicon,temp);
+      if(OVreturn_IS_OK(result)) {
+        I->textType = result.word;
+      }
+    }
+  }
+  if(ok) {
+    OrthoLineType temp;
+    PConvPyStrToStr(PyList_GetItem(list, 9),temp,sizeof(OrthoLineType));
+    I->textType = 0;
+    if(temp[0]) {
+      OVreturn_word result = OVLexicon_GetFromCString(G->Lexicon,temp);
+      if(OVreturn_IS_OK(result)) {
+        I->label = result.word;
+      }
+    }
+  }
+  if(ok) ok = PConvPyStrToStr(PyList_GetItem(list,10),I->ssType,sizeof(SSType));   
+  if(ok) ok = PConvPyIntToChar(PyList_GetItem(list,11),(char*)&I->hydrogen);   
+  if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,12),&I->customType);   
+  if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,13),&I->priority);  
+  if(ok) ok = PConvPyFloatToFloat(PyList_GetItem(list,14),&I->b);  
+  if(ok) ok = PConvPyFloatToFloat(PyList_GetItem(list,15),&I->q);   
+  if(ok) ok = PConvPyFloatToFloat(PyList_GetItem(list,16),&I->vdw); 
+  if(ok) ok = PConvPyFloatToFloat(PyList_GetItem(list,17),&I->partialCharge); 
+  if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,18),&I->formalCharge);  
+  if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,19),&hetatm); 
   if(ok) I->hetatm = hetatm;
-  if(ok) ok = PConvPyListToSIntArrayInPlaceAutoZero(PyList_GetItem(list,20),I->visRep,cRepCnt);
-  if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,21),&I->color);
-  if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,22),&I->id);
-  if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,23),&I->cartoon);
-  if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,24),(int*)&I->flags);
-  if(ok) ok = PConvPyIntToChar(PyList_GetItem(list,25),(char*)&I->bonded);
-  if(ok) ok = PConvPyIntToChar(PyList_GetItem(list,26),(char*)&I->chemFlag);
-  if(ok) ok = PConvPyIntToChar(PyList_GetItem(list,27),(char*)&I->geom);
-  if(ok) ok = PConvPyIntToChar(PyList_GetItem(list,28),(char*)&I->valence);
-  if(ok) ok = PConvPyIntToChar(PyList_GetItem(list,29),(char*)&I->masked);
-  if(ok) ok = PConvPyIntToChar(PyList_GetItem(list,30),(char*)&I->protekted);
-  if(ok) ok = PConvPyIntToChar(PyList_GetItem(list,31),(char*)&I->protons);
-  if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,32),&I->sculpt_id);
-  if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,33),&I->stereo);
-  if(ok&&(ll>34)) ok = PConvPyIntToInt(PyList_GetItem(list,34),&I->discrete_state);
-  if(ok&&(ll>35)) ok = PConvPyFloatToFloat(PyList_GetItem(list,35),&I->bohr_radius);
-  if(ok&&(ll>36)) ok = PConvPyIntToInt(PyList_GetItem(list,36),&I->rank);
-  if(ok&&(ll>37)) ok = PConvPyIntToChar(PyList_GetItem(list,37),(char*)&I->hb_donor);
-  if(ok&&(ll>38)) ok = PConvPyIntToChar(PyList_GetItem(list,38),(char*)&I->hb_acceptor);
-  if(ok&&(ll>39)) 
+  if(ok) ok = PConvPyListToSCharArrayInPlaceAutoZero(
+                          PyList_GetItem(list,20),I->visRep,cRepCnt); 
+  if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,21),&I->color); 
+  if(ok) I->color = ColorConvertOldSessionIndex(G,I->color);
+  if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,22),&I->id); 
+  if(ok) ok = PConvPyIntToChar(PyList_GetItem(list,23),(char*)&I->cartoon); 
+  if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,24),(int*)&I->flags); 
+  if(ok) ok = PConvPyIntToChar(PyList_GetItem(list,25),(char*)&I->bonded); 
+  if(ok) ok = PConvPyIntToChar(PyList_GetItem(list,26),(char*)&I->chemFlag); 
+  if(ok) ok = PConvPyIntToChar(PyList_GetItem(list,27),(char*)&I->geom); 
+  if(ok) ok = PConvPyIntToChar(PyList_GetItem(list,28),(char*)&I->valence); 
+  if(ok) ok = PConvPyIntToChar(PyList_GetItem(list,29),(char*)&I->masked); 
+  if(ok) ok = PConvPyIntToChar(PyList_GetItem(list,30),(char*)&I->protekted); 
+  if(ok) ok = PConvPyIntToChar(PyList_GetItem(list,31),(char*)&I->protons); 
+  if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,32),&I->unique_id); 
+  if(ok && I->unique_id) { /* reserve existing IDs */
+    CAtomInfo *II = G->AtomInfo;
+    AtomInfoPrimeUniqueIDs(G);
+    I->unique_id = SettingUniqueConvertOldSessionID(G,I->unique_id);
+    OVOneToAny_SetKey(II->ActiveIDs, I->unique_id, 1);
+  }
+  if(ok) ok = PConvPyIntToChar(PyList_GetItem(list,33),(char*)&I->stereo); 
+  if(ok&&(ll>34)) ok = PConvPyIntToInt(PyList_GetItem(list,34),&I->discrete_state);  
+  if(ok&&(ll>35)) ok = PConvPyFloatToFloat(PyList_GetItem(list,35),&I->elec_radius); 
+  if(ok&&(ll>36)) ok = PConvPyIntToInt(PyList_GetItem(list,36),&I->rank); 
+  if(ok&&(ll>37)) ok = PConvPyIntToChar(PyList_GetItem(list,37),(char*)&I->hb_donor); 
+  if(ok&&(ll>38)) ok = PConvPyIntToChar(PyList_GetItem(list,38),(char*)&I->hb_acceptor); 
+  if(ok&&(ll>39)) {
     ok = PConvPyIntToInt(PyList_GetItem(list,39),&I->atomic_color);
-  else {
+    if(ok) I->atomic_color = ColorConvertOldSessionIndex(G,I->atomic_color);
+  } else {
     I->atomic_color = AtomInfoGetColor(G,I);
   }
+  if(ok&&(ll>40)) ok = PConvPyIntToChar(PyList_GetItem(list,40),(char*)&I->has_setting);
+
   return(ok);
 #endif
 }
 
+void AtomInfoCopy(PyMOLGlobals *G,AtomInfoType *src,AtomInfoType *dst)
+{
+  /* copy, handling resource management issues... */
+
+  *dst=*src;
+  dst->selEntry=0;
+  if(src->unique_id && src->has_setting) {
+    dst->unique_id = AtomInfoGetNewUniqueID(G);
+    if(!SettingUniqueCopyAll(G, src->unique_id, dst->unique_id)) 
+      dst->has_setting = 0;
+  } else {
+    dst->unique_id=0;
+    dst->has_setting=0;
+  }
+  if(dst->label) {
+    OVLexicon_IncRef(G->Lexicon,dst->label);
+  }
+  if(dst->textType) {
+    OVLexicon_IncRef(G->Lexicon,dst->textType);
+  }
+}
+
+void AtomInfoBondCopy(PyMOLGlobals *G, BondType *src, BondType *dst)
+{
+  *(dst) = *(src);
+
+  if(src->unique_id && src->has_setting) {
+    dst->unique_id = AtomInfoGetNewUniqueID(G);
+    if(!SettingUniqueCopyAll(G, src->unique_id, dst->unique_id)) 
+      dst->has_setting = 0;
+  } else {
+    src->unique_id = 0;
+    src->has_setting = 0;
+  }
+}
+
+void AtomInfoPurgeBond(PyMOLGlobals *G,BondType *bi)
+{
+  CAtomInfo *I=G->AtomInfo;  
+  if(bi->has_setting && bi->unique_id) {
+    SettingUniqueDetachChain(G,bi->unique_id);
+  }
+  if(bi->unique_id && I->ActiveIDs) {
+    OVOneToAny_DelKey(I->ActiveIDs, bi->unique_id);
+    bi->unique_id = 0;
+  }
+}
+
+void AtomInfoPurge(PyMOLGlobals *G,AtomInfoType *ai)
+{
+  CAtomInfo *I=G->AtomInfo;  
+  if(ai->textType) {
+    OVLexicon_DecRef(G->Lexicon,ai->textType);
+  }
+  if(ai->has_setting && ai->unique_id) {
+    SettingUniqueDetachChain(G,ai->unique_id);
+  }
+  if(ai->unique_id && I->ActiveIDs) {
+    OVOneToAny_DelKey(I->ActiveIDs, ai->unique_id);
+  }
+  if(ai->label) {
+    /*    printf("purging %d [%s]\n", OVLexicon_GetNActive(G->Lexicon),
+          OVLexicon_FetchCString(G->Lexicon,ai->label));*/
+    OVLexicon_DecRef(G->Lexicon,ai->label);
+  }
+}
+
 /*========================================================================*/
 void AtomInfoCombine(PyMOLGlobals *G,AtomInfoType *dst,AtomInfoType *src,int mask)
 {
-  if(mask&cAIC_tt) strcpy(dst->textType,src->textType); /* use the new types */
+  if(mask&cAIC_tt) {
+    if(dst->textType) {
+      OVLexicon_DecRef(G->Lexicon,dst->textType);
+      dst->textType = 0;
+    }
+    dst->textType = src->textType;
+  } else if(src->textType) {
+    OVLexicon_DecRef(G->Lexicon,src->textType);
+    src->textType = 0;
+  }
   if(mask&cAIC_ct) dst->customType = src->customType;
   if(mask&cAIC_pc) dst->partialCharge = src->partialCharge;
   if(mask&cAIC_fc) dst->formalCharge = src->formalCharge;
@@ -728,11 +1074,17 @@ void AtomInfoCombine(PyMOLGlobals *G,AtomInfoType *dst,AtomInfoType *src,int mas
   if(mask&cAIC_state) dst->discrete_state = src->discrete_state;
   if(mask&cAIC_rank) dst->rank = src->rank;
   dst->temp1 = src->temp1;
-  dst->sculpt_id = src->sculpt_id;
+  dst->unique_id = src->unique_id;
   /* keep all existing names, identifiers, etc. */
   /* also keep all existing selections,
      colors, masks, and visible representations*/
-
+  { 
+    if(src->label) { /* destroy src label if one exists */
+      OVLexicon_DecRef(G->Lexicon,src->label);
+      src->label = 0;
+    }
+    /* leaves dst->label untouched */
+  }
 }
 /*========================================================================*/
 void AtomInfoUniquefyNames(PyMOLGlobals *G,AtomInfoType *atInfo0,int n0,AtomInfoType *atInfo1,int n1)
@@ -751,6 +1103,10 @@ void AtomInfoUniquefyNames(PyMOLGlobals *G,AtomInfoType *atInfo0,int n0,AtomInfo
   ai1=atInfo1;
   lai0=NULL; /* last atom compared against in each object */
   lai1=NULL;
+  st0 = 0;
+  nd0 = 0;
+  st1 = 0;
+  nd1 = 0;
   c=1;
   /* ai1->name is the atom we're currently on */
 
@@ -774,9 +1130,10 @@ void AtomInfoUniquefyNames(PyMOLGlobals *G,AtomInfoType *atInfo0,int n0,AtomInfo
         AtomInfoBracketResidue(G,atInfo1,n1,ai1,&st1,&nd1);
         lai1=ai1;
       }
+      
       ai0 = atInfo1 + st1;
       for(a=st1;a<=nd1;a++) {
-        if(strcmp(ai1->name,ai0->name))
+        if(!WordMatchExact(G,ai1->name,ai0->name,true))
           ai0++;
         else if(!AtomInfoSameResidue(G,ai1,ai0))
           ai0++;
@@ -803,7 +1160,7 @@ void AtomInfoUniquefyNames(PyMOLGlobals *G,AtomInfoType *atInfo0,int n0,AtomInfo
         }
         ai0 = atInfo0 + st0;
         for(a=st0;a<=nd0;a++) {
-          if(strcmp(ai1->name,ai0->name))
+          if(!WordMatchExact(G,ai1->name,ai0->name,true))
             ai0++;
           else if(!AtomInfoSameResidue(G,ai1,ai0))
             ai0++;
@@ -835,6 +1192,8 @@ void AtomInfoUniquefyNames(PyMOLGlobals *G,AtomInfoType *atInfo0,int n0,AtomInfo
   }
 }
 
+
+/*========================================================================*/
 /*========================================================================*/
 void AtomInfoBracketResidue(PyMOLGlobals *G,AtomInfoType *ai0,int n0,AtomInfoType *ai,int *st,int *nd)
 {
@@ -1233,6 +1592,7 @@ int AtomInfoGetColor(PyMOLGlobals *G,AtomInfoType *at1)
       case 'a': color=ColorGetIndex(G,"lanthanum"); break;
       case 'i': color=ColorGetIndex(G,"lithium"); break;
       case 'r': color=ColorGetIndex(G,"lawrencium"); break;
+      case 'p': color=ColorGetIndex(G,"lonepair"); break;
       case 'u': color=ColorGetIndex(G,"lutetium"); break;
       } break;
     case 'M': switch(c2) {
@@ -1264,6 +1624,7 @@ int AtomInfoGetColor(PyMOLGlobals *G,AtomInfoType *at1)
       case 'm': color=ColorGetIndex(G,"promethium"); break;
       case 'o': color=ColorGetIndex(G,"polonium"); break;
       case 'r': color=ColorGetIndex(G,"praseodymium"); break;
+      case 's': color=ColorGetIndex(G,"pseudoatom"); break;
       case 't': color=ColorGetIndex(G,"platinum"); break;
       case 'u': color=ColorGetIndex(G,"plutonium"); break;
       } break;
@@ -1402,6 +1763,7 @@ int AtomInfoCompare(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2)
                         result = -1;
                       else if(at1->rank>at2->rank)
                         result = 1;
+                      else result = 0;
                     }
                   } else if((!at2->alt[0])||(at1->alt[0]&&((at1->alt[0]<at2->alt[0])))) {
                     result=-1;
@@ -1423,7 +1785,7 @@ int AtomInfoCompare(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2)
             }
           } else {
             /* NOTE: don't forget to synchronize with below */
-            if(SettingGet(G,cSetting_pdb_insertions_go_first)) {
+            if(SettingGetGlobal_b(G,cSetting_pdb_insertions_go_first)) {
               int sl1,sl2;
               sl1 = strlen(at1->resi);
               sl2 = strlen(at2->resi);
@@ -1433,6 +1795,13 @@ int AtomInfoCompare(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2)
                 result = 1;
               else 
                 result = -1;
+            } else if((at1->rank!=at2->rank)&&
+                      SettingGetGlobal_b(G,cSetting_rank_assisted_sorts)) { 
+              /* use rank to resolve insertion code ambiguities */
+              if(at1->rank<at2->rank)
+                result = -1;
+              else
+                result = 1;
             } else {
               result=wc;
             }
@@ -1457,6 +1826,193 @@ int AtomInfoCompare(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2)
   }
   return(result);
 }
+int AtomInfoCompareIgnoreRankHet(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2)
+{
+  register int result;
+  register int wc = 0;
+  /* order by segment, chain, residue value, residue id, residue name, priority,
+	 and lastly by name */
+
+  /*  printf(":%s:%s:%d:%s:%s:%i:%s =? ",
+	  at1->segi,at1->chain,at1->resv,at1->resi,at1->resn,at1->priority,at1->name);
+	  printf(":%s:%s:%d:%s:%s:%i:%s\n",
+	  at2->segi,at2->chain,at2->resv,at2->resi,at2->resn,at2->priority,at2->name);
+  */
+  {
+    /* attempt to optimize the segi comparison for equality */
+    register char *p1,*p2;
+    p1=at1->segi;
+    p2=at2->segi;
+    if((p1[0]!=p2[0]) ||
+       (p1[0] && ((p1[1]!=p2[1]) ||
+		  (p1[1] && ((p1[2]!=p2[2]) ||
+			     (p1[2] && (p1[3]!=p2[3])))))))
+      wc=WordCompare(G,p1,p2,true);
+  }
+  if(!wc) {
+	 if(at1->chain[0]==at2->chain[0]) {
+       if(at1->resv==at2->resv) {
+         wc=WordCompare(G,at1->resi,at2->resi,true);
+         if(!wc) {
+           wc=WordCompare(G,at1->resn,at2->resn,true);
+           if(!wc) {
+             if(at1->discrete_state==at2->discrete_state) {
+               if(at1->priority==at2->priority) {
+                 if(at1->alt[0]==at2->alt[0]) {
+                   result=AtomInfoNameCompare(G,at1->name,at2->name);
+                 } else if((!at2->alt[0])||(at1->alt[0]&&((at1->alt[0]<at2->alt[0])))) {
+                   result=-1;
+                 } else {
+                   result=1;
+                 }
+               } else if(at1->priority<at2->priority) {
+                 result=-1;
+               } else {
+                 result=1;
+               }
+             } else if(at1->discrete_state<at2->discrete_state) {
+               result = -1;
+             } else {
+               result = 1;
+             }
+           } else {
+             result=wc;
+           }
+         } else {
+           /* NOTE: don't forget to synchronize with below */
+           if(SettingGetGlobal_b(G,cSetting_pdb_insertions_go_first)) {
+             int sl1,sl2;
+             sl1 = strlen(at1->resi);
+             sl2 = strlen(at2->resi);
+             if(sl1==sl2)
+               result = wc;
+             else if(sl1<sl2) /* sort residue 188A before 188, etc. */
+               result = 1;
+             else 
+               result = -1;
+           } else if((at1->rank!=at2->rank)&&
+                     SettingGetGlobal_b(G,cSetting_rank_assisted_sorts)) { 
+             /* use rank to resolve insertion code ambiguities */
+             if(at1->rank<at2->rank)
+               result = -1;
+             else
+               result = 1;
+           } else {
+             result=wc;
+           }
+         }
+       } else if(at1->resv<at2->resv) {
+         result=-1;
+       } else {
+         result=1;
+       }
+     } else if((!at2->chain[0])||(at1->chain[0]&&((at1->chain[0]<at2->chain[0])))) {
+      result=-1;
+    } else {
+      result=1;
+    }
+  } else {
+	 result=wc;
+  }
+  return(result);
+}
+
+int AtomInfoCompareIgnoreRank(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2)
+{
+  register int result;
+  register int wc = 0;
+  /* order by segment, chain, residue value, residue id, residue name, priority,
+	 and lastly by name */
+
+  /*  printf(":%s:%s:%d:%s:%s:%i:%s =? ",
+	  at1->segi,at1->chain,at1->resv,at1->resi,at1->resn,at1->priority,at1->name);
+	  printf(":%s:%s:%d:%s:%s:%i:%s\n",
+	  at2->segi,at2->chain,at2->resv,at2->resi,at2->resn,at2->priority,at2->name);
+  */
+  {
+    /* attempt to optimize the segi comparison for equality */
+    register char *p1,*p2;
+    p1=at1->segi;
+    p2=at2->segi;
+    if((p1[0]!=p2[0]) ||
+       (p1[0] && ((p1[1]!=p2[1]) ||
+		  (p1[1] && ((p1[2]!=p2[2]) ||
+			     (p1[2] && (p1[3]!=p2[3])))))))
+      wc=WordCompare(G,p1,p2,true);
+  }
+  if(!wc) {
+	 if(at1->chain[0]==at2->chain[0]) {
+       if(at1->hetatm==at2->hetatm) {
+         if(at1->resv==at2->resv) {
+           wc=WordCompare(G,at1->resi,at2->resi,true);
+           if(!wc) {
+             wc=WordCompare(G,at1->resn,at2->resn,true);
+             if(!wc) {
+               if(at1->discrete_state==at2->discrete_state) {
+                 if(at1->priority==at2->priority) {
+                   if(at1->alt[0]==at2->alt[0]) {
+                     result=AtomInfoNameCompare(G,at1->name,at2->name);
+                   } else if((!at2->alt[0])||(at1->alt[0]&&((at1->alt[0]<at2->alt[0])))) {
+                     result=-1;
+                   } else {
+                     result=1;
+                   }
+                 } else if(at1->priority<at2->priority) {
+                   result=-1;
+                 } else {
+                   result=1;
+                 }
+               } else if(at1->discrete_state<at2->discrete_state) {
+                 result = -1;
+               } else {
+                 result = 1;
+               }
+             } else {
+               result=wc;
+             }
+           } else {
+             /* NOTE: don't forget to synchronize with below */
+             if(SettingGetGlobal_b(G,cSetting_pdb_insertions_go_first)) {
+               int sl1,sl2;
+               sl1 = strlen(at1->resi);
+               sl2 = strlen(at2->resi);
+               if(sl1==sl2)
+                 result = wc;
+               else if(sl1<sl2) /* sort residue 188A before 188, etc. */
+                 result = 1;
+               else 
+                 result = -1;
+             } else if((at1->rank!=at2->rank)&&
+                       SettingGetGlobal_b(G,cSetting_rank_assisted_sorts)) { 
+               /* use rank to resolve insertion code ambiguities */
+               if(at1->rank<at2->rank)
+                 result = -1;
+               else 
+                 result = 1;
+             } else {
+               result=wc;
+             }
+           }
+         } else if(at1->resv<at2->resv) {
+           result=-1;
+         } else {
+           result=1;
+         }
+       } else if(at2->hetatm) {
+         result=-1;
+       } else {
+         result=1;
+       }
+    } else if((!at2->chain[0])||(at1->chain[0]&&((at1->chain[0]<at2->chain[0])))) {
+      result=-1;
+    } else {
+      result=1;
+    }
+  } else {
+	 result=wc;
+  }
+  return(result);
+}
 
 
 int AtomInfoCompareIgnoreHet(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2)
@@ -1474,8 +2030,7 @@ int AtomInfoCompareIgnoreHet(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2
 
   wc=WordCompare(G,at1->segi,at2->segi,true);
   if(!wc) {
-	 if(at1->chain[0]==at2->chain[0]) {
-      /*      if(at1->hetatm==at2->hetatm) {*/
+    if(at1->chain[0]==at2->chain[0]) {
       if(at1->resv==at2->resv) {
         wc=WordCompare(G,at1->resi,at2->resi,true);
         if(!wc) {
@@ -1489,6 +2044,8 @@ int AtomInfoCompareIgnoreHet(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2
                       result = -1;
                     else if(at1->rank>at2->rank)
                       result = 1;
+                    else
+                      result = 0;
                   }
                 } else if((!at2->alt[0])||(at1->alt[0]&&((at1->alt[0]<at2->alt[0])))) {
                   result=-1;
@@ -1511,7 +2068,7 @@ int AtomInfoCompareIgnoreHet(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2
         } else {
           /* NOTE: don't forget to synchronize with above */
 
-            if(SettingGet(G,cSetting_pdb_insertions_go_first)) {
+            if(SettingGetGlobal_b(G,cSetting_pdb_insertions_go_first)) {
               int sl1,sl2;
               sl1 = strlen(at1->resi);
               sl2 = strlen(at2->resi);
@@ -1521,6 +2078,13 @@ int AtomInfoCompareIgnoreHet(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2
                 result = 1;
               else 
                 result = -1;
+            } else if((at1->rank!=at2->rank)&&
+                      SettingGetGlobal_b(G,cSetting_rank_assisted_sorts)) { 
+              /* use rank to resolve insertion code ambiguities */
+              if(at1->rank<at2->rank)
+                result = -1;
+              else
+                result = 1;
             } else {
               result=wc;
             }
@@ -1530,12 +2094,6 @@ int AtomInfoCompareIgnoreHet(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2
       } else {
         result=1;
       }
-      /*
-        } else if(at2->hetatm) {
-        result=-1;
-        } else {
-        result=1;
-        }*/
     } else if((!at2->chain[0])||(at1->chain[0]&&((at1->chain[0]<at2->chain[0])))) {
       result=-1;
     } else {
@@ -1566,23 +2124,6 @@ int AtomInfoNameOrder(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2)
   return(result);
 }
 
-int AtomInfoInOrder(PyMOLGlobals *G,AtomInfoType *atom,int atom1,int atom2)
-{
-  return(AtomInfoCompare(G,atom+atom1,atom+atom2)<=0);
-}
-
-int AtomInfoInOrderIgnoreHet(PyMOLGlobals *G,AtomInfoType *atom,int atom1,int atom2)
-{
-  return(AtomInfoCompareIgnoreHet(G,atom+atom1,atom+atom2)<=0);
-}
-
-int AtomInfoInOrigOrder(PyMOLGlobals *G,AtomInfoType *atom,int atom1,int atom2)
-{
-  if(atom[atom1].rank!=atom[atom2].rank)
-    return (atom[atom1].rank<atom[atom2].rank);
-  else
-    return(AtomInfoCompare(G,atom+atom1,atom+atom2)<=0);
-}
 
 int AtomInfoSameResidue(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2)
 {
@@ -1629,32 +2170,51 @@ int AtomInfoSameSegmentP(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2)
   return 0;
 }
 
-int AtomInfoSequential(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2)
+int AtomInfoSequential(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2,int mode)
 {
   char last1=0,last2=0;
   char *p;
-  if(at1->hetatm==at2->hetatm)
-    if(at1->chain[0]==at2->chain[0]) {
-      if(WordMatch(G,at1->segi,at2->segi,true)<0) {
-        if(at1->resv==at2->resv) {
-          p=at1->resi;
-          while(*p) {
-            last1=(*p++);
+  if(mode>0) {
+    if(at1->hetatm==at2->hetatm) {
+      if(mode>1) {
+        if(WordMatch(G,at1->segi,at2->segi,true)<0) {
+          if(mode>2) {
+            if(at1->chain[0]==at2->chain[0]) {
+              if(mode>3) {
+                if(at1->resv==at2->resv) {
+                  if(mode>4) {
+                    p=at1->resi;
+                    while(*p) {
+                      last1=(*p++);
+                    }
+                    p=at2->resi;
+                    while(*p) {
+                      last2=(*p++);
+                    }
+                    if(last1==last2)
+                      return 1;
+                    if((last1+1)==last2)
+                      return 1;
+                  } else {
+                    return 1; /* no resi check */
+                  }
+                } else if((at1->resv+1)==at2->resv)
+                  return 1; 
+              } else {
+                return 1; /* no resv check */
+              }
+            }
+          } else {
+            return 1; /* no chain check */
           }
-          p=at2->resi;
-          while(*p) {
-            last2=(*p++);
-          }
-          if(last1==last2)
-            return 1;
-          if((last1+1)==last2)
-            return 1;
-        } else {
-          if((at1->resv+1)==at2->resv)
-            return 1;
-        }
+        } 
+      } else {
+        return 1; /* no segi check */
       }
     }
+  } else {
+    return 1; /* no hetatm check */
+  }
   return 0;
 }
 
@@ -1670,6 +2230,19 @@ int AtomInfoMatch(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2)
   return 0;
 }
 
+int AtomInfoIsFreeCation(PyMOLGlobals *G,AtomInfoType *I) {
+  switch(I->protons) {
+  case cAN_Na:
+  case cAN_K:
+  case cAN_Ca:
+  case cAN_Mg:
+  case cAN_Mn: 
+  case cAN_Sr:
+    return true;
+  }
+  return false;
+}
+
 int AtomInfoGetExpectedValence(PyMOLGlobals *G,AtomInfoType *I) {
   int result=-1; /* negative indicates minimum expected valence (abs)
                 but it could be higher  */
@@ -1766,6 +2339,8 @@ static void set_protons(AtomInfoType *I,char *elem)
       switch(e[1]) {
       case 'R':
       case 'r': I->protons=cAN_Br; break;
+      case 'A':
+      case 'a': I->protons=cAN_Ba; break;
       }
       break;
     case 'F':
@@ -1785,12 +2360,26 @@ static void set_protons(AtomInfoType *I,char *elem)
         break;
       }
       break;
+    case 'L': 
+      switch(e[1]) {
+      case 'P':
+      case 'p':
+        I->protons=cAN_LP; break;
+        break;
+      }
+      break;
     case 'M':
       switch(e[1]) {
       case 'G':
       case 'g': I->protons=cAN_Mg; break;
       }
       break;
+    case 'N':
+      switch(e[1]) {
+      case 'A':
+      case 'a': I->protons=cAN_Na; break;
+      }
+      break;
     case 'Z':
       switch(e[1]) {
       case 'N':
@@ -1803,6 +2392,8 @@ static void set_protons(AtomInfoType *I,char *elem)
       case 'i': I->protons=cAN_Si; break;
       case 'E':
       case 'e': I->protons=cAN_Se; break;
+      case 'R':
+      case 'r': I->protons=cAN_Sr; break;
       }
       break;
     default: /* unrecognized element (possible garbage?) */
@@ -1952,15 +2543,26 @@ void AtomInfoAssignParameters(PyMOLGlobals *G,AtomInfoType *I)
             pri=0;
             n++;
             while(*n) {
+              if(*n=='P') {
+                pri-=200;
+                break;
+              } else if((*n=='*')||(*n=='\'')) {
+                pri=(-100)-pri;
+                break;
+              } else if((*n<'0')||(*n>'9'))
+                break;
               pri*=10;
               pri+=(*n-'0');
               n++;
             }
-            pri+=25;
+            pri+=300;
             break;
           default: pri = 500; break;
           }
         break;
+      case 'P': /* this will place the phosphate before CNO numbered atoms */
+        pri = 20;
+        break;
       case 'D' :
       case 'H' :
         switch (*(n+1))
@@ -2101,6 +2703,7 @@ void AtomInfoAssignParameters(PyMOLGlobals *G,AtomInfoType *I)
   case cAN_Mn: vdw=1.73F; break;
   case cAN_Mg: vdw=1.73F; break;
   case cAN_Zn: vdw=1.39F; break;
+  case cAN_LP: vdw=0.5F; break; /* lone pairs @ 0.5 same as MOE? */
   default: vdw=1.80F; break;
   }
   
