@@ -32,6 +32,7 @@ Z* -------------------------------------------------------------------
 /* FLAG 4 - Exclude these atoms when performing simulation, minimization */
 #define cAtomFlag_exclude       0x00000010
 
+
 /* FLAGS 4-7 are reserved for additional molecular modeling tasks */
 
 /* FLAGS 8-15 are free for end users to manipulate */
@@ -80,7 +81,7 @@ Z* -------------------------------------------------------------------
 #define cAtomInfoSingle 1
 #define cAtomInfoNone 5
 
-
+#define cAN_LP  0
 #define cAN_H   1
 #define cAN_He  2
 #define cAN_B   5
@@ -102,7 +103,9 @@ Z* -------------------------------------------------------------------
 #define cAN_Zn 30
 #define cAN_Se 34
 #define cAN_Br 35
+#define cAN_Sr 38
 #define cAN_I  53
+#define cAN_Ba 56
 #define cAN_Hg 80
 
 typedef char Chain[2];
@@ -111,13 +114,63 @@ typedef char SegIdent[cSegiLen+1];
 typedef char ResIdent[cResiLen+1];
 typedef char ResName[cResnLen+1];
 typedef char AtomName[cAtomNameLen+1];
+#if 0
 typedef char TextType[cTextTypeLen+1];
 typedef char LabelType[cLabelTypeLen+1];
+#endif
 
 #define cAtomInfoNoType -9999
 
+typedef struct {
+  int index[2];
+  int order;
+  int id;
+  int unique_id; 
+  short int stereo; /* to preserve 2D rep */
+  short int has_setting; /* setting based on unique_id */
+} BondType;
+
 typedef struct AtomInfoType {
   int resv;
+  int customType;
+  int priority;
+  float b,q,vdw,partialCharge;
+  int formalCharge;
+  int atom;       /* obsolete?? */
+  int selEntry;
+  int color;
+  int id; 
+  unsigned int flags;
+  int temp1; /* kludge fields - to remove */
+  int unique_id; /* introduced in version 0.77 */
+  int discrete_state; /* state+1 for atoms in discrete objects */
+  float elec_radius; /* radius for PB calculations */
+  int rank;
+  int atomic_color; /* what color was this atom originally assigned? */
+  int textType;
+  int label;
+
+  /* be careful not to write at these as (int*) */
+
+  signed char visRep[cRepCnt]; 
+  signed char stereo; /* for 2D representation */
+  signed char hydrogen;
+  signed char cartoon; /* 0 = default which is auto (use ssType) */
+
+  signed char hetatm;
+  signed char bonded; 
+  signed char chemFlag;
+  signed char geom;
+
+  signed char valence;
+  signed char deleteFlag;
+  signed char masked;
+  signed char protekted;
+
+  signed char protons;
+  signed char hb_donor; 
+  signed char hb_acceptor;
+  signed char has_setting; /* setting based on unique_id */
   Chain chain;
   Chain alt;
   ResIdent resi;
@@ -125,47 +178,32 @@ typedef struct AtomInfoType {
   ResName resn;
   AtomName name;
   AtomName elem;
-  TextType textType;
-  LabelType label;
   SSType ssType; /* blank or 'L' = turn/loop, 'H' = helix, 'S' = beta-strand/sheet */
-  int hydrogen;
-  int customType;
-  int priority;
-  float b,q,vdw,partialCharge;
-  int formalCharge;
-  signed char hetatm;
-  short int model; /* obsolete?? */
-  int atom;       /* obsolete?? */
-  int selEntry;
-  short int visRep[cRepCnt]; /* why isn't this a char? */
-  int color;
-  int id; 
-  int cartoon; /* 0 = default which is auto (use ssType) */
-  unsigned int flags;
-  signed char bonded; /* be careful not to write at these as (int*) */
-  signed char chemFlag;
-  signed char geom;
-  signed char valence;
-  signed char deleteFlag;
-  signed char masked;
-  signed char protekted;
-  signed char protons;
-  signed char hb_donor; 
-  signed char hb_acceptor;
-
-  int temp1; /* kludge field - to remove */
-  int sculpt_id; /* introduced in version 0.77 */
-  int stereo; /* for 2D representation  (NOTE: could probably be converted to signed char */
-  int discrete_state; /* state+1 for atoms in discrete objects */
-  float bohr_radius;
-  int rank;
-  int atomic_color; /* what color was this atom originally assigned? */
 } AtomInfoType;
 
 void AtomInfoFree(PyMOLGlobals *G);
 int AtomInfoInit(PyMOLGlobals *G);
+void AtomInfoPurge(PyMOLGlobals *G,AtomInfoType *ai);
+void AtomInfoCopy(PyMOLGlobals *G,AtomInfoType *src,AtomInfoType *dst);
+int AtomInfoReserveUniqueID(PyMOLGlobals *G,int unique_id);
+int AtomInfoIsUniqueIDActive(PyMOLGlobals *G,int unique_id);
+int AtomInfoGetNewUniqueID(PyMOLGlobals *G);
 
+int AtomInfoCheckSetting(PyMOLGlobals *G, AtomInfoType *ai, int setting_id);
+int AtomInfoGetSetting_b(PyMOLGlobals *G, AtomInfoType *ai, int setting_id, int current, int *effective);
+int AtomInfoGetSetting_i(PyMOLGlobals *G, AtomInfoType *ai, int setting_id, int current, int *effective);
+int AtomInfoGetSetting_f(PyMOLGlobals *G, AtomInfoType *ai, int setting_id, float current, float *effective);
+int AtomInfoGetSetting_color(PyMOLGlobals *G, AtomInfoType *ai, int setting_id, int current, int *effective);
 
+void AtomInfoBondCopy(PyMOLGlobals *G, BondType *src, BondType *dst);
+
+int AtomInfoCheckBondSetting(PyMOLGlobals *G, BondType *bi, int setting_id);
+int AtomInfoGetBondSetting_b(PyMOLGlobals *G, BondType *ai, int setting_id, int current, int *effective);
+int AtomInfoGetBondSetting_i(PyMOLGlobals *G, BondType *ai, int setting_id, int current, int *effective);
+int AtomInfoGetBondSetting_f(PyMOLGlobals *G, BondType *ai, int setting_id, float current, float *effective);
+int AtomInfoGetBondSetting_color(PyMOLGlobals *G, BondType *ai, int setting_id, int current, int *effective);
+
+int AtomInfoCheckUniqueID(PyMOLGlobals *G, AtomInfoType *ai);
 int *AtomInfoGetSortedIndex(PyMOLGlobals *G,AtomInfoType *rec,int n,int **outdex);
 void AtomInfoAssignParameters(PyMOLGlobals *G,AtomInfoType *I);
 void AtomInfoFreeSortedIndexes(PyMOLGlobals *G,int *index,int *outdex);
@@ -173,17 +211,23 @@ void AtomInfoPrimeColors(PyMOLGlobals *G);
 void AtomInfoAssignColors(PyMOLGlobals *G,AtomInfoType *at1);
 int AtomInfoGetColor(PyMOLGlobals *G,AtomInfoType *at1);
 int AtomInfoGetExpectedValence(PyMOLGlobals *G,AtomInfoType *I);
+int AtomInfoIsFreeCation(PyMOLGlobals *G,AtomInfoType *I);
 PyObject *AtomInfoAsPyList(PyMOLGlobals *G,AtomInfoType *at);
 int AtomInfoFromPyList(PyMOLGlobals *G,AtomInfoType *at,PyObject *list);
 int AtomInfoMatch(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2);
 int AtomInfoCompare(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2);
+int AtomInfoCompareIgnoreRank(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2);
 int AtomInfoCompareIgnoreHet(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2);
+int AtomInfoCompareIgnoreRankHet(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2);
 float AtomInfoGetBondLength(PyMOLGlobals *G,AtomInfoType *ai1,AtomInfoType *ai2);
 int AtomInfoSameResidue(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2);
 int AtomInfoSameResidueP(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2);
 int AtomInfoSameChainP(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2);
 int AtomInfoSameSegmentP(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2);
-int AtomInfoSequential(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2);
+int AtomInfoSequential(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2,int mode);
+
+int AtomInfoCheckUniqueBondID(PyMOLGlobals *G, BondType *bi);
+void AtomInfoPurgeBond(PyMOLGlobals *G,BondType *bi);
 
 void AtomInfoBracketResidue(PyMOLGlobals *G,AtomInfoType *ai0,int n0,AtomInfoType *ai,int *st,int *nd);
 void AtomInfoBracketResidueFast(PyMOLGlobals *G,AtomInfoType *ai0,int n0,int cur,int *st,int *nd);
@@ -225,201 +269,5 @@ typedef struct  {
   int next;
 } SSEntry;
 
-/* legacy binary file support */
-#if 0
-
-typedef struct AtomInfoType068 {
-  int resv;
-  Chain chain;
-  Chain alt;
-  ResIdent resi;
-  SegIdent segi;
-  ResName resn;
-  AtomName name;
-  AtomName elem;
-  TextType textType;
-  LabelType label;
-  SSType ssType; /* blank or 'L' = turn/loop, 'H' = helix, 'S' = beta-strand/sheet */
-  int hydrogen;
-  int customType;
-  int priority;
-  float b,q,vdw,partialCharge;
-  int formalCharge;
-  signed char hetatm;
-  short int model; 
-  int atom;
-  int selEntry;
-  short int visRep[cRepCnt];
-  int color;
-  int id; 
-  int cartoon; /* 0 = default which is auto (use ssType) */
-  unsigned int flags;
-  signed char bonded; /* be careful not to write at these as (int*) */
-  signed char chemFlag;
-  signed char geom;
-  signed char valence;
-  signed char deleteFlag;
-  signed char masked;
-  signed char protekted;
-  signed char protons;
-} AtomInfoType068;
-
-typedef struct AtomInfoType076 {
-  int resv;
-  Chain chain;
-  Chain alt;
-  ResIdent resi;
-  SegIdent segi;
-  ResName resn;
-  AtomName name;
-  AtomName elem;
-  TextType textType;
-  LabelType label;
-  SSType ssType; /* blank or 'L' = turn/loop, 'H' = helix, 'S' = beta-strand/sheet */
-  int hydrogen;
-  int customType;
-  int priority;
-  float b,q,vdw,partialCharge;
-  int formalCharge;
-  signed char hetatm;
-  short int model; 
-  int atom;
-  int selEntry;
-  short int visRep[cRepCnt];
-  int color;
-  int id; 
-  int cartoon; /* 0 = default which is auto (use ssType) */
-  unsigned int flags;
-  signed char bonded; /* be careful not to write at these as (int*) */
-  signed char chemFlag;
-  signed char geom;
-  signed char valence;
-  signed char deleteFlag;
-  signed char masked;
-  signed char protekted;
-  signed char protons;
-  int temp1; /* kludge field - to remove */
-} AtomInfoType076;
-
-typedef struct AtomInfoType083 {
-  int resv;
-  Chain chain;
-  Chain alt;
-  ResIdent resi;
-  SegIdent segi;
-  ResName resn;
-  AtomName name;
-  AtomName elem;
-  TextType textType;
-  LabelType label;
-  SSType ssType; /* blank or 'L' = turn/loop, 'H' = helix, 'S' = beta-strand/sheet */
-  int hydrogen;
-  int customType;
-  int priority;
-  float b,q,vdw,partialCharge;
-  int formalCharge;
-  signed char hetatm;
-  short int model; 
-  int atom;
-  int selEntry;
-  short int visRep[cRepCnt];
-  int color;
-  int id; 
-  int cartoon; /* 0 = default which is auto (use ssType) */
-  unsigned int flags;
-  signed char bonded; /* be careful not to write at these as (int*) */
-  signed char chemFlag;
-  signed char geom;
-  signed char valence;
-  signed char deleteFlag;
-  signed char masked;
-  signed char protekted;
-  signed char protons;
-  int temp1; /* kludge field - to remove */
-  int sculpt_id; /* introduced in version 0.77 */
-  int stereo; /* for 2D representation */
-} AtomInfoType083;
-
-typedef struct AtomInfoType090 {
-  int resv;
-  Chain chain;
-  Chain alt;
-  ResIdent resi;
-  SegIdent segi;
-  ResName resn;
-  AtomName name;
-  AtomName elem;
-  TextType textType;
-  LabelType label;
-  SSType ssType; /* blank or 'L' = turn/loop, 'H' = helix, 'S' = beta-strand/sheet */
-  int hydrogen;
-  int customType;
-  int priority;
-  float b,q,vdw,partialCharge;
-  int formalCharge;
-  signed char hetatm;
-  short int model; /* obsolete?? */
-  int atom;       /* obsolete?? */
-  int selEntry;
-  short int visRep[cRepCnt];
-  int color;
-  int id; 
-  int cartoon; /* 0 = default which is auto (use ssType) */
-  unsigned int flags;
-  signed char bonded; /* be careful not to write at these as (int*) */
-  signed char chemFlag;
-  signed char geom;
-  signed char valence;
-  signed char deleteFlag;
-  signed char masked;
-  signed char protekted;
-  signed char protons;
-  int temp1; /* kludge field - to remove */
-  int sculpt_id; /* introduced in version 0.77 */
-  int stereo; /* for 2D representation */
-} AtomInfoType090;
-
-typedef struct AtomInfoType095 {
-  int resv;
-  Chain chain;
-  Chain alt;
-  ResIdent resi;
-  SegIdent segi;
-  ResName resn;
-  AtomName name;
-  AtomName elem;
-  TextType textType;
-  LabelType label;
-  SSType ssType; /* blank or 'L' = turn/loop, 'H' = helix, 'S' = beta-strand/sheet */
-  int hydrogen;
-  int customType;
-  int priority;
-  float b,q,vdw,partialCharge;
-  int formalCharge;
-  signed char hetatm;
-  short int model; /* obsolete?? */
-  int atom;       /* obsolete?? */
-  int selEntry;
-  short int visRep[cRepCnt];
-  int color;
-  int id; 
-  int cartoon; /* 0 = default which is auto (use ssType) */
-  unsigned int flags;
-  signed char bonded; /* be careful not to write at these as (int*) */
-  signed char chemFlag;
-  signed char geom;
-  signed char valence;
-  signed char deleteFlag;
-  signed char masked;
-  signed char protekted;
-  signed char protons;
-  int temp1; /* kludge field - to remove */
-  int sculpt_id; /* introduced in version 0.77 */
-  int stereo; /* for 2D representation */
-  int hb_donor;
-  int hb_acceptor;
-} AtomInfoType095;
-
-#endif
 
 #endif
