@@ -28,7 +28,8 @@ Z* -------------------------------------------------------------------
 #include"ButMode.h"
 #include"Matrix.h"
 #include"Sphere.h"
-
+#include"Util.h"
+#include"Feedback.h"
 #include"RepWireBond.h"
 #include"RepCylBond.h"
 #include"RepDot.h"
@@ -43,10 +44,10 @@ Z* -------------------------------------------------------------------
 
 #include"PyMOLGlobals.h"
 
-void CoordSetUpdate(CoordSet *I);
+static void CoordSetUpdate(CoordSet *I,int state);
 
 void CoordSetFree(CoordSet *I);
-void CoordSetRender(CoordSet *I,CRay *ray,Pickable **pick,int pass);
+void CoordSetRender(CoordSet *I,RenderInfo *info);
 void CoordSetEnumIndices(CoordSet *I);
 void CoordSetStrip(CoordSet *I);
 void CoordSetInvalidateRep(CoordSet *I,int type,int level);
@@ -121,7 +122,9 @@ int CoordSetFromPyList(PyMOLGlobals *G,PyObject *list,CoordSet **cs)
         ok = PConvPyListToIntArray(tmp,&I->AtmToIdx);
     }
     if(ok&&(ll>5)) ok = PConvPyStrToStr(PyList_GetItem(list,5),I->Name,sizeof(WordType));
-    if(ok&&(ll>6))  ok = ObjectStateFromPyList(G,PyList_GetItem(list,6),&I->State);
+    if(ok&&(ll>6)) ok = ObjectStateFromPyList(G,PyList_GetItem(list,6),&I->State);
+    if(ok&&(ll>7)) I->Setting = SettingNewFromPyList(G,PyList_GetItem(list,7));
+    if(ok&&(ll>8)) ok = PConvPyListToLabPosVLA(PyList_GetItem(list,8),&I->LabPos);
     if(!ok) {
       if(I)
         CoordSetFree(I);
@@ -142,7 +145,7 @@ PyObject *CoordSetAsPyList(CoordSet *I)
   PyObject *result = NULL;
 
   if(I) {
-    result = PyList_New(7);
+    result = PyList_New(9);
     
     PyList_SetItem(result,0,PyInt_FromLong(I->NIndex));
     PyList_SetItem(result,1,PyInt_FromLong(I->NAtIndex));
@@ -154,7 +157,9 @@ PyObject *CoordSetAsPyList(CoordSet *I)
       PyList_SetItem(result,4,PConvAutoNone(NULL));
     PyList_SetItem(result,5,PyString_FromString(I->Name));
     PyList_SetItem(result,6,ObjectStateAsPyList(&I->State));
-    /* TODO symmetry, spheroid, setting, periodic box ... */
+    PyList_SetItem(result,7,SettingAsPyList(I->Setting));
+    PyList_SetItem(result,8,PConvLabPosVLAToPyList(I->LabPos,I->NIndex));
+    /* TODO symmetry, spheroid, periodic box ... */
   }
   return(PConvAutoNone(result));
 #endif
@@ -205,6 +210,13 @@ void CoordSetMerge(CoordSet *I,CoordSet *cs) /* must be non-overlapping */
     I->AtmToIdx[cs->IdxToAtm[a]] = i0;
     copy3f(cs->Coord+a*3,I->Coord+i0*3);
   }
+  if(cs->LabPos) {
+    if(!I->LabPos) 
+      I->LabPos = VLACalloc(LabPosType,I->NIndex);
+    if(I->LabPos) {
+      UtilCopyMem(I->LabPos+I->NIndex,cs->LabPos,sizeof(LabPosType)*cs->NIndex);
+    }
+  }
   if(I->fInvalidateRep)
     I->fInvalidateRep(I,cRepAll,cRepInvAll);
   I->NIndex = nIndex;
@@ -218,14 +230,15 @@ void CoordSetPurge(CoordSet *I)
   AtomInfoType *ai;
   ObjectMolecule *obj;
   float *c0,*c1;
+  LabPosType *l0,*l1;
   obj=I->Obj;
 
   PRINTFD(I->State.G,FB_CoordSet)
     " CoordSetPurge-Debug: entering..."
     ENDFD;
 
-  c0 = I->Coord;
-  c1 = I->Coord;
+  c0 = c1 = I->Coord;
+  l0 = l1 = I->LabPos;
 
   for(a=0;a<I->NIndex;a++) {
     a1 = I->IdxToAtm[a];
@@ -233,21 +246,32 @@ void CoordSetPurge(CoordSet *I)
     if(ai->deleteFlag) {
       offset--;
       c0+=3;
+      if(l0)
+        l0++;
     } else if(offset) {
         ao=a+offset;
         *(c1++)=*(c0++);
         *(c1++)=*(c0++);
         *(c1++)=*(c0++);
+        if(l0) {
+          *(l1++) = *(l0++);
+        }
         I->AtmToIdx[a1] = ao;
         I->IdxToAtm[ao] = a1; /* no adjustment of these indexes yet...*/
     } else {
       c0+=3;
       c1+=3;
+      if(l0) {
+        l0++;
+        l1++;
+      }
     }
   }
   if(offset) {
     I->NIndex+=offset;
     VLASize(I->Coord,float,I->NIndex*3);
+    if(I->LabPos)
+      VLASize(I->LabPos,LabPosType,I->NIndex);
     I->IdxToAtm=Realloc(I->IdxToAtm,int,I->NIndex);
     PRINTFD(I->State.G,FB_CoordSet)
       " CoordSetPurge-Debug: I->IdxToAtm shrunk to %d\n",I->NIndex
@@ -265,7 +289,7 @@ void CoordSetPurge(CoordSet *I)
 int CoordSetTransformAtomTTTf(CoordSet *I,int at,float *TTT)
 {
   ObjectMolecule *obj;
-  int a1 = 01;
+  int a1 = -1;
   int result = 0;
   float *v1;
 
@@ -289,7 +313,7 @@ int CoordSetTransformAtomTTTf(CoordSet *I,int at,float *TTT)
 int CoordSetTransformAtomR44f(CoordSet *I,int at,float *matrix)
 {
   ObjectMolecule *obj;
-  int a1 = 01;
+  int a1 = -1;
   int result = 0;
   float *v1;
 
@@ -335,7 +359,7 @@ void CoordSetRecordTxfApplied(CoordSet *I,float *matrix,int homogenous)
 int CoordSetMoveAtom(CoordSet *I,int at,float *v,int mode)
 {
   ObjectMolecule *obj;
-  int a1 = 01;
+  int a1 = -1;
   int result = 0;
   float *v1;
 
@@ -345,7 +369,7 @@ int CoordSetMoveAtom(CoordSet *I,int at,float *v,int mode)
       a1=obj->DiscreteAtmToIdx[at];
   } else 
     a1=I->AtmToIdx[at];
-  
+
   if(a1>=0) {
     result = 1;
     v1 = I->Coord+3*a1;
@@ -359,11 +383,47 @@ int CoordSetMoveAtom(CoordSet *I,int at,float *v,int mode)
   return(result);
 }
 /*========================================================================*/
-int CoordSetGetAtomVertex(CoordSet *I,int at,float *v)
+int CoordSetMoveAtomLabel(CoordSet *I,int at,float *v,int mode)
 {
   ObjectMolecule *obj;
   int a1 = -1;
   int result = 0;
+  LabPosType *lp;
+
+  obj = I->Obj;
+  if(obj->DiscreteFlag) {
+    if(I==obj->DiscreteCSet[at])
+      a1=obj->DiscreteAtmToIdx[at];
+  } else 
+    a1=I->AtmToIdx[at];
+  
+  if(a1>=0) {
+    if(!I->LabPos) 
+      I->LabPos = VLACalloc(LabPosType,I->NIndex);
+    if(I->LabPos) {
+      result = 1;
+      lp = I->LabPos+a1;
+      if(!lp->mode) {
+        float *lab_pos = SettingGet_3fv(obj->Obj.G,I->Setting,obj->Obj.Setting,cSetting_label_position);
+        copy3f(lab_pos,lp->pos);
+      }
+      lp->mode=1;
+      if(mode) {
+        add3f(v,lp->offset,lp->offset);
+      } else { 
+        copy3f(v,lp->offset);
+      }
+    }
+  }
+
+  return(result);
+}
+/*========================================================================*/
+int CoordSetGetAtomVertex(CoordSet *I,int at,float *v)
+{
+  register ObjectMolecule *obj;
+  register int a1 = -1;
+  register int result = 0;
 
   obj = I->Obj;
   if(obj->DiscreteFlag) {
@@ -379,11 +439,41 @@ int CoordSetGetAtomVertex(CoordSet *I,int at,float *v)
 
   return(result);
 }
+/*========================================================================*/
+int CoordSetGetAtomTxfVertex(CoordSet *I,int at,float *v)
+{
+  register ObjectMolecule *obj;
+  register int a1 = -1;
+  register int result = 0;
+  
+  obj = I->Obj;
+  if(obj->DiscreteFlag) {
+    if(I==obj->DiscreteCSet[at])
+      a1=obj->DiscreteAtmToIdx[at];
+  } else 
+    a1=I->AtmToIdx[at];
+  
+  if(a1>=0) {
+    result = 1;
+    copy3f(I->Coord+3*a1,v);
+    if(I->State.Matrix && SettingGet_b(I->State.G,
+                                       obj->Obj.Setting,I->Setting,
+                                       cSetting_matrix_mode)>0) {
+      /* apply state transformation */
+      transform44d3f(I->State.Matrix,v,v);
+    }
+    if(obj->Obj.TTTFlag) { /* object transformation */
+      transformTTT44f3f(obj->Obj.TTT, v,v);
+    }
+  }
+  return(result);
+}
+
 /*========================================================================*/
 int CoordSetSetAtomVertex(CoordSet *I,int at,float *v)
 {
   ObjectMolecule *obj;
-  int a1 = 01;
+  int a1 = -1;
   int result = 0;
 
   obj = I->Obj;
@@ -480,6 +570,7 @@ void CoordSetAtomToPDBStrVLA(PyMOLGlobals *G,char **charVLA,int *c,AtomInfoType
   int rl;
   int literal = (int)SettingGet(G,cSetting_pdb_literal_names);
   int reformat = (int)SettingGet(G,cSetting_pdb_reformat_names_mode);
+  WordType x,y,z;
 
   strcpy(resn,ai->resn); 
   if(SettingGetGlobal_b(G,cSetting_pdb_truncate_residue_name)) {
@@ -529,6 +620,7 @@ void CoordSetAtomToPDBStrVLA(PyMOLGlobals *G,char **charVLA,int *c,AtomInfoType
                 strcpy(name+1,ai->name);
               }
               break;
+            case 4:
             default: /* otherwise, start in column 1 */
               name[0]=' ';	
               strcpy(name+1,ai->name);
@@ -580,6 +672,7 @@ void CoordSetAtomToPDBStrVLA(PyMOLGlobals *G,char **charVLA,int *c,AtomInfoType
               strcpy(name,ai->name);
             }
             break;
+          case 4:
           default: /* no changes */
             strcpy(name,ai->name);
             break;
@@ -612,11 +705,25 @@ void CoordSetAtomToPDBStrVLA(PyMOLGlobals *G,char **charVLA,int *c,AtomInfoType
     }
   } else { /* LITERAL mode: preserve what was in the original PDB as best PyMOL can 
             this should enable people to open and save amber pdb files without issues */
-    if(strlen(ai->name)<4) { /* under length 4? */
-      name[0] = ' ';
-      strcpy(name+1,ai->name); /* stick in the second column */
-    } else {
-      strcpy(name,ai->name); /* otherwise, stick in the first */
+    if(strlen(ai->name)==4) {
+      strcpy(name,ai->name); /* save literal contents of field */
+    } else { /* under length 4? check match with atomic symbol */
+      if((toupper(ai->elem[0])==toupper(ai->name[0]))&&
+         ((!ai->elem[1]) || /* symbol len = 1 */
+          (toupper(ai->elem[1])==toupper(ai->name[1])))) { /* matched len 2 */
+        /* starts with corrent atomic symbol, so */
+        if(strlen(ai->elem)>1) { /* if atom symbol is length 2 */
+          strcpy(name,ai->name); /* then start in column 0 */
+        } else { 
+          /* otherwise, start in column 1 */
+          name[0]=' ';	
+          strcpy(name+1,ai->name);
+        } 
+      } else {
+        /* otherwise, start in column 1 */
+        name[0]=' ';	
+        strcpy(name+1,ai->name);
+      }
     }
   }
   if((int)SettingGet(G,cSetting_pdb_retain_ids)) {
@@ -626,21 +733,45 @@ void CoordSetAtomToPDBStrVLA(PyMOLGlobals *G,char **charVLA,int *c,AtomInfoType
     cnt=99998;
 
   if((!pdb_info)||(!pdb_info->is_pqr_file)) { /* relying upon short-circuit */
-
-    (*c)+=sprintf((*charVLA)+(*c),"%6s%5i %-4s%1s%-4s%1s%5s   %8.3f%8.3f%8.3f%6.2f%6.2f      %-4s%2s\n",
+    sprintf(x,"%8.3f",v[0]); x[8]=0;
+    sprintf(y,"%8.3f",v[1]); y[8]=0;
+    sprintf(z,"%8.3f",v[2]); z[8]=0;
+    (*c)+=sprintf((*charVLA)+(*c),"%6s%5i %-4s%1s%-4s%1s%5s   %s%s%s%6.2f%6.2f      %-4s%2s\n",
                   aType,cnt+1,name,ai->alt,resn,
-                  ai->chain,resi,*v,*(v+1),*(v+2),ai->q,ai->b,ai->segi,ai->elem);
+                  ai->chain,resi,x,y,z,ai->q,ai->b,ai->segi,ai->elem);
   } else {
-    if(pdb_info->pqr_no_chain_id) {
+    if(pdb_info->is_pqr_file && pdb_info->pqr_workarounds) {
+      int non_num = false;
+      char *p = resi;
+      while(*p) {
+        if( (((*p)<='0')||((*p)>='9'))
+            &&(*p!=' ')) {
+          non_num=true;
+          break;
+        }
+        p++;
+      }
+      if(non_num) {
+        sprintf(resi,"%d",ai->resv);
+        rl = strlen(resi)-1;
+        if(rl>=0)
+          if((resi[rl]>='0')&&(resi[rl]<='9')) {
+            resi[rl+1]=' ';
+            resi[rl+2]=0;
+            }
+      }
       chain[0]=0;
     } else {
       chain[0] = ai->chain[0];
       chain[1] = 0;
     }
+    sprintf(x,"%8.3f",v[0]); x[8]=0;
+    sprintf(y,"%8.3f",v[1]); y[8]=0;
+    sprintf(z,"%8.3f",v[2]); z[8]=0;
       
-    (*c)+=sprintf((*charVLA)+(*c),"%6s%5i %-4s%1s%-4s%1s%5s   %8.3f%8.3f%8.3f %7.3f %7.3f\n",
+    (*c)+=sprintf((*charVLA)+(*c),"%6s%5i %-4s%1s%-4s%1s%5s   %s%s%s %11.8f %7.3f\n",
                   aType,cnt+1,name,ai->alt,resn,
-                  chain,resi,*v,*(v+1),*(v+2),ai->partialCharge,ai->bohr_radius);
+                  chain,resi,x,y,z,ai->partialCharge,ai->elec_radius);
   }
   
 }
@@ -672,13 +803,19 @@ PyObject *CoordSetAtomToChemPyAtom(PyMOLGlobals *G,AtomInfoType *ai,float *v,int
     PConvFloatToPyObjAttr(atom,"q",ai->q);
     PConvFloatToPyObjAttr(atom,"b",ai->b);
     PConvFloatToPyObjAttr(atom,"vdw",ai->vdw);
-    PConvFloatToPyObjAttr(atom,"bohr",ai->bohr_radius);
+    PConvFloatToPyObjAttr(atom,"elec_radius",ai->elec_radius);
     PConvFloatToPyObjAttr(atom,"partial_charge",ai->partialCharge);
     PConvIntToPyObjAttr(atom,"formal_charge",ai->formalCharge);
     if(ai->customType!=-9999)
       PConvIntToPyObjAttr(atom,"numeric_type",ai->customType);
-    if(ai->textType[0])
-      PConvStringToPyObjAttr(atom,"text_type",ai->textType);      
+    if(ai->textType) {
+      char null_st[1] = "";
+      char *st = null_st;
+      
+      if(ai->textType) st = OVLexicon_FetchCString(G->Lexicon,ai->textType);
+      PConvStringToPyObjAttr(atom,"text_type",st);
+    }
+
     PConvIntToPyObjAttr(atom,"hetatm",ai->hetatm);      
     PConvIntToPyObjAttr(atom,"flags",ai->flags);      
     PConvIntToPyObjAttr(atom,"id",ai->id);  /* not necc. unique */
@@ -767,11 +904,11 @@ void CoordSetInvalidateRep(CoordSet *I,int type,int level)
     }
   if(level>=cRepInvColor) 
 	 VLAFreeP(I->Color);
-  if(type>=0) {
+  if(type>=0) { /* representation specific */
 	 if(type<I->NRep)	{
       a=type;
       if(I->Rep[a]) {
-        if(I->Rep[a]->fInvalidate) 
+        if(I->Rep[a]->fInvalidate && (level<cRepInvPurge))
           I->Rep[a]->fInvalidate(I->Rep[a],I,level);
         else {
           I->Rep[a]->fFree(I->Rep[a]);
@@ -786,40 +923,48 @@ void CoordSetInvalidateRep(CoordSet *I,int type,int level)
       if(level>=cRepInvVisib) /* make active if visibility has changed */
         I->Active[a]=true;
 		if(I->Rep[a]) {
-        if(I->Rep[a]->fInvalidate) 
-          I->Rep[a]->fInvalidate(I->Rep[a],I,level);
-        else {
-          I->Rep[a]->fFree(I->Rep[a]);
-          I->Rep[a] = NULL;
-        }
+          if(I->Rep[a]->fInvalidate && (level<cRepInvPurge))
+            I->Rep[a]->fInvalidate(I->Rep[a],I,level);
+          else {
+            I->Rep[a]->fFree(I->Rep[a]);
+            I->Rep[a] = NULL;
+          }
 		}
 	 }
   }
+  if(level>=cRepInvCoord) { /* if coordinates change, then this map becomes invalid */
+    MapFree(I->Coord2Idx);
+    I->Coord2Idx = NULL;
+  }
   SceneChanged(I->State.G);
 }
 /*========================================================================*/
 
-#define RepUpdateMacro(I,rep,new_fn) {\
+#define RepUpdateMacro(I,rep,new_fn,state) {\
   if(I->Active[rep]) {\
     if(!I->Rep[rep]) {\
-      I->Rep[rep]=new_fn(I);\
+      I->Rep[rep]=new_fn(I,state);\
       if(I->Rep[rep]) \
-         I->Rep[rep]->fNew=(struct Rep *(*)(struct CoordSet *))new_fn;\
+         I->Rep[rep]->fNew=(struct Rep *(*)(struct CoordSet *,int state))new_fn;\
     } else {\
       if(I->Rep[rep]->fUpdate)\
-         I->Rep[rep] = I->Rep[rep]->fUpdate(I->Rep[rep],I,rep);\
+         I->Rep[rep] = I->Rep[rep]->fUpdate(I->Rep[rep],I,state,rep);\
     }\
   }\
 OrthoBusyFast(I->State.G,rep,I->NRep);\
 }
 /*========================================================================*/
-void CoordSetUpdate(CoordSet *I)
+static void CoordSetUpdate(CoordSet *I,int state)
 {
   int a;
   int i;
   ObjectMolecule *obj;
   obj=I->Obj;
-
+  
+  if(Feedback(I->Obj->Obj.G,FB_CoordSet,FB_Blather)) {
+    printf(" CoordSetUpdate-Entered: object %s state %d cset %p\n",
+           I->Obj->Obj.Name, state, I);
+  }
 
   if(!I->Color) /* colors invalidated */
 	 {
@@ -843,37 +988,83 @@ void CoordSetUpdate(CoordSet *I)
 		}
 	 }
   OrthoBusyFast(I->State.G,0,I->NRep);
-  RepUpdateMacro(I, cRepLine,            RepWireBondNew        );
-  RepUpdateMacro(I, cRepCyl,             RepCylBondNew         );
-  RepUpdateMacro(I, cRepDot,             RepDotNew             );
-  RepUpdateMacro(I, cRepMesh,            RepMeshNew            );
-  RepUpdateMacro(I, cRepSphere,          RepSphereNew          );
-  RepUpdateMacro(I, cRepRibbon,          RepRibbonNew          );
-  RepUpdateMacro(I, cRepCartoon,         RepCartoonNew         );
-  RepUpdateMacro(I, cRepSurface,         RepSurfaceNew         );
-  RepUpdateMacro(I, cRepLabel,           RepLabelNew           );
-  RepUpdateMacro(I, cRepNonbonded,       RepNonbondedNew       );
-  RepUpdateMacro(I, cRepNonbondedSphere, RepNonbondedSphereNew );
+  RepUpdateMacro(I, cRepLine,            RepWireBondNew        , state );
+  RepUpdateMacro(I, cRepCyl,             RepCylBondNew         , state );
+  RepUpdateMacro(I, cRepDot,             RepDotNew             , state );
+  RepUpdateMacro(I, cRepMesh,            RepMeshNew            , state );
+  RepUpdateMacro(I, cRepSphere,          RepSphereNew          , state );
+  RepUpdateMacro(I, cRepRibbon,          RepRibbonNew          , state );
+  RepUpdateMacro(I, cRepCartoon,         RepCartoonNew         , state );
+  RepUpdateMacro(I, cRepSurface,         RepSurfaceNew         , state );
+  RepUpdateMacro(I, cRepLabel,           RepLabelNew           , state );
+  RepUpdateMacro(I, cRepNonbonded,       RepNonbondedNew       , state );
+  RepUpdateMacro(I, cRepNonbondedSphere, RepNonbondedSphereNew , state );
 
   for(a=0;a<I->NRep;a++) 
     if(!I->Rep[a])
       I->Active[a]=false;
 
-  SceneDirty(I->State.G);
+  SceneInvalidate(I->State.G);
   OrthoBusyFast(I->State.G,1,1);
+  if(Feedback(I->Obj->Obj.G,FB_CoordSet,FB_Blather)) {
+    printf(" CoordSetUpdate-Leaving: object %s state %d cset %p\n",
+           I->Obj->Obj.Name, state, I);
+  }
+
 }
 /*========================================================================*/
-void CoordSetRender(CoordSet *I,CRay *ray,Pickable **pick,int pass)
+void CoordSetUpdateCoord2IdxMap(CoordSet *I, float cutoff)
 {
+  if(cutoff<R_SMALL4)
+    cutoff = R_SMALL4;
+  if(I->NIndex>10) {
+    if(I->Coord2Idx) {
+      if((I->Coord2IdxDiv<cutoff)||
+         (((cutoff-I->Coord2IdxReq)/I->Coord2IdxReq)<-0.5F)) {
+        MapFree(I->Coord2Idx);
+        I->Coord2Idx = NULL;
+      }
+    }
+    if(I->NIndex && (!I->Coord2Idx)) { /* NOTE: map based on stored coords */
+      I->Coord2IdxReq = cutoff;
+      I->Coord2IdxDiv = cutoff * 1.25F;
+      I->Coord2Idx = MapNew(I->State.G, I->Coord2IdxDiv, I->Coord, I->NIndex, NULL);
+      if(I->Coord2IdxDiv<I->Coord2Idx->Div) 
+        I->Coord2IdxDiv = I->Coord2Idx->Div;
+    }
+  }
+}
+/*========================================================================*/
+void CoordSetRender(CoordSet *I,RenderInfo *info)
+{
+  PyMOLGlobals *G = I->State.G;
+  int pass = info->pass;
+  CRay *ray = info->ray;
+  Picking **pick = info->pick;
   int a,aa;
   Rep *r;
+  int float_labels = SettingGet_i(G,I->Setting,
+                                  I->Obj->Obj.Setting,
+                                  cSetting_float_labels);
 
-  PRINTFD(I->State.G,FB_CoordSet)
+  PRINTFD(G,FB_CoordSet)
     " CoordSetRender: entered (%p).\n",(void*)I
     ENDFD;
 
   if((!pass)&&I->Name[0])
-    ButModeCaption(I->State.G,I->Name);
+    ButModeCaption(G,I->Name);
+  if((!pass)&&I->SculptCGO&&(I->Obj->Obj.RepVis[cRepCGO])) {
+    if(ray) {
+      CGORenderRay(I->SculptCGO,ray,
+                   ColorGet(G,I->Obj->Obj.Color),
+                   I->Setting,I->Obj->Obj.Setting);
+    } else if(G->HaveGUI && G->ValidContext) {
+      if(!pick) {
+        CGORenderGL(I->SculptCGO,ColorGet(G,I->Obj->Obj.Color),
+                    I->Setting,I->Obj->Obj.Setting,info);
+      }
+    }
+  }
   for(aa=0;aa<I->NRep;aa++) {
     if(aa==cRepSurface) { /* reorder */
       a=cRepCell;
@@ -892,29 +1083,44 @@ void CoordSetRender(CoordSet *I,CRay *ray,Pickable **pick,int pass)
           } else {
             if(I->Obj) 
               ray->fWobble(ray,
-                           SettingGet_i(I->State.G,I->Setting,I->Obj->Obj.Setting,cSetting_ray_texture),
-                           SettingGet_3fv(I->State.G,I->Setting,I->Obj->Obj.Setting,cSetting_ray_texture_settings));
+                           SettingGet_i(G,I->Setting,
+                                        I->Obj->Obj.Setting,
+                                        cSetting_ray_texture),
+                           SettingGet_3fv(G,I->Setting,
+                                          I->Obj->Obj.Setting,
+                                          cSetting_ray_texture_settings));
             else
               ray->fWobble(ray,
-                           SettingGet_i(I->State.G,I->Setting,NULL,cSetting_ray_texture),
-                           SettingGet_3fv(I->State.G,I->Setting,NULL,cSetting_ray_texture_settings));
-            ray->fColor3fv(ray,ColorGet(I->State.G,I->Obj->Obj.Color));
+                           SettingGet_i(G,I->Setting,
+                                        NULL,cSetting_ray_texture),
+                           SettingGet_3fv(G,I->Setting, NULL, 
+                                          cSetting_ray_texture_settings));
+            ray->fColor3fv(ray,ColorGet(G,I->Obj->Obj.Color));
           }
         
           if(r->fRender) { /* do OpenGL rendering in three passes */
             if(ray||pick) {
-              r->fRender(r,ray,pick);                
-            } else 
+              
+              /* here we need to iterate through and apply coordinate set matrices */
+
+              r->fRender(r,info);
+            } else {
+
+              /* here we need to iterate through and apply coordinate set matrices */
+
               switch(a) {
               case cRepLabel:
-                if(pass==1) r->fRender(r,ray,pick);
+                if(float_labels && (pass==-1))
+                  r->fRender(r,info);
+                else if(pass==1)
+                  r->fRender(r,info);                  
                 break;
               case cRepNonbondedSphere:
               case cRepRibbon:
               case cRepDot:
               case cRepCGO:
               case cRepCallback:
-                if(pass==1) r->fRender(r,ray,pick);
+                if(pass==1) r->fRender(r,info);
                 break;
               case cRepLine:
               case cRepMesh:
@@ -922,55 +1128,64 @@ void CoordSetRender(CoordSet *I,CRay *ray,Pickable **pick,int pass)
               case cRepNonbonded:
               case cRepCell:
               case cRepExtent:
-                if(!pass) r->fRender(r,ray,pick);                
+                if(!pass) r->fRender(r,info);                
                 break;
               case cRepCyl: /* render sticks differently depending on transparency */
-                if(SettingGet_f(I->State.G,r->cs->Setting,
+                if(SettingGet_f(G,r->cs->Setting,
                                 r->obj->Setting,
                                 cSetting_stick_transparency)>0.0001) {
                   if(pass==-1)
-                    r->fRender(r,ray,pick);                                
+                    r->fRender(r,info);                                
                 } else if(pass==1)
-                  r->fRender(r,ray,pick);
+                  r->fRender(r,info);
                 break;
 
               case cRepSurface:
-                /*                if(pass==-1) r->fRender(r,ray,pick);              */
-                if(SettingGet_f(I->State.G,r->cs->Setting,
-                                r->obj->Setting,
-                                cSetting_transparency)>0.0001) {
-                  if(pass==-1)
-                    r->fRender(r,ray,pick);                                
-                } else if(pass==1)
-                  r->fRender(r,ray,pick);
+                if(info->alpha_cgo) {
+                   if(pass == 1)
+                    r->fRender(r,info);
+                } else {
+                  if(SettingGet_f(G,r->cs->Setting,
+                                  r->obj->Setting,
+                                  cSetting_transparency)>0.0001) {
+                    if(pass==-1)
+                      r->fRender(r,info);                                
+                  } else if(pass==1)
+                    r->fRender(r,info);
+                }
                 break;
               case cRepSphere: /* render spheres differently depending on transparency */
-                if(SettingGet_f(I->State.G,r->cs->Setting,
+                if(SettingGet_f(G,r->cs->Setting,
                                 r->obj->Setting,
                                 cSetting_sphere_transparency)>0.0001) {
                   if(pass==-1)
-                    r->fRender(r,ray,pick);                                
+                    r->fRender(r,info);                                
                 } else if(pass==1)
-                  r->fRender(r,ray,pick);
+                  r->fRender(r,info);
                 break;
               case cRepCartoon:
-                if(SettingGet_f(I->State.G,r->cs->Setting,
-                                r->obj->Setting,
-                                cSetting_cartoon_transparency)>0.0001) {
-                  if(pass==-1)
-                    r->fRender(r,ray,pick);                                
-                } else if(pass==1)
-                  r->fRender(r,ray,pick);
+                if(info->alpha_cgo) {
+                  if(pass==1)
+                    r->fRender(r,info);
+                } else {
+                  if(SettingGet_f(G,r->cs->Setting,
+                                  r->obj->Setting,
+                                  cSetting_cartoon_transparency)>0.0001) {
+                    if(pass==-1)
+                      r->fRender(r,info);                                
+                  } else if(pass==1)
+                    r->fRender(r,info);
+                }
                 break;
               }
 
-            
+            }
           }
           /*          if(ray)
                       ray->fWobble(ray,0,NULL);*/
         }
   }
-  PRINTFD(I->State.G,FB_CoordSet)
+  PRINTFD(G,FB_CoordSet)
     " CoordSetRender: leaving...\n"
     ENDFD;
 
@@ -1011,7 +1226,12 @@ CoordSet *CoordSetNew(PyMOLGlobals *G)
   I->SpheroidSphereSize = I->State.G->Sphere->Sphere[1]->nDot; /* does this make any sense? */
   for(a=0;a<I->NRep;a++)
 	 I->Rep[a] = NULL;
-  I->Setting=NULL;
+  I->Setting = NULL;
+  I->Coord2Idx = NULL;
+  I->NMatrix = 0;
+  I->MatrixVLA = NULL;
+  I->SculptCGO = NULL;
+  I->LabPos = NULL;
   return(I);
 }
 /*========================================================================*/
@@ -1024,9 +1244,11 @@ CoordSet *CoordSetCopy(CoordSet *cs)
   OOAlloc(cs->State.G,CoordSet);
 
   (*I)=(*cs);
+  ObjectStateCopy(&cs->State,&I->State);
   I->Symmetry=SymmetryCopy(cs->Symmetry);
   if(I->PeriodicBox) I->PeriodicBox=CrystalCopy(I->PeriodicBox);
   I->Coord = VLAlloc(float,I->NIndex*3);
+  
   v0=I->Coord;
   v1=cs->Coord;
   for(a=0;a<I->NIndex;a++) {
@@ -1034,6 +1256,10 @@ CoordSet *CoordSetCopy(CoordSet *cs)
     *(v0++)=*(v1++);
     *(v0++)=*(v1++);
   }
+  if(cs->LabPos) {
+    I->LabPos = VLACalloc(LabPosType,I->NIndex);
+    UtilCopyMem(I->LabPos,cs->LabPos,sizeof(LabPosType)*I->NIndex);
+  }
   if(I->AtmToIdx) {
     nAtom = cs->Obj->NAtom;
     I->AtmToIdx = Alloc(int,nAtom);
@@ -1042,6 +1268,14 @@ CoordSet *CoordSetCopy(CoordSet *cs)
     for(a=0;a<nAtom;a++)
       *(i0++)=*(i1++);
   }
+  
+  
+  if(cs->MatrixVLA) { /* not used yet */
+    I->MatrixVLA = VLAlloc(double,16*cs->NMatrix*sizeof(double));
+    if(I->MatrixVLA) {
+      UtilCopyMem(I->MatrixVLA,cs->MatrixVLA,sizeof(double)*16*cs->NMatrix);
+    }
+  }
 
   I->IdxToAtm = Alloc(int,I->NIndex);
   i0=I->IdxToAtm;
@@ -1062,6 +1296,7 @@ CoordSet *CoordSetCopy(CoordSet *cs)
   I->Color=NULL;
   I->Spheroid=NULL;
   I->SpheroidNormal=NULL;
+  I->Coord2Idx = NULL;
   return(I);
 }
 /*========================================================================*/
@@ -1185,6 +1420,7 @@ void CoordSetFree(CoordSet *I)
     FreeP(I->AtmToIdx);
     FreeP(I->IdxToAtm);
     VLAFreeP(I->Color);
+    MapFree(I->Coord2Idx);
     VLAFreeP(I->Coord);
     /*    VLAFreeP(I->Rep);*/
     VLAFreeP(I->TmpBond);
@@ -1194,6 +1430,8 @@ void CoordSetFree(CoordSet *I)
     FreeP(I->SpheroidNormal);
     SettingFreeP(I->Setting);
     ObjectStatePurge(&I->State);
+    CGOFree(I->SculptCGO);
+    VLAFreeP(I->LabPos);
     OOFreeP(I);
   }
 }
