@@ -22,11 +22,12 @@ Z* -------------------------------------------------------------------
 
 typedef struct GadgetSet {
   void (*fUpdate)(struct GadgetSet *I);
-  void (*fRender)(struct GadgetSet *I,CRay *ray,Pickable **pick,int pass);
+  void (*fRender)(struct GadgetSet *I,RenderInfo *info);
   void (*fFree)(struct GadgetSet *I);
   void (*fInvalidateRep)(struct GadgetSet *I,int type,int level);
   PyMOLGlobals *G;
-  struct ObjectGadget *Obj;
+  struct ObjectGadget *Obj; /* NOT pickled -- restore manually */
+  int State; /* NOT pickled -- restore manually */
   float *Coord;
   float *Normal;
   float *Color;
