@@ -197,7 +197,7 @@ void ObjectCGORecomputeExtent(ObjectCGO *I)
 /*========================================================================*/
 static void ObjectCGOUpdate(ObjectCGO *I)
 {
-  SceneDirty(I->Obj.G);/* needed ?*/
+  SceneInvalidate(I->Obj.G);/* needed ?*/
 }
 
 /*========================================================================*/
@@ -208,9 +208,13 @@ static int ObjectCGOGetNState(ObjectCGO *I) {
 
 /*========================================================================*/
 
-static void ObjectCGORender(ObjectCGO *I,int state,CRay *ray,Pickable **pick,int pass)
+static void ObjectCGORender(ObjectCGO *I,RenderInfo *info)
 {
   register PyMOLGlobals *G = I->Obj.G;
+  int state = info->state;
+  CRay *ray = info->ray;
+  Picking **pick = info->pick;
+  int pass = info->pass;
   ObjectCGOState *sobj = NULL;
   int a;
   float *color;
@@ -219,7 +223,7 @@ static void ObjectCGORender(ObjectCGO *I,int state,CRay *ray,Pickable **pick,int
 
   color = ColorGet(G,I->Obj.Color);
 
-  if(!pass) {
+  if((pass == 1) || info->ray) {
     if(I->Obj.RepVis[cRepCGO]) {
       
       if(state<I->NState) {
@@ -238,7 +242,7 @@ static void ObjectCGORender(ObjectCGO *I,int state,CRay *ray,Pickable **pick,int
               if(pick) {
               } else {
                 if(sobj->std)
-                  CGORenderGL(sobj->std,color,I->Obj.Setting,NULL);
+                  CGORenderGL(sobj->std,color,I->Obj.Setting,NULL,info);
               }
             }
           }
@@ -261,7 +265,7 @@ static void ObjectCGORender(ObjectCGO *I,int state,CRay *ray,Pickable **pick,int
           } else {
             if(sobj)
               if(sobj->std)
-                CGORenderGL(sobj->std,color,I->Obj.Setting,NULL);
+                CGORenderGL(sobj->std,color,I->Obj.Setting,NULL,info);
           }
         }
       }
@@ -282,7 +286,7 @@ ObjectCGO *ObjectCGONew(PyMOLGlobals *G)
   I->Obj.type = cObjectCGO;
   I->Obj.fFree = (void (*)(struct CObject *))ObjectCGOFree;
   I->Obj.fUpdate =(void (*)(struct CObject *)) ObjectCGOUpdate;
-  I->Obj.fRender =(void (*)(struct CObject *, int, CRay *, Pickable **,int))ObjectCGORender;
+  I->Obj.fRender =(void (*)(struct CObject *, RenderInfo *))ObjectCGORender;
   I->Obj.fGetNFrame = (int (*)(struct CObject *)) ObjectCGOGetNState;
 
   return(I);
@@ -318,6 +322,28 @@ static CGO *ObjectCGOPyListFloatToCGO(PyMOLGlobals *G,PyObject *list)
 }
 #endif
 
+/*========================================================================*/
+static CGO *ObjectCGOFloatArrayToCGO(PyMOLGlobals *G,float *raw, int len, int quiet)
+{
+  CGO *cgo=NULL;
+  int ok = true;
+  int result;
+
+  if(raw) {
+    if(ok) {
+      cgo=CGONewSized(G,len);
+      if(cgo) {
+        result = CGOFromFloatArray(cgo,raw,len); 
+        if(result&&!quiet) {
+          PRINTF " FloatToCGO: error encountered on element %d\n", result ENDF(G);
+        }
+        CGOStop(cgo);
+      }
+    }
+  }
+  return(cgo);
+}
+
 /*========================================================================*/
 ObjectCGO *ObjectCGOFromCGO(PyMOLGlobals *G,ObjectCGO *obj,CGO *cgo,int state)
 {
@@ -405,9 +431,11 @@ ObjectCGO *ObjectCGODefine(PyMOLGlobals *G,ObjectCGO *obj,PyObject *pycgo,int st
 
   if(I->State[state].std) {
     CGOFree(I->State[state].std);
+    I->State[state].std = NULL;
   }
   if(I->State[state].ray) {
     CGOFree(I->State[state].ray);
+    I->State[state].ray = NULL;
   }
   if(PyList_Check(pycgo)) {
     if(PyList_Size(pycgo)) {
@@ -442,3 +470,57 @@ ObjectCGO *ObjectCGODefine(PyMOLGlobals *G,ObjectCGO *obj,PyObject *pycgo,int st
   return(I);
 #endif
 }
+
+ObjectCGO *ObjectCGOFromFloatArray(PyMOLGlobals *G,ObjectCGO *obj,
+                                   float *array, int size, int state, int quiet)
+{
+  ObjectCGO *I = NULL;
+  
+  CGO *cgo,*font_cgo;
+  int est;
+  
+  if(obj) {
+    if(obj->Obj.type!=cObjectCGO) /* TODO: handle this */
+      obj=NULL;
+  }
+  if(!obj) {
+    I=ObjectCGONew(G);
+  } else {
+    I=obj;
+  }
+  if(state<0) state=I->NState;
+  if(I->NState<=state) {
+    VLACheck(I->State,ObjectCGOState,state);
+    I->NState=state+1;
+  }
+  if(I->State[state].std) {
+    CGOFree(I->State[state].std);
+  }
+  if(I->State[state].ray) {
+    CGOFree(I->State[state].ray);
+  }
+  cgo=ObjectCGOFloatArrayToCGO(G,array,size,quiet);
+  if(cgo) {
+    est=CGOCheckForText(cgo);
+    if(est) {
+      CGOPreloadFonts(cgo);
+      font_cgo = CGODrawText(cgo,est,NULL);
+      CGOFree(cgo);
+      cgo=font_cgo;
+    }
+    est=CGOCheckComplex(cgo);
+    if(est) {
+      I->State[state].ray=cgo;
+      I->State[state].std=CGOSimplify(cgo,est);
+    } else 
+      I->State[state].std=cgo;
+  } else if(!quiet) {
+    ErrMessage(G,"ObjectCGO","could not parse CGO.");
+  }
+  if(I) {
+    ObjectCGORecomputeExtent(I);
+  }
+  SceneChanged(G);
+  SceneCountFrames(G);
+  return(I);
+}
