@@ -35,14 +35,15 @@ static void ObjectCallbackFree(ObjectCallback *I);
 static void ObjectCallbackFree(ObjectCallback *I) {
 #ifndef _PYMOL_NOPY
   int a;
-  PBlock();
+  PyMOLGlobals *G = I->Obj.G;
+  PBlock(G);
   for(a=0;a<I->NState;a++) {
     if(I->State[a].PObj) {
       Py_DECREF(I->State[a].PObj);
       I->State[a].PObj=NULL;
     }
   }
-  PUnblock();
+  PUnblock(G);
 #endif 
   VLAFreeP(I->State);
   ObjectPurge(&I->Obj);
@@ -52,14 +53,18 @@ static void ObjectCallbackFree(ObjectCallback *I) {
 /*========================================================================*/
 
 static void ObjectCallbackUpdate(ObjectCallback *I) {
-  SceneDirty(I->Obj.G);
+  SceneInvalidate(I->Obj.G);
 }
 
 /*========================================================================*/
 
-static void ObjectCallbackRender(ObjectCallback *I,int state,CRay *ray,Pickable **pick,int pass)
+static void ObjectCallbackRender(ObjectCallback *I,RenderInfo *info)
 {
 #ifndef _PYMOL_NOPY
+  int state = info->state;
+  CRay *ray = info->ray;
+  Picking **pick = info->pick;
+  int pass = info->pass;
   PyMOLGlobals *G = I->Obj.G;
   ObjectCallbackState *sobj = NULL;
   int a;
@@ -75,7 +80,7 @@ static void ObjectCallbackRender(ObjectCallback *I,int state,CRay *ray,Pickable
       }
       if(state<0) {
         if(I->State) {
-          PBlock();
+          PBlock(G);
           for(a=0;a<I->NState;a++) {
             sobj = I->State+a;
             pobj=sobj->PObj;
@@ -91,7 +96,7 @@ static void ObjectCallbackRender(ObjectCallback *I,int state,CRay *ray,Pickable
               }
             }
           }
-          PUnblock();
+          PUnblock(G);
         }
       } else {
         if(!sobj) {
@@ -104,13 +109,13 @@ static void ObjectCallbackRender(ObjectCallback *I,int state,CRay *ray,Pickable
           } else {
             if(sobj) {
               pobj=sobj->PObj;
-              PBlock();
+              PBlock(G);
               if(PyObject_HasAttrString(pobj,"__call__")) {
                 Py_XDECREF(PyObject_CallMethod(pobj,"__call__",""));
               }
               if(PyErr_Occurred())
                 PyErr_Print();
-              PUnblock();
+              PUnblock(G);
             }
           }
         }
@@ -138,7 +143,7 @@ ObjectCallback *ObjectCallbackNew(PyMOLGlobals *G)
   I->Obj.type = cObjectCallback;
   I->Obj.fFree = (void (*)(struct CObject *))ObjectCallbackFree;
   I->Obj.fUpdate =  (void (*)(struct CObject *)) ObjectCallbackUpdate;
-  I->Obj.fRender =(void (*)(struct CObject *, int, CRay *, Pickable **,int pass))
+  I->Obj.fRender =(void (*)(struct CObject *, RenderInfo *))
     ObjectCallbackRender;
   I->Obj.fGetNFrame = (int (*)(struct CObject *)) ObjectCallbackGetNStates;
 
