@@ -30,13 +30,64 @@ Z* -------------------------------------------------------------------
 #include"ObjectMolecule.h"
 #include"Feedback.h"
 
-void ObjectDistRender(ObjectDist *I,int frame,CRay *ray,Pickable **pick,int pass);
 void ObjectDistFree(ObjectDist *I);
 void ObjectDistUpdate(ObjectDist *I);
 int ObjectDistGetNFrames(ObjectDist *I);
 void ObjectDistUpdateExtents(ObjectDist *I);
 
-static DistSet *ObjectDistGetDistSetFromM4XBond(PyMOLGlobals *G,ObjectMolecule *obj, M4XBondType *hb, int n_hb,
+int ObjectDistGetLabelTxfVertex(ObjectDist *I,int state,int index,float *v)
+{
+  int result = 0;
+  if(I->DSet) { 
+    if(state<0) state=SettingGet_i(I->Obj.G,NULL,I->Obj.Setting,cSetting_state)-1;
+    if(state<0) state=SceneGetState(I->Obj.G); 
+    if(I->NDSet==1) state=0; /* static singletons always active here it seems */
+    state = state % I->NDSet;
+    {
+	DistSet *ds = I->DSet[state];
+    if((!ds)&&(SettingGet_b(I->Obj.G,I->Obj.Setting,NULL,cSetting_all_states))) {
+      state=0;
+      ds = I->DSet[state];
+    }
+    if(ds) {
+      result = DistSetGetLabelVertex(ds,index,v);
+    }}
+  }
+  return(result);
+}
+
+int ObjectDistMoveLabel(ObjectDist *I,int state,int index,float *v,int mode,int log)
+{
+  int result = 0;
+  DistSet *ds;
+  if(state<0) state=0;
+  if(I->NDSet==1) state=0;
+  state = state % I->NDSet;
+  if((!I->DSet[state])&&(SettingGet_b(I->Obj.G,I->Obj.Setting,NULL,cSetting_all_states)))
+    state=0;
+  ds = I->DSet[state];
+  if(ds) {
+    result = DistSetMoveLabel(I->DSet[state],index,v,mode);
+    ds->fInvalidateRep(ds,cRepLabel,cRepInvCoord);
+    /*      ExecutiveUpdateCoordDepends(I->Obj.G,I);*/
+  }
+#if 0
+  if(log) {
+    OrthoLineType line,buffer;
+    if(SettingGet(I->Obj.G,cSetting_logging)) {
+      ObjectMoleculeGetAtomSele(I,index,buffer);
+      sprintf(line,"cmd.translate_atom(\"%s\",%15.9f,%15.9f,%15.9f,%d,%d,%d)\n",
+              buffer,v[0],v[1],v[2],state+1,mode,0);
+      PLog(G,line,cPLog_no_flush);
+    }
+  }
+#endif
+  return(result);
+}
+
+static DistSet *ObjectDistGetDistSetFromM4XBond(PyMOLGlobals *G,
+                                                ObjectMolecule *obj, 
+                                                M4XBondType *hb, int n_hb,
                                                 int state,int nbr_sele)
 {
   int min_id,max_id,range,*lookup = NULL;
@@ -342,7 +393,7 @@ void ObjectDistUpdate(ObjectDist *I)
 	   OrthoBusySlow(I->Obj.G,a,I->NDSet);
       /*	   printf(" ObjectDist: updating state %d of \"%s\".\n" , a+1, I->Obj.Name);*/
       if(I->DSet[a]->fUpdate)
-        I->DSet[a]->fUpdate(I->DSet[a]);
+        I->DSet[a]->fUpdate(I->DSet[a],a);
 	 }
 }
 /*========================================================================*/
@@ -360,42 +411,65 @@ void ObjectDistInvalidateRep(ObjectDist *I,int rep)
 	 }
 }
 /*========================================================================*/
-void ObjectDistRender(ObjectDist *I,int frame,CRay *ray,Pickable **pick,int pass)
+static void ObjectDistRender(ObjectDist *I,RenderInfo *info)
 {
   int a;
-  if(!pass) {
+  int state = info->state;
+  int pass = info->pass;
+  CRay *ray = info->ray;
 
+  if((pass==0)||(pass==-1)) {
     ObjectPrepareContext(&I->Obj,ray);
-    if(frame<0) {
+    if(state<0) {
       for(a=0;a<I->NDSet;a++)
         if(I->DSet[a])
           if(I->DSet[a]->fRender)
-            I->DSet[a]->fRender(I->DSet[a],ray,pick,pass);        
-    } else if(frame<I->NDSet) {
-      I->CurDSet=frame % I->NDSet;
+            I->DSet[a]->fRender(I->DSet[a],info);
+    } else if(state<I->NDSet) {
+      I->CurDSet=state % I->NDSet;
       if(I->DSet[I->CurDSet]) {
         if(I->DSet[I->CurDSet]->fRender)
-          I->DSet[I->CurDSet]->fRender(I->DSet[I->CurDSet],ray,pick,pass);
+          I->DSet[I->CurDSet]->fRender(I->DSet[I->CurDSet],info);
       }
     } else if(I->NDSet==1) { /* if only one coordinate set, assume static */
-      if(I->DSet[0]->fRender)
-        I->DSet[0]->fRender(I->DSet[0],ray,pick,pass);    
+      if(I->DSet[0]->fRender) {
+        if(SettingGet_b(I->Obj.G,I->Obj.Setting,NULL,cSetting_static_singletons)) {
+          I->DSet[0]->fRender(I->DSet[0],info);
+        }
+      }
     }
   }
 }
 
+static CSetting **ObjectDistGetSettingHandle(ObjectDist *I,int state)
+{
+  if(state<0) {
+    return(&I->Obj.Setting);
+  } else if(state<I->NDSet) {
+    if(I->DSet[state]) {
+      return(&I->DSet[state]->Setting);
+    } else {
+      return(NULL);
+    }
+  } else {
+    return(NULL);
+  }
+}
+
 /*========================================================================*/
 ObjectDist *ObjectDistNew(PyMOLGlobals *G)
 {
   OOAlloc(G,ObjectDist);
   ObjectInit(G,(CObject*)I);
-  I->Obj.type=cObjectDist;
+  I->Obj.type=cObjectMeasurement;
   I->DSet=VLAMalloc(10,sizeof(DistSet*),5,true); /* auto-zero */
   I->NDSet=0;
-  I->Obj.fRender=(void (*)(struct CObject *, int, CRay *, Pickable **,int))ObjectDistRender;
+  I->Obj.fRender=(void (*)(struct CObject *, RenderInfo *info))ObjectDistRender;
   I->Obj.fFree= (void (*)(struct CObject *))ObjectDistFree;
   I->Obj.fUpdate= (void (*)(struct CObject *)) ObjectDistUpdate;
   I->Obj.fGetNFrame = (int (*)(struct CObject *)) ObjectDistGetNFrames;
+  I->Obj.fGetSettingHandle = (CSetting **(*)(struct CObject *,int state))
+    ObjectDistGetSettingHandle;
   I->Obj.fDescribeElement = NULL;
   I->CurDSet=0;
   I->Obj.Color=ColorGetIndex(G,"dash");
@@ -418,7 +492,7 @@ static void ObjectDistReset(PyMOLGlobals *G,ObjectDist *I)
 /*========================================================================*/
 ObjectDist *ObjectDistNewFromSele(PyMOLGlobals *G,ObjectDist *oldObj,
                                   int sele1,int sele2,int mode,float cutoff,
-                                  int labels,int reset,float *result)
+                                  int labels,int reset,float *result,int state)
 {
   int a,mn;
   float dist_sum=0.0,dist;
@@ -434,38 +508,41 @@ ObjectDist *ObjectDistNewFromSele(PyMOLGlobals *G,ObjectDist *oldObj,
   }
   *result = 0.0;
   mn = 0;
-  SelectorUpdateTable(G);
+  SelectorUpdateTable(G,state,-1);
   n_state1 = SelectorGetSeleNCSet(G,sele1);
   n_state2 = SelectorGetSeleNCSet(G,sele2);
   mn = n_state1;
   if(n_state2>mn) mn = n_state2;
   if(mn) {
-    for(a=0;a<mn;a++)
-      {
-        VLACheck(I->DSet,DistSet*,a);
-        if(n_state1>1)
-          state1=a;
-        else
-          state1=0;
-        if(n_state2>1)
-          state2=a;
-        else
-          state2=0;
-        I->DSet[a] = SelectorGetDistSet(G,I->DSet[a],sele1,state1,sele2,
-                                        state2,mode,cutoff,&dist);
-        if(I->DSet[a]) {
-          dist_sum+=dist;
-          dist_cnt++;
-          I->DSet[a]->Obj = I;
-          I->NDSet=a+1;
-        }
-      }  
+    for(a=0;a<mn;a++) {
+      if(state>=0) {
+        if(state>=mn)
+          break;
+        a = state;
+      }
+        
+      VLACheck(I->DSet,DistSet*,a);
+      if(n_state1>1)
+        state1=a;
+      else
+        state1=0;
+      if(n_state2>1)
+        state2=a;
+      else
+        state2=0;
+      I->DSet[a] = SelectorGetDistSet(G,I->DSet[a],sele1,state1,sele2,
+                                      state2,mode,cutoff,&dist);
+      if(I->DSet[a]) {
+        dist_sum+=dist;
+        dist_cnt++;
+        I->DSet[a]->Obj = I;
+        I->NDSet=a+1;
+      }
+
+      if(state>=0)
+        break;
+    }
   } 
-  /* else {
-     VLAFreeP(I->DSet);
-     OOFreeP(I);
-     }
-  */
   ObjectDistUpdateExtents(I);
   ObjectDistInvalidateRep(I,cRepAll);
 
@@ -476,8 +553,9 @@ ObjectDist *ObjectDistNewFromSele(PyMOLGlobals *G,ObjectDist *oldObj,
 }
 
 ObjectDist *ObjectDistNewFromAngleSele(PyMOLGlobals *G,ObjectDist *oldObj,
-                                        int sele1, int sele2, int sele3, int mode,
-                                        int labels,float *result, int reset)
+                                       int sele1, int sele2, int sele3, int mode,
+                                       int labels,float *result, int reset,
+                                       int state)
 {
   int a,mn;
   float angle_sum=0.0;
@@ -497,7 +575,7 @@ ObjectDist *ObjectDistNewFromAngleSele(PyMOLGlobals *G,ObjectDist *oldObj,
 
   /* count number of states in each selection */
 
-  SelectorUpdateTable(G);
+  SelectorUpdateTable(G,state,-1);
   n_state1 = SelectorGetSeleNCSet(G,sele1);
   n_state2 = SelectorGetSeleNCSet(G,sele2);  
   n_state3 = SelectorGetSeleNCSet(G,sele3);
@@ -509,34 +587,40 @@ ObjectDist *ObjectDistNewFromAngleSele(PyMOLGlobals *G,ObjectDist *oldObj,
   if(n_state3>mn) mn = n_state3;
 
   if(mn) {
-    for(a=0;a<mn;a++)
-      {
-        /* treat selections with one state as static singletons */
-
-        if(n_state1>1)
-          state1=a;
-        else
-          state1=0;
-        if(n_state2>1)
-          state2=a;
-        else
-          state2=0;
-        if(n_state3>1)
-          state3=a;
-        else
-          state3=0;
-        
-        VLACheck(I->DSet,DistSet*,a);
-        I->DSet[a] = SelectorGetAngleSet(G,I->DSet[a],sele1,state1,sele2,
-                                         state2,sele3,state3,mode,&angle_sum,
-                                         &angle_cnt);
-
-        if(I->DSet[a]) {
-          I->DSet[a]->Obj = I;
-          if(I->NDSet<=a) I->NDSet = a+1;
-        }
-      }  
-  } 
+    for(a=0;a<mn;a++) {
+      if(state>=0) {
+        if(state>mn)
+          break;
+        a = state;
+      }
+      /* treat selections with one state as static singletons */
+      
+      if(n_state1>1)
+        state1=a;
+      else
+        state1=0;
+      if(n_state2>1)
+        state2=a;
+      else
+        state2=0;
+      if(n_state3>1)
+        state3=a;
+      else
+        state3=0;
+      
+      VLACheck(I->DSet,DistSet*,a);
+      I->DSet[a] = SelectorGetAngleSet(G,I->DSet[a],sele1,state1,sele2,
+                                       state2,sele3,state3,mode,&angle_sum,
+                                       &angle_cnt);
+      
+      if(I->DSet[a]) {
+        I->DSet[a]->Obj = I;
+        if(I->NDSet<=a) I->NDSet = a+1;
+      }
+      if(state>=0)
+        break;
+    }  
+  }
   /* else {
      VLAFreeP(I->DSet);
      OOFreeP(I);
@@ -552,8 +636,9 @@ ObjectDist *ObjectDistNewFromAngleSele(PyMOLGlobals *G,ObjectDist *oldObj,
 }
 
 ObjectDist *ObjectDistNewFromDihedralSele(PyMOLGlobals *G,ObjectDist *oldObj,
-                                        int sele1, int sele2, int sele3, int sele4,
-                                        int mode, int labels, float *result, int reset)
+                                          int sele1, int sele2, int sele3, int sele4,
+                                          int mode, int labels, float *result,
+                                          int reset,int state)
 {
   int a,mn;
   float angle_sum=0.0;
@@ -573,7 +658,8 @@ ObjectDist *ObjectDistNewFromDihedralSele(PyMOLGlobals *G,ObjectDist *oldObj,
 
   /* count number of states in each selection */
 
-  SelectorUpdateTable(G);
+  SelectorUpdateTable(G,state,-1);
+
   n_state1 = SelectorGetSeleNCSet(G,sele1);
   n_state2 = SelectorGetSeleNCSet(G,sele2);  
   n_state3 = SelectorGetSeleNCSet(G,sele3);
@@ -587,38 +673,45 @@ ObjectDist *ObjectDistNewFromDihedralSele(PyMOLGlobals *G,ObjectDist *oldObj,
   if(n_state4>mn) mn = n_state4;
 
   if(mn) {
-    for(a=0;a<mn;a++)
-      {
-        /* treat selections with one state as static singletons */
+    for(a=0;a<mn;a++) {
+      if(state>=0) {
+        if(state>mn)
+          break;
+        a = state;
+      }
+      /* treat selections with one state as static singletons */
+      
+      if(n_state1>1)
+        state1=a;
+      else
+        state1=0;
+      if(n_state2>1)
+        state2=a;
+      else
+        state2=0;
+      if(n_state3>1)
+        state3=a;
+      else
+        state3=0;
+      if(n_state4>1)
+        state4=a;
+      else
+        state4=0;
+      
+      VLACheck(I->DSet,DistSet*,a);
+      I->DSet[a] = SelectorGetDihedralSet(G,I->DSet[a],sele1,state1,sele2,
+                                          state2,sele3,state3,sele4,state4,
+                                          mode,&angle_sum,
+                                          &angle_cnt);
+      
+      if(I->DSet[a]) {
+        I->DSet[a]->Obj = I;
+        if(I->NDSet<=a) I->NDSet = a+1;
+      }
 
-        if(n_state1>1)
-          state1=a;
-        else
-          state1=0;
-        if(n_state2>1)
-          state2=a;
-        else
-          state2=0;
-        if(n_state3>1)
-          state3=a;
-        else
-          state3=0;
-        if(n_state4>1)
-          state4=a;
-        else
-          state4=0;
-        
-        VLACheck(I->DSet,DistSet*,a);
-        I->DSet[a] = SelectorGetDihedralSet(G,I->DSet[a],sele1,state1,sele2,
-                                         state2,sele3,state3,sele4,state4,
-                                         mode,&angle_sum,
-                                         &angle_cnt);
-
-        if(I->DSet[a]) {
-          I->DSet[a]->Obj = I;
-          if(I->NDSet<=a) I->NDSet = a+1;
-        }
-      }  
+      if(state>=0)
+        break;
+    }
   } 
   /* else {
      VLAFreeP(I->DSet);
