@@ -34,7 +34,7 @@ Z* -------------------------------------------------------------------
 
 CGO *ObjectGadgetPyListFloatToCGO(PyObject *list);
 
-void ObjectGadgetRender(ObjectGadget *I,int state,CRay *ray,Pickable **pick,int pass);
+
 
 int ObjectGadgetGetVertex(ObjectGadget *I,int index,int base,float *v)
 {
@@ -189,7 +189,7 @@ ObjectGadget *ObjectGadgetTest(PyMOLGlobals *G)
   cgo = CGONewSized(G,100);
   CGODotwidth(cgo,5);
 
-  CGOPickColor(cgo,0,0);
+  CGOPickColor(cgo,0,cPickableGadget);
 
   /* top */
   CGOBegin(cgo,GL_TRIANGLE_STRIP);
@@ -230,6 +230,7 @@ ObjectGadget *ObjectGadgetTest(PyMOLGlobals *G)
   gs->PickShapeCGO = cgo;
 
   gs->Obj = I;
+  gs->State = 0;
 
   I->GSet[0] = gs;
   I->NGSet = 1;
@@ -289,7 +290,10 @@ static int ObjectGadgetGSetFromPyList(ObjectGadget *I,PyObject *list,int version
     VLACheck(I->GSet,GadgetSet*,I->NGSet);
     for(a=0;a<I->NGSet;a++) {
       if(ok) ok = GadgetSetFromPyList(I->Obj.G,PyList_GetItem(list,a),&I->GSet[a],version);
-      if(ok&&I->GSet[a]) I->GSet[a]->Obj = I;
+      if(ok&&I->GSet[a]) {
+        I->GSet[a]->Obj = I;
+        I->GSet[a]->State = a;
+      }
     }
   }
   return(ok);
@@ -334,12 +338,18 @@ int ObjectGadgetNewFromPyList(PyMOLGlobals *G,PyObject *list,ObjectGadget **resu
   int ok = true;
   ObjectGadget *I=NULL;
   int gadget_type = -1;
+  PyObject *plain;
   (*result) = NULL;
 
   if(ok) ok=(list!=NULL);
   if(ok) ok=PyList_Check(list);
 
-  if(ok) ok=PConvPyIntToInt(PyList_GetItem(list,1),&gadget_type);
+  /* NOTE there is a serious screw-up here...ramp gadgets aren't saved right, but
+     we've got to maintain backward compat...ugh */
+
+  if(ok) ok=((plain=PyList_GetItem(list,0))!=NULL); 
+  if(ok) ok=PyList_Check(plain);
+  if(ok) ok=PConvPyIntToInt(PyList_GetItem(plain,1),&gadget_type);
   if(ok) switch(gadget_type) { /* call the right routine to restore the gadget! */
   case cGadgetRamp:
     ok = ObjectGadgetRampNewFromPyList(G,list,(ObjectGadgetRamp**)result,version);
@@ -450,8 +460,11 @@ static int ObjectGadgetGetNState(ObjectGadget *I) {
 }
 
 /*========================================================================*/
-void ObjectGadgetRender(ObjectGadget *I,int state,CRay *ray,Pickable **pick,int pass)
+static void ObjectGadgetRender(ObjectGadget *I,RenderInfo *info)
 {
+  int state = info->state;
+  CRay *ray = info->ray;
+  int pass = info->pass;
   int a;
   if(!pass) {
 
@@ -460,16 +473,16 @@ void ObjectGadgetRender(ObjectGadget *I,int state,CRay *ray,Pickable **pick,int
       for(a=0;a<I->NGSet;a++)
         if(I->GSet[a])
           if(I->GSet[a]->fRender)
-            I->GSet[a]->fRender(I->GSet[a],ray,pick,pass);        
+            I->GSet[a]->fRender(I->GSet[a],info);
     } else if(state<I->NGSet) {
       I->CurGSet=state;
       if(I->GSet[I->CurGSet]) {
         if(I->GSet[I->CurGSet]->fRender)
-          I->GSet[I->CurGSet]->fRender(I->GSet[I->CurGSet],ray,pick,pass);
+          I->GSet[I->CurGSet]->fRender(I->GSet[I->CurGSet],info);
       }
     } else if(I->NGSet==1) { /* if only one coordinate set, assume static */
       if(I->GSet[0]->fRender)
-        I->GSet[0]->fRender(I->GSet[0],ray,pick,pass);    
+        I->GSet[0]->fRender(I->GSet[0],info);
       I->CurGSet=0;
     }
   }
@@ -487,7 +500,7 @@ void ObjectGadgetInit(PyMOLGlobals *G,ObjectGadget *I)
 
   I->Obj.fFree = (void (*)(struct CObject *))ObjectGadgetFree;
   I->Obj.fUpdate =(void (*)(struct CObject *)) ObjectGadgetUpdate;
-  I->Obj.fRender =(void (*)(struct CObject *, int, CRay *, Pickable **,int))ObjectGadgetRender;
+  I->Obj.fRender =(void (*)(struct CObject *, RenderInfo *info))ObjectGadgetRender;
   I->Obj.fGetNFrame = (int (*)(struct CObject *)) ObjectGadgetGetNState;
   I->Obj.fDescribeElement = NULL;
   I->CurGSet=0;
