@@ -39,6 +39,7 @@ Z* -------------------------------------------------------------------
 ObjectMesh *ObjectMeshNew(PyMOLGlobals *G);
 
 static void ObjectMeshFree(ObjectMesh *I);
+static void ObjectMeshInvalidate(ObjectMesh *I,int rep,int level,int state);
 void ObjectMeshStateInit(PyMOLGlobals *G,ObjectMeshState *ms);
 void ObjectMeshRecomputeExtent(ObjectMesh *I);
 
@@ -47,7 +48,7 @@ static PyObject *ObjectMeshStateAsPyList(ObjectMeshState *I)
 {
   PyObject *result = NULL;
 
-  result = PyList_New(14);
+  result = PyList_New(15);
   
   PyList_SetItem(result,0,PyInt_FromLong(I->Active));
   PyList_SetItem(result,1,PyString_FromString(I->MapName));
@@ -66,27 +67,8 @@ static PyObject *ObjectMeshStateAsPyList(ObjectMeshState *I)
   } else {
     PyList_SetItem(result,12,PConvAutoNone(NULL));
   }
-  PyList_SetItem(result,13,PyInt_FromLong(I->DotFlag));
-
-#if 0
-  char MapName[ObjNameMax];
-  int MapState;
-  CCrystal Crystal;
-  int Active;
-  int *N;
-  float *V;
-  int Range[6];
-  float ExtentMin[3],ExtentMax[3];
-  int ExtentFlag;
-  float Level,Radius;
-  int RefreshFlag;
-  int ResurfaceFlag;
-  float *AtomVertex;
-  int CarveFlag;
-  float CarveBuffer;
-  int DotFlag;
-  CGO *UnitCellCGO;
-#endif
+  PyList_SetItem(result,13,PyInt_FromLong(I->MeshMode));
+  PyList_SetItem(result,14,PyFloat_FromDouble(I->AltLevel));
 
   return(PConvAutoNone(result));  
 }
@@ -114,7 +96,7 @@ static PyObject *ObjectMeshAllStatesAsPyList(ObjectMesh *I)
 static int ObjectMeshStateFromPyList(PyMOLGlobals *G,ObjectMeshState *I,PyObject *list)
 {
   int ok=true;
-  int ll;
+  int ll=0;
   PyObject *tmp;
   if(ok) ok=(list!=NULL);
   if(ok) {
@@ -129,7 +111,7 @@ static int ObjectMeshStateFromPyList(PyMOLGlobals *G,ObjectMeshState *I,PyObject
          Always check ll when adding new PyList_GetItem's */
       
       if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,0),&I->Active);
-      if(ok) ok = PConvPyStrToStr(PyList_GetItem(list,1),I->MapName,ObjNameMax);
+      if(ok) ok = PConvPyStrToStr(PyList_GetItem(list,1),I->MapName,WordLength);
       if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,2),&I->MapState);
       if(ok) ok = CrystalFromPyList(&I->Crystal,PyList_GetItem(list,3));
       if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,4),&I->ExtentFlag);
@@ -147,11 +129,16 @@ static int ObjectMeshStateFromPyList(PyMOLGlobals *G,ObjectMeshState *I,PyObject
         else 
           ok = PConvPyListToFloatVLA(tmp,&I->AtomVertex);
       }
-      if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,13),&I->DotFlag);
+      if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,13),&I->MeshMode);
       if(ok) {
         I->RefreshFlag=true;
         I->ResurfaceFlag=true;
       }
+      if(ok&&(ll>14)) {
+        ok = PConvPyFloatToFloat(PyList_GetItem(list,14),&I->AltLevel);
+      } else {
+        I->AltLevel = I->Level;
+      }
     }
   }
   return(ok);
@@ -239,13 +226,15 @@ static void ObjectMeshStateFree(ObjectMeshState *ms)
         }
       } else {
         char buffer[255]; /* pass this off to the main thread */
-        sprintf(buffer,"_cmd.gl_delete_lists(%d,%d)\n",ms->displayList,1);
-        PParse(buffer);
+        sprintf(buffer,"_cmd.gl_delete_lists(cmd._COb,%d,%d)\n",ms->displayList,1);
+        PParse(ms->State.G,buffer);
       }
     }
   }
   VLAFreeP(ms->N);
   VLAFreeP(ms->V);
+  FreeP(ms->VC);
+  FreeP(ms->RC);
   VLAFreeP(ms->AtomVertex);
   if(ms->UnitCellCGO)
     CGOFree(ms->UnitCellCGO);
@@ -263,6 +252,23 @@ static void ObjectMeshFree(ObjectMesh *I) {
   OOFreeP(I);
 }
 
+int ObjectMeshInvalidateMapName(ObjectMesh *I,char *name)
+{
+  int a;
+  ObjectMeshState *ms;
+  int result=false;
+  for(a=0;a<I->NState;a++) {
+    ms=I->State+a;
+    if(ms->Active) {
+      if(strcmp(ms->MapName,name)==0) {
+        ObjectMeshInvalidate(I,cRepAll,cRepInvAll,a);
+        result=true;
+      }
+    }
+  }
+  return result;
+}
+
 void ObjectMeshDump(ObjectMesh *I,char *fname,int state)
 {
   float *v;
@@ -280,7 +286,7 @@ void ObjectMeshDump(ObjectMesh *I,char *fname,int state)
         while(*n)
           {
             c=*(n++);
-            if(!I->State[state].DotFlag) {
+            if(!I->State[state].MeshMode) {
               fprintf(f,"\n");
             }
             while(c--) {
@@ -296,25 +302,71 @@ void ObjectMeshDump(ObjectMesh *I,char *fname,int state)
   }
 }
 
+#if 0
+static char *ObjectMeshGetCaption(ObjectMesh *I)
+{
+  int state = ObjectGetCurrentState((CObject*)I,false);
+  if(state>=0) {
+    if(state<I->NState) {
+      char *p;
+      ObjectMeshState *ms = I->State + state;
+      sprintf(ms->caption,"@ %1.4f\n",ms->Level);
+      p=ms->caption+strlen(ms->caption);
+      while(p>ms->caption) {
+        p--;
+        if(*p!='0')
+          break;
+      }
+      return ms->caption;
+    }
+  }
+  return NULL;
+}
+#endif
+
 static void ObjectMeshInvalidate(ObjectMesh *I,int rep,int level,int state)
 {
   int a;
   int once_flag=true;
+  if(level>=cRepInvExtents) {
+    I->Obj.ExtentFlag=false;
+  }
   for(a=0;a<I->NState;a++) {
     if(state<0) once_flag=false;
     if(!once_flag) state=a;
     I->State[state].RefreshFlag=true;
-    if(level>=cRepInvAll) {
+    if((level>=cRepInvAll)||(rep==cRepMesh)) {
       I->State[state].ResurfaceFlag=true;      
       SceneChanged(I->Obj.G);
+    } else if(level>=cRepInvColor) {
+      I->State[state].RecolorFlag=true;
+      SceneChanged(I->Obj.G);
     } else {
-      SceneDirty(I->Obj.G);
+      SceneInvalidate(I->Obj.G);
     }
-
     if(once_flag) break;
   }
 }
 
+int ObjectMeshGetLevel(ObjectMesh *I,int state, float *result)
+{
+  int ok=true;
+  ObjectMeshState *ms;
+  if(state>=I->NState) {
+    ok=false;
+  } else {
+    if(state<0) {
+      state = 0;
+    }
+    ms = I->State + state;
+    if(ms->Active && result) {
+      *result = ms->Level;
+    } else 
+      ok = false;
+  }
+  return(ok);
+}
+
 int ObjectMeshSetLevel(ObjectMesh *I,float level,int state)
 {
   int a;
@@ -345,6 +397,76 @@ int ObjectMeshSetLevel(ObjectMesh *I,float level,int state)
   return(ok);
 }
 
+static void ObjectMeshStateUpdateColors(ObjectMesh *I, ObjectMeshState *ms)
+{
+  int one_color_flag = true;
+  int cur_color = SettingGet_color(I->Obj.G, I->Obj.Setting, NULL, cSetting_mesh_color);
+
+  if(cur_color == -1)
+    cur_color = I->Obj.Color;
+
+  if(ColorCheckRamped(I->Obj.G,cur_color))
+    one_color_flag = false;
+
+  ms->OneColor = cur_color;
+  if(ms->V) {
+    int ramped_flag = false;
+    float *v = ms->V;
+    float *vc;
+    int *rc;
+    int a;
+    int state = ms - I->State;
+    int n_vert = VLAGetSize(ms->V)/3;
+    int base_n_vert = ms->base_n_V/3;
+    
+    if(ms->VC && (ms->VCsize<n_vert)) {
+      FreeP(ms->VC);
+      FreeP(ms->RC);
+    }
+    
+    if(!ms->VC) {
+      ms->VCsize = n_vert;
+      ms->VC = Alloc(float,n_vert*3);
+    }
+    if(!ms->RC) {
+      ms->RC = Alloc(int,n_vert);
+    }
+    rc = ms->RC;
+    vc = ms->VC;
+    if(vc) {
+      for(a=0;a<n_vert;a++) {
+        if(a==base_n_vert) {
+          int new_color = SettingGet_color(I->Obj.G, I->Obj.Setting, 
+                                           NULL, cSetting_mesh_negative_color);
+          if(new_color==-1) new_color=cur_color;
+          if(new_color!=cur_color) {
+            one_color_flag = false;
+            cur_color = new_color;
+          }
+        }
+        if(ColorCheckRamped(I->Obj.G,cur_color)) {
+          ColorGetRamped(I->Obj.G,cur_color,v,vc,state);
+          *rc = cur_color;
+          ramped_flag=true;
+        } else {
+          float *col = ColorGet(I->Obj.G,cur_color);
+          copy3f(col,vc);
+        }
+        rc++;
+        vc+=3;
+        v+=3; 
+      }
+    }
+
+    if(one_color_flag && (!ramped_flag)) {
+      FreeP(ms->VC);
+      FreeP(ms->RC);
+    } else if( (!ramped_flag) || (!SettingGet_b(I->Obj.G,NULL,I->Obj.Setting,cSetting_ray_color_ramps))) {
+      FreeP(ms->RC);
+    }
+  }
+}
+
 static void ObjectMeshUpdate(ObjectMesh *I) 
 {
   PyMOLGlobals *G = I->Obj.G;
@@ -368,6 +490,8 @@ static void ObjectMeshUpdate(ObjectMesh *I)
   int h,k,l;
   int i,j;
   int ok=true;
+  int mesh_skip = SettingGet_i(G,I->Obj.Setting,NULL,cSetting_mesh_skip);
+
   MapType *voxelmap; /* this has nothing to do with isosurfaces... */
   
   for(a=0;a<I->NState;a++) {
@@ -400,13 +524,15 @@ static void ObjectMeshUpdate(ObjectMesh *I)
             ObjectStateResetMatrix(&ms->State);
           }
           ms->RefreshFlag=false;
+          ms->displayListInvalid = true;
         }
       }
       
       if(map&&oms&&ms->N&&ms->V&&I->Obj.RepVis[cRepMesh]) {
         if(ms->ResurfaceFlag) {
+          ms->RecolorFlag=true;          
           ms->ResurfaceFlag=false;
-          PRINTFB(G,FB_ObjectMesh,FB_Actions)
+          PRINTFB(G,FB_ObjectMesh,FB_Details)
             " ObjectMesh: updating \"%s\".\n" , I->Obj.Name 
           ENDFB(G);
           if(oms->Field) {
@@ -427,19 +553,71 @@ static void ObjectMeshUpdate(ObjectMesh *I)
               IsosurfGetRange(I->Obj.G,oms->Field,oms->Crystal,
                               min_ext,max_ext,ms->Range);
             }
-            /*          printf("%d %d %d %d %d %d\n",
+            /*                      printf("DEBUG: %d %d %d %d %d %d\n",
                    ms->Range[0],
                    ms->Range[1],
                    ms->Range[2],
                    ms->Range[3],
                    ms->Range[4],
                    ms->Range[5]);*/
-                
-            IsosurfVolume(I->Obj.G,oms->Field,
+            IsosurfVolume(I->Obj.G,I->Obj.Setting,NULL,
+                          oms->Field,
                           ms->Level,
                           &ms->N,&ms->V,
                           ms->Range,
-                          ms->DotFlag); 
+                          ms->MeshMode,
+                          mesh_skip,ms->AltLevel);
+
+                       
+
+            if(!SettingGet_b(I->Obj.G,I->Obj.Setting,NULL,cSetting_mesh_negative_visible)) { 
+              ms->base_n_V = VLAGetSize(ms->V);
+            } else if(ms->MeshMode!=3) {
+              /* do we want the negative surface too? */
+
+              int *N2 = VLAlloc(int,10000);
+              float *V2 =  VLAlloc(float,10000);
+
+              IsosurfVolume(I->Obj.G,I->Obj.Setting,NULL,
+                            oms->Field,
+                            -ms->Level,
+                            &N2,&V2,
+                            ms->Range,
+                            ms->MeshMode,
+                            mesh_skip,ms->AltLevel);
+              
+
+              if(N2&&V2) {
+                
+                int base_n_N = VLAGetSize(ms->N);
+                int base_n_V = VLAGetSize(ms->V);
+                int addl_n_N = VLAGetSize(N2);
+                int addl_n_V = VLAGetSize(V2);
+
+                ms->base_n_V = base_n_V;
+
+                /* make room */
+
+                VLASize(ms->N, int, base_n_N + addl_n_N);
+                VLASize(ms->V, float, base_n_V + addl_n_V);
+                
+                /* copy vertex data */
+
+                memcpy(((char*)ms->V)+(sizeof(float)*base_n_V),
+                       V2, sizeof(float)*addl_n_V);
+
+                /* copy strip counts */
+
+                memcpy(((char*)ms->N)+(sizeof(int)*(base_n_N-1)),
+                       N2, sizeof(int)*addl_n_N);
+                ms->N[ base_n_N + addl_n_N - 1] = 0;
+                
+                VLAFreeP(N2);
+                VLAFreeP(V2);
+              }
+
+            }
+
 
             if(ms->State.Matrix && VLAGetSize(ms->N)&&VLAGetSize(ms->V)) { 
               int count;
@@ -534,24 +712,37 @@ static void ObjectMeshUpdate(ObjectMesh *I)
             }
           }
         }
+        if(ms->RecolorFlag) {
+          ObjectMeshStateUpdateColors(I,ms);
+          ms->RecolorFlag=false;
+        }
       }
     }
-    SceneDirty(I->Obj.G);
+    SceneInvalidate(I->Obj.G);
+  }
+  if(!I->Obj.ExtentFlag) {
+    ObjectMeshRecomputeExtent(I);
+    if(I->Obj.ExtentFlag)
+      SceneInvalidate(I->Obj.G);
   }
 }
 
-static void ObjectMeshRender(ObjectMesh *I,int state,CRay *ray,Pickable **pick,int pass)
+static void ObjectMeshRender(ObjectMesh *I,RenderInfo *info)
 {
   PyMOLGlobals *G = I->Obj.G;
   float *v = NULL;
   float *vc;
+  int *rc;
   float radius;
-  int *n = NULL;
+  int state = info->state;
+  CRay *ray = info->ray;
+  Picking **pick = info->pick;
+  int pass = info->pass;  int *n = NULL;
   int c;
   int a=0;
   ObjectMeshState *ms = NULL;
 
-  ObjectPrepareContext(&I->Obj,ray);
+   ObjectPrepareContext(&I->Obj,ray);
   
   if(state>=0) 
     if(state<I->NState) 
@@ -578,7 +769,7 @@ static void ObjectMeshRender(ObjectMesh *I,int state,CRay *ray,Pickable **pick,i
           if(ms->UnitCellCGO&&(I->Obj.RepVis[cRepCell]))
             CGORenderRay(ms->UnitCellCGO,ray,ColorGet(I->Obj.G,I->Obj.Color),
                          I->Obj.Setting,NULL);
-          if(!ms->DotFlag) {
+          if(ms->MeshMode!=1) {
             radius=SettingGet_f(I->Obj.G,I->Obj.Setting,NULL,cSetting_mesh_radius);
             
             if(radius==0.0F) {
@@ -589,91 +780,122 @@ static void ObjectMeshRender(ObjectMesh *I,int state,CRay *ray,Pickable **pick,i
             if(radius==0.0F) {
               radius = ray->PixelRadius*SettingGet_f(I->Obj.G,I->Obj.Setting,NULL,cSetting_dot_width)/1.4142F;
             } 
-          }
-                  
+          }                  
 
           if(n&&v&&I->Obj.RepVis[cRepMesh]) {
-            vc = ColorGet(I->Obj.G,I->Obj.Color);
-            if(ms->DotFlag) {
-              ray->fColor3fv(ray,vc);
-              while(*n)
-                {
-                  c=*(n++);
-                  if(c--)
-                    {
-                      v+=3;
-                      while(c--)
-                        {
-                          ray->fSphere3fv(ray,v,radius);
-                          v+=3;
-                        }
+            float cc[3];
+            float colA[3],colB[3];
+            ColorGetEncoded(G,ms->OneColor,cc);
+            vc = ms->VC;
+            rc = ms->RC;
+            /* vc = ColorGet(I->Obj.G,I->Obj.Color); */
+            if(ms->MeshMode==1) {
+              ray->fColor3fv(ray,cc);
+              while(*n) {
+                c=*(n++);
+                while(c--) {
+                  if(vc) {
+                    register float *cA=vc;
+                    if(rc) {
+                      if(rc[0]<-1) ColorGetEncoded(G,rc[0], (cA=colA));
+                      rc++;
                     }
+                    ray->fColor3fv(ray,cA);
+                    ray->fSphere3fv(ray,v,radius);
+                    vc+=3;
+                  } else {
+                    ray->fSphere3fv(ray,v,radius);
+                  }
+                  v+=3;
                 }
+              }
             } else {
-              while(*n)
-                {
-                  c=*(n++);
-                  if(c--)
-                    {
-                      v+=3;
-                      while(c--)
-                        {
-                          ray->fSausage3fv(ray,v-3,v,radius,vc,vc);
-                          v+=3;
-                        }
+              while(*n) {
+                c=*(n++);
+                if(c--) {
+                  v+=3;
+                  if(vc) {
+                    vc+=3;
+                    if(rc) rc++;
+                  }
+                  while(c--) {
+                    if(vc) {
+                      register float *cA=vc-3, *cB=vc;
+                      if(rc) {
+                        if(rc[-1]<-1) ColorGetEncoded(G,rc[-1], (cA=colA));
+                        if(rc[0]<-1) ColorGetEncoded(G,rc[0], (cB=colB));
+                        rc++;
+                      }
+                      ray->fSausage3fv(ray,v-3,v,radius,cA,cB);
+                      vc+=3;
+                    } else {
+                      ray->fSausage3fv(ray,v-3,v,radius,cc,cc);
                     }
+                    v+=3;
+                  }
                 }
+              }
             }
           }
         } else if(G->HaveGUI && G->ValidContext) {
           if(pick) {
           } else {
             if(!pass) {
-
-            int use_dlst;
-            if(ms->UnitCellCGO&&(I->Obj.RepVis[cRepCell]))
-              CGORenderGL(ms->UnitCellCGO,ColorGet(I->Obj.G,I->Obj.Color),
-                          I->Obj.Setting,NULL);
-
-            SceneResetNormal(I->Obj.G,false);
-            ObjectUseColor(&I->Obj);
-            use_dlst = (int)SettingGet(I->Obj.G,cSetting_use_display_lists);
-            if(use_dlst&&ms->displayList) {
-              glCallList(ms->displayList);
-            } else { 
+              int use_dlst;
+              if(ms->UnitCellCGO&&(I->Obj.RepVis[cRepCell]))
+                CGORenderGL(ms->UnitCellCGO,ColorGet(I->Obj.G,I->Obj.Color),
+                            I->Obj.Setting,NULL,info);
               
-              if(use_dlst) {
-                if(!ms->displayList) {
-                  ms->displayList = glGenLists(1);
-                  if(ms->displayList) {
-                    glNewList(ms->displayList,GL_COMPILE_AND_EXECUTE);
+
+              glDisable(GL_LIGHTING); 
+              SceneResetNormal(I->Obj.G,false);
+              ObjectUseColor(&I->Obj);
+              use_dlst = (int)SettingGet(I->Obj.G,cSetting_use_display_lists);
+              
+              if(use_dlst && ms->displayList && ms->displayListInvalid) {
+                glDeleteLists(ms->displayList,1);
+                ms->displayList = 0;
+                ms->displayListInvalid = false;
+              }
+              
+              if(use_dlst&&ms->displayList) {
+                glCallList(ms->displayList);
+              } else { 
+                
+                if(use_dlst) {
+                  if(!ms->displayList) {
+                    ms->displayList = glGenLists(1);
+                    if(ms->displayList) {
+                      glNewList(ms->displayList,GL_COMPILE_AND_EXECUTE);
+                    }
                   }
                 }
+                
+                if(n&&v&&I->Obj.RepVis[cRepMesh]) {
+                  vc = ms->VC;              
+                  if(ms->MeshMode==1) 
+                    glPointSize(SettingGet_f(I->Obj.G,I->Obj.Setting,NULL,cSetting_dot_width));
+                  else
+                    glLineWidth(SettingGet_f(I->Obj.G,I->Obj.Setting,NULL,cSetting_mesh_width));
+                  while(*n) {
+                    c=*(n++);
+                    if(ms->MeshMode==1) 
+                      glBegin(GL_POINTS);
+                    else 
+                      glBegin(GL_LINE_STRIP);
+                    while(c--) {
+                      if(vc) { glColor3fv(vc); vc+=3;}
+                      glVertex3fv(v);
+                      v+=3;
+                    }
+                    glEnd();
+                  }
+                }
+                if(use_dlst&&ms->displayList) {
+                  glEndList();
+                }
               }
-
-            if(n&&v&&I->Obj.RepVis[cRepMesh]) {
-              if(ms->DotFlag) 
-                glPointSize(SettingGet_f(I->Obj.G,I->Obj.Setting,NULL,cSetting_dot_width));
-              else
-                glLineWidth(SettingGet_f(I->Obj.G,I->Obj.Setting,NULL,cSetting_mesh_width));
-              while(*n)
-                {
-                  c=*(n++);
-                  if(ms->DotFlag) 
-                    glBegin(GL_POINTS);
-                  else 
-                    glBegin(GL_LINE_STRIP);
-                  while(c--) {
-                    glVertex3fv(v);
-                    v+=3;
-                  }
-                  glEnd();
-                }
-            }
-            if(use_dlst&&ms->displayList) {
-              glEndList();
-            }
-            }
+              glEnable(GL_LIGHTING);
             }
           }
         }
@@ -706,9 +928,10 @@ ObjectMesh *ObjectMeshNew(PyMOLGlobals *G)
   
   I->Obj.fFree = (void (*)(struct CObject *))ObjectMeshFree;
   I->Obj.fUpdate =  (void (*)(struct CObject *)) ObjectMeshUpdate;
-  I->Obj.fRender =(void (*)(struct CObject *, int, CRay *, Pickable **,int ))ObjectMeshRender;
+  I->Obj.fRender =(void (*)(struct CObject *, RenderInfo *))ObjectMeshRender;
   I->Obj.fInvalidate =(void (*)(struct CObject *,int,int,int))ObjectMeshInvalidate;
   I->Obj.fGetNFrame = (int (*)(struct CObject *)) ObjectMeshGetNStates;
+  /*  I->Obj.fGetCaption = (char *(*)(struct CObject *))ObjectMeshGetCaption;*/
   return(I);
 }
 
@@ -730,20 +953,23 @@ void ObjectMeshStateInit(PyMOLGlobals *G,ObjectMeshState *ms)
   ms->N[0]=0;
   ms->Active=true;
   ms->ResurfaceFlag=true;
+  ms->RecolorFlag=false;
   ms->ExtentFlag=false;
   ms->CarveFlag=false;
   ms->CarveBuffer=0.0;
   ms->AtomVertex=NULL;
   ms->UnitCellCGO=NULL;
   ms->displayList=0;
+  ms->displayListInvalid = true;
+  ms->caption[0]=0;
 }
 
 /*========================================================================*/
 ObjectMesh *ObjectMeshFromBox(PyMOLGlobals *G,ObjectMesh *obj,ObjectMap *map,
                               int map_state,
                               int state,float *mn,float *mx,
-                              float level,int dotFlag,
-                              float carve,float *vert_vla)
+                              float level,int meshMode,
+                              float carve,float *vert_vla,float alt_level)
 {
   ObjectMesh *I;
   ObjectMeshState *ms;
@@ -769,8 +995,18 @@ ObjectMesh *ObjectMeshFromBox(PyMOLGlobals *G,ObjectMesh *obj,ObjectMap *map,
   oms = ObjectMapGetState(map,map_state);
 
   ms->Level = level;
-  ms->DotFlag = dotFlag;
+  ms->AltLevel = alt_level;
+  ms->MeshMode = meshMode;
   if(oms) {
+
+    if((meshMode==3) && (ms->AltLevel < ms->Level)) {
+      /* gradient object -- need to auto-set range */
+      if(!ObjectMapStateGetDataRange(G,oms,&ms->Level,&ms->AltLevel)) {
+        ms->Level = -1.0F;
+        ms->AltLevel = 1.0F;
+      }
+    }
+
     copy3f(mn,ms->ExtentMin); /* this is not exactly correct...should actually take vertex points from range */
     copy3f(mx,ms->ExtentMax);
 
@@ -835,6 +1071,18 @@ void ObjectMeshRecomputeExtent(ObjectMesh *I)
       }
     }
   }
+
   I->Obj.ExtentFlag=extent_flag;
+
+  if(I->Obj.TTTFlag && I->Obj.ExtentFlag) {
+    float *ttt;
+    double tttd[16];
+    if(ObjectGetTTT(&I->Obj,&ttt,-1)) {
+      convertTTTfR44d(ttt,tttd);
+      MatrixTransformExtentsR44d3f(tttd,
+                                   I->Obj.ExtentMin,I->Obj.ExtentMax,
+                                   I->Obj.ExtentMin,I->Obj.ExtentMax);
+    }
+  }
 }
 
