@@ -17,27 +17,33 @@ Z* -------------------------------------------------------------------
 #define _H_ObjectMesh
 
 #include"ObjectMap.h"
+#include"Word.h"
 
 typedef struct {
   CObjectState State;
-  char MapName[ObjNameMax];
+  ObjectNameType MapName;
   int MapState;
   CCrystal Crystal;
   int Active;
-  int *N;
-  float *V;
+  int *N,*RC,VCsize,base_n_V;
+  int OneColor;
+  float *V,*VC;
   int Range[6];
   float ExtentMin[3],ExtentMax[3];
   int ExtentFlag;
   float Level,Radius;
   int RefreshFlag;
   int ResurfaceFlag;
+  int RecolorFlag;
   float *AtomVertex;
   int CarveFlag;
   float CarveBuffer;
-  int DotFlag;
+  int MeshMode;
   CGO *UnitCellCGO;
   int displayList;
+  int displayListInvalid;
+  WordType caption;
+  float AltLevel;
 } ObjectMeshState;
 
 typedef struct ObjectMesh {
@@ -46,16 +52,19 @@ typedef struct ObjectMesh {
   int NState;
 } ObjectMesh;
 
+
 ObjectMesh *ObjectMeshFromBox(PyMOLGlobals *G,ObjectMesh *obj,ObjectMap* map,
                               int map_state,
                               int state,float *mn,float *mx,
-                              float level,int dotFlag,
-                              float carve,float *vert_vla);
+                              float level,int meshMode,
+                              float carve,float *vert_vla,float alt_level);
 void ObjectMeshDump(ObjectMesh *I,char *fname,int state);
 
 PyObject *ObjectMeshAsPyList(ObjectMesh *I);
 int ObjectMeshNewFromPyList(PyMOLGlobals *G,PyObject *list,ObjectMesh **result);
 int ObjectMeshSetLevel(ObjectMesh *I,float level,int state);
+int ObjectMeshGetLevel(ObjectMesh *I,int state, float *result);
+int ObjectMeshInvalidateMapName(ObjectMesh *I,char *name);
 
 #endif
 
