@@ -31,14 +31,7 @@ Z* -------------------------------------------------------------------
 #define cKeywordCenter "center"
 #define cKeywordOrigin "origin"
 
-#define cUndoMask 0x7
-
-typedef struct {
-  int index[2];
-  int order;
-  int id;
-  int stereo; /* to preserve 2D rep */
-} BondType;
+#define cUndoMask 0xF
 
 typedef struct ObjectMoleculeBPRec {
   int *dist;
@@ -58,6 +51,8 @@ typedef struct ObjectMolecule {
   int DiscreteFlag,NDiscrete;
   int *DiscreteAtmToIdx;
   struct CoordSet **DiscreteCSet;
+  /* proposed, for storing uniform trajectory data more efficiently:
+     int *UniformAtmToIdx, *UniformIdxToAtm;  */
   int CurCSet;
   int SeleBase; /* for internal usage by  selector & only valid during selection process */
   CSymmetry *Symmetry;
@@ -70,7 +65,9 @@ typedef struct ObjectMolecule {
   int BondCounter;
   int AtomCounter;
   struct CSculpt *Sculpt;
-  short int RepVisCache[cRepCnt]; /* for transient storage during updates */
+  signed char RepVisCache[cRepCnt]; /* for transient storage during updates */
+
+  /* not stored */
 } ObjectMolecule;
 
 typedef struct ObjectMoleculeOpRec {
@@ -79,7 +76,7 @@ typedef struct ObjectMoleculeOpRec {
   int cs1,cs2;
   int i1,i2,i3,i4,*vc1,*i1VLA,*ii1,*vp1;
   float f1,f2,*f1VLA,*f2VLA,*ff1;
-  double d[3][3];
+  double d[3][3],d1;
   float *vv1,*vv2;
   char *charVLA;
   char *s1;
@@ -106,9 +103,10 @@ typedef struct {
 } PDBScale;
 
 typedef struct {
-  int is_pqr_file, pqr_no_chain_id;
+  int is_pqr_file, pqr_workarounds;
   PDBScale scale;
   int ignore_header_names;
+  int multi_object_status; /* 0 = unknown, 1 = is multi_object, -1 is not multi_object */
   int multiplex;
 } PDBInfoRec;
 
@@ -172,6 +170,10 @@ typedef struct {
 #define OMOP_OnOff 53
 #define OMOP_Pop 54
 #define OMOP_TransformR44f 55
+#define OMOP_FixHydrogens 56
+#define OMOP_Sort 57
+#define OMOP_SetAtomicSetting 58
+#define OMOP_CSetSumSqDistToPt 59
 
 #include"CoordSet.h"
 
@@ -203,7 +205,7 @@ typedef struct {
 } M4XContextType;
 
 typedef struct {
-  char target[ObjNameMax];
+  ObjectNameType target;
   int n_point;
   int *id_at_point;
   float *fitness;
@@ -215,12 +217,12 @@ typedef struct {
   int n_context;
   M4XContextType *context;
   int xname_flag;
-  char xname[ObjNameMax];
+  ObjectNameType xname;
   M4XAlignType *align;
 } M4XAnnoType;
 
 typedef struct {
-  char name[ObjNameMax];
+  ObjectNameType name;
 } ObjMolMultiplexType;
 
 void M4XAnnoInit(M4XAnnoType *m4x);
@@ -230,10 +232,13 @@ void M4XAlignInit(M4XAlignType *align);
 void M4XAlignPurge(M4XAlignType *align);
 
 int ObjectMoleculeGetMatrix(ObjectMolecule *I,int state,double **history);
+int ObjectMoleculeSetMatrix(ObjectMolecule *I,int state,double *matrix);
 int ObjectMoleculeGetTopNeighbor(PyMOLGlobals *G,
                                  ObjectMolecule *I, 
                                  int start, int excluded);
 
+int ObjectMoleculeGetNearestAtomIndex(ObjectMolecule *I, float *point, float cutoff, int state, float *dist);
+
 int *ObjectMoleculeGetPrioritizedOtherIndexList(ObjectMolecule *I,struct CoordSet *cs);
 int ObjectMoleculeGetPrioritizedOther(int *other, int a1, int a2, int *double_sided);
 
@@ -249,18 +254,24 @@ PyObject *ObjectMoleculeAsPyList(ObjectMolecule *I);
 int ObjectMoleculeGetSerial(ObjectMolecule *I);
 int ObjectMoleculeSetStateTitle(ObjectMolecule *I,int state,char *text);
 char *ObjectMoleculeGetStateTitle(ObjectMolecule *I,int state);
-
+int ObjectMoleculeCheckFullStateSelection(ObjectMolecule *I,int sele, int state);
 void ObjectMoleculeFree(ObjectMolecule *I); /* only for friends of ObjectMolecule */
 
+int ObjectMoleculeAddPseudoatom(ObjectMolecule *I,int sele_index, char *name, 
+                                char *resn, char *resi, char *chain,
+                                char *segi, char *elem, float vdw, 
+                                int hetatm, float b, float q, char *label, 
+
+                                float *pos, int color, 
+                                int state, int more, int quiet);
+
 ObjectMolecule *ObjectMoleculeNew(PyMOLGlobals *G,int discreteFlag);
 void ObjectMoleculeSort(ObjectMolecule *I);
 ObjectMolecule *ObjectMoleculeCopy(ObjectMolecule *obj);
 void ObjectMoleculeFixChemistry(ObjectMolecule *I, int sele1, int sele2, int invalidate);
 
 ObjectMolecule *ObjectMoleculeLoadXYZFile(PyMOLGlobals *G,ObjectMolecule *obj,char *fname,int frame,int discrete);
-ObjectMolecule *ObjectMoleculeLoadPDBFile(PyMOLGlobals *G,ObjectMolecule *obj,char *fname,int frame,int discrete,M4XAnnoType *m4x,PDBInfoRec *pdb_info);
 ObjectMolecule *ObjectMoleculeLoadPMOFile(PyMOLGlobals *G,ObjectMolecule *obj,char *fname,int frame,int discrete);
-ObjectMolecule *ObjectMoleculeLoadMOLFile(PyMOLGlobals *G,ObjectMolecule *obj,char *fname,int frame,int discrete);
 ObjectMolecule *ObjectMoleculeLoadMMDFile(PyMOLGlobals *G,ObjectMolecule *obj,char *fname,
                                           int frame,char *sepPrefix,int discrete);
 ObjectMolecule *ObjectMoleculeLoadTOPFile(PyMOLGlobals *G,ObjectMolecule *obj,char *fname,int frame,int discrete);
@@ -269,20 +280,15 @@ ObjectMolecule *ObjectMoleculeLoadChemPyModel(PyMOLGlobals *G,ObjectMolecule *I,
 ObjectMolecule *ObjectMoleculeLoadTRJFile(PyMOLGlobals *G,ObjectMolecule *obj,char *fname,int frame,
                                           int interval,int average,int start,
                                           int stop,int max,char *sele,int image,
-                                          float *shift);
+                                          float *shift,int quiet);
 
-ObjectMolecule *ObjectMoleculeLoadRSTFile(PyMOLGlobals *G,ObjectMolecule *obj,char *fname,int frame);
+ObjectMolecule *ObjectMoleculeLoadRSTFile(PyMOLGlobals *G,ObjectMolecule *obj,
+                                          char *fname,int frame,int quiet);
 
 ObjectMolecule *ObjectMoleculeLoadCoords(PyMOLGlobals *G,ObjectMolecule *I,PyObject *coords,int frame);
 
 ObjectMolecule *ObjectMoleculeReadPMO(PyMOLGlobals *G,ObjectMolecule *obj,CRaw *pmo,int frame,int discrete);
 
-ObjectMolecule *ObjectMoleculeReadMOLStr(PyMOLGlobals *G,ObjectMolecule *obj,char *molstr,
-                                         int frame,int discrete,int finish);
-ObjectMolecule *ObjectMoleculeReadMOL2Str(PyMOLGlobals *G,ObjectMolecule *I,
-                                          char *MOLStr,int frame,int discrete,
-                                          int quiet,int multiplex, char *new_name,
-                                          char **next_entry);
 ObjectMolecule *ObjectMoleculeReadStr(PyMOLGlobals *G,ObjectMolecule *I,
                                    char *st,int content_format, int frame,int discrete,
                                    int quiet,int multiplex, char *new_name,
@@ -291,7 +297,7 @@ ObjectMolecule *ObjectMoleculeReadStr(PyMOLGlobals *G,ObjectMolecule *I,
 ObjectMolecule *ObjectMoleculeReadPDBStr(PyMOLGlobals *G,ObjectMolecule *obj,char *molstr,
                                          int frame,int discrete,
                                          M4XAnnoType *m4x,char *pdb_name,
-                                         char **next_pdb,PDBInfoRec *pdb_info,int quiet);
+                                         char **next_pdb,PDBInfoRec *pdb_info,int quiet, int *model_number);
 
 ObjectMolecule *ObjectMoleculeReadMMDStr(PyMOLGlobals *G,ObjectMolecule *I,char *MMDStr,int frame,int discrete);
 ObjectMolecule *ObjectMoleculeReadXYZStr(PyMOLGlobals *G,ObjectMolecule *I,char *PDBStr,int frame,int discrete);
@@ -300,7 +306,7 @@ void ObjectMoleculeExtendIndices(ObjectMolecule *I);
 
 void ObjectMoleculeInvalidate(ObjectMolecule *I,int rep,int level,int state);
 
-void ObjectMoleculeRenderSele(ObjectMolecule *I,int curState,int sele);
+void ObjectMoleculeRenderSele(ObjectMolecule *I,int curState,int sele,int vis_only);
 
 void ObjectMoleculeSeleOp(ObjectMolecule *I,int sele,ObjectMoleculeOpRec *op);
 
@@ -312,11 +318,14 @@ void ObjectMoleculeMerge(ObjectMolecule *I,AtomInfoType *ai,
 void ObjectMoleculeUpdateNonbonded(ObjectMolecule *I);
 void ObjectMoleculeUpdateNeighbors(ObjectMolecule *I);
 int ObjectMoleculeMoveAtom(ObjectMolecule *I,int state,int index,float *v,int mode,int log);
+int ObjectMoleculeMoveAtomLabel(ObjectMolecule *I,int state,int index,float *v,int mode,int log);
 int ObjectMoleculeGetAtomVertex(ObjectMolecule *I,int state,int index,float *v);
+int ObjectMoleculeGetAtomTxfVertex(ObjectMolecule *I,int state,int index,float *v);
 int ObjectMoleculeGetAtomIndex(ObjectMolecule *I,int sele);
 int ObjectMoleculeTransformSelection(ObjectMolecule *I,int state,
                                       int sele,float *TTT,int log,
-                                     char *sname,int homogenous);
+                                     char *sname,int homogenous,
+                                     int global);
 int ObjectMoleculeDoesAtomNeighborSele(ObjectMolecule *I, int index, int sele);
 void ObjectMoleculeInferChemFromNeighGeom(ObjectMolecule *I,int state);
 void ObjectMoleculeInferChemForProtein(ObjectMolecule *I,int state);
@@ -335,7 +344,10 @@ void ObjectMoleculeReplaceAtom(ObjectMolecule *I,int index,AtomInfoType *ai);
 void ObjectMoleculePreposReplAtom(ObjectMolecule *I,int index,AtomInfoType *ai);
 void ObjectMoleculeCreateSpheroid(ObjectMolecule *I,int average);
 int ObjectMoleculeSetAtomVertex(ObjectMolecule *I,int state,int index,float *v);
-int ObjectMoleculeVerifyChemistry(ObjectMolecule *I);
+int ObjectMoleculeVerifyChemistry(ObjectMolecule *I,int state);
+int ObjectMoleculeFindOpenValenceVector(ObjectMolecule *I,int state,
+                                        int index,float *v,float *seek,
+                                        int ignore_index);
 int ObjectMoleculeFillOpenValences(ObjectMolecule *I,int index);
 int ObjectMoleculeGetTotalAtomValence(ObjectMolecule *I,int atom);
 int ObjectMoleculeAdjustBonds(ObjectMolecule *I,int sele0,int sele1,int mode,int order);
@@ -344,7 +356,7 @@ void ObjectMoleculeFuse(ObjectMolecule *I,int index0,ObjectMolecule *src,int ind
 void ObjectMoleculeRenameAtoms(ObjectMolecule *I,int force);
 int ObjectMoleculeAreAtomsBonded(ObjectMolecule *I,int i0,int i1);
 void ObjectGotoState(ObjectMolecule *I,int state);
-float ObjectMoleculeGetAvgHBondVector(ObjectMolecule *I,int atom,int state,float *v);
+float ObjectMoleculeGetAvgHBondVector(ObjectMolecule *I,int atom,int state,float *v,float *incoming);
 int ObjectMoleculeCheckBondSep(ObjectMolecule *I,int a0,int a1,int dist);
 int ObjectMoleculeGetPhiPsi(ObjectMolecule *I,int ca,float *phi,float *psi,int state);
 void ObjectMoleculeGetAtomSele(ObjectMolecule *I,int index, char *buffer);
@@ -353,8 +365,8 @@ void ObjectMoleculeGetAtomSeleLog(ObjectMolecule *I,int index, char *buffer,int
 int ObjectMoleculeMultiSave(ObjectMolecule *I,char *fname,int state,int append);
 void ObjectMoleculeUpdateIDNumbers(ObjectMolecule *I);
 
-void ObjectMoleculeSculptImprint(ObjectMolecule *I,int state);
-float ObjectMoleculeSculptIterate(ObjectMolecule *I,int state,int n_cycle);
+void ObjectMoleculeSculptImprint(ObjectMolecule *I,int state,int match_state,int match_by_segment);
+float ObjectMoleculeSculptIterate(ObjectMolecule *I,int state,int n_cycle,float *center);
 void ObjectMoleculeSculptClear(ObjectMolecule *I);
 int ObjectMoleculeGetBondPaths(ObjectMolecule *I,int atom, int max, ObjectMoleculeBPRec *bp);
 int ObjectMoleculeInitBondPath(ObjectMolecule *I,ObjectMoleculeBPRec *bp);
@@ -363,10 +375,12 @@ int ObjectMoleculeGetBondPath(ObjectMolecule *I,int atom,int max,ObjectMoleculeB
 int ***ObjectMoleculeGetBondPrint(ObjectMolecule *I,int max_bond,int max_type,int *dim);
 
 int ObjectMoleculeConnect(ObjectMolecule *I,BondType **bond,AtomInfoType *ai,struct CoordSet *cs,int searchFlag);
+int ObjectMoleculeSetDiscrete(PyMOLGlobals *G,ObjectMolecule *I,int discrete);
+
 float ObjectMoleculeGetMaxVDW(ObjectMolecule *I);
-int ObjectMoleculeFindOpenValenceVector(ObjectMolecule *I,int state,
-                                        int index,float *v,float *seek);
-int ObjectMoleculeGetCheckHBond(ObjectMolecule *don_obj,
+int ObjectMoleculeGetCheckHBond(int *h_is_real,
+                                float *h_crd_ret,
+                                ObjectMolecule *don_obj,
                                 int don_atom,
                                 int don_state,
                                 ObjectMolecule *acc_obj,
@@ -382,6 +396,9 @@ int ObjectMoleculeConvertIDsToIndices(ObjectMolecule *I,int *id,int n_id);
 ObjectMolecule *ObjectMoleculeDummyNew(PyMOLGlobals *G,int mode);
 void ObjectMoleculeDummyUpdate(ObjectMolecule *I,int mode);
 
+void ObjectMoleculeTransformState44f(ObjectMolecule *I,int state,float *matrix,
+                                     int log_trans,int homogenous,int transformed);
+
 /* internal to ObjectMolecule */
 
 struct CoordSet *ObjectMoleculePDBStr2CoordSet(PyMOLGlobals *G,
@@ -393,7 +410,7 @@ struct CoordSet *ObjectMoleculePDBStr2CoordSet(PyMOLGlobals *G,
                                                char *pdb_name,
                                                char **next_pdb,
                                                PDBInfoRec *pdb_info,
-                                               int quiet);
+                                               int quiet, int *model_number);
 
 #if 0
 /* legacy binary file suppoort */
@@ -410,6 +427,23 @@ typedef struct {
 } BondType083;
 #endif
 
+#if 0
+ObjectMolecule *ObjectMoleculeReadMOL2Str(PyMOLGlobals *G,ObjectMolecule *I,
+                                          char *MOLStr,int frame,int discrete,
+                                          int quiet,int multiplex, char *new_name,
+                                          char **next_entry);
+
+
+ObjectMolecule *ObjectMoleculeReadMOLStr(PyMOLGlobals *G,ObjectMolecule *obj,char *molstr,
+                                         int frame,int discrete,int finish);
+
+ObjectMolecule *ObjectMoleculeLoadMOLFile(PyMOLGlobals *G,ObjectMolecule *obj,char *fname,int frame,int discrete);
+
+ObjectMolecule *ObjectMoleculeLoadPDBFile(PyMOLGlobals *G,ObjectMolecule *obj,char *fname,int frame,int discrete,M4XAnnoType *m4x,PDBInfoRec *pdb_info);
+
+#endif
+
+
 #endif
 
 
