@@ -75,7 +75,7 @@ static PyObject *ObjectSliceStateAsPyList(ObjectSliceState *I)
   
 #if 0
   int Active;
-  char MapName[ObjNameMax];
+  ObjectNameType MapName;
   int MapState;
   float ExtentMin[3];
   float ExtentMax[3];
@@ -124,7 +124,7 @@ static int ObjectSliceStateFromPyList(PyMOLGlobals *G,ObjectSliceState *I,PyObje
          Always check ll when adding new PyList_GetItem's */      
 
       if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,0),&I->Active);
-      if(ok) ok = PConvPyStrToStr(PyList_GetItem(list,1),I->MapName,ObjNameMax);
+      if(ok) ok = PConvPyStrToStr(PyList_GetItem(list,1),I->MapName,WordLength);
       if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,2),&I->MapState);
       if(ok) ok = PConvPyListToFloatArrayInPlace(PyList_GetItem(list,3),I->ExtentMin,3);
       if(ok) ok = PConvPyListToFloatArrayInPlace(PyList_GetItem(list,4),I->ExtentMax,3);
@@ -218,8 +218,8 @@ static void ObjectSliceStateFree(ObjectSliceState *oss)
         }
       } else {
         char buffer[255]; /* pass this off to the main thread */
-        sprintf(buffer,"_cmd.gl_delete_lists(%d,%d)\n",oss->displayList,1);
-        PParse(buffer);
+        sprintf(buffer,"_cmd.gl_delete_lists(cmd._COb,%d,%d)\n",oss->displayList,1);
+        PParse(oss->G,buffer);
       }
     }
   }
@@ -301,7 +301,7 @@ static void ObjectSliceStateUpdate(ObjectSlice *I,ObjectSliceState *oss, ObjectM
   if(grid<0.01F)
     grid=0.01F;
   
-  /* for the given map, compute a new set of interpolated point with accompanying levels */
+  /* for the given map, compute a new set of interpolated points with accompanying levels */
 
   /* first, find the limits of the enclosing rectangle, starting at the slice origin, 
      via a simple brute-force approach... */
@@ -702,6 +702,7 @@ static void ObjectSliceUpdate(ObjectSlice *I)
         
         if(oss->RefreshFlag) {
           oss->RefreshFlag=false;
+          oss->displayListInvalid = true;
           PRINTFB(I->Obj.G,FB_ObjectSlice,FB_Blather)
             " ObjectSlice: updating \"%s\".\n" , I->Obj.Name 
             ENDFB(I->Obj.G);
@@ -722,7 +723,7 @@ static void ObjectSliceUpdate(ObjectSlice *I)
           }
         }
       }
-      SceneDirty(I->Obj.G);
+      SceneInvalidate(I->Obj.G);
     }
   }
 }
@@ -739,6 +740,7 @@ void ObjectSliceDrag(ObjectSlice *I, int state, int mode, float *pt, float *mov,
   if(oss) {
     switch(mode) {
     case cButModeRotFrag: /* rotated about origin */
+    case cButModeRotObj:
       {
         float v3[3];
         float n0[3];
@@ -767,11 +769,14 @@ void ObjectSliceDrag(ObjectSlice *I, int state, int mode, float *pt, float *mov,
         multiply33f33f(mat,oss->system,oss->system);
         
         ObjectSliceInvalidate(I,cRepSlice,cRepAll,state);
-        SceneDirty(I->Obj.G);
+        SceneInvalidate(I->Obj.G);
         
       }
       break;
     case cButModeMovFrag: /* move along "up" direction */
+    case cButModeMovFragZ:
+    case cButModeMovObj:
+    case cButModeMovObjZ:
       {
         float up[3],v1[3];
         up[0]=oss->system[2];
@@ -781,7 +786,7 @@ void ObjectSliceDrag(ObjectSlice *I, int state, int mode, float *pt, float *mov,
         project3f(mov,up,v1);
         add3f(v1,oss->origin,oss->origin);
         ObjectSliceInvalidate(I,cRepSlice,cRepAll,state);        
-        SceneDirty(I->Obj.G);
+        SceneInvalidate(I->Obj.G);
       }
       break;
     case cButModeTorFrag: 
@@ -848,10 +853,14 @@ int ObjectSliceGetOrigin(ObjectSlice *I,int state,float *origin)
   return ok;
 }
 
-static void ObjectSliceRender(ObjectSlice *I,int state,CRay *ray,Pickable **pick,int pass)
+static void ObjectSliceRender(ObjectSlice *I,RenderInfo *info)
 {
 
   PyMOLGlobals *G = I->Obj.G;
+  int state = info->state;
+  CRay *ray = info->ray;
+  Picking **pick = info->pick;
+  int pass = info->pass;
   int cur_state = 0;
   float alpha;
   int track_camera = SettingGet_b(G,NULL,I->Obj.Setting,cSetting_slice_track_camera);
@@ -1009,12 +1018,12 @@ static void ObjectSliceRender(ObjectSlice *I,int state,CRay *ray,Pickable **pick
         } else if(G->HaveGUI && G->ValidContext) {
           if(pick) {
 
-          int i=(*pick)->index;
+          int i=(*pick)->src.index;
           int j;
-          Pickable p;
+          Picking p;
           
-          p.ptr = (void*)I;
-          p.index = state+1;
+          p.context.object = (void*)I;
+          p.src.index = state+1;
           
           if(I->Obj.RepVis[cRepSlice]) {
             int *strip = oss->strips;
@@ -1049,10 +1058,10 @@ static void ObjectSliceRender(ObjectSlice *I,int state,CRay *ray,Pickable **pick
                   if(tri_count>=3) {
                     
                     i++;
-                    if(!(*pick)[0].ptr) {
+                    if(!(*pick)[0].src.bond) {
                       /* pass 1 - low order bits */
                       glColor3ub((uchar)((i&0xF)<<4),(uchar)((i&0xF0)|0x8),(uchar)((i&0xF00)>>4)); 
-                      VLACheck((*pick),Pickable,i);
+                      VLACheck((*pick),Picking,i);
                       (*pick)[i] = p; /* copy object and atom info */
                     } else { 
                       /* pass 2 - high order bits */
@@ -1060,7 +1069,7 @@ static void ObjectSliceRender(ObjectSlice *I,int state,CRay *ray,Pickable **pick
                       j=i>>12;
                       glColor3ub((uchar)((j&0xF)<<4),(uchar)((j&0xF0)|0x8),(uchar)((j&0xF00)>>4)); 
                     }
-                    p.bond = offset0 + 1;
+                    p.src.bond = offset0 + 1;
                     
                     if(tri_count&0x1) { /* get the handedness right ... */
                       glVertex3fv(point+3*offset0);
@@ -1080,7 +1089,7 @@ static void ObjectSliceRender(ObjectSlice *I,int state,CRay *ray,Pickable **pick
               glEnd();
             }
           }
-          (*pick)[0].index = i; /* pass the count */
+          (*pick)[0].src.index = i; /* pass the count */
           } else {
 
           int render_now = false;
@@ -1096,6 +1105,13 @@ static void ObjectSliceRender(ObjectSlice *I,int state,CRay *ray,Pickable **pick
             SceneResetNormal(G,false);
             ObjectUseColor(&I->Obj);
             use_dlst = (int)SettingGet(G,cSetting_use_display_lists);
+
+            if(use_dlst && oss->displayList && oss->displayListInvalid) {
+              glDeleteLists(oss->displayList,1);
+              oss->displayList = 0;
+              oss->displayListInvalid = false;
+            }
+
             if(use_dlst&&oss->displayList) {
               glCallList(oss->displayList);
             } else { 
@@ -1208,7 +1224,7 @@ ObjectSlice *ObjectSliceNew(PyMOLGlobals *G)
   
   I->Obj.fFree = (void (*)(struct CObject *))ObjectSliceFree;
   I->Obj.fUpdate =  (void (*)(struct CObject *)) ObjectSliceUpdate;
-  I->Obj.fRender =(void (*)(struct CObject *, int, CRay *, Pickable **,int ))ObjectSliceRender;
+  I->Obj.fRender =(void (*)(struct CObject *, RenderInfo *))ObjectSliceRender;
   I->Obj.fInvalidate =(void (*)(struct CObject *,int,int,int))ObjectSliceInvalidate;
   I->Obj.fGetNFrame = (int (*)(struct CObject *)) ObjectSliceGetNStates;
   return(I);
