@@ -73,7 +73,7 @@ static PyObject *ObjectSurfaceStateAsPyList(ObjectSurfaceState *I)
   PyList_SetItem(result,15,PyInt_FromLong(I->Side));
 
 #if 0
-  char MapName[ObjNameMax];
+  ObjectNameType MapName;
   int MapState;
   CCrystal Crystal;
   int Active;
@@ -127,7 +127,7 @@ static int ObjectSurfaceStateFromPyList(PyMOLGlobals *G,ObjectSurfaceState *I,Py
       if(ok) ok=PyList_Check(list);
       if(ok) ll=PyList_Size(list);
       if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,0),&I->Active);
-      if(ok) ok = PConvPyStrToStr(PyList_GetItem(list,1),I->MapName,ObjNameMax);
+      if(ok) ok = PConvPyStrToStr(PyList_GetItem(list,1),I->MapName,WordLength);
       if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,2),&I->MapState);
       if(ok) ok = CrystalFromPyList(&I->Crystal,PyList_GetItem(list,3));
       if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,4),&I->ExtentFlag);
@@ -233,8 +233,8 @@ static void ObjectSurfaceStateFree(ObjectSurfaceState *ms)
         }
       } else {
         char buffer[255]; /* pass this off to the main thread */
-        sprintf(buffer,"_cmd.gl_delete_lists(%d,%d)\n",ms->displayList,1);
-        PParse(buffer);
+        sprintf(buffer,"_cmd.gl_delete_lists(cmd._COb,%d,%d)\n",ms->displayList,1);
+        PParse(ms->State.G,buffer);
       }
     }
   }
@@ -242,6 +242,8 @@ static void ObjectSurfaceStateFree(ObjectSurfaceState *ms)
   
   VLAFreeP(ms->N);
   VLAFreeP(ms->V);
+  FreeP(ms->VC);
+  FreeP(ms->RC);
   VLAFreeP(ms->AtomVertex);
   if(ms->UnitCellCGO)
     CGOFree(ms->UnitCellCGO);
@@ -300,10 +302,14 @@ void ObjectSurfaceDump(ObjectSurface *I,char *fname,int state)
   }
 }
 
+
 static void ObjectSurfaceInvalidate(ObjectSurface *I,int rep,int level,int state)
 {
   int a;
   int once_flag=true;
+  if(level>=cRepInvExtents) {
+    I->Obj.ExtentFlag=false;
+  }
   for(a=0;a<I->NState;a++) {
     if(state<0) once_flag=false;
     if(!once_flag) state=a;
@@ -311,13 +317,158 @@ static void ObjectSurfaceInvalidate(ObjectSurface *I,int rep,int level,int state
     if(level>=cRepInvAll) {
       I->State[state].ResurfaceFlag=true;     
       SceneChanged(I->Obj.G);
+    } else if(level>=cRepInvColor) {
+      I->State[state].RecolorFlag=true;
+      SceneChanged(I->Obj.G);
     } else {
-      SceneDirty(I->Obj.G);
+      SceneInvalidate(I->Obj.G);
     }
     if(once_flag) break;
   }
 }
 
+int ObjectSurfaceInvalidateMapName(ObjectSurface *I,char *name)
+{
+  int a;
+  ObjectSurfaceState *ms;
+  int result=false;
+  for(a=0;a<I->NState;a++) {
+    ms=I->State+a;
+    if(ms->Active) {
+      if(strcmp(ms->MapName,name)==0) {
+        ObjectSurfaceInvalidate(I,cRepAll,cRepInvAll,a);
+        result=true;
+      }
+    }
+  }
+  return result;
+}
+
+static void ObjectSurfaceStateUpdateColors(ObjectSurface *I, ObjectSurfaceState *ms)
+{ 
+  int one_color_flag = true;
+  int cur_color = SettingGet_color(I->Obj.G, I->Obj.Setting, NULL, cSetting_surface_color);
+
+  if(cur_color == -1)
+    cur_color = I->Obj.Color;
+
+  if(ColorCheckRamped(I->Obj.G,cur_color))
+    one_color_flag = false;
+
+  ms->OneColor = cur_color;
+  if(ms->V) {
+    int ramped_flag = false;
+    float *v = ms->V;
+    float *vc;
+    int *rc;
+    int a;
+    int state = ms - I->State;
+    int base_n_vert = ms->base_n_V;
+    switch(ms->Mode) {
+    case 3:
+    case 2:
+      {
+        int n_vert = VLAGetSize(ms->V)/6;
+        base_n_vert /=6;
+
+        if(ms->VC && (ms->VCsize<n_vert)) {
+          FreeP(ms->VC);
+          FreeP(ms->RC);
+        }
+        
+        if(!ms->VC) {
+          ms->VCsize = n_vert;
+          ms->VC = Alloc(float,n_vert*3);
+        }
+        if(!ms->RC) {
+          ms->RC = Alloc(int,n_vert);
+        }
+        rc = ms->RC;
+        vc = ms->VC;
+        v+=3;
+        if(vc) {
+          for(a=0;a<n_vert;a++) {
+            if(a==base_n_vert) {
+              int new_color = SettingGet_color(I->Obj.G, I->Obj.Setting, 
+                                               NULL, cSetting_surface_negative_color);
+              if(new_color==-1) new_color=cur_color;
+              if(new_color!=cur_color) {
+                one_color_flag = false;
+                cur_color = new_color;
+              }
+            }
+            if(ColorCheckRamped(I->Obj.G,cur_color)) {
+              ColorGetRamped(I->Obj.G,cur_color,v,vc,state);
+              *rc = cur_color;
+              ramped_flag=true;
+            } else {
+              float *col = ColorGet(I->Obj.G,cur_color);
+              copy3f(col,vc);
+            }
+            rc++;
+            vc+=3;
+            v+=6; /* alternates with normals */
+          }
+        }
+      }
+      break;
+    case 1:
+    case 0:
+    default:
+      {
+        int n_vert = VLAGetSize(ms->V)/3;
+        base_n_vert /=3;
+        if(ms->VC && (ms->VCsize<n_vert)) {
+          FreeP(ms->VC);
+          FreeP(ms->RC);
+        }
+        
+        if(!ms->VC) {
+          ms->VCsize = n_vert;
+          ms->VC = Alloc(float,n_vert*3);
+        }
+        if(!ms->RC) {
+          ms->RC = Alloc(int,n_vert);
+        }
+        rc = ms->RC;
+        vc = ms->VC;
+        if(vc) {
+          for(a=0;a<n_vert;a++) {
+            if(a==base_n_vert) {
+              int new_color = SettingGet_color(I->Obj.G, I->Obj.Setting, 
+                                               NULL, cSetting_surface_negative_color);
+              if(new_color==-1) new_color=cur_color;
+              if(new_color!=cur_color)
+                one_color_flag = false;
+                cur_color = new_color;
+            }
+
+            if(ColorCheckRamped(I->Obj.G,cur_color)) {
+              ColorGetRamped(I->Obj.G,cur_color,v,vc,state);
+              *rc = cur_color;
+              ramped_flag=true;
+            } else {
+              float *col = ColorGet(I->Obj.G,cur_color);
+              copy3f(col,vc);
+            }
+            rc++;
+            vc+=3;
+            v+=3; 
+          }
+        }
+      }
+      break;
+    }
+
+    if(one_color_flag && (!ramped_flag)) {
+      FreeP(ms->VC);
+      FreeP(ms->RC);
+    } else if( (!ramped_flag) || (!SettingGet_b(I->Obj.G,NULL,I->Obj.Setting,cSetting_ray_color_ramps))) {
+      FreeP(ms->RC);
+    }
+  }
+}
+
 static void ObjectSurfaceUpdate(ObjectSurface *I) 
 {
   int a;
@@ -356,16 +507,15 @@ static void ObjectSurfaceUpdate(ObjectSurface *I)
           } else if(ms->State.Matrix) {
             ObjectStateResetMatrix(&ms->State);
           }
-          
+          ms->displayListInvalid = true;          
           ms->RefreshFlag=false;
         }
       }
       if(map&&ms&&oms&&ms->N&&ms->V&&I->Obj.RepVis[cRepSurface]) {
         if(ms->ResurfaceFlag) {
-
-
           ms->ResurfaceFlag=false;
-          PRINTFB(I->Obj.G,FB_ObjectSurface,FB_Actions)
+          ms->RecolorFlag=true;
+          PRINTFB(I->Obj.G,FB_ObjectSurface,FB_Details)
            " ObjectSurface: updating \"%s\".\n" , I->Obj.Name 
             ENDFB(I->Obj.G);
           if(oms->Field) {
@@ -383,7 +533,7 @@ static void ObjectSurfaceUpdate(ObjectSurface *I)
                 max_ext = ms->ExtentMax;
               }
               
-              TetsurfGetRange(oms->Field,oms->Crystal,
+              TetsurfGetRange(I->Obj.G,oms->Field,oms->Crystal,
                               min_ext,max_ext,ms->Range);
             }
 
@@ -409,6 +559,56 @@ static void ObjectSurfaceUpdate(ObjectSurface *I)
                                  ms->CarveBuffer,
                                  ms->Side);
             
+            if(!SettingGet_b(I->Obj.G,I->Obj.Setting,NULL,cSetting_surface_negative_visible)) { 
+              ms->base_n_V = VLAGetSize(ms->V);
+            } else {
+              /* do we want the negative surface too? */
+
+              int nT2;
+              int *N2 = VLAlloc(int,10000);
+              float *V2 =  VLAlloc(float,10000);
+
+              nT2 = TetsurfVolume(I->Obj.G,oms->Field,
+                                  -ms->Level,
+                                  &N2,&V2,
+                                  ms->Range,
+                                  ms->Mode,
+                                  voxelmap,
+                                  ms->AtomVertex,
+                                  ms->CarveBuffer,
+                                  ms->Side);
+              if(N2&&V2) {
+                
+                int base_n_N = VLAGetSize(ms->N);
+                int base_n_V = VLAGetSize(ms->V);
+                int addl_n_N = VLAGetSize(N2);
+                int addl_n_V = VLAGetSize(V2);
+
+                ms->base_n_V = base_n_V;
+
+                /* make room */
+
+                VLASize(ms->N, int, base_n_N + addl_n_N);
+                VLASize(ms->V, float, base_n_V + addl_n_V);
+                
+                /* copy vertex data */
+
+                memcpy(((char*)ms->V)+(sizeof(float)*base_n_V),
+                       V2, sizeof(float)*addl_n_V);
+
+                /* copy strip counts */
+
+                memcpy(((char*)ms->N)+(sizeof(int)*(base_n_N-1)),
+                       N2, sizeof(int)*addl_n_N);
+                ms->N[ base_n_N + addl_n_N - 1] = 0;
+
+                ms->nT += nT2;
+                VLAFreeP(N2);
+                VLAFreeP(V2);
+              }
+
+            }
+            
             if(voxelmap)
               MapFree(voxelmap);
 
@@ -464,12 +664,19 @@ static void ObjectSurfaceUpdate(ObjectSurface *I)
             }
           }
         }
+        if(ms->RecolorFlag) {
+          ObjectSurfaceStateUpdateColors(I,ms);
+          ms->RecolorFlag=false;
+        }
       }
     }
   }
-  SceneDirty(I->Obj.G);
+  if(!I->Obj.ExtentFlag) {
+    ObjectSurfaceRecomputeExtent(I);
+  }
+  SceneInvalidate(I->Obj.G);
 }
-
+#if 0
 static int ZOrderFn(float *array,int l,int r)
 {
   return (array[l]<=array[r]);
@@ -479,12 +686,17 @@ static int ZRevOrderFn(float *array,int l,int r)
 {
   return (array[l]>=array[r]);
 }
-
-static void ObjectSurfaceRender(ObjectSurface *I,int state,CRay *ray,Pickable **pick,int pass)
+#endif
+static void ObjectSurfaceRender(ObjectSurface *I,RenderInfo *info)
 {
   PyMOLGlobals *G = I->Obj.G;
+  int state = info->state;
+  CRay *ray = info->ray;
+  Picking **pick = info->pick;
+  int pass = info->pass;
   float *v = NULL;
-  float *vc;
+  float *vc = NULL;
+  int *rc = NULL;
   float *col;
   int *n = NULL;
   int c;
@@ -494,8 +706,7 @@ static void ObjectSurfaceRender(ObjectSurface *I,int state,CRay *ray,Pickable **
 
   ObjectPrepareContext(&I->Obj,ray);
 
-  alpha = SettingGet_f(G,NULL,I->Obj.Setting,cSetting_transparency);
-  alpha=1.0F-alpha;
+  alpha = 1.0F - SettingGet_f(G,NULL,I->Obj.Setting,cSetting_transparency);
   if(fabs(alpha-1.0)<R_SMALL4)
     alpha=1.0F;
   
@@ -523,11 +734,20 @@ static void ObjectSurfaceRender(ObjectSurface *I,int state,CRay *ray,Pickable **
             CGORenderRay(ms->UnitCellCGO,ray,ColorGet(G,I->Obj.Color),
                          I->Obj.Setting,NULL);
 
-
           ray->fTransparentf(ray,1.0F-alpha);       
           ms->Radius=SettingGet_f(G,I->Obj.Setting,NULL,cSetting_mesh_radius);
+          if(ms->Radius==0.0F) {
+            ms->Radius = ray->PixelRadius*
+              SettingGet_f(I->Obj.G,I->Obj.Setting,NULL,cSetting_mesh_width)/2.0F;
+          }
+          
           if(n&&v&&I->Obj.RepVis[cRepSurface]) {
-            vc = ColorGet(G,I->Obj.Color);
+            float cc[3];
+            float colA[3],colB[3],colC[3];
+            ColorGetEncoded(G,ms->OneColor,cc);
+            vc = ms->VC;
+
+            rc = ms->RC;
             while(*n)
               {
                 c=*(n++);
@@ -535,12 +755,26 @@ static void ObjectSurfaceRender(ObjectSurface *I,int state,CRay *ray,Pickable **
                 case 3:
                 case 2:
                   v+=12;
+                  if(vc) vc+=6;
                   c-=4;
                   while(c>0) {
-                    
-                    ray->fTriangle3fv(ray,v-9,v-3,v+3,
-                                      v-12,v-6,v,
-                                      vc,vc,vc);
+                    if(vc) {
+                      register float *cA=vc-6, *cB=vc-3, *cC=vc;
+                      if(rc) {
+                        if(rc[0]<-1) ColorGetEncoded(G,rc[0], (cA=colA));
+                        if(rc[1]<-1) ColorGetEncoded(G,rc[1], (cB=colB));
+                        if(rc[2]<-1) ColorGetEncoded(G,rc[2], (cC=colC));
+                        rc++;
+                      }
+                      ray->fTriangle3fv(ray,v-9,v-3,v+3,
+                                        v-12,v-6,v,
+                                        cA,cB,cC);
+                      vc+=3;
+                    } else {
+                      ray->fTriangle3fv(ray,v-9,v-3,v+3,
+                                        v-12,v-6,v,
+                                        cc,cc,cc);
+                    }
                     v+=6;
                     c-=2;
                   }
@@ -548,8 +782,19 @@ static void ObjectSurfaceRender(ObjectSurface *I,int state,CRay *ray,Pickable **
                 case 1:
                   c--;
                   v+=3;
+                  if(vc) vc+=3;
                   while(c>0) {
-                    ray->fSausage3fv(ray,v-3,v,ms->Radius,vc,vc);
+                    if(vc) {
+                      register float *cA=vc-3, *cB=vc;
+                      if(rc) {
+                        if(rc[0]<-1) ColorGetEncoded(G,rc[0], (cA=colA));
+                        if(rc[1]<-1) ColorGetEncoded(G,rc[1], (cB=colB));
+                        rc++;
+                      }
+                      ray->fSausage3fv(ray,v-3,v,ms->Radius,cA,cB);
+                      vc+=3;
+                    } else 
+                      ray->fSausage3fv(ray,v-3,v,ms->Radius,cc,cc);
                     v+=3;
                     c--;
                   }
@@ -557,6 +802,10 @@ static void ObjectSurfaceRender(ObjectSurface *I,int state,CRay *ray,Pickable **
                 case 0:
                 default:
                   while(c>0) {
+                    if(vc) {
+                      ray->fColor3fv(ray,vc);
+                      vc+=3;
+                    }
                     ray->fSphere3fv(ray,v,ms->Radius);
                     v+=3;
                     c--;
@@ -570,25 +819,39 @@ static void ObjectSurfaceRender(ObjectSurface *I,int state,CRay *ray,Pickable **
         } else if(G->HaveGUI && G->ValidContext) {
           if(pick) {
           } else {
-            
             int render_now = false;
-            if(alpha>0.0001) {
-              render_now = (pass==-1);
+            int t_mode;
+            int use_dlst = SettingGetGlobal_b(G,cSetting_use_display_lists);
+
+            t_mode  = SettingGet_i(G, NULL, I->Obj.Setting, cSetting_transparency_mode);
+
+            if(info && info->alpha_cgo) {
+              render_now = (pass == 1);
+              t_mode = 0; 
+              if(alpha != 1.0F)
+                use_dlst = false;
+            } else if(alpha < 1.0F) {
+              use_dlst = false;
+              render_now = (pass == -1);
             } else 
-              render_now = (!pass);
+              render_now = (pass == 1);
             if(render_now) {
-              int use_dlst;
               
               if(ms->UnitCellCGO&&(I->Obj.RepVis[cRepCell]))
                 CGORenderGL(ms->UnitCellCGO,ColorGet(G,I->Obj.Color),
-                            I->Obj.Setting,NULL);
+                            I->Obj.Setting,NULL,info);
               
               SceneResetNormal(G,false);
-              col = ColorGet(G,I->Obj.Color);
+              col = ColorGet(G,ms->OneColor);
               glColor4f(col[0],col[1],col[2],alpha);
-              
-              use_dlst = (int)SettingGet(G,cSetting_use_display_lists);
-              if(use_dlst&&ms->displayList) {
+
+              if(use_dlst && ms->displayList && ms->displayListInvalid) {
+                glDeleteLists(ms->displayList,1);
+                ms->displayList = 0;
+                ms->displayListInvalid = false;
+              }
+
+              if(use_dlst && ms->displayList) {
                 glCallList(ms->displayList);
               } else { 
                 
@@ -601,18 +864,14 @@ static void ObjectSurfaceRender(ObjectSurface *I,int state,CRay *ray,Pickable **
                   }
                 }
                 
-                
                 if(n&&v&&I->Obj.RepVis[cRepSurface]) {
                   
-                  if((ms->Mode>1)&&(alpha!=1.0)) { /* transparent */
-                    
-                    int t_mode;
-                    
-                    t_mode  = SettingGet_i(G,NULL,I->Obj.Setting,cSetting_transparency_mode);
+                  if( (ms->Mode > 1) && (alpha != 1.0)) { /* transparent */
                     
                     if(t_mode) { /* high quality (sorted) transparency? */
                       
                       float **t_buf=NULL,**tb;
+                      float **c_buf=NULL,**tc;
                       float *z_value=NULL,*zv;
                       int *ix=NULL;
                       int n_tri = 0;
@@ -622,18 +881,26 @@ static void ObjectSurfaceRender(ObjectSurface *I,int state,CRay *ray,Pickable **
                       glGetFloatv(GL_MODELVIEW_MATRIX,matrix);
                       
                       t_buf = Alloc(float*,ms->nT*9);
-                      
+                      vc = ms->VC;
+
+                      if(vc) {
+                        c_buf = Alloc(float*,ms->nT*9);
+                      }
+
                       z_value = Alloc(float,ms->nT);
                       ix = Alloc(int,ms->nT);
                       
                       zv = z_value;
                       tb = t_buf;
-                      
+                      tc = c_buf;
+
                       while(*n)
                         {
                           parity=true;
                           c=*(n++);
                           v+=12;
+                          if(vc)
+                            vc+=6;
                           c-=4;
                           while(c>0) {
                             
@@ -644,6 +911,11 @@ static void ObjectSurfaceRender(ObjectSurface *I,int state,CRay *ray,Pickable **
                               *(tb++) = v-3;
                               *(tb++) = v;
                               *(tb++) = v+3;
+                              if(vc) {
+                                *(tc++) = vc-6;
+                                *(tc++) = vc-3;
+                                *(tc++) = vc;
+                              }
                             } else {
                               *(tb++) = v-12;
                               *(tb++) = v-9;
@@ -651,6 +923,11 @@ static void ObjectSurfaceRender(ObjectSurface *I,int state,CRay *ray,Pickable **
                               *(tb++) = v+3;
                               *(tb++) = v-6;
                               *(tb++) = v-3;
+                              if(vc) {
+                                *(tc++) = vc-6;
+                                *(tc++) = vc;
+                                *(tc++) = vc-3;
+                              }
                             }
                             
                             parity=!parity;
@@ -661,33 +938,42 @@ static void ObjectSurfaceRender(ObjectSurface *I,int state,CRay *ray,Pickable **
                             *(zv++) = matrix[2]*sum[0]+matrix[6]*sum[1]+matrix[10]*sum[2];
                             n_tri++;
                             
+                            if(vc) vc+=3;
                             v+=6;
                             c-=2;
                           }
                         }
+
+
                       switch(t_mode) {
                       case 1:
-                        UtilSortIndex(n_tri,z_value,ix,(UtilOrderFn*)ZOrderFn);
+                        UtilSemiSortFloatIndex(n_tri,z_value,ix,true);
+                        /* UtilSortIndex(n_tri,z_value,ix,(UtilOrderFn*)ZOrderFn);*/
                         break;
                       default:
-                        UtilSortIndex(n_tri,z_value,ix,(UtilOrderFn*)ZRevOrderFn);
+                        UtilSemiSortFloatIndex(n_tri,z_value,ix,false);
+                        /*UtilSortIndex(n_tri,z_value,ix,(UtilOrderFn*)ZRevOrderFn); */
                         break;
                       }
                       
                       c=n_tri;
                       
-                      col=ColorGet(G,I->Obj.Color);
+                      col=ColorGet(G,ms->OneColor);
                       
                       glColor4f(col[0],col[1],col[2],alpha);
                       glBegin(GL_TRIANGLES);
                       for(c=0;c<n_tri;c++) {
                         
                         tb = t_buf+6*ix[c];
-                        
+                        if(vc) tc = c_buf+3*ix[c];
+
+                        if(vc) { glColor4f(tc[0][0],tc[0][1],tc[0][2],alpha); tc++;}
                         glNormal3fv(*(tb++));
                         glVertex3fv(*(tb++));
+                        if(vc) { glColor4f(tc[0][0],tc[0][1],tc[0][2],alpha); tc++;}
                         glNormal3fv(*(tb++));
                         glVertex3fv(*(tb++));
+                        if(vc) { glColor4f(tc[0][0],tc[0][1],tc[0][2],alpha); tc++;}
                         glNormal3fv(*(tb++));
                         glVertex3fv(*(tb++));
                       }
@@ -696,24 +982,69 @@ static void ObjectSurfaceRender(ObjectSurface *I,int state,CRay *ray,Pickable **
                       FreeP(ix);
                       FreeP(z_value);
                       FreeP(t_buf);
-                    } else { 
-                      while(*n)
-                        {
+                      FreeP(c_buf);
+                    } else { /* t_mode is zero */
+
+                      if(info->alpha_cgo) { /* global transparency */
+                        
+                        vc = ms->VC;
+                        while(*n) {
+                          int parity = 1;
+                          c=*(n++);
+
+                          v+=6;
+                          if(vc) vc+=3;
+                          c-=2;
+
+                          v+=6;
+                          if(vc) vc+=3;
+                          c-=2;
+
+                          while(c>0) {
+                            
+                            if(vc) {
+                              CGOAlphaTriangle(info->alpha_cgo,
+                                               v+(3-6), v+(3-12), v+3,
+                                               v - 6, v - 12, v,
+                                               vc - 3, vc - 6, vc,
+                                               alpha, alpha, alpha, parity);
+                            } else {
+                              CGOAlphaTriangle(info->alpha_cgo,
+                                               v+(3-6), v+(3-12), v+3,
+                                               v - 6, v - 12, v,
+                                               col, col, col,
+                                               alpha, alpha, alpha, parity);
+                            }
+                            v+=6;
+                            if(vc) vc+=3;
+                            c-=2;
+                            parity=!parity;
+                          }
+                        }
+                      } else {  /* fast, but unoptimized transparency */
+                        vc = ms->VC;
+                        while(*n) {
                           c=*(n++);
                           
                           glBegin(GL_TRIANGLE_STRIP);
                           while(c>0) {
                             glNormal3fv(v);
                             v+=3;
+                            if(vc) {
+                              glColor3fv(vc);
+                              vc+=3;
+                            }
                             glVertex3fv(v);
                             v+=3;
                             c-=2;
                           }
                           glEnd();
                         }
+                      }
                     }
-                  } else {
+                  } else { /* opaque, triangles */
                     glLineWidth(SettingGet_f(G,I->Obj.Setting,NULL,cSetting_mesh_width));
+                    vc = ms->VC;
                     while(*n)
                       {
                         c=*(n++);
@@ -724,6 +1055,10 @@ static void ObjectSurfaceRender(ObjectSurface *I,int state,CRay *ray,Pickable **
                           while(c>0) {
                             glNormal3fv(v);
                             v+=3;
+                            if(vc) {
+                              glColor3fv(vc);
+                              vc+=3;
+                            }
                             glVertex3fv(v);
                             v+=3;
                             c-=2;
@@ -733,6 +1068,10 @@ static void ObjectSurfaceRender(ObjectSurface *I,int state,CRay *ray,Pickable **
                         case 1:
                           glBegin(GL_LINES);
                           while(c>0) {
+                            if(vc) {
+                              glColor3fv(vc);
+                              vc+=3;
+                            }
                             glVertex3fv(v);
                             v+=3;
                             c--;
@@ -743,6 +1082,10 @@ static void ObjectSurfaceRender(ObjectSurface *I,int state,CRay *ray,Pickable **
                         default:
                           glBegin(GL_POINTS);
                           while(c>0) {
+                            if(vc) {
+                              glColor3fv(vc);
+                              vc+=3;
+                            }
                             glVertex3fv(v);
                             v+=3;
                             c--;
@@ -790,7 +1133,7 @@ ObjectSurface *ObjectSurfaceNew(PyMOLGlobals *G)
   
   I->Obj.fFree = (void (*)(struct CObject *))ObjectSurfaceFree;
   I->Obj.fUpdate =  (void (*)(struct CObject *)) ObjectSurfaceUpdate;
-  I->Obj.fRender =(void (*)(struct CObject *, int, CRay *, Pickable **,int ))ObjectSurfaceRender;
+  I->Obj.fRender =(void (*)(struct CObject *, RenderInfo *info))ObjectSurfaceRender;
   I->Obj.fInvalidate =(void (*)(struct CObject *,int,int,int))ObjectSurfaceInvalidate;
   I->Obj.fGetNFrame = (int (*)(struct CObject *)) ObjectSurfaceGetNStates;
   return(I);
@@ -814,14 +1157,19 @@ void ObjectSurfaceStateInit(PyMOLGlobals *G,ObjectSurfaceState *ms)
 
   ms->N[0]=0;
   ms->nT=0;
+  ms->VC = NULL;
+  ms->RC = NULL;
+  ms->VCsize = 0;
   ms->Active=true;
   ms->ResurfaceFlag=true;
+  ms->RecolorFlag=false;
   ms->ExtentFlag=false;
   ms->CarveFlag=false;
   ms->AtomVertex=NULL;
   ms->UnitCellCGO=NULL;
   ms->Side = 0;
   ms->displayList = 0;
+  ms->displayListInvalid = true;
 }
 
 /*========================================================================*/
@@ -880,7 +1228,7 @@ float carve,float *vert_vla,int side)
         max_ext = ms->ExtentMax;
       }
       
-      TetsurfGetRange(oms->Field,oms->Crystal,min_ext,max_ext,ms->Range);
+      TetsurfGetRange(G,oms->Field,oms->Crystal,min_ext,max_ext,ms->Range);
     }
     ms->ExtentFlag = true;
   }
@@ -912,6 +1260,25 @@ float carve,float *vert_vla,int side)
   return(I);
 }
 
+int ObjectSurfaceGetLevel(ObjectSurface *I,int state, float *result)
+{
+  int ok=true;
+  ObjectSurfaceState *ms;
+  if(state>=I->NState) {
+    ok=false;
+  } else {
+    if(state<0) {
+      state = 0;
+    }
+    ms = I->State + state;
+    if(ms->Active && result) {
+      *result = ms->Level;
+    } else 
+      ok = false;
+  }
+  return(ok);
+}
+
 int ObjectSurfaceSetLevel(ObjectSurface *I,float level,int state)
 {
   int a;
@@ -966,5 +1333,16 @@ void ObjectSurfaceRecomputeExtent(ObjectSurface *I)
     }
   }
   I->Obj.ExtentFlag=extent_flag;
+
+  if(I->Obj.TTTFlag && I->Obj.ExtentFlag) {
+    float *ttt;
+    double tttd[16];
+    if(ObjectGetTTT(&I->Obj,&ttt,-1)) {
+      convertTTTfR44d(ttt,tttd);
+      MatrixTransformExtentsR44d3f(tttd,
+                                   I->Obj.ExtentMin,I->Obj.ExtentMax,
+                                   I->Obj.ExtentMin,I->Obj.ExtentMax);
+    }
+  }
 }
 
