@@ -20,18 +20,23 @@ Z* -------------------------------------------------------------------
 
 typedef struct {
   CObjectState State;
-  char MapName[ObjNameMax];
+  ObjectNameType MapName;
   int MapState;
   CCrystal Crystal;
   int Active;
-  int *N,nT;
+  int *N,nT,base_n_V;
   float *V;
+  float *VC;
+  int *RC;
+  int OneColor;
+  int VCsize;
   int Range[6];
   float ExtentMin[3],ExtentMax[3];
   int ExtentFlag;
   float Level,Radius;
   int RefreshFlag;
   int ResurfaceFlag;
+  int RecolorFlag;
   float *AtomVertex;
   int CarveFlag;
   float CarveBuffer;
@@ -40,6 +45,7 @@ typedef struct {
   CGO *UnitCellCGO;
   int Side;
   int displayList;
+  int displayListInvalid;
 } ObjectSurfaceState;
 
 typedef struct ObjectSurface {
@@ -58,6 +64,8 @@ void ObjectSurfaceDump(ObjectSurface *I,char *fname,int state);
 int ObjectSurfaceNewFromPyList(PyMOLGlobals *G,PyObject *list,ObjectSurface **result);
 PyObject *ObjectSurfaceAsPyList(ObjectSurface *I);
 int ObjectSurfaceSetLevel(ObjectSurface *I,float level,int state);
+int ObjectSurfaceGetLevel(ObjectSurface *I,int state,float *result);
+int ObjectSurfaceInvalidateMapName(ObjectSurface *I,char *name);
 
 #endif
 
