@@ -36,8 +36,8 @@ typedef struct RepDot {
   float Width;
 } RepDot;
 
-Rep *RepDotNew(CoordSet *cset);
-Rep *RepDotDoNew(CoordSet *cs,int mode);
+Rep *RepDotNew(CoordSet *cset,int state);
+Rep *RepDotDoNew(CoordSet *cs,int mode,int state);
 void RepDotInit(void);
 
 #endif
