@@ -36,7 +36,7 @@ Z* -------------------------------------------------------------------
 #include"P.h"
 
 struct _CEditor {
-  ObjectMolecule *Obj;
+  ObjectMolecule *DihedObject;
   WordType DragSeleName;
   int Active;
   int ActiveState;
@@ -51,13 +51,43 @@ struct _CEditor {
   float V0[3],V1[3],Axis[3],Center[3],DragBase[3];
   float *PosVLA;
   int ShowFrags;
+  int DihedralInvalid;
+  int MouseInvalid;
+  int FavorOrigin;
+  float FavoredOrigin[3];
 } ;
 
+int EditorGetScheme(PyMOLGlobals *G)
+{
+  register CEditor *I = G->Editor;
+  int scheme = EDITOR_SCHEME_OBJ;
+
+  if(EditorActive(G)) 
+    scheme = EDITOR_SCHEME_FRAG;
+  else if(I->DragObject) {
+    if(I->DragIndex>=0) {
+      scheme = EDITOR_SCHEME_OBJ;
+    } else {
+      scheme = EDITOR_SCHEME_DRAG;
+    }
+  }
+  return scheme;
+}
+
+void EditorFavorOrigin(PyMOLGlobals *G, float *v1)
+{
+  register CEditor *I = G->Editor;
+  if(v1) {
+    I->FavorOrigin = true;
+    copy3f(v1,I->FavoredOrigin);
+  } else {
+    I->FavorOrigin = false;
+  }
+}
 
 static void EditorDrawDihedral(PyMOLGlobals *G)
 {
-
-  if(EditorActive(G)&&EditorIsBondMode(G)) {
+  if(EditorActive(G)&&EditorIsBondMode(G)&&SettingGetGlobal_b(G,cSetting_editor_auto_dihedral)) {
     ObjectMolecule *obj1, *obj2;
     int at1, at2, at0, at3;
     int sele1 = SelectorIndexByName(G,cEditorSele1);
@@ -66,30 +96,176 @@ static void EditorDrawDihedral(PyMOLGlobals *G)
       obj1 = SelectorGetFastSingleAtomObjectIndex(G,sele1,&at1);
       obj2 = SelectorGetFastSingleAtomObjectIndex(G,sele2,&at2);
       if(obj1 && (obj1 == obj2) ) {
+        register CEditor *I = G->Editor;
         
+        I->DihedObject = obj1;
         at0 = ObjectMoleculeGetTopNeighbor(G,obj1, at1,at2);
         at3 = ObjectMoleculeGetTopNeighbor(G,obj1, at2,at1);
         
         if((at0>=0) &&( at3>=0)) {
           int sele0, sele3;
-          
+          float result;
+
           /* find the highest priority atom attached to index1 */
           SelectorCreateOrderedFromObjectIndices(G,cEditorDihe1,obj1,&at0,1);
           SelectorCreateOrderedFromObjectIndices(G,cEditorDihe2,obj2,&at3,1);
           sele0 = SelectorIndexByName(G,cEditorDihe1);
           sele3 = SelectorIndexByName(G,cEditorDihe2);
           
-          ExecutiveDihedral(G,cEditorDihedral,cEditorDihe1,cEditorSele1,cEditorSele2,cEditorDihe2,
-                            0,true,true,false,true);
+          ExecutiveDihedral(G,&result,cEditorDihedral,cEditorDihe1,
+                            cEditorSele1,cEditorSele2,cEditorDihe2,
+                            0,true,true,false,true,-1);
           ExecutiveColor(G,cEditorDihedral,"white",1,true);
-          ExecutiveSetSettingFromString(G, cSetting_label_font_id, "4", cEditorDihedral, 0, true, true);
-          ExecutiveSetSettingFromString(G, cSetting_label_color, "30", cEditorDihedral, 0, true, true);
+          ExecutiveSetSettingFromString(G, cSetting_float_labels,
+                                        "1", cEditorDihedral, 0, true, true);
+#ifndef _PYMOL_FREETYPE
+          ExecutiveSetSettingFromString(G, cSetting_label_font_id, 
+                                        "4", cEditorDihedral, 0, true, true);
+#else
+          ExecutiveSetSettingFromString(G, cSetting_label_font_id, 
+                                        "8", cEditorDihedral, 0, true, true);
+          ExecutiveSetSettingFromString(G, cSetting_label_size,
+                                        "20", cEditorDihedral, 0, true, true);
+#endif
+          ExecutiveSetSettingFromString(G, cSetting_label_color, 
+                                        "brightorange", cEditorDihedral,
+                                        0, true, true);
         }
       }
     }
   }
 }
 
+void EditorDihedralInvalid(PyMOLGlobals *G,ObjectMolecule *obj)
+{
+  register CEditor *I = G->Editor;
+  if(!obj) 
+    I->DihedralInvalid = true;
+  else if(obj==I->DihedObject)
+    I->DihedralInvalid = true;    
+}
+
+void EditorMouseInvalid(PyMOLGlobals *G)
+{
+  register CEditor *I = G->Editor;
+  I->MouseInvalid = true;
+}
+
+static void EditorConfigMouse(PyMOLGlobals *G)
+{
+
+  int scheme = EditorGetScheme(G);
+  char *mouse_mode = SettingGetGlobal_s(G,cSetting_button_mode_name);
+
+  if(mouse_mode && 
+     ((!strcmp(mouse_mode,"3-Button Editing")==0)||
+      (!strcmp(mouse_mode,"3-Button Motions")==0))) { /* WEAK! */
+    int button;
+
+    button = cButModeMiddleShft;
+
+    {
+      int action = ButModeGet(G,button);
+      if ((action == cButModeMovFrag)||
+          (action == cButModeMovObj)||
+          (action == cButModeMovDrag)) {
+        switch(scheme) {
+        case EDITOR_SCHEME_OBJ: action = cButModeMovObj; break;
+        case EDITOR_SCHEME_FRAG: action = cButModeMovFrag; break;
+        case EDITOR_SCHEME_DRAG: action = cButModeMovDrag; break;
+        }
+        ButModeSet(G,button, action);
+      }
+    }
+
+    button = cButModeLeftShft;
+    {
+      int action = ButModeGet(G,button);
+      if ((action == cButModeRotFrag)||
+          (action == cButModeRotObj)||
+          (action == cButModeRotDrag)) {
+        switch(scheme) {
+        case EDITOR_SCHEME_OBJ: action = cButModeRotObj; break;
+        case EDITOR_SCHEME_FRAG: action = cButModeRotFrag; break;
+        case EDITOR_SCHEME_DRAG: action = cButModeRotDrag; break;
+        }
+        ButModeSet(G,button, action);
+      }
+    }
+
+    button = cButModeRightShft;
+    {
+      int action = ButModeGet(G,button);
+      if ((action == cButModeMovFragZ)||
+          (action == cButModeMovObjZ)||
+          (action == cButModeMovDragZ)) {
+        switch(scheme) {
+        case EDITOR_SCHEME_OBJ: action = cButModeMovObjZ; break;
+        case EDITOR_SCHEME_FRAG: action = cButModeMovFragZ; break;
+        case EDITOR_SCHEME_DRAG: action = cButModeMovDragZ; break;
+        }
+        ButModeSet(G,button, action);
+      }
+    }
+
+    button = cButModeLeftCtrl;
+    {
+      int action = ButModeGet(G,button);
+      if ((action == cButModeMoveAtom)||
+          (action == cButModeTorFrag)) {
+        switch(scheme) {
+        case EDITOR_SCHEME_OBJ: action = cButModeMoveAtom; break;
+        case EDITOR_SCHEME_FRAG: action = cButModeTorFrag; break;
+        case EDITOR_SCHEME_DRAG: action = cButModeMoveAtom; break;
+        }
+        ButModeSet(G,button, action);
+      }
+    }
+
+    button = cButModeLeftDouble;
+    {
+      int action = ButModeGet(G,button);
+      if ((action == cButModeMoveAtom)||
+          (action == cButModeTorFrag)) {
+        switch(scheme) {
+        case EDITOR_SCHEME_OBJ: action = cButModeMoveAtom; break;
+        case EDITOR_SCHEME_FRAG: action = cButModeTorFrag; break;
+        case EDITOR_SCHEME_DRAG: action = cButModeMoveAtom; break;
+        }
+        ButModeSet(G,button, action);
+      }
+    }
+
+    button = cButModeLeftCtSh;
+    {
+      int action = ButModeGet(G,button);
+      if ((action == cButModeMoveAtom)||
+          (action == cButModeMoveAtomZ)) {
+        switch(scheme) {
+        case EDITOR_SCHEME_OBJ: action = cButModeMoveAtomZ; break;
+        case EDITOR_SCHEME_FRAG: action = cButModeMoveAtom; break;
+        case EDITOR_SCHEME_DRAG: action = cButModeMoveAtomZ; break;
+        }
+        ButModeSet(G,button, action);
+      }
+    }
+  }
+  
+}
+
+void EditorUpdate(PyMOLGlobals *G) 
+{
+  register CEditor *I =G->Editor;
+  if(I->DihedralInvalid) {
+    EditorDrawDihedral(G);
+    I->DihedralInvalid = false;
+  }
+  if(I->MouseInvalid) {
+    EditorConfigMouse(G);
+    I->MouseInvalid = false;
+  }
+}
+
 static int EditorGetEffectiveState(PyMOLGlobals *G,ObjectMolecule *obj,int state)
 {
   if(!obj) obj = SelectorGetFastSingleObjectMolecule(G,
@@ -325,7 +501,7 @@ int EditorLogState(PyMOLGlobals *G,int pkresi)
     int pkbond = 1;
     
     if(!EditorActive(G)) {
-      PLog("edit",cPLog_pml);
+      PLog(G,"edit",cPLog_pml);
     } else {
       int sele1,sele2,sele3,sele4;
       ObjectMolecule *obj1=NULL,*obj2=NULL,*obj3=NULL,*obj4=NULL;
@@ -372,7 +548,7 @@ int EditorLogState(PyMOLGlobals *G,int pkresi)
       sprintf(buffer,"cmd.edit(%s,%s,%s,%s,pkresi=%d,pkbond=%d)",
               buf1,buf2,buf3,buf4,pkresi ? 1: 0, pkbond ? 1: 0);
 
-      PLog(buffer,cPLog_pym);
+      PLog(G,buffer,cPLog_pym);
 
     }
   }
@@ -441,7 +617,7 @@ int EditorInvert(PyMOLGlobals *G,int quiet)
                (!ObjectMoleculeDoesAtomNeighborSele(obj0,ia0,sele2)) &&
                (!ObjectMoleculeDoesAtomNeighborSele(obj0,ia1,sele2))) {
               found = true;
-              ok = ObjectMoleculeTransformSelection(obj0,state,sele2,m,false,NULL,false);
+              ok = ObjectMoleculeTransformSelection(obj0,state,sele2,m,false,NULL,false,false);
             }
           }
           if(found) {
@@ -456,7 +632,7 @@ int EditorInvert(PyMOLGlobals *G,int quiet)
               ENDFB(G);
           }
 
-          SceneDirty(G);
+          SceneInvalidate(G);
           I->DragIndex=-1;
           I->DragSelection=-1;
           I->DragObject=NULL;
@@ -518,16 +694,17 @@ int EditorTorsion(PyMOLGlobals *G,float angle)
             
             theta=(float)(cPI*angle/180.0);
             get_rotation_about3f3fTTTf(theta, n0, v1, m);
-            ok = ObjectMoleculeTransformSelection(obj2,state,sele2,m,false,NULL,false);
-            SceneDirty(G);
+            ok = ObjectMoleculeTransformSelection(obj2,state,sele2,m,false,NULL,false,false);
+            SceneInvalidate(G);
             
             I->DragIndex=-1;
             I->DragSelection=-1;
             I->DragObject=NULL;
 
+            
             if(I->BondMode && 
                SettingGetGlobal_b(G,cSetting_editor_auto_dihedral)) 
-              EditorDrawDihedral(G);
+              EditorDihedralInvalid(G,NULL);
           }
         }
       }
@@ -537,7 +714,8 @@ int EditorTorsion(PyMOLGlobals *G,float angle)
 }
 
 /*========================================================================*/
-int EditorSelect(PyMOLGlobals *G,char *s0,char *s1,char *s2,char *s3,int pkresi,int pkbond,int quiet)
+int EditorSelect(PyMOLGlobals *G,char *s0,char *s1,char *s2,
+                 char *s3,int pkresi,int pkbond,int quiet)
 {
   int i0=-1;
   int i1=-1;
@@ -590,13 +768,13 @@ int EditorSelect(PyMOLGlobals *G,char *s0,char *s1,char *s2,char *s3,int pkresi,
 
   if(ok) {
     if(obj0)
-      ObjectMoleculeVerifyChemistry(obj0);  
+      ObjectMoleculeVerifyChemistry(obj0,-1);  
     if(obj1&&(obj1!=obj0))
-      ObjectMoleculeVerifyChemistry(obj1);  
+      ObjectMoleculeVerifyChemistry(obj1,-1);  
     if(obj2&&(obj2!=obj0)&&(obj2!=obj1))
-      ObjectMoleculeVerifyChemistry(obj2);  
+      ObjectMoleculeVerifyChemistry(obj2,-1);  
     if(obj3&&(obj3!=obj0)&&(obj3!=obj1)&&(obj3!=obj2))
-      ObjectMoleculeVerifyChemistry(obj3);  
+      ObjectMoleculeVerifyChemistry(obj3,-1);  
 
     if(i0>=0) SelectorCreate(G,cEditorSele1,s0,NULL,quiet,NULL);
     if(i1>=0) SelectorCreate(G,cEditorSele2,s1,NULL,quiet,NULL);
@@ -609,7 +787,7 @@ int EditorSelect(PyMOLGlobals *G,char *s0,char *s1,char *s2,char *s3,int pkresi,
     if(pkresi)
       EditorDefineExtraPks(G);
     
-    SceneDirty(G);
+    SceneInvalidate(G);
     result=true;
 
   } else {
@@ -666,7 +844,7 @@ void EditorCycleValence(PyMOLGlobals *G,int quiet)
         obj0 = SelectorGetFastSingleObjectMolecule(G,sele0);
         obj1 = SelectorGetFastSingleObjectMolecule(G,sele1);
         if((obj0==obj1)&&I->BondMode) {
-          ObjectMoleculeVerifyChemistry(obj0);
+          ObjectMoleculeVerifyChemistry(obj0,-1);
           ObjectMoleculeAdjustBonds(obj0,sele0,sele1,0,0);
         }
       }
@@ -694,24 +872,28 @@ void EditorAttach(PyMOLGlobals *G,char *elem,int geom,int valence,
       obj1 = SelectorGetFastSingleObjectMolecule(G,sele1);
 
       if(obj0) {
-        ObjectMoleculeVerifyChemistry(obj0); /* remember chemistry for later */
-        state = SceneGetState(G);
-        if(obj1) {
-          if(obj0==obj1) {
-            /* bond mode - behave like replace */
-            EditorReplace(G,elem,geom,valence,name,quiet);
-          }
+        if(obj0->DiscreteFlag) {
+          ErrMessage(G,"Remove","Can't attach atoms onto discrete objects.");
         } else {
-          /* atom mode */
-          i0 = ObjectMoleculeGetAtomIndex(obj0,sele0); /* slow */
-          if(i0>=0) {
-            UtilNCopy(ai->elem,elem,sizeof(AtomName));
-            ai->geom=geom;
-            ai->valence=valence;
-            if(name[0])
-              UtilNCopy(ai->name,name,sizeof(AtomName));
-            ObjectMoleculeAttach(obj0,i0,ai); /* will free ai */
-            ai = NULL;
+          ObjectMoleculeVerifyChemistry(obj0,-1); /* remember chemistry for later */
+          state = SceneGetState(G);
+          if(obj1) {
+            if(obj0==obj1) {
+              /* bond mode - behave like replace */
+              EditorReplace(G,elem,geom,valence,name,quiet);
+            }
+          } else {
+            /* atom mode */
+            i0 = ObjectMoleculeGetAtomIndex(obj0,sele0); /* slow */
+            if(i0>=0) {
+              UtilNCopy(ai->elem,elem,sizeof(AtomName));
+              ai->geom=geom;
+              ai->valence=valence;
+              if(name[0])
+                UtilNCopy(ai->name,name,sizeof(AtomName));
+              ObjectMoleculeAttach(obj0,i0,ai); /* will free ai */
+              ai = NULL;
+            }
           }
         }
       }
@@ -733,7 +915,7 @@ void EditorRemove(PyMOLGlobals *G,int hydrogen,int quiet)
   if(EditorActive(G)) {
     sele0 = SelectorIndexByName(G,cEditorSele1);
     obj0 = SelectorGetFastSingleObjectMolecule(G,sele0);
-    ObjectMoleculeVerifyChemistry(obj0); /* remember chemistry for later */
+    ObjectMoleculeVerifyChemistry(obj0,-1); /* remember chemistry for later */
     if((sele0>=0)&&obj0) {
       sele1 = SelectorIndexByName(G,cEditorSele2);
       obj1 = SelectorGetFastSingleObjectMolecule(G,sele1);
@@ -777,7 +959,7 @@ void EditorHFill(PyMOLGlobals *G,int quiet)
   if(EditorActive(G)) {
     sele0 = SelectorIndexByName(G,cEditorSele1);
     obj0 = SelectorGetFastSingleObjectMolecule(G,sele0);    
-    ObjectMoleculeVerifyChemistry(obj0); /* remember chemistry for later */
+    ObjectMoleculeVerifyChemistry(obj0,-1); /* remember chemistry for later */
     if(sele0>=0) {
       sele1 = SelectorIndexByName(G,cEditorSele2);
       if(sele0>=0) {
@@ -812,8 +994,34 @@ void EditorHFill(PyMOLGlobals *G,int quiet)
       }
     }
   }
-  
 }
+
+/*========================================================================*/
+void EditorHFix(PyMOLGlobals *G,char *sele,int quiet)
+{
+  int sele0,sele1;
+  ObjectMolecule *obj0,*obj1;
+
+  if((!sele)||(!sele[0])) { /* if selection is empty, then apply to picked atoms */
+    if(EditorActive(G)) {
+      sele0 = SelectorIndexByName(G,cEditorSele1);
+      if(sele0>=0) {
+        obj0 = SelectorGetFastSingleObjectMolecule(G,sele0);    
+        ObjectMoleculeVerifyChemistry(obj0,-1); 
+        ExecutiveFixHydrogens(G,cEditorSele1,quiet);
+      }
+      sele1 = SelectorIndexByName(G,cEditorSele2);
+      if(sele1>=0) {
+        obj1 = SelectorGetFastSingleObjectMolecule(G,sele1);    
+        ObjectMoleculeVerifyChemistry(obj1,-1); 
+        ExecutiveFixHydrogens(G,cEditorSele2,quiet);
+      }
+    }
+  } else {
+    ExecutiveFixHydrogens(G,sele,quiet);
+  }
+}
+
 /*========================================================================*/
 void EditorReplace(PyMOLGlobals *G,char *elem,int geom,int valence,char *name,int quiet)
 {
@@ -827,26 +1035,30 @@ void EditorReplace(PyMOLGlobals *G,char *elem,int geom,int valence,char *name,in
   if(EditorActive(G)) {
     sele0 = SelectorIndexByName(G,cEditorSele1);
     obj0 = SelectorGetFastSingleObjectMolecule(G,sele0);    
-    ObjectMoleculeVerifyChemistry(obj0); /* remember chemistry for later */
-
-    state = SceneGetState(G);
-
-    if(sele0>=0) {
-      i0 = ObjectMoleculeGetAtomIndex(obj0,sele0); /* slow */
-      if(i0>=0) {
-        UtilNCopy(ai.elem,elem,sizeof(AtomName));
-        if(name[0])
-          UtilNCopy(ai.name,name,sizeof(AtomName));
-        ai.geom=geom;
-        ai.valence=valence;
-        ObjectMoleculePrepareAtom(obj0,i0,&ai);
-        ObjectMoleculePreposReplAtom(obj0,i0,&ai);
-        ObjectMoleculeReplaceAtom(obj0,i0,&ai); /* invalidates */
-        ObjectMoleculeVerifyChemistry(obj0);
-        ObjectMoleculeFillOpenValences(obj0,i0);
-        ObjectMoleculeSort(obj0);
-        ObjectMoleculeUpdateIDNumbers(obj0);
-        EditorInactivate(G);
+    if(obj0->DiscreteFlag) {
+      ErrMessage(G,"Remove","Can't attach atoms onto discrete objects.");
+    } else {
+      ObjectMoleculeVerifyChemistry(obj0,-1); /* remember chemistry for later */
+      
+      state = SceneGetState(G);
+      
+      if(sele0>=0) {
+        i0 = ObjectMoleculeGetAtomIndex(obj0,sele0); /* slow */
+        if(i0>=0) {
+          UtilNCopy(ai.elem,elem,sizeof(AtomName));
+          if(name[0])
+            UtilNCopy(ai.name,name,sizeof(AtomName));
+          ai.geom=geom;
+          ai.valence=valence;
+          ObjectMoleculePrepareAtom(obj0,i0,&ai);
+          ObjectMoleculePreposReplAtom(obj0,i0,&ai);
+          ObjectMoleculeReplaceAtom(obj0,i0,&ai); /* invalidates */
+          ObjectMoleculeVerifyChemistry(obj0,-1);
+          ObjectMoleculeFillOpenValences(obj0,i0);
+          ObjectMoleculeSort(obj0);
+          ObjectMoleculeUpdateIDNumbers(obj0);
+          EditorInactivate(G);
+        }
       }
     }
   }
@@ -1192,8 +1404,8 @@ void EditorRender(PyMOLGlobals *G,int state)
       if((sele1>=0) && (sele2>=0) && I->BondMode && obj1 && obj2) {
         /* bond mode */
 
-        ObjectMoleculeGetAtomVertex(obj1,state,index1,v0);
-        ObjectMoleculeGetAtomVertex(obj2,state,index2,v1);
+        ObjectMoleculeGetAtomTxfVertex(obj1,state,index1,v0);
+        ObjectMoleculeGetAtomTxfVertex(obj2,state,index2,v1);
         draw_bond(G,v0,v1);
         
       } else {
@@ -1202,28 +1414,28 @@ void EditorRender(PyMOLGlobals *G,int state)
         vv = vp;
 
         if(obj1) {
-          if(ObjectMoleculeGetAtomVertex(obj1,state,index1,vv)) {
+          if(ObjectMoleculeGetAtomTxfVertex(obj1,state,index1,vv)) {
             draw_globe(G,vv,1);
             vv+=3;
           }
         }
 
         if(obj2) {
-          if(ObjectMoleculeGetAtomVertex(obj2,state,index2,vv)) {
+          if(ObjectMoleculeGetAtomTxfVertex(obj2,state,index2,vv)) {
             draw_globe(G,vv,2);
             vv+=3;
           }
         }
 
         if(obj3) {
-          if(ObjectMoleculeGetAtomVertex(obj3,state,index3,vv)) {
+          if(ObjectMoleculeGetAtomTxfVertex(obj3,state,index3,vv)) {
             draw_globe(G,vv,3);
             vv+=3;
           }
         }
 
         if(obj4) {
-          if(ObjectMoleculeGetAtomVertex(obj4,state,index4,vv)) {
+          if(ObjectMoleculeGetAtomTxfVertex(obj4,state,index4,vv)) {
             draw_globe(G,vv,4);
             vv+=3;
           }
@@ -1268,6 +1480,8 @@ void EditorInactivate(PyMOLGlobals *G)
     " EditorInactivate-Debug: callend.\n"
     ENDFD;
 
+  I->DihedObject = NULL;
+  I->DragObject = NULL;
   I->BondMode = false;
   I->ShowFrags = false;
   I->NFrag = 0;
@@ -1287,11 +1501,12 @@ void EditorInactivate(PyMOLGlobals *G)
   ExecutiveDelete(G,cEditorDihedral);
   ExecutiveDelete(G,cEditorDihe1);
   ExecutiveDelete(G,cEditorDihe2);
-  /*  if(SettingGet(G,cSetting_log_conformations)) PLogFlush();
+  /*  if(SettingGet(G,cSetting_log_conformations)) PLogFlush(G);
       TODO: resolve this problem:
       we can't assume that Python interpreter isn't blocked
   */
-  SceneDirty(G);
+  EditorMouseInvalid(G);
+  SceneInvalidate(G);
 }
 /*========================================================================*/
 void EditorActivate(PyMOLGlobals *G,int state,int enable_bond)
@@ -1334,14 +1549,32 @@ void EditorActivate(PyMOLGlobals *G,int state,int enable_bond)
       ExecutiveHideSelections(G);
 
     if(I->BondMode && SettingGetGlobal_b(G,cSetting_editor_auto_dihedral)) 
-      EditorDrawDihedral(G);
+      EditorDihedralInvalid(G,NULL);
   } else {
     EditorInactivate(G);
   }
-
+  EditorMouseInvalid(G);
 }
 /*========================================================================*/
-void EditorPrepareDrag(PyMOLGlobals *G,ObjectMolecule *obj,int index,int state)
+void EditorSetDrag(PyMOLGlobals *G,ObjectMolecule *obj,int sele, int quiet,int state)
+{
+  EditorInactivate(G);
+  state = EditorGetEffectiveState(G,obj,state);
+  if(ObjectMoleculeCheckFullStateSelection(obj,sele,state)) {
+    sele = -1;
+  }
+  EditorPrepareDrag(G,obj,sele,-1,state,0);
+}
+void EditorReadyDrag(PyMOLGlobals *G,int state)
+{
+  register CEditor *I = G->Editor;
+  if(I->DragObject && (I->DragIndex==-1)) {
+    EditorPrepareDrag(G,I->DragObject,I->DragSelection,-1,state,0);
+  }
+}
+/*========================================================================*/
+void EditorPrepareDrag(PyMOLGlobals *G,ObjectMolecule *obj,
+                       int sele, int index, int state, int mode)
 {
   int frg;
   int sele0=-1,sele1=-1,sele2=-1,sele3=-1;
@@ -1359,16 +1592,52 @@ void EditorPrepareDrag(PyMOLGlobals *G,ObjectMolecule *obj,int index,int state)
 
   state = EditorGetEffectiveState(G,obj,state);
 
-  if(!EditorActive(G)) { /* non-anchored */
-    /* need to modify this code to move a complete covalent structure */
+  /* TODO: if user is drags label, then the editor must be deactivated */
+  
+  if(!EditorActive(G)) { 
+    /* non-anchored dragging of objects and now selections */
 
+    float mn[3],mx[3];
+    
     I->DragObject=obj;
-    I->DragIndex=index;
-    I->DragSelection=-1;
+    I->DragIndex=index; /* set to -1 when in "mouse drag" mode */
+    I->DragSelection=sele;
+    I->DragHaveBase = false;
+    if(sele>=0) {
+      char *sele_name = SelectorGetNameFromIndex(G,sele);
+      if(sele_name) {
+        strcpy(I->DragSeleName,sele_name);
+        if(SettingGetGlobal_b(G,cSetting_editor_auto_origin)) {
+          if(I->FavorOrigin) {
+            I->DragHaveBase = true;
+            copy3f(I->FavoredOrigin, I->DragBase);
+          } else {
+            if(ExecutiveGetExtent(G,sele_name,mn,mx,true,state,true)) {
+              average3f(mn,mx,I->DragBase);
+              I->DragHaveBase = true;
+            }
+          }
+        }
+      } else {
+        I->DragSeleName[0]=0;
+      }
+    } else {
+      if(SettingGetGlobal_b(G,cSetting_editor_auto_origin)) {
+        if(I->FavorOrigin) {
+          I->DragHaveBase = true;
+          copy3f(I->FavoredOrigin, I->DragBase);
+        } else {
+          if(ExecutiveGetExtent(G,obj->Obj.Name,mn,mx,true,state,true)) {
+            average3f(mn,mx,I->DragBase);
+            I->DragHaveBase = true;
+          }
+        }
+      }
+    }
+    
   } else {
 
-
-    /* anchored */
+    /* anchored / fragment dragging  */
     for(frg=1;frg<=I->NFrag;frg++) {
       sprintf(name,"%s%1d",cEditorFragPref,frg);
       drag_sele = SelectorIndexByName(G,name);
@@ -1400,8 +1669,9 @@ void EditorPrepareDrag(PyMOLGlobals *G,ObjectMolecule *obj,int index,int state)
       if(sele1>=0) {
         i1 = ObjectMoleculeGetAtomIndex(obj,sele1);
         if(i1>=0) {
-          ObjectMoleculeGetAtomVertex(obj,state,i1,I->DragBase);
+          ObjectMoleculeGetAtomTxfVertex(obj,state,i1,I->DragBase);
           I->DragHaveBase = true;
+          /*printf("base %s\n",name);*/
         }
       }
 
@@ -1446,29 +1716,29 @@ void EditorPrepareDrag(PyMOLGlobals *G,ObjectMolecule *obj,int index,int state)
           
           zero3f(I->Center);
           if(i0>=0) {
-            ObjectMoleculeGetAtomVertex(obj,state,i0,I->V0);          
+            ObjectMoleculeGetAtomTxfVertex(obj,state,i0,I->V0);          
           } else if(i1>=0) {
-            ObjectMoleculeGetAtomVertex(obj,state,i1,I->V0);          
+            ObjectMoleculeGetAtomTxfVertex(obj,state,i1,I->V0);          
           } else if(i2>=0) {
-            ObjectMoleculeGetAtomVertex(obj,state,i2,I->V0);          
+            ObjectMoleculeGetAtomTxfVertex(obj,state,i2,I->V0);          
           } else if(i3>=0) {
-            ObjectMoleculeGetAtomVertex(obj,state,i3,I->V0);          
+            ObjectMoleculeGetAtomTxfVertex(obj,state,i3,I->V0);          
           }
           
           if(i0>=0) {
-            ObjectMoleculeGetAtomVertex(obj,state,i0,I->V1);          
+            ObjectMoleculeGetAtomTxfVertex(obj,state,i0,I->V1);          
             add3f(I->V1,I->Center,I->Center);
           } 
           if(i1>=0) {
-            ObjectMoleculeGetAtomVertex(obj,state,i1,I->V1);          
+            ObjectMoleculeGetAtomTxfVertex(obj,state,i1,I->V1);          
             add3f(I->V1,I->Center,I->Center);
           }
           if(i2>=0) {
-            ObjectMoleculeGetAtomVertex(obj,state,i2,I->V1);          
+            ObjectMoleculeGetAtomTxfVertex(obj,state,i2,I->V1);          
             add3f(I->V1,I->Center,I->Center);
           }
           if(i3>=0) {
-            ObjectMoleculeGetAtomVertex(obj,state,i3,I->V1);          
+            ObjectMoleculeGetAtomTxfVertex(obj,state,i3,I->V1);          
             add3f(I->V1,I->Center,I->Center);
           }
           
@@ -1482,23 +1752,42 @@ void EditorPrepareDrag(PyMOLGlobals *G,ObjectMolecule *obj,int index,int state)
           normalize3f(I->Axis);
           I->DragHaveAxis=true;
 
+          if(SettingGetGlobal_b(G,cSetting_editor_auto_origin)) {
+            if(I->FavorOrigin) {
+              I->DragHaveBase = true;
+              copy3f(I->FavoredOrigin, I->DragBase);
+            } else {
+              float mn[3],mx[3];
+
+              if(ExecutiveGetExtent(G,I->DragSeleName,mn,mx,true,state,true)) {
+                average3f(mn,mx,I->DragBase);
+                I->DragHaveBase = true;
+              }
+            }
+          }
+
         } else { /* atom mode */
           
           if(i0>=0) {
-            ObjectMoleculeGetAtomVertex(obj,state,i0,I->V0);          
+            ObjectMoleculeGetAtomTxfVertex(obj,state,i0,I->V0);          
           } else if(i1>=0) {
-            ObjectMoleculeGetAtomVertex(obj,state,i1,I->V0);          
+            ObjectMoleculeGetAtomTxfVertex(obj,state,i1,I->V0);          
           } else if(i2>=0) {
-            ObjectMoleculeGetAtomVertex(obj,state,i2,I->V0);          
+            ObjectMoleculeGetAtomTxfVertex(obj,state,i2,I->V0);          
           } else if(i3>=0) {
-            ObjectMoleculeGetAtomVertex(obj,state,i3,I->V0);          
+            ObjectMoleculeGetAtomTxfVertex(obj,state,i3,I->V0);          
           }
           if(I->DragHaveBase) {
+            
             copy3f(I->DragBase,I->V1);
             subtract3f(I->V1,I->V0,I->Axis);
             average3f(I->V1,I->V0,I->Center);
             normalize3f(I->Axis);
             I->DragHaveAxis=true;
+            if(mode==cButModeRotFrag) {
+              copy3f(I->V0,I->DragBase);
+            }
+
           }
         }
       }
@@ -1529,7 +1818,7 @@ void EditorPrepareDrag(PyMOLGlobals *G,ObjectMolecule *obj,int index,int state)
       if(sele0>=0) { /* just provide a base vector, no valid axis exists */
         i1 = ObjectMoleculeGetAtomIndex(obj,sele0);
         if(i1>=0) {
-          ObjectMoleculeGetAtomVertex(obj,state,i1,I->DragBase);
+          ObjectMoleculeGetAtomTxfVertex(obj,state,i1,I->DragBase);
           I->DragHaveBase=true;
           I->DragBondFlag=true;
         }
@@ -1547,10 +1836,10 @@ void EditorPrepareDrag(PyMOLGlobals *G,ObjectMolecule *obj,int index,int state)
     if(SettingGet(G,cSetting_auto_sculpt)) {
       SettingSet(G,cSetting_sculpting,1);
       if(!I->DragObject->Sculpt)
-        ObjectMoleculeSculptImprint(I->DragObject,state);
+        ObjectMoleculeSculptImprint(I->DragObject,state,-1,0);
     }
   }
-  if(log_trans) PLogFlush();
+  if(log_trans) PLogFlush(G);
 
   PRINTFD(G,FB_Editor)
     " EditorPrepDrag-Debug: leaving Index %d Sele %d Object %p\n Axis %d Base %d BondFlag %d SlowFlag %d seleFlag %d\n",
@@ -1570,7 +1859,6 @@ void EditorDrag(PyMOLGlobals *G,ObjectMolecule *obj,int index,int mode,int state
   float m[16];
   int log_trans = (int)SettingGet(G,cSetting_log_conformations);
 
-
   PRINTFD(G,FB_Editor)
     " EditorDrag-Debug: entered. obj %p state %d index %d mode %d \nIndex %d Sele %d Object %p\n Axis %d Base %d BondFlag %d SlowFlag %d\n", 
     (void*)obj,state,index,mode,
@@ -1579,15 +1867,47 @@ void EditorDrag(PyMOLGlobals *G,ObjectMolecule *obj,int index,int mode,int state
     I->DragBondFlag,I->DragSlowFlag
     ENDFD;
 
+  if((index<0)&&(!obj)) obj=I->DragObject;
+
   state = EditorGetEffectiveState(G,obj,state);
 
   if((index==I->DragIndex)&&(obj==I->DragObject)) {
     if(!EditorActive(G)) {
+      int matrix_mode = SettingGet_b(G,I->DragObject->Obj.Setting,
+                                      NULL,cSetting_matrix_mode);
+
       /* non-achored actions */
       switch(mode) {
+      case cButModeRotDrag:
+        if(I->DragHaveBase) {
+          copy3f(I->DragBase,v3);
+        } else {
+          SceneOriginGet(G,v3);
+        }
+        get_rotation_about3f3fTTTf(pt[0], mov, v3, m); 
+        if(matrix_mode&&(I->DragSelection<0)) {
+          switch(matrix_mode) {
+          case 1:
+            ObjectCombineTTT(&obj->Obj,m,false);
+            break;
+          case 2:
+            ObjectMoleculeTransformState44f(obj,state,m,log_trans,false,true);
+            break;
+          }
+        } else {
+          ObjectMoleculeTransformSelection(obj,state,I->DragSelection,
+                                           m,log_trans,I->DragSeleName,false,true);
+        }
+        SceneInvalidate(G);
+        break;
       case cButModeRotFrag:
-        SceneOriginGet(G,v3);
-
+      case cButModeRotObj:
+      case cButModeRotView:
+        if(I->DragHaveBase) {
+          copy3f(I->DragBase,v3);
+        } else {
+          SceneOriginGet(G,v3);
+        }
         subtract3f(pt,v3,n0);
         add3f(pt,mov,n1);
         subtract3f(n1,v3,n1);
@@ -1596,25 +1916,71 @@ void EditorDrag(PyMOLGlobals *G,ObjectMolecule *obj,int index,int mode,int state
         cross_product3f(n0,n1,cp);
         theta = (float)asin(length3f(cp));
         normalize23f(cp,n2);        
-        get_rotation_about3f3fTTTf(theta, n2, v3, m);
-        ObjectMoleculeTransformSelection(obj,state,I->DragSelection,m,log_trans,I->DragSeleName,false);
-        SceneDirty(G);
+        get_rotation_about3f3fTTTf(theta, n2, v3, m); 
+        /* matrix m now contains a valid TTT rotation in global
+           coordinate space that could be applied directly to the
+           coordinates to effect the desired rotation */
+        if(mode==cButModeRotView) {
+          /* modify the object's TTT */
+          ObjectCombineTTT(&obj->Obj, m,false);
+        } else {
+          if(matrix_mode) {
+            switch(matrix_mode) {
+            case 1:
+              ObjectCombineTTT(&obj->Obj,m,false);
+              break;
+            case 2:
+              ObjectMoleculeTransformState44f(obj,state,m,log_trans,false,true);
+              break;
+            }
+          } else {
+            ObjectMoleculeTransformSelection(obj,state,I->DragSelection,
+                                             m,log_trans,I->DragSeleName,false,true);
+          }
+        }
+        SceneInvalidate(G);
         break;
       case cButModeTorFrag:
         ObjectMoleculeMoveAtom(obj,state,index,mov,1,log_trans);
-        SceneDirty(G);
+        SceneInvalidate(G);
         break;
+      case cButModeMovView:
+      case cButModeMovViewZ:
+        ObjectTranslateTTT(&obj->Obj,mov);
+        break;
+      case cButModeMovObj:
+      case cButModeMovObjZ:
       case cButModeMovFrag:
-        identity44f(m);
-        copy3f(mov,m+12); /* questionable... */
-        ObjectMoleculeTransformSelection(obj,state,I->DragSelection,m,log_trans,I->DragSeleName,false);
-        SceneDirty(G);
+      case cButModeMovFragZ:
+      case cButModeMovDrag:
+      case cButModeMovDragZ:
+        if(matrix_mode && (I->DragSelection<0)) {
+          identity44f(m);
+          m[3]=mov[0];
+          m[7]=mov[1];
+          m[11]=mov[2];
+          switch(matrix_mode) {
+          case 1:
+            ObjectCombineTTT(&obj->Obj,m,false);
+            break;
+          case 2:
+            ObjectMoleculeTransformState44f(obj,state,m,log_trans,true,true);
+            break;
+          }
+        } else {
+          identity44f(m);
+          copy3f(mov,m+12); /* questionable... */
+          ObjectMoleculeTransformSelection(obj,state,I->DragSelection,m,
+                                           log_trans,I->DragSeleName,false,true);
+        }
+        SceneInvalidate(G);
         break;
       }
     } else {
       switch(mode) {
       case cButModeRotFrag:
-        if(I->DragHaveBase&&I->DragBondFlag) {
+      case cButModeRotObj:
+        if(I->DragHaveBase) {
           copy3f(I->DragBase,v3);
         } else {
           copy3f(I->V0,v3);
@@ -1638,8 +2004,9 @@ void EditorDrag(PyMOLGlobals *G,ObjectMolecule *obj,int index,int mode,int state
         normalize23f(cp,n2);        
         
         get_rotation_about3f3fTTTf(theta, n2, v3, m);
-        ObjectMoleculeTransformSelection(obj,state,I->DragSelection,m,log_trans,I->DragSeleName,false);
-        SceneDirty(G);
+        ObjectMoleculeTransformSelection(obj,state,I->DragSelection,
+                                         m,log_trans,I->DragSeleName,false,true);
+        SceneInvalidate(G);
         break;
       case cButModeTorFrag:
       case cButModePkTorBnd:
@@ -1669,27 +2036,29 @@ void EditorDrag(PyMOLGlobals *G,ObjectMolecule *obj,int index,int mode,int state
               theta=-theta;
             get_rotation_about3f3fTTTf(theta, n0, v1, m);
             ObjectMoleculeTransformSelection(obj,state,I->DragSelection,m,
-                                             log_trans,I->DragSeleName,false);
+                                             log_trans,I->DragSeleName,false,true);
           } else {
 
             cross_product3f(I->Axis,z_dir,d0);
             theta = -dot_product3f(d0,mov);
             get_rotation_about3f3fTTTf(theta, n0, v1, m);
             ObjectMoleculeTransformSelection(obj,state,I->DragSelection,m,
-                                             log_trans,I->DragSeleName,false);
+                                             log_trans,I->DragSeleName,false,true);
             
           }
           if(I->BondMode && SettingGetGlobal_b(G,cSetting_editor_auto_dihedral))
-            EditorDrawDihedral(G);
+            EditorDihedralInvalid(G,NULL);
         }
 
-        SceneDirty(G);
+        SceneInvalidate(G);
         break;
       case cButModeMovFrag:
+      case cButModeMovFragZ:
         identity44f(m);
         copy3f(mov,m+12); /* questionable */
-        ObjectMoleculeTransformSelection(obj,state,I->DragSelection,m,log_trans,I->DragSeleName,false);
-        SceneDirty(G);
+        ObjectMoleculeTransformSelection(obj,state,I->DragSelection,
+                                         m,log_trans,I->DragSeleName,false,true);
+        SceneInvalidate(G);
         break;
       }
     }
@@ -1706,7 +2075,7 @@ int EditorInit(PyMOLGlobals *G)
   register CEditor *I=NULL;
   if( (I=(G->Editor=Calloc(CEditor,1)))) {
     
-    I->Obj = NULL;
+    I->DihedObject = NULL;
     I->NFrag= 0;
     I->Active = false;
     I->DragObject=NULL;
@@ -1715,6 +2084,10 @@ int EditorInit(PyMOLGlobals *G)
     I->NextPickSele = 0;
     I->BondMode = false;
     I->PosVLA = VLAlloc(float,30);
+    I->DihedralInvalid = false;
+    I->MouseInvalid = false;
+    I->FavorOrigin = false;
+
     return 1;
   } else 
     return 0;
