@@ -35,14 +35,22 @@ Z* -------------------------------------------------------------------
 #define cEditorDihedral "_pkdihe"
 #define cEditorDihe1    "_pkdihe1"
 #define cEditorDihe2    "_pkdihe2"
+#define cEditorDrag    "_drag"
+
+#define EDITOR_SCHEME_OBJ 1
+#define EDITOR_SCHEME_FRAG 2
+#define EDITOR_SCHEME_DRAG 3
 
 int EditorInit(PyMOLGlobals *G);
 int EditorActive(PyMOLGlobals *G); 
 void EditorRender(PyMOLGlobals *G,int state);
 int EditorLogState(PyMOLGlobals *G,int pkresi);
+void EditorFavorOrigin(PyMOLGlobals *G, float *v1);
 
 void EditorFree(PyMOLGlobals *G);
-void EditorPrepareDrag(PyMOLGlobals *G,ObjectMolecule *obj,int index,int state);
+void EditorSetDrag(PyMOLGlobals *G,ObjectMolecule *obj,int sele, int quiet,int state);
+void EditorReadyDrag(PyMOLGlobals *G,int state);
+void EditorPrepareDrag(PyMOLGlobals *G,ObjectMolecule *obj,int sele, int index,int state, int mode);
 void EditorDrag(PyMOLGlobals *G,ObjectMolecule *obj,int index,int mode,int state,
                 float *pt,float *mov,float *z_dir);
 
@@ -52,6 +60,7 @@ void EditorReplace(PyMOLGlobals *G,char *elem,int geom,int valence,char *name,in
 void EditorAttach(PyMOLGlobals *G,char *elem,int geom,int valence,char *name,int quiet);
 void EditorRemove(PyMOLGlobals *G,int hydrogen,int quiet);
 void EditorHFill(PyMOLGlobals *G,int quiet);
+void EditorHFix(PyMOLGlobals *G,char *sele,int quiet);
 void EditorCycleValence(PyMOLGlobals *G,int quiet);
 void EditorInactivate(PyMOLGlobals *G);
 void EditorUpdateState(PyMOLGlobals *G);
@@ -70,5 +79,9 @@ int EditorIsBondMode(PyMOLGlobals *G);
 int EditorDeselectIfSelected(PyMOLGlobals *G,ObjectMolecule *obj,int index,int update);
 void EditorDefineExtraPks(PyMOLGlobals *G);
 int EditorGetNFrag(PyMOLGlobals *G);
+void EditorUpdate(PyMOLGlobals *G);
+void EditorMouseInvalid(PyMOLGlobals *G);
+int EditorGetScheme(PyMOLGlobals *G);
+void EditorDihedralInvalid(PyMOLGlobals *G,ObjectMolecule *obj);
 
 #endif
