@@ -51,11 +51,11 @@ static void SeekerBuildSeleFromAtomList(PyMOLGlobals *G,char *obj_name,int *atom
   ObjectMolecule *obj = ExecutiveFindObjectMoleculeByName(G,obj_name);
 
   if(start_fresh) {
-    SelectorCreateOrderedFromObjectIndices(G,sele_name, obj, atom_list,-1);
+    SelectorCreateFromObjectIndices(G,sele_name, obj, atom_list,-1);
   } else {
     OrthoLineType buf1;
 
-    SelectorCreateOrderedFromObjectIndices(G,cTempSeekerSele2, obj,atom_list,-1);    
+    SelectorCreateFromObjectIndices(G,cTempSeekerSele2, obj,atom_list,-1);    
 
     sprintf(buf1,"?%s|?%s",sele_name,cTempSeekerSele2);
     SelectorCreate(G,sele_name,buf1,NULL,true,NULL);      
@@ -67,7 +67,7 @@ static void SeekerSelectionToggleRange(PyMOLGlobals *G,CSeqRow* rowVLA,int row_n
                                   int col_first,int col_last,int inc_or_excl,
                                   int start_over)
 {
-  char selName[ObjNameMax];
+  char selName[WordLength];
   OrthoLineType buf1,buf2;
 
   if(row_num>=0) {
@@ -119,7 +119,7 @@ static void SeekerSelectionToggleRange(PyMOLGlobals *G,CSeqRow* rowVLA,int row_n
         
         if(!WizardDoSelect(G,cTempSeekerSele)) {
           
-          ExecutiveGetActiveSeleName(G,selName,true);
+          ExecutiveGetActiveSeleName(G,selName,true,logging);
           
           /* selection or deselecting? */
           
@@ -140,20 +140,20 @@ static void SeekerSelectionToggleRange(PyMOLGlobals *G,CSeqRow* rowVLA,int row_n
           SelectorCreate(G,selName,buf1,NULL,true,NULL);
           {
             sprintf(buf2,"%scmd.select(\"%s\",\"%s\")\n",prefix,selName,buf1);
-            PLog(buf2,cPLog_no_flush);
+            PLog(G,buf2,cPLog_no_flush);
           }
         }
         
         ExecutiveDelete(G,cTempSeekerSele);
         if(logging) {
           sprintf(buf2,"%scmd.delete(\"%s\")\n",prefix,cTempSeekerSele);
-          PLog(buf2,cPLog_no_flush);
-          PLogFlush();
+          PLog(G,buf2,cPLog_no_flush);
+          PLogFlush(G);
         }
         
         if(SettingGet(G,cSetting_auto_show_selections))
-          ExecutiveSetObjVisib(G,selName,1);
-        SceneDirty(G);
+          ExecutiveSetObjVisib(G,selName,1,false);
+        SceneInvalidate(G);
       }
     }
   }
@@ -163,7 +163,7 @@ static void SeekerSelectionToggle(PyMOLGlobals *G,CSeqRow* rowVLA,int row_num,
                                   int col_num,int inc_or_excl,
                                   int start_over)
 {
-  char selName[ObjNameMax];
+  char selName[WordLength];
   OrthoLineType buf1,buf2;
 
   if(row_num>=0) {
@@ -191,7 +191,7 @@ static void SeekerSelectionToggle(PyMOLGlobals *G,CSeqRow* rowVLA,int row_num,
         
         if(!WizardDoSelect(G,cTempSeekerSele)) {
           
-          ExecutiveGetActiveSeleName(G,selName,true);
+          ExecutiveGetActiveSeleName(G,selName,true,logging);
           
           /* selection or deselecting? */
 
@@ -221,20 +221,20 @@ static void SeekerSelectionToggle(PyMOLGlobals *G,CSeqRow* rowVLA,int row_num,
           SelectorCreate(G,selName,buf1,NULL,true,NULL);
           {
             sprintf(buf2,"%scmd.select(\"%s\",\"%s\")\n",prefix,selName,buf1);
-            PLog(buf2,cPLog_no_flush);
+            PLog(G,buf2,cPLog_no_flush);
           }
         }
 
         ExecutiveDelete(G,cTempSeekerSele);
         if(logging) {
           sprintf(buf2,"%scmd.delete(\"%s\")\n",prefix,cTempSeekerSele);
-          PLog(buf2,cPLog_no_flush);
-          PLogFlush();
+          PLog(G,buf2,cPLog_no_flush);
+          PLogFlush(G);
         }
         
         if(SettingGet(G,cSetting_auto_show_selections))
-          ExecutiveSetObjVisib(G,selName,1);
-        SceneDirty(G);
+          ExecutiveSetObjVisib(G,selName,1,false);
+        SceneInvalidate(G);
       }
   }
 }
@@ -285,30 +285,30 @@ static void SeekerSelectionCenter(PyMOLGlobals *G,int action)
   
   switch(action) {
   case 0: /* center cumulative*/
-    ExecutiveCenter(G,cTempCenterSele,-1,true,-1,NULL);
+    ExecutiveCenter(G,cTempCenterSele,-1,true,-1,NULL,true);
     if(logging) {
       sprintf(buf2,"%scmd.center(\"%s\")\n",prefix,cTempCenterSele);
-      PLog(buf2,cPLog_no_flush);
-      PLogFlush();
+      PLog(G,buf2,cPLog_no_flush);
+      PLogFlush(G);
     }
     break;
   case 1: /* zoom */
-    ExecutiveWindowZoom(G,cTempCenterSele,0.0,-1,false,-1);
+    ExecutiveWindowZoom(G,cTempCenterSele,0.0,-1,false,-1,true);
     if(logging) {
       sprintf(buf2,"%scmd.zoom(\"%s\")\n",prefix,cTempCenterSele);
-      PLog(buf2,cPLog_no_flush);
-      PLogFlush();
+      PLog(G,buf2,cPLog_no_flush);
+      PLogFlush(G);
     }
     break;
   case 2: /* center seeker */
     {
-      char selName[ObjNameMax];
-      if(ExecutiveGetActiveSeleName(G,selName,true)) {
-        ExecutiveCenter(G,selName,-1,true,-1,NULL);
+      char selName[WordLength];
+      if(ExecutiveGetActiveSeleName(G,selName,true,logging)) {
+        ExecutiveCenter(G,selName,-1,true,-1,NULL,true);
         if(logging) {
           sprintf(buf2,"%scmd.center(\"%s\")\n",prefix,selName);
-          PLog(buf2,cPLog_no_flush);
-          PLogFlush();
+          PLog(G,buf2,cPLog_no_flush);
+          PLogFlush(G);
         }
       }
     }
@@ -322,23 +322,23 @@ static CSeqRow* SeekerClick(PyMOLGlobals *G,CSeqRow* rowVLA,int button,int row_n
 {
   CSeqRow *row;
   CSeqCol *col;
-  /*  char selName[ObjNameMax]; */
+  /*  char selName[WordLength]; */
   register CSeeker *I = G->Seeker;    
+  int logging = SettingGetGlobal_i(G,cSetting_logging);
   int continuation = false;
   if((row_num<0)||(col_num<0)) {
     switch(button) {
     case P_GLUT_LEFT_BUTTON:
       if((UtilGetSeconds(G)-I->LastClickTime)<cDoubleTime) {
         OrthoLineType buf2;
-        char name[ObjNameMax];
-        if(ExecutiveGetActiveSeleName(G,name, false)) {
+        char name[WordLength];
+        if(ExecutiveGetActiveSeleName(G,name, false,false)) {
           SelectorCreate(G,name,"none",NULL,true,NULL);
           if(SettingGet(G,cSetting_logging)) {
             sprintf(buf2,"cmd.select('%s','none')\n",name);
-            PLog(buf2,cPLog_no_flush);
+            PLog(G,buf2,cPLog_no_flush);
           }
           SeqDirty(G);
-          
         }
       }
       I->LastClickTime = UtilGetSeconds(G);
@@ -364,9 +364,9 @@ static CSeqRow* SeekerClick(PyMOLGlobals *G,CSeqRow* rowVLA,int button,int row_n
     case P_GLUT_RIGHT_BUTTON:
       {
         ObjectMolecule *obj;
-        char name[ObjNameMax];
+        char name[WordLength];
 
-        if(ExecutiveGetActiveSeleName(G,name, false) && col->inverse) {
+        if(ExecutiveGetActiveSeleName(G,name, false,logging) && col->inverse) {
           MenuActivate2Arg(G,x,y+16,x,y,false,"pick_sele",name,name);
         } else if( (obj = ExecutiveFindObjectMoleculeByName(G,row->name) )) {
           OrthoLineType buffer;
@@ -390,7 +390,7 @@ static CSeqRow* SeekerClick(PyMOLGlobals *G,CSeqRow* rowVLA,int button,int row_n
                 SeekerBuildSeleFromAtomList(G,row->name,atom_list,cTempSeekerSele,true);
                 if(logging) SelectorLogSele(G,cTempSeekerSele);
                 
-                MenuActivate2Arg(G,x,y+16,x,y,false,"seq_option",buffer,cTempSeekerSele); 
+                MenuActivate2Arg(G,x,y+16,x,y,false,"seq_option",cTempSeekerSele,buffer); 
                 
               }
             }
@@ -482,6 +482,9 @@ static void SeekerRefresh(PyMOLGlobals *G,CSeqRow *rowVLA)
     int b;
     ObjectMolecule *obj;
 
+    if(sele<0)
+      sele = SelectorIndexByName(G,"_seeker_hilight");
+
     for(b=0;b<nRow;b++) {
       row = rowVLA + b;
       
@@ -706,7 +709,7 @@ static CSeqRow* SeekerRelease(PyMOLGlobals *G,CSeqRow* rowVLA,int button,
   return NULL;
 }
 
-char SeekerGetAbbr(PyMOLGlobals *G,char *abbr)
+char SeekerGetAbbr(PyMOLGlobals *G,char *abbr,char water,char unknown)
 {
   
   switch(abbr[0]) {
@@ -774,14 +777,14 @@ char SeekerGetAbbr(PyMOLGlobals *G,char *abbr)
     case 'O': 
       switch(abbr[2]) {
       case 'H':
-        return 'O';
+        return water;
         break;
       }
       break;
     case '2': 
       switch(abbr[2]) {
       case 'O':
-        return 'O';
+        return water;
         break;
       }
       break;
@@ -854,7 +857,7 @@ char SeekerGetAbbr(PyMOLGlobals *G,char *abbr)
     case 'O':  /* SOL -- gromacs solvent residue */
       switch(abbr[2]) {
       case 'L':
-        return 'O';
+        return water;
         break;
       }
       break;
@@ -872,7 +875,7 @@ char SeekerGetAbbr(PyMOLGlobals *G,char *abbr)
     case 'I': 
       switch(abbr[2]) {
       case 'P':
-        return 'O';
+        return water;
         break;
       }
       break;
@@ -908,7 +911,7 @@ char SeekerGetAbbr(PyMOLGlobals *G,char *abbr)
     case 'A': 
       switch(abbr[2]) {
       case 'T':
-        return 'O';
+        return water;
         break;
       }
       break;
@@ -917,20 +920,20 @@ char SeekerGetAbbr(PyMOLGlobals *G,char *abbr)
 
   }
 
-  return 0;
+  return unknown;
 }
 
-static int SeekerFindColor(PyMOLGlobals *G,AtomInfoType *ai,int n_more)
+static int SeekerFindColor(PyMOLGlobals *G,AtomInfoType *ai,int n_more_plus_one)
 {
-  int result = ai->color; /* default -- use first atom color */
-  AtomInfoType *ai0 =ai;
+  register int result = ai->color; /* default -- use first atom color */
+  register AtomInfoType *ai0 =ai;
   while(1) {
     if(ai0->flags & cAtomFlag_guide) /* best use guide color */
       return ai0->color;
     if(ai0->protons == cAN_C) /* or use carbon color */
       result = ai0->color;
-    n_more--;
-    if(n_more>0) {
+    n_more_plus_one--;
+    if(n_more_plus_one>0) {
       ai0++;
       if(!AtomInfoSameResidueP(G,ai,ai0))
         break;
@@ -940,6 +943,207 @@ static int SeekerFindColor(PyMOLGlobals *G,AtomInfoType *ai,int n_more)
   return result;
 }
 
+static int SeekerFindTag(PyMOLGlobals *G,AtomInfoType *ai,int sele, int codes,int n_more_plus_one)
+{
+  register int result = 0;/* default -- no tag */
+  register AtomInfoType *ai0 =ai;
+  while(1) {
+    int tag = SelectorIsMember(G,ai0->selEntry, sele);
+    if(tag && (codes<2) && (ai0->flags & cAtomFlag_guide)) /* use guide atom if present */
+      return tag;
+    if(result<tag) {
+      if(!result)
+        result = tag;
+      else if((codes<2) && (ai0->flags & cAtomFlag_guide)) /* residue based and on guide atom */
+        result = tag;
+    }
+    n_more_plus_one--;
+    if(n_more_plus_one>0) {
+      int do_break = false;
+      ai0++;
+      switch(codes) {
+      case 0:
+      case 1:
+        if(!AtomInfoSameResidueP(G,ai,ai0))
+          do_break = true;
+        break;
+      case 2: /* atoms */
+        do_break = true;
+        break;
+      case 3: /* chains */
+        if(!AtomInfoSameChainP(G,ai,ai0))
+          do_break = true;
+        break;
+      }
+      if(do_break)
+        break;
+    } else 
+      break;
+  }
+  return result;
+}
+
+PyObject *SeekerGetRawAlignment(PyMOLGlobals *G, int align_sele, int active_only) 
+{
+  PyObject *result = NULL;
+#ifdef _PYMOL_NOPY
+  return NULL;
+#else
+  int nRow = 0;
+  int nCol = 0;
+  CSeqRow *row_vla = NULL,*row;
+  void *hidden = NULL;
+  ObjectMolecule *obj;
+
+  if(align_sele<0) {
+    align_sele = ExecutiveGetActiveAlignmentSele(G);  
+  }
+  if(align_sele>=0) {
+
+    row_vla = VLACalloc(CSeqRow,10);
+
+    /* first, find out which objects are included in the alignment */
+
+    while(ExecutiveIterateObjectMolecule(G,&obj,&hidden)) {
+      if((obj->Obj.Enabled || !active_only) && (obj->Obj.Name[0]!='_')) {
+        int a;
+        AtomInfoType *ai = obj->AtomInfo;
+        for(a=0;a<obj->NAtom;a++) {
+          if(SelectorIsMember(G,ai->selEntry,align_sele)) {      
+            VLACheck(row_vla,CSeqRow,nRow);
+            row = row_vla + nRow;
+            row->obj = obj;
+            row->nCol = obj->NAtom;
+            nRow++;
+            break;
+          }
+          ai++;
+        }
+      }
+    }
+
+    /* next, figure out how many aligned columns exist */
+
+    {
+      int done = false;
+      while(!done) {
+        int a;
+        int min_tag = -1;
+        done = true;
+        for(a=0;a<nRow;a++) {
+          row = row_vla + a;
+          while(row->cCol<row->nCol) { /* advance to next tag in each row & find lowest */
+            AtomInfoType *ai = row->obj->AtomInfo + row->cCol;
+            int tag = SelectorIsMember(G,ai->selEntry, align_sele);     
+            if(!tag) {
+              row->cCol++;
+            } else { /* we're at a tagged atom... */
+              if(min_tag>tag)
+                min_tag = tag; 
+              else if(min_tag<0)
+                min_tag = tag;
+              done = false;
+              break;
+            }
+          }
+        }
+        if(min_tag>=0) {
+          nCol++;
+          for(a=0;a<nRow;a++) {
+            row = row_vla + a;
+            if(row->cCol<row->nCol) {
+              AtomInfoType *ai = row->obj->AtomInfo + row->cCol;
+              int tag = SelectorIsMember(G,ai->selEntry, align_sele);     
+              if(tag == min_tag) { /* advance past this tag */
+                row->cCol++;
+              }
+            }
+          }
+        }
+      }
+    }
+
+    /* now populate the table */
+
+    result = PyList_New(nCol);
+  
+    if(nCol) {
+      int done = false;
+      nCol = 0;
+      { /* reset start points for our second pass */
+        int a;
+        for(a=0;a<nRow;a++) {
+          row = row_vla + a;
+          row->cCol = 0;
+        }
+      }
+      while(!done) {
+        int a;
+        int min_tag = -1;
+        done = true;
+        for(a=0;a<nRow;a++) {
+          row = row_vla + a;
+          while(row->cCol<row->nCol) { /* advance to next tag in each row & find lowest */
+            AtomInfoType *ai = row->obj->AtomInfo + row->cCol;
+            int tag = SelectorIsMember(G,ai->selEntry, align_sele);     
+            if(!tag) {
+              row->cCol++;
+            } else { /* we're at a tagged atom... */
+              if(min_tag>tag)
+                min_tag = tag; 
+              else if(min_tag<0)
+                min_tag = tag;
+              done = false;
+              break;
+            }
+          }
+        }
+        if(min_tag>=0) {
+          int n_member = 0;
+
+          for(a=0;a<nRow;a++) {
+            row = row_vla + a;
+            if(row->cCol<row->nCol) {
+              AtomInfoType *ai = row->obj->AtomInfo + row->cCol;
+              int tag = SelectorIsMember(G,ai->selEntry, align_sele);     
+              if(tag == min_tag) { /* participates */
+                n_member++;
+              }
+            }
+          }
+          {
+            PyObject *column_list = PyList_New(n_member);
+            n_member = 0;
+
+            for(a=0;a<nRow;a++) {
+              row = row_vla + a;
+              if(row->cCol<row->nCol) {
+                AtomInfoType *ai = row->obj->AtomInfo + row->cCol;
+                int tag = SelectorIsMember(G,ai->selEntry, align_sele);     
+                if(tag == min_tag) { /* participates */
+
+                  PyObject *tup = PyTuple_New(2);
+                  PyTuple_SetItem(tup,0,PyString_FromString(row->obj->Obj.Name));
+                  PyTuple_SetItem(tup,1,PyInt_FromLong(row->cCol));
+                  PyList_SetItem(column_list,n_member,tup);
+
+                  row->cCol++; /* advance past this tag */
+                  n_member++;
+                }
+              }
+            }
+            PyList_SetItem(result,nCol,column_list);
+          }
+          nCol++;
+        }
+      }
+    }
+  }
+  VLAFreeP(row_vla);
+  return result;
+#endif
+}
+
 void SeekerUpdate(PyMOLGlobals *G)
 {
   /*  CObject *o = NULL;
@@ -953,11 +1157,16 @@ void SeekerUpdate(PyMOLGlobals *G)
   int codes = 0;
   int max_row = 50;
   int default_color = 0;
+  int align_sele = -1; /* alignment selection */
   CSeqRow *row_vla,*row,*lab=NULL;
   row_vla = VLACalloc(CSeqRow,10);
   /* FIRST PASS: get all the residues represented properly */
   label_mode = SettingGetGlobal_i(G,cSetting_seq_view_label_mode);
 
+#if 1
+  align_sele = ExecutiveGetActiveAlignmentSele(G);
+#endif
+
   while(ExecutiveIterateObjectMolecule(G,&obj,&hidden)) {
     if(obj->Obj.Enabled&&(SettingGet_b(G,obj->Obj.Setting,NULL,cSetting_seq_view))&&
        (obj->Obj.Name[0]!='_')) {
@@ -965,7 +1174,8 @@ void SeekerUpdate(PyMOLGlobals *G)
       AtomInfoType *last = NULL,*last_segi=NULL,*last_chain = NULL;
       CoordSet *last_disc = NULL;
       int last_state;
-      int last_abbr = false;
+      int last_abbr = true;
+      int last_spacer = false;
       int nCol = 0;
       int nListEntries = 1; /* first list starts at 1 always... */
       int est_col = obj->NAtom/5+1;
@@ -991,15 +1201,14 @@ void SeekerUpdate(PyMOLGlobals *G)
       */
 
       VLACheck(row_vla,CSeqRow,nRow);
-      if((label_mode==2)||((label_mode==1)&&(!nRow)))
-        {
+      if((label_mode==2)||((label_mode==1)&&(!nRow))) {
           lab = row_vla + nRow++;
           lab->txt = VLAlloc(char,est_char);
           lab->col = VLACalloc(CSeqCol,est_col);
           lab->label_flag = true;
-        }
-      else
+      } else {
         lab = NULL;
+      }
 
       VLACheck(row_vla,CSeqRow,nRow);
 
@@ -1007,6 +1216,7 @@ void SeekerUpdate(PyMOLGlobals *G)
       if(lab) lab = row-1; /* critical! */
       row->txt = VLAlloc(char,est_char);
       row->col = VLACalloc(CSeqCol,est_col);
+      row->fill = VLACalloc(CSeqCol,est_col/8);
       row->atom_lists = VLACalloc(int,obj->NAtom+est_col+1);
       row->atom_lists[0] = -1; /* terminate the blank listQ (IMPORTANT!) */
       row->char2col = VLACalloc(int,est_char);
@@ -1039,7 +1249,7 @@ void SeekerUpdate(PyMOLGlobals *G)
           r1->stop = row->len;
           r1->spacer = true;
           nCol++;
-        }
+        } 
       } 
       if(label_mode<2) { /* no label rows, so put object name into left-hand column */
 
@@ -1055,8 +1265,16 @@ void SeekerUpdate(PyMOLGlobals *G)
         row->column_label_flag = true;
         row->title_width = row->len;
         nCol++;
-
+      } else if(label_mode==3) { /* otherwise just insert a blank zero-length column */
+        VLACheck(row->col,CSeqCol,nCol);
+        r1 = row->col + nCol;
+        r1->start = row->len;
+        UtilConcatVLA(&row->txt,&row->len,"");
+        r1->stop = row->len;
+        r1->spacer = true;
+        nCol++;
       }
+
       if(lab) {
         
         int st_len;
@@ -1088,8 +1306,17 @@ void SeekerUpdate(PyMOLGlobals *G)
         r1->stop = row->len;
         r1->spacer = true;
         nCol++;
+      } else { /* if no labels, just insert a space in row below */
+        VLACheck(row->col,CSeqCol,nCol);
+        r1 = row->col + nCol;
+        r1->start = row->len;
+        UtilConcatVLA(&row->txt,&row->len," ");
+        r1->stop = row->len;
+        r1->spacer = true;
+        nCol++;
       }
 
+
       last_state=-1;
       for(a=0;a<obj->NAtom;a++) {
         first_atom_in_label = false;
@@ -1125,7 +1352,7 @@ void SeekerUpdate(PyMOLGlobals *G)
           nCol++;
           
           last_abbr = false;
-          
+          last_spacer = true;
           last_segi = ai;
           last_chain = ai;
 
@@ -1159,6 +1386,7 @@ void SeekerUpdate(PyMOLGlobals *G)
           nCol++;
           
           last_abbr = false;
+          last_spacer = true;
           last_chain = ai;
         }
 
@@ -1179,8 +1407,10 @@ void SeekerUpdate(PyMOLGlobals *G)
 
             first_atom_in_label = true;
 
-            abbr[0] = SeekerGetAbbr(G,ai->resn);
-            
+            abbr[0] = SeekerGetAbbr(G,ai->resn,'O',0);
+
+            r1->hint_no_space = last_abbr || last_spacer;
+
             if(!abbr[0]) {
               if(last_abbr) {
                 UtilConcatVLA(&row->txt,&row->len," ");                
@@ -1197,12 +1427,18 @@ void SeekerUpdate(PyMOLGlobals *G)
               UtilConcatVLA(&row->txt,&row->len," ");
             } else {
               UtilConcatVLA(&row->txt,&row->len,abbr);
+              r1->is_abbr = true;
               r1->stop = row->len;
             }
             if(default_color<0)
               r1->color = SeekerFindColor(G,ai,obj->NAtom-a);
             else
               r1->color = default_color;
+            if(align_sele>=0) {
+              r1->tag = SeekerFindTag(G,ai,align_sele,codes,obj->NAtom-a);
+            } else {
+              r1->tag = 0;
+            }
             nCol++;
             last_abbr=abbr[0];
           }
@@ -1228,6 +1464,11 @@ void SeekerUpdate(PyMOLGlobals *G)
               r1->color = SeekerFindColor(G,ai,obj->NAtom-a);
             else
               r1->color = default_color;
+            if(align_sele>=0) {
+              r1->tag = SeekerFindTag(G,ai,align_sele,codes,obj->NAtom-a);
+            } else {
+              r1->tag = 0;
+            }
             UtilConcatVLA(&row->txt,&row->len," ");
             nCol++;
           }
@@ -1246,10 +1487,15 @@ void SeekerUpdate(PyMOLGlobals *G)
             r1->color = ai->color;
           else
             r1->color = default_color;
+          if(align_sele>=0) {
+            r1->tag = SeekerFindTag(G,ai,align_sele,codes,obj->NAtom-a);
+          } else {
+            r1->tag = 0;
+          }
           UtilConcatVLA(&row->txt,&row->len," ");
           nCol++;
           break;
-        case 3:
+        case 3: /* chains */
           if(!AtomInfoSameChainP(G,last,ai)) {
             last = ai;
 
@@ -1267,6 +1513,11 @@ void SeekerUpdate(PyMOLGlobals *G)
               r1->color = SeekerFindColor(G,ai,obj->NAtom-a);
             else
               r1->color = default_color;
+            if(align_sele>=0) {
+              r1->tag = SeekerFindTag(G,ai,align_sele,codes,obj->NAtom-a);
+            } else {
+              r1->tag = 0;
+            }
             UtilConcatVLA(&row->txt,&row->len," ");
             nCol++;
           }
@@ -1331,6 +1582,8 @@ void SeekerUpdate(PyMOLGlobals *G)
             }
           }
           break;
+        case 5: /* movie frames */
+          break;
         }
 
         if(first_atom_in_label) {
@@ -1378,14 +1631,13 @@ void SeekerUpdate(PyMOLGlobals *G)
   }
 
   /* SECOND PASS: align columns to reflect current alignment and fixed labels */
-  if(nRow)
-    {
+  if(nRow) {
     int a,b;
     int nCol;
     int maxCol = 0;
     int done_flag = false;
     /* find out the maximum number of columns */
-
+    
     for(a=0;a<nRow;a++) {
       row = row_vla + a;
       nCol = row->nCol;
@@ -1395,10 +1647,9 @@ void SeekerUpdate(PyMOLGlobals *G)
         maxCol = nCol;
     }
 
-    /* in the simplest mode, just start each sequence in the same column */
+    if(align_sele<0) {
+      /* in the simplest mode, just start each sequence in the same column */
 
-    switch(0) {
-    case 0:
       b = 0;
       while(!done_flag) {
         int max_offset = 0;
@@ -1406,7 +1657,7 @@ void SeekerUpdate(PyMOLGlobals *G)
         for(a=0;a<nRow;a++) {
           row = row_vla + a;
           if(!row->label_flag) {
-            if(b< row->nCol) {
+            if(b < row->nCol) {
               CSeqCol *r1 = row->col + b;
               done_flag = false;
               
@@ -1421,7 +1672,7 @@ void SeekerUpdate(PyMOLGlobals *G)
           if(!row->label_flag) {
             if(b<row->nCol) {
               CSeqCol *r1 = row->col + b;
-              if(b<3) {
+              if(b<3) { 
                 if(r1->offset<max_offset) {
                   row->accum += max_offset - r1->offset;
                 }
@@ -1432,9 +1683,250 @@ void SeekerUpdate(PyMOLGlobals *G)
         }
         b++;
       }
-      break;
+    } else {
+      /* in alignment mode, line up the tags */
+      int stagger = false;
+      /* intialize current columns and get the starting character */
+      int current = 0;
+      int first = true;
+      switch(SettingGetGlobal_i(G,cSetting_seq_view_unaligned_mode)) {
+      case 0:
+      case 1:
+      case 2:
+        stagger = false;
+        break;
+      default:
+        stagger = true;
+        break;
+      }
+      for(a=0;a<nRow;a++) {       
+        row = row_vla + a;
+        row->cCol = 0;
+        if((!row->label_flag) && ((row->cCol < row->nCol))) {
+          if(current < row->accum)
+            current = row->accum;
+        }
+      }
+      done_flag = false;
+      while(!done_flag) {
+        int hint_tagged_no_space = true;
+        done_flag=true;
+        {
+#if 0
+          if(all_spacers) {
+            /* this column is only spacers, so line them up like normal */
+            int max_width = 0;
+            int width;
+            int space_added = false;
+            int rep;
+            for(rep=0;rep<2;rep++)
+              for(a=0;a<nRow;a++) {       
+                row = row_vla + a;
+                if((!row->label_flag) && (row->cCol < row->nCol)) {
+                  CSeqCol *r1 = row->col + row->cCol;
+                  if( (!first) && (!space_added) &&
+                      (codes || (((!r1->is_abbr) && (!r1->spacer))) ||
+                       (r1->is_abbr&&(!r1->hint_no_space)))) {
+                    current++;
+                    space_added = true;
+                  }
+                  done_flag = false;
+                  first = false;
+                  r1->offset = current;
+                  width =  (r1->stop-r1->start);
+                  if(max_width<width)
+                    max_width = width;
+                  row->cCol++;
+                }
+              }
+            current+=max_width;
+          }
+#endif
+          {
+            /* insert untagged entries into their own columns */
+            int untagged_flag = true;
+            int saw_untagged_no_abbr = false;
+            int hint_untagged_space = false;
+            while(untagged_flag) {
+              int space_added = false;
+              int max_width = 0;
+              untagged_flag = false;
+              
+              /* first get the spaces in...*/
+
+              for(a=0;a<nRow;a++) {       
+                row = row_vla + a;
+                if((!row->label_flag) && (row->cCol < row->nCol)) {
+                  CSeqCol *r1 = row->col + row->cCol;
+                  if(!r1->tag) { /* not aligned */
+                    int text_len = (r1->stop-r1->start);
+                    if((!first) && (!space_added) && (row->cCol>2) &&
+                       (codes || (((!r1->is_abbr) && (!r1->spacer) )) ||
+                        hint_untagged_space ||
+                        (r1->is_abbr&&(!r1->hint_no_space)))) {
+                      /* insert space */
+                      current++;
+                      space_added = true;
+                    }
+                    if(max_width<text_len)
+                      max_width = text_len;
+                  }
+                }
+              }
+              
+              /* then do the rest */
+
+              for(a=0;a<nRow;a++) {       
+                row = row_vla + a;
+                if((!row->label_flag) && (row->cCol < row->nCol)) {
+                  CSeqCol *r1 = row->col + row->cCol;
+                  if(!r1->tag) { /* not aligned */
+                    int text_len = (r1->stop-r1->start);
+                    untagged_flag = true;
+                    done_flag = false;
+                    saw_untagged_no_abbr |= (!r1->is_abbr)&&(!r1->spacer);
+
+                    first = false;
+                    r1->offset = current;
+                    r1->unaligned = true;
+                  
+                    if(!r1->spacer) {
+                      int aa;
+
+                      for(aa=0;aa<nRow;aa++) { /* infill populate other rows with dashes */
+                        if(aa!=a) {       
+                          CSeqRow *row2 = row_vla + aa;
+                          if(!row2->label_flag) { 
+                            if(row2->cCol < row2->nCol) {
+                              CSeqCol *r2 = row2->col + row2->cCol;
+                              if(stagger||r2->tag||r2->spacer) {
+                                VLACheck(row2->fill,CSeqCol,row2->nFill);
+                                r2 = row2->fill + row2->nFill;
+                                r2->stop = text_len;
+                                r2->offset = current;
+                                row2->nFill++;
+                              }
+                            } else {
+                              CSeqCol *r2 = row2->col + row2->cCol;
+                              VLACheck(row2->fill,CSeqCol,row2->nFill);
+                              r2 = row2->fill + row2->nFill;
+                              r2->stop = text_len;
+                              r2->offset = current;
+                              row2->nFill++;
+                            }
+                          }
+                        }
+                      }
+                    }
+                    if(stagger) 
+                      current += text_len;
+                    else if(max_width<text_len)
+                      max_width = text_len;
+                  }
+                }
+              }
+              if(!stagger) 
+                current += max_width;
+              if(saw_untagged_no_abbr) {
+                hint_untagged_space = true;
+                hint_tagged_no_space = false;
+              } else {
+                hint_untagged_space = false;
+                hint_tagged_no_space = true;
+              }
+              saw_untagged_no_abbr = false;
+              for(a=0;a<nRow;a++) {       
+                row = row_vla + a;
+                if((!row->label_flag) && (row->cCol < row->nCol)) {
+                  CSeqCol *r1 = row->col + row->cCol;
+                  if(!r1->tag) {
+                    row->cCol++;
+                  }
+                }
+              }
+            }
+          }
+        }
+        
+        {
+          /* next insert match-tagged entries into the same column */
+          int min_tag = 0;
+          for(a=0;a<nRow;a++) {       
+            row = row_vla + a;
+            if((!row->label_flag) && (row->cCol < row->nCol)) {
+              CSeqCol *r1 = row->col + row->cCol;
+              if(r1->tag && ((min_tag>r1->tag)||(!min_tag))) {
+                min_tag = r1->tag;
+              }
+            }
+          }
+          if(min_tag) {
+            int width, max_width = 0;
+            int space_added = false;
+            int rep;
+            for(rep=0;rep<2;rep++) 
+              for(a=0;a<nRow;a++) {       
+                row = row_vla + a;
+                if((!row->label_flag) && (row->cCol < row->nCol)) {
+                  CSeqCol *r1 = row->col + row->cCol;
+                  if(r1->tag == min_tag) {
+                    if((!first) && (!space_added) &&
+                       (codes || (((!r1->is_abbr) && (!r1->spacer))) ||
+                        (r1->is_abbr&&(!(r1->hint_no_space||hint_tagged_no_space))))) {
+                      /* insert space */
+                      current++;
+                      space_added = true;
+                    }
+                    done_flag = false;
+                    first = false;
+                    r1->offset = current;
+                    width = (r1->stop-r1->start);
+                    if(max_width<width)
+                      max_width = width;
+                    /*   row->cCol++;  */
+                  }
+                }
+              }
+            {
+              int aa;
+              for(aa=0;aa<nRow;aa++) { /* infill populate other rows with dashes */
+                CSeqRow *row2 = row_vla + aa;
+                if(!row2->label_flag) {
+                  if(row2->cCol < row2->nCol) {
+                    CSeqCol *r1 = row2->col + row2->cCol;
+                    if( r1->tag != min_tag) {
+                      CSeqCol *r2;
+                      VLACheck(row2->fill,CSeqCol,row2->nFill);
+                      r2 = row2->fill + row2->nFill;
+                      r2->stop = max_width;
+                      r2->offset = current;
+                      row2->nFill++;
+                    }
+                  } else {
+                    CSeqCol *r2;
+                    VLACheck(row2->fill,CSeqCol,row2->nFill);
+                    r2 = row2->fill + row2->nFill;
+                    r2->stop = max_width;
+                    r2->offset = current;
+                    row2->nFill++;
+                  }
+                }
+              }
+            }
+            for(a=0;a<nRow;a++) {       
+              row = row_vla + a;
+              if((!row->label_flag) && (row->cCol < row->nCol)) {
+                CSeqCol *r1 = row->col + row->cCol;
+                if(r1->tag == min_tag) {
+                  row->cCol++;
+                }
+              }
+            }
+            current+=max_width;
+          }
+        }
+      }
     }
-    
 
     for(a=0;a<nRow;a++) {
       row = row_vla + a;
@@ -1445,7 +1937,8 @@ void SeekerUpdate(PyMOLGlobals *G)
         for(b=0;b<nCol;b++) {
           CSeqCol *r1 = row->col + b,*l1=NULL;
           if(lab) { 
-            l1 = lab->col + b; /* if a fixed label is present, get the final offset from the residue line */
+            l1 = lab->col + b; /* if a fixed label is present, 
+                                  get the final offset from the residue line */
             if(l1->stop) 
               l1->offset = r1->offset;
           }
@@ -1454,7 +1947,7 @@ void SeekerUpdate(PyMOLGlobals *G)
       }
     }
     
-    }
+  }
 
   /* THIRD PASS: fill in labels, based on actual residue spacing */
 
@@ -1513,6 +2006,11 @@ void SeekerUpdate(PyMOLGlobals *G)
               n_skipped = 0;
               last_ai = ai;
               l1->start = lab->len;
+              if(codes==2) {
+                UtilConcatVLA(&lab->txt,&lab->len,ai->resn);
+                UtilConcatVLA(&lab->txt,&lab->len,"`");
+              }
+                
               UtilConcatVLA(&lab->txt,&lab->len,ai->resi);
               l1->stop = lab->len;
               st_len = l1->stop - l1->start + 1;
