@@ -19,8 +19,12 @@ Z* -------------------------------------------------------------------
 #include"Ortho.h"
 #include"ObjectMolecule.h"
 
-void SeekerInit(void);
-void SeekerFree(void);
-void SeekerUpdate(void);
+int SeekerInit(PyMOLGlobals *G);
+void SeekerFree(PyMOLGlobals *G);
+void SeekerUpdate(PyMOLGlobals *G);
+char SeekerGetAbbr(PyMOLGlobals *G,char *abbr,char water,char unknown);
+
+PyObject *SeekerGetRawAlignment(PyMOLGlobals *G, int align_sele, int active_only);
+
 
 #endif
