@@ -75,12 +75,13 @@ static Block *PopUpRecursiveFind(Block *block,int x, int y)
 {
   PyMOLGlobals *G=block->G;
   CPopUp *I = (CPopUp*)block->reference;
+  if(I->Child) { /* favor the child */
+    if(PopUpRecursiveFind(I->Child,x,y)==I->Child)
+      return block;
+  } 
   if(BlockRecursiveFind(block,x,y) == block) {
     OrthoGrab(G,block);
     return block;
-  } else if(I->Child) {
-    if(PopUpRecursiveFind(I->Child,x,y)==I->Child)
-      return block;
   }
   return NULL;
 }
@@ -97,7 +98,7 @@ Block *PopUpNew(PyMOLGlobals *G,int x,int y,int last_x,int last_y,
   int dim[2];
   PyObject *elem;
   char *str,*c;
-
+  int blocked = PAutoBlock(G);
   OOAlloc(G,CPopUp);
 
   I->Block = OrthoNewBlock(G,NULL);
@@ -152,7 +153,7 @@ Block *PopUpNew(PyMOLGlobals *G,int x,int y,int last_x,int last_y,
   
   dim[0]=I->NLine+1;
   dim[1]=mx+1;
-  I->Text = (char**)UtilArrayMalloc((unsigned int*)dim,2,1);
+  I->Text = (char**)UtilArrayCalloc((unsigned int*)dim,2,1);
 
   mx=1;
   for(a=0;a<I->NLine;a++) {
@@ -164,7 +165,7 @@ Block *PopUpNew(PyMOLGlobals *G,int x,int y,int last_x,int last_y,
   }
   dim[0]=I->NLine+1;
   dim[1]=mx+1;
-  I->Command = (char**)UtilArrayMalloc((unsigned int*)dim,2,1);
+  I->Command = (char**)UtilArrayCalloc((unsigned int*)dim,2,1);
 
   I->Code = Alloc(int,I->NLine+1);
   I->Sub = Calloc(PyObject*,I->NLine+1);
@@ -202,6 +203,9 @@ Block *PopUpNew(PyMOLGlobals *G,int x,int y,int last_x,int last_y,
 
   if(passive) 
     PyMOL_SetPassive(G->PyMOL,true);
+
+  PAutoUnblock(G,blocked);
+
   return I->Block;
 #endif
 
@@ -313,11 +317,13 @@ static void PopUpFree(Block *block)
 #ifndef _PYMOL_NOPY
   { /* purge code */
     int a;
+    int blocked = PAutoBlock(G);
     for(a=0;a<I->NLine;a++)
       if(I->Sub[a]) {
         Py_DECREF(I->Sub[a]);
       }
-  }
+    PAutoUnblock(G,blocked);
+}
 #endif
 
   OrthoDetach(G,I->Block);
@@ -387,9 +393,9 @@ int PopUpRelease(Block *block,int button,int x,int y,int mod)
     PopUpRecursiveDetach(block);
     if(!I->NeverDragged) 
       if((I->Selected>=0)&&(!I->Sub[I->Selected])) {
-        PLog(I->Command[I->Selected],cPLog_pym);
-        PParse(I->Command[I->Selected]);
-        PFlush();
+        PLog(G,I->Command[I->Selected],cPLog_pym);
+        PParse(G,I->Command[I->Selected]);
+        PFlush(G);
       }
     PopUpRecursiveFree(block);
   }
@@ -418,13 +424,13 @@ int PopUpDrag(Block *block,int x,int y,int mod)
   x-=I->Block->rect.left;
   y =(I->Block->rect.top -cPopUpCharMargin) -y -1;
 
-  if((x<0)||(x>I->Width)) {
+  if((x<-2)||(x>(I->Width+2))) {
     int handled_flag = false;
     if(I->Child) {
       if(PopUpRecursiveFind(I->Child,I->LastX,I->LastY) == I->Child) {
         I->Selected = I->ChildLine;
         handled_flag = true;
-      }
+      } 
     }
     if(!handled_flag) {
       if(I->Parent) { /* are we back in the parent window? */
