@@ -57,7 +57,7 @@ static int TestPyMOL_00_00(PyMOLGlobals *G)
   
   obj = ObjectMapNew(G);
   if(obj) {
-    ms = ObjectMapNewStateFromDesc(G,obj,md,0);    
+    ms = ObjectMapNewStateFromDesc(G,obj,md,0,true);    
     ms->Active=true;
   }
   if(obj) {
@@ -90,7 +90,7 @@ static char *get_st(const char array[][STR_MAX])
   return result;
 }
 
-const static char pdb_01_01[][STR_MAX] = {
+static const char pdb_01_01[][STR_MAX] = {
   "ATOM      1  N   ASP E   1       4.868 -17.809  25.188  1.00 34.37      E    N\n",
   "ATOM      2  CA  ASP E   1       3.984 -16.723  25.698  1.00 33.85      E    C\n",
   "ATOM      3  CB  ASP E   1       4.633 -16.020  26.888  1.00 35.91      E    C\n",
@@ -201,7 +201,7 @@ const static char pdb_01_01[][STR_MAX] = {
   "END\n",
   ""};
 
-const static char mol_01_02[][STR_MAX] = {
+static const char mol_01_02[][STR_MAX] = {
 "MFCD02681585\n",
 "  ChemPy            3D                             0\n",
 "\n",
@@ -292,17 +292,17 @@ int TestPyMOLRun(PyMOLGlobals *G,int group,int test)
       TestPyMOL_00_00(G); 
       break;
     case 1: 
-      PBlock();
+      PBlock(G);
       VFontLoad(G,1,0,0,true); 
-      PUnblock();
+      PUnblock(G);
       break;
     case 2: 
       {
         CObject *obj = NULL;
         float pos[3] = {0.0,0.0,0.0};
-        PBlock();
+        PBlock(G);
         obj = (CObject*)ObjectCGONewVFontTest(G,"hello",pos);
-        PUnblock();
+        PUnblock(G);
         if(obj) {
           ObjectSetName(obj,"hello");
           ExecutiveManageObject(G,obj,-1,false);
@@ -339,7 +339,7 @@ int TestPyMOLRun(PyMOLGlobals *G,int group,int test)
       {
         char *st = get_st(pdb_01_01);
 
-        PyMOL_Load(G->PyMOL, st, "c_string", 0, "pdb", "test_01_01", 0, false, true, true, false, PYMOL_DEFAULT);
+        PyMOL_CmdLoad(G->PyMOL, st, "string", "pdb", "test_01_01", 0, false, true, true, false, PYMOL_DEFAULT);
         ExecutiveSetRepVisib(G,"test_01_01",cRepCyl,1);
         ExecutiveSetRepVisib(G,"test_01_01",cRepLine,0);
         SettingSetGlobal_f(G,cSetting_sweep_speed,3.0F);
@@ -351,7 +351,7 @@ int TestPyMOLRun(PyMOLGlobals *G,int group,int test)
     case 2: 
       {
         char *st = get_st(pdb_01_01);
-        PyMOL_Load(G->PyMOL,st, "c_string", 0, "pdb", "test_01_02", 0, false, true, true, false, PYMOL_DEFAULT);
+        PyMOL_CmdLoad(G->PyMOL,st, "string", "pdb", "test_01_02", 0, false, true, true, false, PYMOL_DEFAULT);
         ExecutiveSetRepVisib(G,"test_01_02",cRepLine,0);
         ExecutiveSetRepVisib(G,"test_01_02",cRepSurface,1);
         ControlRock(G,1);
@@ -362,7 +362,7 @@ int TestPyMOLRun(PyMOLGlobals *G,int group,int test)
     case 3: 
       {
         char *st = get_st(pdb_01_01);
-        PyMOL_Load(G->PyMOL,st, "c_string", 0, "pdb", "test_01_03", 0, false, true, true, false, PYMOL_DEFAULT);
+        PyMOL_CmdLoad(G->PyMOL,st, "string", "pdb", "test_01_03", 0, false, true, true, false, PYMOL_DEFAULT);
         ExecutiveSetRepVisib(G,"test_01_03",cRepLine,0);
         ExecutiveSetRepVisib(G,"test_01_03",cRepCartoon,1);
         SettingSetGlobal_f(G,cSetting_sweep_speed,1.50F);
@@ -374,7 +374,7 @@ int TestPyMOLRun(PyMOLGlobals *G,int group,int test)
     case 4: 
       {
         char *st = get_st(pdb_01_01);
-        PyMOL_Load(G->PyMOL,st, "c_string", 0, "pdb", "test_01_04", 0, false, true, true, false, PYMOL_DEFAULT);
+        PyMOL_CmdLoad(G->PyMOL,st, "string", "pdb", "test_01_04", 0, false, true, true, false, PYMOL_DEFAULT);
         ExecutiveSetRepVisib(G,"test_01_04",cRepLine,0);
         ExecutiveSetRepVisib(G,"test_01_04",cRepDot,1);
         SettingSetGlobal_f(G,cSetting_sweep_speed,1.50F);
@@ -386,7 +386,7 @@ int TestPyMOLRun(PyMOLGlobals *G,int group,int test)
     case 5: 
       {
         char *st = get_st(pdb_01_01);
-        PyMOL_Load(G->PyMOL,st, "c_string", 0, "pdb", "test_01_05", 0, false, true, true, false, PYMOL_DEFAULT);
+        PyMOL_CmdLoad(G->PyMOL,st, "string", "pdb", "test_01_05", 0, false, true, true, false, PYMOL_DEFAULT);
         ExecutiveSetRepVisib(G,"test_01_05",cRepLine,0);
         ExecutiveSetRepVisib(G,"test_01_05",cRepSphere,1);
         SettingSetGlobal_f(G,cSetting_sweep_speed,4.50F);
@@ -398,7 +398,7 @@ int TestPyMOLRun(PyMOLGlobals *G,int group,int test)
     case 6: 
       {
         char *st = get_st(pdb_01_01);
-        PyMOL_Load(G->PyMOL,st, "c_string", 0, "pdb", "test_01_06", 0, false, true, true, false, PYMOL_DEFAULT);
+        PyMOL_CmdLoad(G->PyMOL,st, "string", "pdb", "test_01_06", 0, false, true, true, false, PYMOL_DEFAULT);
         SettingSetGlobal_f(G,cSetting_sweep_speed,4.50F);
         ControlRock(G,1);
         FreeP(st);
@@ -407,11 +407,8 @@ int TestPyMOLRun(PyMOLGlobals *G,int group,int test)
       break;
     case 7: 
       {
-        ObjectMolecule *obj;
         char *st = get_st(mol_01_02);
-        obj=ObjectMoleculeReadMOLStr(G,NULL,st,0,0,1);
-        ObjectSetName((CObject*)obj,"test_01_07");
-        ExecutiveManageObject(G,(CObject*)obj,-1,true);
+        ExecutiveLoad(G,NULL,st,-1,cLoadTypeMOLStr,"test_01_07",0,-1,0,1,0,1,NULL);
         ExecutiveSetRepVisib(G,"test_01_07",cRepCyl,1);
         ExecutiveSetRepVisib(G,"test_01_07",cRepLine,0);
         SettingSetGlobal_b(G,cSetting_valence,1);
@@ -424,11 +421,8 @@ int TestPyMOLRun(PyMOLGlobals *G,int group,int test)
       break;
     case 8: 
       {
-        ObjectMolecule *obj;
         char *st = get_st(mol_01_02);
-        obj=ObjectMoleculeReadMOLStr(G,NULL,st,0,0,1);
-        ObjectSetName((CObject*)obj,"test_01_08");
-        ExecutiveManageObject(G,(CObject*)obj,-1,true);
+        ExecutiveLoad(G,NULL,st,-1,cLoadTypeMOLStr,"test_01_08",0,-1,0,1,0,1,NULL);
         SettingSetGlobal_b(G,cSetting_valence,1);
         ControlRock(G,1);
         FreeP(st);
@@ -437,11 +431,8 @@ int TestPyMOLRun(PyMOLGlobals *G,int group,int test)
       break;
     case 9: 
       {
-        ObjectMolecule *obj;
         char *st = get_st(mol_01_02);
-        obj=ObjectMoleculeReadMOLStr(G,NULL,st,0,0,1);
-        ObjectSetName((CObject*)obj,"test_01_09");
-        ExecutiveManageObject(G,(CObject*)obj,-1,true);
+        ExecutiveLoad(G,NULL,st,-1,cLoadTypeMOLStr,"test_01_09",0,-1,0,1,0,1,NULL);
         ExecutiveSetRepVisib(G,"test_01_09",cRepMesh,1);
         ExecutiveSetRepVisib(G,"test_01_09",cRepLine,0);
         SettingSetGlobal_b(G,cSetting_valence,1);
